Package: tclust
Type: Package
Encoding: UTF-8
Title: Robust Trimmed Clustering
Version: 2.2-0
VersionNote: Released 2.1-2 on 2025-06-29 on CRAN
Authors@R: c(person("Valentin", "Todorov", role = c("aut", "cre"), email = "valentin@todorov.at", comment=c(ORCID = "0000-0003-4215-0245"))
         , person("Luis Angel", "García Escudero", role = "aut")
         , person("Agustín", "Mayo Iscar", role = "aut")
         , person("Javier", "Crespo Guerrero", role = "aut")
         , person("Heinrich", "Fritz", role = "aut")
         )
Maintainer: Valentin Todorov <valentin@todorov.at>
Description: Provides functions for robust trimmed clustering. The methods are 
    described in Garcia-Escudero (2008) <doi:10.1214/07-AOS515>, 
    Fritz et al. (2012) <doi:10.18637/jss.v047.i12>, 
    Garcia-Escudero et al. (2011)  <doi:10.1007/s11222-010-9194-z> and others.
Depends: R(>= 3.6.2)
Imports: Rcpp (>= 1.0.7), doParallel, parallel, foreach, MASS, rlang,
        methods
Suggests: mclust, cluster, sn
LazyLoad: yes
License: GPL-3
LinkingTo: Rcpp, RcppArmadillo
NeedsCompilation: yes
RoxygenNote: 7.3.3
URL: https://github.com/valentint/tclust
BugReports: https://github.com/valentint/tclust/issues
Repository: CRAN
Author: Valentin Todorov [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4215-0245>),
  Luis Angel García Escudero [aut],
  Agustín Mayo Iscar [aut],
  Javier Crespo Guerrero [aut],
  Heinrich Fritz [aut]
Packaged: 2026-04-25 20:39:00 UTC; valen
Date/Publication: 2026-04-26 10:20:03 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-26 23:56:44 UTC; unix
Archs: tclust.so.dSYM

Package: tclust
Type: Package
Encoding: UTF-8
Title: Robust Trimmed Clustering
Version: 2.2-0
VersionNote: Released 2.1-2 on 2025-06-29 on CRAN
Authors@R: c(person("Valentin", "Todorov", role = c("aut", "cre"), email = "valentin@todorov.at", comment=c(ORCID = "0000-0003-4215-0245"))
         , person("Luis Angel", "García Escudero", role = "aut")
         , person("Agustín", "Mayo Iscar", role = "aut")
         , person("Javier", "Crespo Guerrero", role = "aut")
         , person("Heinrich", "Fritz", role = "aut")
         )
Maintainer: Valentin Todorov <valentin@todorov.at>
Description: Provides functions for robust trimmed clustering. The methods are 
    described in Garcia-Escudero (2008) <doi:10.1214/07-AOS515>, 
    Fritz et al. (2012) <doi:10.18637/jss.v047.i12>, 
    Garcia-Escudero et al. (2011)  <doi:10.1007/s11222-010-9194-z> and others.
Depends: R(>= 3.6.2)
Imports: Rcpp (>= 1.0.7), doParallel, parallel, foreach, MASS, rlang,
        methods
Suggests: mclust, cluster, sn
LazyLoad: yes
License: GPL-3
LinkingTo: Rcpp, RcppArmadillo
NeedsCompilation: yes
RoxygenNote: 7.3.3
URL: https://github.com/valentint/tclust
BugReports: https://github.com/valentint/tclust/issues
Repository: CRAN
Author: Valentin Todorov [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4215-0245>),
  Luis Angel García Escudero [aut],
  Agustín Mayo Iscar [aut],
  Javier Crespo Guerrero [aut],
  Heinrich Fritz [aut]
Packaged: 2026-04-25 20:39:00 UTC; valen
Date/Publication: 2026-04-26 10:20:03 UTC

useDynLib(tclust, .registration = TRUE)
importFrom(Rcpp, evalCpp)

importFrom("stats", "mad", "median", "na.omit", "rnorm", "runif")
importFrom("stats", "heatmap", "princomp", "qchisq", "cov")
importFrom("stats", "mahalanobis", "qnorm", "quantile", "sd")
importFrom("stats", "density")
importFrom("utils", "flush.console", "setTxtProgressBar", "txtProgressBar")
importFrom("grDevices", "dev.interactive", "n2mfrow", "grey")
importFrom("graphics", "barplot", "layout", "legend", "lines", "par", "text")
importFrom("graphics", "axis", "box", "mtext", "plot.new", "points", "title")
importFrom("graphics", "abline", "polygon")
importFrom("graphics", "boxplot", "pairs")
importFrom("MASS", "mvrnorm")
importFrom("parallel", "detectCores", "makeCluster", "stopCluster")
importFrom("methods", "is")
import("foreach", "foreach")
import(doParallel)

importFrom("rlang", "check_dots_used")

export(
    simula.rlg,
    simula.tclust,
    rlg,
#    print.rlg,
    plot.rlg,
    tclust,
    print.tclust,
    plot.tclust,
    DiscrFact,
    print.DiscrFact,
    summary.DiscrFact,
    plot.DiscrFact,
#    plot_DiscrFact_p2,
#    plot_DiscrFact_p3,
    ctlcurves,
    print.ctlcurves,
    plot.ctlcurves,
    
#    tclust_c1,
#    tclust_c2,
    
    tkmeans,
    print.tkmeans,
    plot.tkmeans,
    tclustIC,
    tclustICsol,
    randIndex,
    FowlkesMallowsIndex
)

#   S3method (print, rlg)
S3method (plot, rlg)
S3method (print, tclust)
S3method (plot, tclust)
S3method (print, DiscrFact)
S3method (summary, DiscrFact)
S3method (plot, DiscrFact)
S3method (print, ctlcurves)
S3method (plot, ctlcurves)
S3method (print, tkmeans)
S3method (plot, tkmeans)
S3method (print, tclustIC)
S3method (summary, tclustIC)
S3method (print, summary.tclustIC)
S3method (plot, tclustIC)
S3method (print, tclustICsol)
S3method (summary, tclustICsol)
S3method (print, summary.tclustICsol)
S3method (plot, tclustICsol)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# `tclust`: Robust Trimmed Clustering

<!-- badges: start -->

[![CRAN
version](https://www.r-pkg.org/badges/version/tclust)](https://cran.r-project.org/package=tclust)
[![R-CMD-check](https://github.com/valentint/tclust/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/valentint/tclust/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/valentint/tclust/graph/badge.svg)](https://app.codecov.io/gh/valentint/tclust)
[![downloads](https://cranlogs.r-pkg.org/badges/tclust)](https://cran.r-project.org/package=tclust)
[![downloads](https://cranlogs.r-pkg.org/badges/grand-total/tclust)](https://cran.r-project.org/package=tclust)
[![license](https://img.shields.io/badge/license-GPL--3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0.en.html)
<!-- badges: end -->

The package `tclust` provides functions for robust trimmed clustering.
The methods are described in Garcia-Escudero (2008)
[doi:10.1214/07-AOS515](https://doi.org/10.1214/07-AOS515), Fritz et
al. (2012)
[doi:10.18637/jss.v047.i12](https://doi.org/10.18637/jss.v047.i12),
Garcia-Escudero et al. (2011)
[doi:10.1007/s11222-010-9194-z](https://doi.org/10.1007/s11222-010-9194-z)
and others.

## Installation

The `tclust` package is on CRAN (The Comprehensive R Archive Network)
and the latest release can be easily installed using the command

    install.packages("tclust")
    library(tclust)

## Building from source

To install the latest stable development version from GitHub, you can
pull this repository and install it using

    ## install.packages("remotes")
    remotes::install_github("valentint/tclust", build_opts = c("--no-build-vignettes"))

Of course, if you have already installed `remotes`, you can skip the
first line (I have commented it out).

## Example

Outlying data can heavily influence standard clustering methods. At the
same time, clustering principles can be useful when robustifying
statistical procedures. These two reasons motivate the development of
feasible robust model-based clustering approaches. Instead of trying to
“fit” noisy data, a proportion α of the most outlying observations is
trimmed. The `tclust` package efficiently handles different cluster
scatter constraints. Graphical exploratory tools are also provided to
help the user make sensible choices for the trimming proportion as well
as the number of clusters to search for.

``` r
library(tclust)
#> Robust Trimmed Clustering (version 2.2-0)
data (M5data)
x <- M5data[, 1:2]

clus.a <- tclust (x, k = 3, alpha = 0.1, restr.fact =  1,
                  restr = "eigen", equal.weights = TRUE)
clus.b <- tclust (x, k = 3, alpha = 0.1, restr.fact =  1,
                   equal.weights = TRUE)
clus.c <- tclust (x, k = 3, alpha = 0.1, restr.fact =  1,
                  restr = "deter", equal.weights = TRUE)
clus.d <- tclust (x, k = 3, alpha = 0.1, restr.fact = 50,
                  restr = "eigen", equal.weights = FALSE)

pa <- par (mfrow = c (2, 2))
plot (clus.a, main = "(a) tkmeans")
plot (clus.b, main = "(b) Gallegos and Ritter")
plot (clus.c, main = "(c) Gallegos")
plot (clus.d, main = "(d) tclust")
```

![](README-example-1-1.png)<!-- -->

``` r
par (pa)
```

The trimmed k-means clustering method by Cuesta-Albertos, Gordaliza and
Matran (1997) optimizes the k-means criterion under trimming a portion
of the points:

``` r
library(tclust)
data (swissbank)
## Two clusters and 8% trimming level
clus <- tkmeans (swissbank, k = 2, alpha = 0.08)

## Pairs plot of the clustering solution
pairs (swissbank, col = clus$cluster + 1)
```

![](README-example-2-1.png)<!-- -->

``` r
                                  # Two coordinates
plot (swissbank[, 4], swissbank[, 6], col = clus$cluster + 1,
     xlab = "Distance of the inner frame to lower border",
     ylab = "Length of the diagonal")
```

![](README-example-2-2.png)<!-- -->

``` r
plot (clus)
```

![](README-example-2-3.png)<!-- -->

## Community guidelines

### Report issues and request features

If you experience any bugs or issues or if you have any suggestions for
additional features, please submit an issue via the
[*Issues*](https://github.com/valentint/tclust/issues) tab of this
repository. Please have a look at existing issues first to see if your
problem or feature request has already been discussed.

### Contribute to the package

If you want to contribute to the package, you can fork this repository
and create a pull request after implementing the desired functionality.

### Ask for help

If you need help using the package, or if you are interested in
collaborations related to this project, please get in touch with the
package maintainer.

### R code from vignette source 'tclust.rnw'

###################################################
### code chunk number 1: intro
###################################################

options(prompt = "R> ", continue = "+  ", width = 70, useFancyQuotes = FALSE)

## standard margins
mmar <- c(5.1, 4.1, 4.1, 1.1)
og <- grey(0)   ## color for outliers


###################################################
### code chunk number 2: fig-1
###################################################
library(tclust)
set.seed(100)
data(geyser2)
clus <- tkmeans(geyser2, k = 3, alpha=0.03)
plot(clus, col = c(og, 2:4), tol.lwd = 1, tol.lty = 2)


###################################################
### code chunk number 3: chunk2
###################################################

################
##  Figure 2  ##
################

data(M5data)
cl <- M5data[, "cluster"]
plot(M5data[, 1:2], col = cl + 1, pch = cl + 1, main = "The M5data data set")



###################################################
### code chunk number 4: chunk3
###################################################

################
##  Figure 3  ##
################

data(M5data)
x <- M5data[, 1:2]
set.seed(100)

## applying tclust with restr.fact = 1, restr= "eigen"
res.a <- tclust(x, k = 3, alpha=0.1, restr.fact = 1, restr= "eigen",
                 equal.weights = TRUE)

## applying tclust with restr.fact = 50, restr= "eigen"
res.b <- tclust(x, k = 3, alpha=0.1, restr.fact = 50, restr= "eigen",
                 equal.weights = FALSE)

## applying tclust with restr.fact = 1, restr= "deter"
res.c <- tclust(x, k = 3, alpha=0.1, restr.fact = 1, restr= "deter",
                 equal.weights = TRUE)

## applying tclust with restr.fact = 50, restr= "deter"
res.d <- tclust(x, k = 3, alpha=0.1, restr.fact = 50, restr= "deter",
                 equal.weights = TRUE)

op <- par(mfrow = c (2, 2), mar = c (2.1, 2.1, 3.6, 1.1))
col <- c(og, 4, 2, 3)
pch <- c(1, 4, 2, 3)

  ## plotting results..
plot(res.a, col = col, pch = pch, tol.lty = 2, xlab = "",
     ylab = "", main.pre ="(a)", main = "/r")

plot(res.b, col = col, pch = pch, tol.lty = 2, xlab = "",
     ylab = "", main.pre ="(b)", main = "/r")

plot(res.c, col = col, pch = pch, tol.lty = 2, xlab = "",
     ylab = "", main.pre ="(c)", main = "/r")
plot(res.d, col = col, pch = pch, tol.lty = 2, xlab = "",
     ylab = "", main.pre ="(d)", main = "/r")
par(op)




###################################################
### code chunk number 5: chunk4
###################################################

################
##  Figure 4  ##
################

  ## data generation
library(MASS)
set.seed (10)
x <- rbind(MASS::mvrnorm(200, c (0, 0), diag (2)),
           MASS::mvrnorm(200, c (5, 0), diag (2)))

  ## applying tclust
clus.4 <- tclust(x, k = 3, alpha = 0.0, restr.fact = 1)

  ## plotting results..
plot(clus.4, xlim = c (-3, 9), ylim = c (-6, 6))




###################################################
### code chunk number 6: chunk5
###################################################
################
##  Figure 5  ##
################

library(cluster)
data(geyser2)

set.seed(0)

op <- par(mfrow = c (1, 3), mar = c (3.1, 3.1, 3.1, 2.1))

xlab <- "" #names (geyser2)[1]
ylab <- "" #names (geyser2)[2]

  ## applying PAM on the geyser2 data set
clus <- pam(geyser2, 3)
plot(geyser2, xlab = xlab, ylab = ylab, col = clus$clustering + 1,
      main = "(a) PAM with Geyser")
points(clus$medoids, pch = "X", cex = 1.4)

  ## modifying the geyser2 data set
geyser2.mod <- geyser2
idx <- c(16,  21, 36, 171, 236, 265)
geyser2.mod[idx,] <- geyser2[idx,] - 12

  ## applying PAM on the modified geyser2 data set
clus <- pam(geyser2.mod, 3)
plot(geyser2.mod, xlab = xlab, ylab = ylab, col = clus$clustering + 1,
      main = "(b) PAM with modified Geyser")

  ## applying tkmeans on the modified geyser2 data set
clus <- tkmeans(geyser2.mod, k = 3, alpha = 0.03)
plot(clus, xlab = xlab, ylab = ylab,
      main = "(c) TCLUST with modified Geyser", sub = "")
par(op)



###################################################
### code chunk number 7: chunk6
###################################################
################
##  Figure 6  ##
################

fig.elt <- list()

library(sn)

op <- par(mfrow = c (2, 3), mar = c (3.1, 2, 3.1, 2))

set.seed (3)

n <- 200
xlab <- "" #"x1"
ylab <- "" #"x2"

## data generation: Mixture of elliptical t's
xi <- c (0.5, -1)
Omega <- diag (2) / 2 + 0.5
alpha <- c (0, 0)
rnd1 <- rmst (n, xi, Omega, alpha, nu = 2)
xi <- c (6, -3)
Omega <- matrix (c (4, 1, 1, 2), nrow = 2, ncol = 2)
alpha <- c (0, 0)
rnd2 <- rmst (n, xi, Omega, alpha, nu = 3)
xi <- c (-3, 3)
Omega <- matrix (c (2, 1, 1, 4), nrow = 2, ncol = 2)
alpha <- c (0, 0)
rnd3 <- rmst (n, xi, Omega, alpha, nu = 4)
rnd.1 <- rbind (rnd1, rnd2, rnd3)
real.clus1 <- c (rep (1, n), rep (2, n), rep (3, n))

ylim <- c (-25, 15)

 ## Plotting the real clusters
plot(rnd.1, xlim = c (-15, 80), ylim = ylim, xlab = xlab, ylab = ylab,
      col = real.clus1 + 1, pch = real.clus1 + 1)
title("(a) Elliptical t components", line = 1.6)

  ## applying tclust without trimming
clus <- tclust(rnd.1, k = 3, alpha = 0.0)
plot(clus, xlim = c (-15, 80), ylim = ylim, xlab = xlab, ylab = ylab,
      main.pre = "(b)")

 ## applying tclust with trimming (5%)
clus <- tclust(rnd.1, k = 3, alpha = 0.05)
plot(clus, xlim = c (-15, 80), ylim = ylim, xlab = xlab, ylab = ylab,
      main.pre = "(c)")

## data generation: Mixture of non-elliptical t's
xi <- c (0.5, -1)
Omega <- diag (2) / 2 + 0.5
alpha <- c (2, 80)
rnd1 <- rmst (n, xi, Omega, alpha, nu = 2)
xi <- c (6, -3)
Omega <- matrix (c (4, 1, 1, 2), nrow = 2, ncol = 2)
alpha <- c (2, 2)
rnd2 <- rmst (n, xi, Omega, alpha, nu = 3)
xi <- c (-3, 3)
Omega <- matrix (c (2, 1, 1, 4), nrow = 2, ncol = 2)
alpha <- c (4, 4)
rnd3 <- rmst (n, xi, Omega, alpha, nu = 4)

rnd.2 <- rbind (rnd1, rnd2, rnd3)
real.clus2 <- c (rep (1, n), rep (2, n), rep(3, n))

ylim <- c (-10, 20)
  ## Plotting the real clusters
plot (rnd.2, xlim = c (-10, 20), ylim = ylim, xlab = xlab, ylab = ylab,
      col = real.clus2 + 1, pch=real.clus2 + 1)
title ("(d) Non-elliptical t components", line = 1.6)

 ## applying tclust without trimming
clus <- tclust(rnd.2, k = 3, alpha = 0.0)
plot(clus, xlim = c (-10, 20), ylim = ylim, xlab = xlab, ylab = ylab,
      main.pre = "(e)")

 ## applying tclust with trimming (5%)
clus <- tclust(rnd.2, k = 3, alpha = 0.05)
plot(clus, xlim = c (-10, 20), ylim = ylim, xlab = xlab, ylab = ylab,
      main.pre = "(f)")

par(op)




###################################################
### code chunk number 8: chunk7
###################################################

################
##  Figure 7  ##
################

library(mclust)

  ## function for plotting mclust-objects in tclust-style.
mclustplottcluststyle2d <- function (x, clus, clus.perm, tol.lty = 3,
                                     size = sqrt (qchisq (0.95, 2)), ...)
{
  if (missing (clus.perm))
    cuse <- clus$classification
  else
    cuse <- clus.perm [clus$classification + 1]

  plot (x, col = cuse + 1, pch = cuse + 1, ...)

  k <- ncol (clus$parameters$mean)

  for (i in 1:k)
    tclust:::.doEllipses(cov = clus$parameters$variance$sigma[,, i],
                          center = clus$parameters$mean[, i], size = size,
                          lty = tol.lty)
}

op <- par (mfrow = c (2, 2), mar = c (3.1, 3.1, 3.1, 2.1))

  ## example 1 - uniformly distributed noise

  ## data generation

set.seed (0)
x <- rbind (
            MASS::mvrnorm(360, c (0.0,  0), matrix (c ( 1,  0,  0, 1), ncol = 2)),
            MASS::mvrnorm(540, c (5.0, 10), matrix (c ( 6, -2, -2, 6), ncol = 2)),
            cbind (runif (100, -5, 15), runif (100, -10, 20)))

  ## applying mclust
noiseInit <- sample (c (TRUE, FALSE), size = nrow (x),
                     replace = TRUE, prob = c (0.1, 0.9))
clus <- Mclust(x, initialization = list (noise = noiseInit), G = 2)
mclustplottcluststyle2d (x, clus, c (0, 2, 1), xlab = "", ylab = "",
                         main = "(a) mclust", tol.lty = 2)

  ##  applying tclust
clus <- tclust(x, k = 2, alpha = 0.1)
plot (clus, xlab = "", ylab = "", main = "(b) tclust", sub = "", tol.lty = 2)


  ## example 2 - structured noise

  ## data generation
set.seed (0)
v <- runif (100, -2 * pi, 2 * pi)
noise <- cbind (100 + 25 * sin (v), 10 + 5 * v)

x <- rbind (
            MASS::mvrnorm(360, c (0.0,  0), matrix (c (1,  0,  0, 1), ncol = 2)),
            MASS::mvrnorm(540, c (5.0, 10), matrix (c (6, -2, -2, 6), ncol = 2)),
            noise)

  ##  applying mclust
noiseInit <- sample (c (TRUE, FALSE), size = nrow (x),
                     replace = TRUE, prob = c (0.1, 0.9))
clus <- Mclust (x, initialization = list (noise = noiseInit), G = 2)
mclustplottcluststyle2d (x, clus, c (0, 2, 1), xlab = "", ylab = "",
                         main = "(c) mclust", tol.lty = 2)

  ##  applying tclust
clus <- tclust(x, k = 2, alpha = 0.1)
plot (clus, xlab = "", ylab = "", main = "(d) tclust", sub = "", tol.lty = 2)

par (op)




###################################################
### code chunk number 9: chunk8
###################################################

################
##  Figure 8  ##
################

  ## data generation
set.seed (100)
mixt <- rbind(MASS::mvrnorm(360, c (  0,  0), matrix (c (1,  0,  0,  1), ncol = 2)),
              MASS::mvrnorm(540, c (  5, 10), matrix (c (6, -2, -2,  6), ncol = 2)),
              MASS::mvrnorm(100, c (2.5,  5), matrix (c (50, 0,  0, 50), ncol = 2)))

  ## applying tclust
set.seed (100)
clus.1 <- tclust(mixt, k = 3, alpha =  0.0, restr.fact = 50)
clus.2 <- tclust(mixt, k = 2, alpha = 0.05, restr.fact =  8)

  ## plotting results
op <- par (mfrow = c (1, 2))
plot(clus.1, by.clust = TRUE, col = c (og, 2, 3, 4), pch = c (1, 2, 3, 4),
     tol.lty = 2, main.pre = "(a)")
plot(clus.2, by.clust = TRUE, col = c (og, 2, 3), pch = c (1, 2, 3),
     tol.lty = 2, main.pre = "(b)")
par (op)




###################################################
### code chunk number 10: chunk9
###################################################

################
##  Figure 9  ##
################

  ## computing ctlcurves
ctl <- ctlcurves(mixt, restr.fact = 50, alpha = seq (0, 0.2, by = 0.05), trace=0)

op <- par(mfrow = c (1, 1))
plot (ctl)
par (op)



###################################################
### code chunk number 11: fig-10
###################################################
#################
##  Figure 10  ##
#################

data (geyser2)
op <- par (mfrow = c (1, 2))
set.seed (10)

  ## creating a one dimensional data "matrix"
geyser1 <- geyser2[, 1, drop = FALSE]
  ## applying tkmeans
plot(tkmeans(geyser1, k = 2, alpha = 0.03), jitter = TRUE, tol.lwd = 2,
      main.pre = "(a)")

  ## adding a random dimension to geyser2
geyser3 <- cbind(geyser2, rnorm (nrow (geyser2)))
  ## applying tkmeans
plot(tkmeans(geyser3, k = 3, alpha = 0.03), main.pre = "(b)")

par(op)



###################################################
### code chunk number 12: fig-11
###################################################

#################
##  Figure 11  ##
#################

## data generation
set.seed (100)
mixt2 <- rbind (
  MASS::mvrnorm(360, c (0,   0), matrix (c (1,  0,  0,  1), ncol = 2)),
  MASS::mvrnorm(540, c (5,  10), matrix (c (6, -2, -2,  6), ncol = 2)),
  MASS::mvrnorm(100, c (2.5, 5), matrix (c (50, 0,  0, 50), ncol = 2)))

  ## applying tclust
set.seed (100)
clus.w <- tclust(mixt2, k = 3, alpha = 0.1, restr.fact = 1,
                  equal.weights = TRUE)

  ## applying DiscrFact
discr.clus.w <- DiscrFact(clus.w)

  ## plotting results
op <- par(mfrow = c(1, 3), mar = mmar)

plot(clus.w, col = c(og, 2, 3, 4), tol.lty = 2, main.pre = "(a)")
tclust:::plot_DiscrFact_p2(discr.clus.w, xlim = c (-70, 0), main.pre = "(b)")
tclust:::plot_DiscrFact_p3(discr.clus.w, tol.lty = 2, main.pre = "(c)")

par(op)


###################################################
### code chunk number 13: fig12
###################################################
#################
##  Figure 12  ##
#################

data (swissbank)
set.seed (0)
fig6.ctl <- ctlcurves(swissbank, k = 1:4, alpha = seq (0, 0.3, by = 0.025), restr.fact = 50, trace=0)

op <- par (mfrow = c (1, 1), mar = mmar)
plot (fig6.ctl)

par (op)


###################################################
### code chunk number 14: fig13
###################################################
#################
##  Figure 13  ##
#################

fig7.clus <- tclust(swissbank, k = 2, alpha = .1, restr.fact = 50)
fig7.discrfact <- DiscrFact(fig7.clus, threshold = .000125)

op <- par (mfrow = c (1, 3), mar = mmar)
plot (fig7.discrfact, enum = TRUE)
par (op)


###################################################
### code chunk number 15: fig14
###################################################

#################
##  Figure 14  ##
#################


clus <- tclust(swissbank, k = 2, alpha=.1, restr.fact=50)
discrfact <- DiscrFact(clus)
pch <- c (rep ("G", 100), rep ("F", 100))
condition <- discrfact$assignfact > log (.000125)
cl <- clus$cluster
data (swissbank)
op <- par (mfrow = c (1, 3), mar = mmar)

xlab <- "Distance of the inner frame to lower border"
ylab <- "Length of the diagonal"

plot(swissbank[, 4], swissbank[, 6], col = "darkgrey", pch = pch,
      main = "(a) Cluster1", xlab = xlab, ylab = ylab)

cl1 <- cl == 1
points(swissbank[cl1, 4], swissbank[cl1, 6], pch = pch[cl1], col = 2)
idx <- (cl1) & condition
points(swissbank[idx, 4], swissbank[idx, 6], pch = 1, cex = 4, col = "blue")

plot(swissbank[, 4], swissbank[, 6], col = "darkgrey", pch = pch,
      main = "(b) Cluster2", xlab = xlab, ylab = ylab)
cl2 <- cl == 2
points(swissbank[cl2, 4], swissbank[cl2, 6], pch = pch[cl2], col = 3)
idx <- (cl2) & condition
points(swissbank[idx, 4], swissbank[idx, 6], pch = 1, cex = 4, col = "blue")

cl0 <- cl == 0
plot(swissbank[, 4], swissbank[, 6], col = "darkgrey", pch = pch,
      main = "(c) Trimmed", xlab = xlab, ylab = ylab)

points(swissbank[cl0, 4], swissbank[cl0, 6], pch = pch[cl0])
idx <- (cl0) & condition
points(swissbank[idx, 4], swissbank[idx, 6], pch = 1, cex = 4, col = "blue")
par(op)

if(FALSE)
{
    library(tclust)
    library(MASS)
    
    x <- as.matrix(iris[, -5])
    dm1 <- tclust:::dmvnrm(x, colMeans(x), cov(x))
    dm2 <- .dmnorm(x, colMeans(x), cov(x))
    dm3 <- mvtnorm::dmvnorm(x, colMeans(x), cov(x))
    cbind.data.frame(dm1=dm1, dm2=dm2, dm3=dm3, diff1=round(dm1-dm2, 12), diff2=round(dm1-dm3, 12))
}

##  Multivariate normal density
.dmnorm <- function (X, mu, sigma)
{
  ((2 * pi)^(-length(mu) / 2)) *
  (det(sigma)^(-1/ 2)) *
  exp (-0.5 * .evmaha (X, mu, sigma))
}

.evmaha <- function (X, mu, sigma)    ##  calculate mahalanobis distances 
                                      ##  using the eigenvalues and eigenvectors. 
                                      ##  thus no singularity problems arise.
{                                     ##  Mahalanobis distance == Inf is possible.
    v <- eigen (sigma)
    Zt <- t (v$vectors) %*% (t (X) - mu)
    colSums ((Zt * v$values^(-.5))^2)
}

##  VT::11.10.2023 - this will render the output independent
##  from the version of the package
suppressPackageStartupMessages(library(tclust))

require(tclust)
require(MASS)
#--- EXAMPLE 1 ------------------------------------------

set.seed(123)
sig <- diag (2)
cen <- rep (1,2)
x <- rbind(MASS::mvrnorm(360, cen * 0,   sig),
           MASS::mvrnorm(540, cen * 5,   sig * 6 - 2),
           MASS::mvrnorm(100, cen * 2.5, sig * 50)
           )

# Two groups and 10% trimming level
(clus <- tclust(x, k = 2, alpha = 0.1, restr.fact = 8))


# Three groups (one of them very scattered) and 0% trimming level
(clus <- tclust(x, k = 3, alpha=0.0, restr.fact = 100))


#--- EXAMPLE 2 ------------------------------------------
data(geyser2)
set.seed(123)
(clus <- tclust(geyser2, k=3, alpha=0.03))
plot(clus)
 
#--- EXAMPLE 3 ------------------------------------------
data (M5data)
set.seed(123)
x <- M5data[, 1:2]

(clus.a <- tclust(x, k=3, alpha=0.1, restr.fact=1,
                  restr = "eigen", equal.weights=TRUE))
(clus.b <- tclust(x, k=3, alpha=0.1, restr.fact=50,
                    restr="eigen", equal.weights=TRUE))
(clus.c <- tclust(x, k=3, alpha=0.1, restr.fact=1,
                    restr="deter", equal.weights=TRUE))
(clus.d <- tclust(x, k=3, alpha=0.1, restr.fact=50,
                    restr="eigen", equal.weights=FALSE))

#--- EXAMPLE 4 ------------------------------------------
data (swissbank)
set.seed(123)
# Two clusters and 8% trimming level
(clus <- tclust(swissbank, k = 2, alpha = 0.08, restr.fact = 50))

# Three clusters and 0% trimming level
(clus <- tclust(swissbank, k = 3, alpha = 0.0, restr.fact = 110))


#--- EXAMPLE 5 ------------------------------------------
data (flea)
# Three clusters and 8% trimming level
set.seed(123)
(clus <- tclust(flea[, 1:6], k = 3, alpha = 0.08, restr.fact = 50))
##  adjustedRand(clus$cluster, as.integer(flea[,7]))

# Three clusters and 0% trimming level
set.seed(123)
(clus <- tclust(flea[,1:6], k = 3, alpha = 0.0, restr.fact = 110))
##  adjustedRand(clus$cluster, as.integer(flea[,7]))

# Three clusters and 10% trimming level
set.seed(123)
(clus <- tclust(flea[,1:6], k = 3, alpha = 0.1, restr="deter", restr.fact = 110))
##  adjustedRand(clus$cluster, as.integer(flea[,7]))

##### Discriminant Factor Analysis for tclust Objects ####################
sig <- diag (2)
cen <- rep (1, 2)
x <- rbind(MASS::mvrnorm(360, cen * 0,   sig),
	       MASS::mvrnorm(540, cen * 5,   sig * 6 - 2),
	       MASS::mvrnorm(100, cen * 2.5, sig * 50)
)
(clus.1 <- tclust(x, k = 2, alpha = 0.1, restr.fact = 12))

(clus.2 <- tclust(x, k = 3, alpha = 0.1, restr.fact = 1))
  ##  restr.fact and k are chosen improperly for pointing out the
  ##    difference in the plot of DiscrFact

(dsc.1 <- DiscrFact(clus.1))
(dsc.2 <- DiscrFact(clus.2))


########## Classification Trimmed Likelihood Curves  ################### 

## Do not run - it takes too long and can show differences on some
##  architectures due to the random numbers.
##
#--- EXAMPLE 1 ------------------------------------------

if(FALSE) {
    sig <- diag (2)
    cen <- rep (1, 2)
    x <- rbind(MASS::mvrnorm(108, cen * 0,   sig),
    	       MASS::mvrnorm(162, cen * 5,   sig * 6 - 2),
    	       MASS::mvrnorm(30, cen * 2.5, sig * 50)
    )

    (ctl <- ctlcurves(x, k = 1:4))
}

#--- EXAMPLE 2 ------------------------------------------
if(FALSE) {
    data (geyser2)
    (ctl <- ctlcurves(geyser2, k = 1:5))
}

##  roxygen2::roxygenise("C:/users/valen/onedrive/myrepo/r/tclust", load_code=roxygen2:::load_installed)

#'
#' Computes the Fowlkes and Mallows index
#' 
#' @name FowlkesMallowsIndex
#' @aliases print.FowlkesMallowsIndex
#' @description  Fowlkes-Mallows index is an external evaluation method
#'  that is used to determine the similarity between two clusterings
#'  (clusters obtained after a clustering algorithm). This measure of
#'  similarity could be either between two hierarchical clusterings or a
#'  clustering and a benchmark classification. A higher the value for the
#'  Fowlkes-Mallows index indicates a greater similarity between the clusters
#'  and the benchmark classifications.
#'  This index can be used to compare either two cluster label sets or a
#'  cluster label set with a true label set. The formula of the
#'  adjusted Fowlkes-Mallows index (ABk) is given in the details.
#'
#' @param c1 Labels of the first partition or contingency table. A numeric or 
#'  character vector containing the class labels of the first partition or 
#'  a 2-dimensional numeric matrix which contains the cross-tabulation 
#'  of cluster assignments.
#' @param c2 Labels of the second partition. A numeric or character vector 
#'  containing the class labels of the second partition. 
#'  The length of vector c2 must be equal to the length of vector c1. 
#'  This second input is required only if \code{c1} is not a 2-dimensional numeric matrix.
#'
#' @param noisecluster Label or number associated to the \emph{noise class} or \emph{noise level}.
#'  Number or character label which denotes the points which do not belong to any cluster.
#'  These points are not takern into account for the  computation of the Fowlkes and Mallows index
#'
#' @return A list containing the following components:
#'  \item{ABK}{Adjusted Fowlkes and Mallows index. A number between -1 and 1. The adjusted 
#'  Fowlkes and Mallows index is the corrected-for-chance version of the Fowlkes and Mallows index.}
#'  \item{BK}{Value of the Fowlkes and Mallows index. A number between 0 and 1. }
#'  \item{EBk}{Expected value of the index computed under the null hypothesis of no-relation.}
#'  \item{VarBk}{Variance of the Fowlkes and Mallows index. Variance of the index computed under the null
#'      hypothesis of no-relation.}
#'
#' @details The formula of the adjusted Fowlkes-Mallows index (ABk) is as follows:
#'
#'  \deqn{ABk=\frac{Bk-Expected~value~of~Bk}{Max~Index - Expected~value~of~Bk}}
#'
#' @references
#'  Fowlkes, E.B. and Mallows, C.L. (1983), A Method for Comparing Two Hierarchical Clusterings, 
#'  \emph{Journal of the American Statistical Association}, Vol. 78, pp. 553--569.
#'
#' @seealso \code{\link{randIndex}}
#' @examples
#'
#'  ##  1. FowlkesMallowsIndex (adjusted) with the two vectors as input
#'  c <- matrix(c(1, 1, 1, 2, 2, 1,2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3), 
#'          ncol=2, byrow=TRUE)
#'
#'  ##  c1 - numeric vector containing the labels of the first partition
#'  c1 <- c[, 1]
#'
#'  ##  c2 - numeric vector containing the labels of the second partition
#'  c2 <- c[, 2]
#'
#'  (FM <- FowlkesMallowsIndex(c1, c2))
#'
#'  ##  2. FM index (adjusted) with the contingency table as input.
#'  T <- matrix(c(1, 1, 0, 1, 2, 1, 0, 0, 4), ncol=3, byrow=TRUE)
#'  (FM <- FowlkesMallowsIndex(T))
#'
#'
#'  ##  3. Compare FM (unadjusted) for iris data (true classification against 
#'  ##      tclust classification).
#'
#'  ##  First partition c1 is the true partition
#'  c1 <- iris$Species
#'
#'  ##  Second partition c2 is the output of tclust clustering procedure
#'  out <- tclust(iris[, 1:4], k=3, alpha=0, restr.fact=100)
#'  c2<- out$cluster
#'
#'  (FM <- FowlkesMallowsIndex(c1, c2))
#'
#'  ##  4. Compare FM index (unadjusted) for iris data (exclude unassigned units from tclust).
#'
#'  ##  First partition c1 is the true partition
#'  c1 <- iris$Species
#'
#'  ##  Second partition c2 is the output of tclust clustering procedure
#'  out <- tclust(iris[, 1:4], k=3, alpha=0.1, restr.fact=100)
#'  c2<- out$cluster
#'
#'  ##  Units inside c2 which contain number 0 are referred to trimmed observations
#'  noisecluster <- 0
#'  (FM <- FowlkesMallowsIndex(c1, c2, noisecluster=noisecluster))
#'

FowlkesMallowsIndex <- function(c1, c2 = NULL, noisecluster = NULL) {

    if(!is.matrix(c1))
        c1 <- as.matrix(c1)
    if(!missing(c2) && !is.matrix(c2))
        c2 <- as.matrix(c2)
        
    if(missing(c2)) {
        if(ncol(c1) < 2) {
            stop("FowlkesMallowsIndex: Requires a contingency table with at least two columns")
        }
        
        M <- c1
    } else {
        if(min(dim(c1)) > 1 || min(dim(c2)) > 1) {
            stop("FowlkesMallowsIndex: Requires two vector arguments")
        }
    
        if(!is.null(noisecluster)) {
            boo1 <- if (is.character(c1) || is.factor(c1)) c1 == noisecluster else c1 == noisecluster
            boo2 <- if (is.character(c2) || is.factor(c2)) c2 == noisecluster else c2 == noisecluster
            keep <- !(boo1 | boo2)
            c1 <- c1[keep]
            c2 <- c2[keep]
        }
    
        if (is.character(c1) || is.factor(c1) || is.character(c2) || is.factor(c2)) {
            M <- table(c1, c2)
        } else {
            class_c1 <- sort(unique(c1))
            class_c2 <- sort(unique(c2))
            Rows <- length(class_c1)
            Cols <- length(class_c2)
            M <- matrix(0, nrow = Rows, ncol = Cols)
            for (i in seq_len(Rows)) {
                for (j in seq_len(Cols)) {
                    M[i, j] <- sum(c1 == class_c1[i] & c2 == class_c2[j])
                }
            }
            
        }    
    }

    midot <- rowSums(M)
    mdotj <- colSums(M)
    n <- sum(M)
    
    Tk <- sum(M^2) - n
    Pk <- sum(midot^2) - n
    Qk <- sum(mdotj^2) - n
    Bk <- Tk / sqrt(Pk * Qk)
    
    EBk <- sqrt(Pk * Qk) / (n * (n - 1))
    
    Pk2 <- sum(midot * (midot - 1) * (midot - 2))
    Qk2 <- sum(mdotj * (mdotj - 1) * (mdotj - 2))

    VarBk <- 2 / (n * (n - 1)) +
        4 * Pk2 * Qk2 / ((n * (n - 1) * (n - 2)) * Pk * Qk) +
        (Pk - 2 - 4 * Pk2 / Pk) * (Qk - 2 - 4 * Qk2 / Qk) /
        (n * (n - 1) * (n - 2) * (n - 3)) -
        Pk * Qk / (n^2 * (n - 1)^2)

    ABk <- (Bk - EBk) / (1 - EBk)

    return(list(ABk = ABk, Bk = Bk, EBk = EBk, VarBk = VarBk))
}

#' Plot an 'rlg' object
#'
#' @description Different plots for the results of 'rlg' analysis, stored in an 
#'    \code{rlg} object, see Details.
#'
#' @param x An \code{rlg} object to plot.
#' @param which Select the required plot.
#' @param sort Whether to sort.
#' @param ask if \code{TRUE}, the user is \emph{ask}ed before each plot, see \code{par(ask=.)}. 
#'  Default is \code{ask = which=="all" && dev.interactive()}.  
#' @param \dots Other parameters to be passed to the lower level functions.
#'
#' @examples
#'  data (LG5data)
#'  x <- LG5data[, 1:10]
#'  clus <- rlg(x, d = c(2,2,2), alpha=0.1)
#'  plot(clus, which="eigenvalues") 
#'  plot(clus, which="scores") 
#'
plot.rlg <- function(x, which=c("all", "scores", "loadings", "eigenvalues"), 
    sort = TRUE,
    ask = (which=="all" && dev.interactive(TRUE)), ...){
    
    ## option "clusters" removed
    which <- match.arg(which)
      
    if(!inherits(x, "rlg"))
        stop("Use only with 'rlg' objects")
    
    op <- if (ask) par(ask = TRUE) else list()
    on.exit(par(op))
    
    if(which == "all" || which == "scores")
        .plot.scores(x)
    if(which == "all" || which == "loadings")
        .plot.loadings(x)
    if(which == "all" || which == "eigenvalues")
        .plot.eigenvalues(x)
    
    invisible(x)
}

.plot.clusters <- function(x, sort){
  index <- .sort(x, sort)
  
  p <- ncol(x$x)
  mat <- matrix(rep(x$cluster, each=p), ncol=p, byrow=T)
  
  rownames(mat) <- .get.rownames(x)
  colnames(mat) <- .get.colnames(x)
  
  heatmap(mat[index,], Rowv = NA, Colv = NA, scale = "none", col = 1:(length(x$d)+1))
}

.plot.scores <- function(x){
  
  K <- which(x$dimensions >= 2)
  
  if(length(K) > 0) {
      par(mfrow=rev(n2mfrow(length(K))))
      for(k in K){
        scores <- princomp(x$x[x$cluster == k, ],cor = TRUE)$scores
        plot(scores[,1], scores[,2],
             main=paste("Cluster", k), xlab="dim 1", ylab="dim 2", col="white")
        text(scores[,1], scores[,2],
             as.character(substr(.get.rownames(x)[x$cluster==k], 1, 3)))
      }
  }
}

.plot.loadings <- function(x){
  dim <- rev(n2mfrow(length(x$dimensions)))
  d.max <- min(max(x$dimensions),3)
  dim[1] <- dim[1]*d.max
  mat <- matrix(1:(dim[1]*dim[2]), nrow = dim[1], ncol = dim[2])
  layout(mat)
  for(k in 1:length(x$dimensions)){
    n2 <- min(x$dimensions[k], 3)
    for(i2 in 1:d.max){
      if(i2 <= n2){
        title <- ifelse(i2 == 1, paste("Cluster", k, "\nDim", i2) , paste("Dim", i2))
        barplot(x$U[[k]][,i2], main=title)
      }else{
        plot(0,type='n',axes=FALSE,ann=FALSE)
      }
    }
  }
  par(mfrow=c(1,1))
}

.plot.eigenvalues <- function(x){
  p <- ncol(x$x)
  eigenvalues <- matrix(NA,nrow=length(x$dimensions),ncol=p)
  for (k in 1:length(x$dimensions)){
    eigenvalues[k,] <- princomp(x$x[x$cluster == k, ],cor = TRUE)$sdev^2
  }
  lim.sup <- max(eigenvalues)
  plot(eigenvalues[1,],ylim=c(0,lim.sup),type="n",xlab="intrisic dimensions",ylab="eigenvalues")
  for (k in 1:length(x$dimensions)){
    lines(eigenvalues[k,],col=k+1,type="b",pch=19)
  }
  legend("topright",
         legend=paste("Cluster",as.character(1:length(x$dimensions))),
         col=1:length(x$dimensions)+1, lty=rep(1,length(x$dimensions)),
         pch=rep(19,length(x$dimensions)))
}

.sort <- function(x, sort){
  if(sort)
    index <- order(x$cluster)
  else
    index <- 1:length(x$cluster)
  
  return(index)
}

.get.rownames <- function(x){
  if(!is.null(rownames(x$x)))
    return(rownames(x$x))
  else
    return(1:nrow(x$x))
}

.get.colnames <- function(x){
  if(!is.null(colnames(x$x)))
    return(colnames(x$x))
  else
    return(1:ncol(x$x))
}

##  roxygen2::roxygenise("C:/users/valen/onedrive/myrepo/r/tclust", load_code=roxygen2:::load_installed)

#'
#' Extracts a set of best relevant solutions obtained by \code{\link{tclustIC}}
#' 
#' @name tclustICsol
#' @aliases print.tclustICsol
##' @description  The function \code{tclustICsol()} takes as input an object of class
#'  \code{\link{tclustIC}}, the output
#'  of function \code{\link{tclustIC}} (that is a series of matrices which contain
#'  the values of the information criteria BIC/ICL/CLA for different values of \code{k}
#'  and \code{c}) and extracts the first best solutions. Two solutions are considered
#'  equivalent if the value of the adjusted Rand index (or the adjusted Fowlkes and
#'  Mallows index) is above a certain threshold. For each tentative solution the program
#'  checks the adjacent values of \code{c} for which the solution is stable.
#'  A matrix with adjusted Rand indexes is given for the extracted solutions.
#'
#' @param obj An S3 object of class \code{\link{tclustIC}}
#'  (output of \code{\link{tclustIC}}) containing the values
#'  of the information criteria BIC (MIXMIX), ICL (MIXCLA) or CLA (CLACLA),
#'  for different values of \code{k} (number of groups) and different
#'  values of \code{c} (restriction factor), for a prespecified level of trimming.
#' @param whichIC A character value which Specifies the information criterion to 
#'  use to extract best solutions. Possible values for \code{whichIC} are:
#'  \itemize{
#'   \item "MIXMIX": a mixture model is fitted and for computing the information criterion
#'      the mixture likelihood is used. This option corresponds to the use of the Bayesian
#'      Information criterion (BIC). In output just the matrix \code{MIXMIX} is given.
#'  \item "MIXCLA": a mixture model is fitted but to compute the information criterion
#'      the classification likelihood is used. This option corresponds to the use of the
#'      Integrated Complete Likelihood (ICL). In the output just the matrix \code{MIXCLA} is given.
#'  \item "CLACLA": everything is based on the classification likelihood. This information
#'      criterion will be called CLA. In the output just the matrix \code{CLACLA} is given.
#'  \item "ALL": both classification and mixture likelihood are used. In this case all
#'      three information criteria CLA, ICL and BIC are computed. In the output all
#'      three matrices \code{MIXMIX}, \code{MIXCLA} and \code{CLACLA} are given.
#'  }
#' @param nsol Number of best solutions to extract from BIC/ICL matrix.
#'  The default value of NumberOfBestSolutions is 5
#' @param index Index to use to compare partitions. If \code{index=Rand} (default) 
#'  the adjusted Rand index is used, else, \code{index="FM"}, the adjusted 
#'  Fowlkes and Mallows index is used
#' @param thresholdRI Threshold to identify spurious solutions - the threshold
#'  of the adjusted Rand index to use to consider two solutions as equivalent.
#' The default value of ThreshRandIndex is 0.7
#' @param trace Whether to print intermediate results. Default is \code{trace=FALSE}.
#' @return The function returns an S3 object of type \code{tclustICsol} containing the following components:
#'  \item{call}{the matched call}
#'  \item{kk}{a vector containing the values of \code{k} (number of components) which have been considered.
#'      This vector is identical to the optional argument \code{kk} (default is \code{kk=1:5}.}
#'  \item{cc}{a vector containing the values of \code{c} (values of the restriction factor) which
#'      have been considered. This vector is identical to the optional argument \code{cc}
#'   (defalt is \code{cc=c(1, 2, 4, 8, 16, 32, 64, 128)}.}
#'  \item{alpha}{trimming level}
#'  \item{whichIC}{Information criteria used}
#'  \item{MIXMIXbs}{a matrix of lists of size \code{NumberOfBestSolutions-times-5} which
#'  contains the details of the best solutions for MIXMIX (BIC). Each row refers to a
#'  solution. The information which is stored in the columns is as follows.
#'  \itemize{
#'  \item 1st col = value of k for which solution takes place
#'  \item 2nd col = value of c for which solution takes place;
#'  \item 3rd col = a vector of length \code{d} which contains the values of \code{c}
#'      for which the solution is uniformly better.
#'  \item 4th col = a vector of length \code{d + r} which contains the values of \code{c}
#'      for which the solution is considered stable (i.e. for which the value
#'      of the adjusted Rand index (or the adjusted Fowlkes and Mallows index)
#'      does not go below the threshold defined in input option \code{ThreshRandIndex}).
#'  \item 5th col = string which contains 'true' or 'spurious'. The solution is labelled
#'      spurious if the value of the adjusted Rand index with the previous solutions
#'      is greater than ThreshRandIndex.
#'  }
#'
#'  Remark: the field MIXMIXbs is present only if \code{whichIC=ALL} or \code{whichIC="MIXMIX"}.
#'  }
#'  \item{MIXMIXbsari}{a matrix of adjusted Rand indexes (or Fowlkes and Mallows indexes)
#'      associated with the best solutions for MIXMIX. A matrix of size \code{NumberOfBestSolutions-times-NumberOfBestSolutions}
#'      whose \code{i,j}-th entry contains the adjusted Rand index between classification produced by solution
#'      \code{i} and solution \code{j}, \code{i,j=1,2, ...,NumberOfBestSolutions}.
#'
#'  Remark: the field \code{MIXMIXbsari} is present only if \code{whichIC=ALL} or \code{whichIC="MIXMIX"}.
#'  }
#'  \item{ARIMIX}{a matrix of adjusted Rand indexes between two consecutive value of \code{c}.
#'      Matrix of size \code{k-by-length(cc)-1}. The first column contains the ARI indexes
#'      between \code{cc[2]} and \code{cc[1]} given \code{k}.
#'      The second column contains the the ARI indexes between \code{cc[3]} and \code{cc[2]} given \code{k}.
#'
#'  Remark: the field \code{ARIMIX} is present only if \code{whichIC=ALL} or \code{whichIC="MIXMIX"} or \code{whichIC="MIXCLA"}.
#'  }
#'  \item{MIXCLAbs}{has the same structure as \code{MIXMIXbs} but referres to MIXCLA.
#'
#'  Remark: the field MIXCLAbs is present only if \code{whichIC=ALL} or \code{whichIC="MIXCLA"}.
#'  }
#'  \item{MIXCLAbsari}{has the same structure as \code{MIXMIXbsari} but referres to MIXCLA.
#'
#'  Remark: the field \code{MIXMIXbsari} is present only if \code{whichIC=ALL} or \code{whichIC="MIXCLA"}.
#'  }
#'  \item{CLACLAbs}{has the same structure as \code{MIXMIXbs} but referres to CLACLA.
#'
#'  Remark: the field CLACLAbs is present only if \code{whichIC=ALL} or \code{whichIC="CLACLA"}.
#'  }
#'  \item{CLACLAbsari}{has the same structure as \code{MIXMIXbsari} but referres to CLACLA.
#'
#'  Remark: the field \code{CLACLAbsari} is present only if \code{whichIC=ALL} or \code{whichIC="CLACLA"}.
#'  }
#'  \item{ARICLA}{a matrix of adjusted Rand indexes between two consecutive value of \code{c}.
#'      Matrix of size \code{k-by-length(cc)-1}. The first column contains the ARI indexes
#'      between \code{cc[2]} and \code{cc[1]} given \code{k}.
#'      The second column contains the the ARI indexes between \code{cc[3]} and \code{cc[2]} given \code{k}.
#'
#'  Remark: the field \code{ARICLA} is present only if \code{whichIC=ALL} or \code{whichIC="CLACLA"}.
#'  }
#'  \item{x}{the input data matrix of size \code{n-times-p} with which
#'      the Information Criteria were computed.}
#'
#' @references
#'      Cerioli, A., Garcia-Escudero, L.A., Mayo-Iscar, A. and Riani M. (2017).
#'      Finding the Number of Groups in Model-Based Clustering via Constrained Likelihoods,
#'      \emph{Journal of Computational and Graphical Statistics}, pp. 404--416,
#'      https://doi.org/10.1080/10618600.2017.1390469.
#'
#'  Hubert L. and Arabie P. (1985), Comparing Partitions, \emph{Journal of Classification}, 
#'      Vol. 2, pp. 193--218.
#'
#' @seealso \code{\link{tclust}}, \code{\link{tclustIC}}
#' @examples
#'
#'  #--- EXAMPLE 1 ------------------------------------------
#'  \donttest{
#'  data(geyser2)
#'  (out <- tclustIC(geyser2, whichIC="MIXMIX", alpha=0.1))
#'
#'  ##  Show the first two best solutions using as Information criterion MIXMIX
#'  cat("\nBest solutions using MIXMIX\n")
#'  outsol <- tclust::tclustICsol(out, whichIC="MIXMIX", nsol=2)
#'  print(outsol$MIXMIXbs)
#'  }
#'
#'  #--- EXAMPLE 2 ------------------------------------------
#'  \donttest{
#'  data(flea)
#'  Y <- as.matrix(flea[, 1:(ncol(flea)-1)])    # select only the numeric variables
#'  rownames(Y) <- 1:nrow(Y)
#'  head(Y)
#'
#'  (out <- tclustIC(Y, whichIC="CLACLA", alpha=0.1))
#'  ## Find the smallest value inside the table and write the corresponding
#'  ## values of k (number of groups) and c (restriction factor)
#'  inds <- which(out$CLACLA == min(out$CLACLA), arr.ind=TRUE)
#'  vals <- out$CLACLA[inds]
#'  cat("\nThe Smallest value of the IC is ", vals, 
#'      " and takes place for k=", out$kk[inds[1]], " and c=",   
#'      out$cc[inds[2]], "\n")
#'
#'  ##  Show the first two best solutions using as Information criterion CLACLA
#'  cat("\nBest solutions using CLACLA\n")
#'  outsol <- tclust::tclustICsol(out, whichIC="CLACLA", nsol=2)
#'  print(outsol$CLACLAbs)
#'  
#'  }
#'
#'  #--- EXAMPLE 3 ------------------------------------------
#'  \donttest{
#'  data(swissbank)
#'  (out <- tclustIC(swissbank, whichIC="ALL"))
#'  
#'  outsol <- tclust::tclustICsol(out, whichIC="ALL", nsol=2)
#'  print(outsol$CLACLAbs)
#'  
#'  }
#'

tclustICsol <- function(obj, whichIC=c("ALL", "MIXMIX", "MIXCLA", "CLACLA"), 
        nsol=5, index=c("Rand", "FM"), thresholdRI=0.7, trace=FALSE) {

    if(!inherits(obj, "tclustIC"))
      stop ("parameter obj: expected object of type \x22tclustIC\x22")

    whichIC <- match.arg(whichIC)
    index <- match.arg(index)

    if(whichIC != obj$whichIC && obj$whichIC != "ALL") {        # Not OK combinations
        if(whichIC == "ALL")
            stop(paste0("The \x22tclustIC\x22 object \x22obj\x22 was computed with IC=\x22", obj$whichIC, "\x22!"))
        if(whichIC == "CLACLA" && obj$whichIC != "CLACLA")
            stop("The \x22tclustIC\x22 object \x22obj\x22 was not computed with IC=\x22CLACLA\x22!")
        if(whichIC != "CLACLA" && obj$whichIC == "CLACLA")
            stop("The \x22tclustIC\x22 object \x22obj\x22 was computed with IC=\x22CLACLA\x22!")
    }
    
    kk <- obj$kk
    cc <- obj$cc
    alpha <- obj$alpha
    nkk <- length(kk)
    ncc <- length(cc)

    ARIMIX <- ARICLA <- NULL
    if(whichIC != "CLACLA")                     
        ARIMIX <- matrix(1, nrow=nkk, ncol=ncc)
    if(whichIC == "CLACLA" || whichIC == "ALL")  
        ARICLA <- matrix(1, nrow=nkk, ncol=ncc)
    
    ## loop for different values of k (number of groups)    
    for(k in 1:nkk) {
        if(kk[k] == 1)   # do the computations for ARI or FM indexes when k is different from 1
            next
        
        ## loop through the different values of c
        for(j in 2:ncc) {
            if(whichIC != "CLACLA") {
                idjm1 <- obj$IDXMIX[[k, j-1]]
                idj <- obj$IDXMIX[[k, j]]
                ARIMIX[k, j] <- if(index == "Rand") randIndex(idjm1, idj)$AR else 
                                                    FowlkesMallowsIndex(idjm1, idj)$ABk
            }
            if(whichIC == "CLACLA" || whichIC == "ALL") {
                idjm1 <- obj$IDXCLA[[k, j-1]]
                idj <- obj$IDXCLA[[k, j]]
                ARICLA[k, j] <- if(index == "Rand") randIndex(idjm1, idj)$AR else 
                                                    FowlkesMallowsIndex(idjm1, idj)$ABk
            }
        }
    }

    ret <- list(call=match.call(), kk=kk, cc=cc, alpha=alpha, whichIC=whichIC, x=obj$x, ARIMIX=ARIMIX, ARICLA=ARICLA)

    if(whichIC == "MIXMIX" || whichIC == "ALL") {
    
        objbs <- findBestSolutions(pll=obj$MIXMIX, ARI=ARIMIX, IDX=obj$IDXMIX, 
            kk=kk, cc=cc, nsol=nsol, thresholdRI=thresholdRI, trace=trace)
        
        ret$MIXMIXbs <- objbs$Bestsols
        ret$MIXMIXbsari <- objbs$ARIbest
        
        ret$ARIMIX <- ARIMIX[, 2:ncol(ARIMIX)]

        ## Store matrix which contains in the columns the details of the
        ## classification             
        ##  - nrow(ret$MIXMIXbs) is the number of existing solutions - could be less than nsol             
        for(i in 1:nrow(ret$MIXMIXbs)) {
            kx <- which(kk == ret$MIXMIXbs[[i, 1]])
            cx <- which(cc == ret$MIXMIXbs[[i, 2]])
            IDX <- obj$IDXMIX[[kx, cx]]
            if(i == 1)
                ret$MIXMIXbsIDX <- matrix(NA, nrow=length(IDX), ncol=nsol)
            ret$MIXMIXbsIDX[, i] <- IDX    
        }
    }
   
    if(whichIC == "MIXCLA" || whichIC == "ALL") {
    
        objbs <- findBestSolutions(pll=obj$MIXCLA, ARI=ARIMIX, IDX=obj$IDXMIX, 
            kk=kk, cc=cc, nsol=nsol, thresholdRI=thresholdRI, trace=trace)
        
        ret$MIXCLAbs <- objbs$Bestsols
        ret$MIXCLAbsari <- objbs$ARIbest
        
        ret$ARIMIX <- ARIMIX[, 2:ncol(ARIMIX)]

        ## Store matrix which contains in the columns the details of the
        ## classification             
        ##  - nrow(ret$MIXCLAbs) is the number of existing solutions - could be less than nsol             
        for(i in 1:nrow(ret$MIXCLAbs)) {
            kx <- which(kk == ret$MIXCLAbs[[i, 1]])
            cx <- which(cc == ret$MIXCLAbs[[i, 2]])
            IDX <- obj$IDXMIX[[kx, cx]]
            if(i == 1)
                ret$MIXCLAbsIDX <- matrix(NA, nrow=length(IDX), ncol=nsol)
            ret$MIXCLAbsIDX[, i] <- IDX    
        }
    }
   
    if(whichIC == "CLACLA" || whichIC == "ALL") {
    
        objbs <- findBestSolutions(pll=obj$CLACLA, ARI=ARICLA, IDX=obj$IDXCLA, 
            kk=kk, cc=cc, nsol=nsol, thresholdRI=thresholdRI, trace=trace)

        ret$CLACLAbs <- objbs$Bestsols
        ret$CLACLAbsari <- objbs$ARIbest
        
        ret$ARICLA <- ARICLA[, 2:ncol(ARICLA)]

        ## Store matrix which contains in the columns the details of the
        ## classification.
        ##  - nrow(ret$CLACLAbs) is the number of existing solutions - could be less than nsol             
        for(i in 1:nrow(ret$CLACLAbs)) {
            kx <- which(kk == ret$CLACLAbs[[i, 1]])
            cx <- which(cc == ret$CLACLAbs[[i, 2]])
            IDX <- obj$IDXCLA[[kx, cx]]
            if(i == 1)
                ret$CLACLAbsIDX <- matrix(NA, nrow=length(IDX), ncol=nsol)
            ret$CLACLAbsIDX[, i] <- IDX    
        }
    }
   
    xkk <- paste0("k=", kk)
    xcc <- paste0("c", cc[2:length(cc)], "_vs_c", cc[1:(length(cc)-1)])

    if(!is.null(ret$ARIMIX))
        dimnames(ret$ARIMIX) <- list(xkk, xcc)
    if(!is.null(ret$ARICLA))
        dimnames(ret$ARICLA) <- list(xkk, xcc)

    if(!is.null(ret$MIXMIXbs))
        dimnames(ret$MIXMIXbs) <- list(1:nrow(ret$MIXMIXbs), c("k", "c", "c (uniformly better)", "c (stable)", "Solution"))
    if(!is.null(ret$MIXCLAbs))
        dimnames(ret$MIXCLAbs) <- list(1:nrow(ret$MIXCLAbs), c("k", "c", "c (uniformly better)", "c (stable)", "Solution"))
    if(!is.null(ret$CLACLAbs))
        dimnames(ret$CLACLAbs) <- list(1:nrow(ret$CLACLAbs), c("k", "c", "c (uniformly better)", "c (stable)", "Solution"))

    if(!is.null(ret$MIXMIXbsari))
        dimnames(ret$MIXMIXbsari) <- list(1:nrow(ret$MIXMIXbsari), 1:ncol(ret$MIXMIXbsari))
    if(!is.null(ret$MIXCLAbsari))
        dimnames(ret$MIXCLAbsari) <- list(1:nrow(ret$MIXCLAbsari), 1:ncol(ret$MIXCLAbsari))
    if(!is.null(ret$CLACLAbsari))
        dimnames(ret$CLACLAbsari) <- list(1:nrow(ret$CLACLAbsari), 1:ncol(ret$CLACLAbsari))

    if(!is.null(ret$MIXMIXbsIDX))
        dimnames(ret$MIXMIXbsIDX) <- list(1:nrow(ret$MIXMIXbsIDX), 1:ncol(ret$MIXMIXbsIDX))
    if(!is.null(ret$MIXCLAbsIDX))
        dimnames(ret$MIXCLAbsIDX) <- list(1:nrow(ret$MIXCLAbsIDX), 1:ncol(ret$MIXCLAbsIDX))
    if(!is.null(ret$CLACLAbsIDX))
        dimnames(ret$CLACLAbsIDX) <- list(1:nrow(ret$CLACLAbsIDX), 1:ncol(ret$CLACLAbsIDX))

    class(ret) <- "tclustICsol"
    ret
}

##  pll - matrix of size length(kk) x length(cc) containing penalized log likelihood
##      (MIXMIX, MIXCLA or CLACLA returned by tclustIC()).
##  ARI - matrix of size length(kk) x length(cc) containing Adjusted Rand
##      indexes between two consecutive values of c for fixed k
##  IDX - matrix of lists of size length(kk) x length(cc) containing the 
##      classification vector for each k (rows) and c (columns)
##  kk  - vector containing values of k which have to be considered
##  cc  - vector containing values of c which have to be considered
findBestSolutions <- function(pll, ARI, IDX, kk, cc, nsol, thresholdRI, trace=FALSE) {

    ARI <- t(ARI)
    Xcmod <- t(pll)
    lcc <- length(cc)
    seqcc <- 1:lcc
    seqkk <- 1:length(kk)

    ## Initialize Bestsols as a matrix of lists
    Bestsols <- matrix(vector("list", nsol * 5), nrow = nsol, ncol = 5)
    Bestsols[[1, 5]] <- 'true'

    endofloop <- FALSE
    lab <- "c="
    NumberOfExistingSolutions <- nsol

    for (z in 1:nsol) {
        valmin <- apply(Xcmod, 2, min)
        indmin <- apply(Xcmod, 2, which.min)
        minBICk <- which.min(valmin)
        minBICc <- indmin[minBICk]

        if(min(valmin) < Inf) {
            if(trace) {
                cat("Best solution number:", z, "\n")
                cat("k=", kk[minBICk], "\n")
                cat(lab, cc[minBICc], "\n")
            }

            Xcmod[indmin[minBICk], minBICk] <- Inf
            cwithbestsol <- rep(NA, length(cc))
            cwithbestsol[minBICc] <- 1
            Bestsols[[z, 1]] <- kk[minBICk]
            Bestsols[[z, 2]] <- cc[minBICc]
            
            if(trace) {
                cat("Checking adjacent values for extension\n")
            }

            XcmodWithoutBestk <- Xcmod
            XcmodWithoutBestk[, minBICk] <- Inf
            minICconstr <- min(XcmodWithoutBestk)
            cctoadd <- rep(0, length(cc))

            candcabove <- seqcc[seqcc > minBICc]
            if (length(candcabove) > 0) {
                for (r in candcabove) {
                    if (ARI[r, minBICk] > thresholdRI && Xcmod[r, minBICk] < minICconstr) {
                        Xcmod[r, minBICk] <- Inf
                        cctoadd[r] <- 1
                    } else {
                        break
                    }
                }
            }

            candcbelow <- seqcc[seqcc < minBICc]
            if (length(candcbelow) > 0) {
                for (r in rev(candcbelow)) {
                    if (ARI[r + 1, minBICk] > thresholdRI && Xcmod[r, minBICk] < minICconstr) {
                        Xcmod[r, minBICk] <- Inf
                        cctoadd[r] <- 1
                    } else {
                        break
                    }
                }
            }

            cwithbestsol[cctoadd == 1] <- 1
            
            ## Store the values of c associated to the best solution
            Bestsols[[z, 3]] <- cc[which(cwithbestsol == 1)]

            ##  The interval of values of c for which the solution is uniformly
            ##  better has been found, however, now we have to make sure that we
            ##  do not consider anymore the solutions for the same k which have a
            ##  value of R index adjacent to those which have already been found
            ##  greater than a certain threshold
            ##  Remark. If minBICk =1

            if (minBICk == 1) {
                Bestsols[[z, 4]] <- cc
                Xcmod[, minBICk] <- Inf
            } else {
                intc <- seqcc[which(cwithbestsol == 1)]
                outc <- cc[which(is.na(cwithbestsol))]
                cctoadd <- rep(0, length(cc))
                
                if(length(outc) > 0) {

                    ##  candcbelow - indexes of the set of candidate values for c which are smaller than
                    ##  those associated with best solutions found so far. 
                    ##  For example, if candc =(1 2) it means that we check whether for the same value of k
                    ##  solution with cc(2) and cc(1) have an R index greater than a certain threshold.
                    ##  If it is the case this means that these solutions do not have to be considered anymore
                
                    candcbelow <- seqcc[seqcc < min(intc)]
                    if (length(candcbelow) > 0) {
                        for (r in rev(candcbelow)) {
                            if (ARI[r + 1, minBICk] > thresholdRI) {
                                Xcmod[r, minBICk] <- Inf
                                cctoadd[r] <- 1
                            } else {
                                break
                            }
                        }
                    }
            
                    candcabove <- seqcc[seqcc > max(intc)]
                    if (length(candcabove) > 0) {
                        for (r in candcabove) {
                            if (ARI[r, minBICk] > thresholdRI) {
                                Xcmod[r, minBICk] <- Inf
                                cctoadd[r] <- 1
                            } else {
                                break
                            }
                        }
                    }
            
                    Bestsols[[z, 4]] <- cc[which(cctoadd == 1)]
                }
            }

            ##  get out of the loop because in this case there is a new candidate
            ##  solution (with the same k because the R index was smaller than a
            ##  certain threshold, or with a different k)
                    
            ##  Before getting out of the loop, check if the solution
            ##  which has just been found, has a Rand index greater than a
            ##  certain threshold with those which have already been found
            if(z > 1) {
                k_idx <- which(kk == Bestsols[[z, 1]])
                c_idx <- which(cc == Bestsols[[z, 2]])
                idxcurrentz <- IDX[[k_idx, c_idx]]
                
                for(j in 1:(z - 1)) {
                    prev_k_idx <- which(kk == Bestsols[[j, 1]])
                    prev_c_idx <- which(cc == Bestsols[[j, 2]])
                    idxpreviousz <- IDX[[prev_k_idx, prev_c_idx]]
                    
                    if(randIndex(idxpreviousz, idxcurrentz, 0)$AR > thresholdRI && Bestsols[[j, 5]] == 'true') {
                        Bestsols[[z, 5]] <- 'spurious'
                        break
                    } else {
                        Bestsols[[z, 5]] <- 'true'
                    }
                }
            }
        } else {    ## if Xcmod is full of inf than get out of the loop
            NumberOfExistingSolutions <- z - 1
            Bestsols <- Bestsols[1:NumberOfExistingSolutions, , drop = FALSE]
            endofloop <- TRUE
            break
        }
    }

    if(endofloop && trace) {
        cat("There are at most", NumberOfExistingSolutions, "different solutions\n")
        
        ##  break
    }

    ##  Find matrix of ARI for the z solutions which have been found
    ARIbest <- matrix(0, NumberOfExistingSolutions, NumberOfExistingSolutions)
    for (i in 1:NumberOfExistingSolutions) {
        for (j in 1:NumberOfExistingSolutions) {
            k_idx_i <- which(kk == Bestsols[[i, 1]])
            c_idx_i <- which(cc == Bestsols[[i, 2]])
            idxi <- IDX[[k_idx_i, c_idx_i]]
            
            k_idx_j <- which(kk == Bestsols[[j, 1]])
            c_idx_j <- which(cc == Bestsols[[j, 2]])
            idxj <- IDX[[k_idx_j, c_idx_j]]

            idxi[idxi < 0] <- 0
            idxj[idxj < 0] <- 0
            ARIbest[i, j] <- randIndex(idxi, idxj, 0)$AR
        }
    }

    return(list(Bestsols = Bestsols, ARIbest = ARIbest))
} 

#' @export
print.tclustICsol <- function(x, digits = max(3, getOption("digits") - 3), ...) {
    cat("\nCall:\n", deparse(x$call), "\n", sep = "")
    cat("\nInformation criteria for TCLUST:", x$whichIC, "\n", paste0("Trimming = ", x$alpha))
    cat("\nNumber of mixture components (clusters):", x$kk)
    cat("\nvalues of the restriction factor:", x$cc, "\n")
    if(!is.null(x$MIXMIXbs)) {
        cat("\n\nBest solutions for MIXMIX (BIC):\n")
        ##  print.default(format(x$MIXMIXbs, digits = digits), print.gap = 2, quote = FALSE)
        print(x$MIXMIXbs)
    }
    if(!is.null(x$CLACLAbs)) {
        cat("\n\nBest solutions for CLACLA (CLA):\n")
        ##  print.default(format(x$CLACLAbs, digits = digits), print.gap = 2, quote = FALSE)
        print(x$CLACLAbs)
    }
    if(!is.null(x$MIXCLAbs)) {
        cat("\n\nBest solutions for MIXCLA (IC):\n")
        ##  print.default(format(x$MIXCLAbs, digits = digits), print.gap = 2, quote = FALSE)
        print(x$MIXCLAbs)
    }

     invisible(x)
}

#' @export
summary.tclustICsol <- function (object, ...) {
    ans <- list(tclustICobj=object)
    class(ans) <- "summary.tclustICsol"
    ans
}

#' @export
print.summary.tclustICsol <- function(x, digits = max(3, getOption("digits") - 3), ...) {
    cat("\nCall:\n",
	paste(deparse(x$tclustICobj$call), sep = "\n", collapse = "\n"), "\n", sep = "")

    cat("\nInformation criteria for TCLUST:", x$tclustICobj$whichIC, "\n", paste0("Trimming = ", x$tclustICobj$alpha))
    cat("\nNumber of mixture components (clusters):", x$tclustICobj$kk)
    cat("\nValues of the restriction factor:", x$tclustICobj$cc, "\n")

    invisible(x)
}

#' Simulate contaminated data set for applying rlg
#' 
#' @description Simulate alpha*100\% contaminated data set for applying rlg by 
#'  generating a k=3 components with equal size  and            # 
#'  common underlying dimension q_1=q_2=q_3=q
#' 
#' @param n number of observations
#' @param p dimension (\code{p >= 2} and \code{p > q})
#' @param q intrinsic dimension
#' @param var The smaller 'var' the smaller the scatter around the lower dimensional space
#' @param sep.means Parameter controlling the location vectors separation
#' @param alpha contamination level
#' 
#' @return a list with the following items
#' \itemize{
#'     \item x - The generated dataset
#'     \item true - The true classification
#' }
#' 
#' @export
#' 
#' @examples 
#'  res <- simula.rlg(q=5, p=200, n=150, var=0.01, sep.means=0.00)
#'  plot(res$x,col=res$true+1)
#'
simula.rlg <- function (q = 2, p = 10, n = 200, var = 0.01, sep.means = 0, 
          alpha = 0.05) 
{
    if (q >= p) 
        stop("Invalid intrinsic dimension")
    
    ## 21.02.2024: with the fix of Luis Angel: replace the ceiling() by floor()
    ##  then extend the third partas necessary to obtain a data set with n rows.
    ##
    ##  par_n = max(2, ceiling(n * (1 - alpha)/3))
    par_n = max(2, floor(n * (1 - alpha)/3))
    par_sd = sqrt(var)
    rot1 = qr.Q(qr(matrix(rnorm(p * p), ncol = p)))
    rot2 = qr.Q(qr(matrix(rnorm(p * p), ncol = p)))
    U1 = qr.Q(qr(matrix(rnorm(p * q), ncol = q)))
    U2 = rot1 %*% U1
    U3 = rot2 %*% U2
    Q1 = matrix(runif(q * par_n), ncol = par_n)
    Q2 = matrix(runif(q * par_n), ncol = par_n)
    X1 = rnorm(p, 0, sep.means) %*% t(rep(1, par_n)) + U1 %*% 
        Q1 + matrix(rnorm(p * par_n, sd = par_sd), ncol = par_n)
    X2 = rnorm(p, 0, sep.means) %*% t(rep(1, par_n)) + U2 %*% 
        Q2 + matrix(rnorm(p * par_n, sd = par_sd), ncol = par_n)
    
    n.cont <- floor(n*alpha)
    n3 <- n-2*par_n-n.cont
    Q3 = matrix(runif(q * n3), ncol = n3)
    X3 = rnorm(p, 0, sep.means) %*% t(rep(1, n3)) + U3 %*% 
        Q3 + matrix(rnorm(p * n3, sd = par_sd), ncol = n3)
    
    X <- t(cbind(X1, X2, X3))
    rr <- max(1, abs(X))
    X.cont <- matrix(runif(n.cont * p, -rr, rr), ncol = p)
    X <- rbind(X, X.cont)
    true <- c(rep(1:2, each = par_n), rep(3,n3), rep(0, n.cont))
    
    return(list(x = X, true = true))
}

#'
#' @export

print.tclust <- function (x, ...)
{
  cat ("* Results for TCLUST algorithm: *\n")
  cat ("opt=", x$par$opt, ", trim = ", x$par$alpha, ", k = ", x$k, 
    "\nRestriction on: ", ifelse(x$par$restr=="eigen", "eigenvalues", "determinants"), "\n\n", sep = "")

  cat ("Classification (trimmed points are indicated by 0", "):\n")

  print (x$cluster)
  cat ("Means:\n")
  print (x$centers)
  if (x$obj < (-1e+20))
    warning ("The solution is not reliable. More iterations are probably needed.")
  cat ("\nTrimmed objective function: ", x$obj, "\n")

    if (!is.null (x$par$restr.fact))
        cat ("Selected restriction factor:", x$par$restr.fact, "\n")
  
    ##  cat (round (x$int$iter.converged / x$int$iter.successful* 100), 
    ##        "% of iterations converged successfully.\n", sep= "")

  invisible(x)
}

#' @export

print.tkmeans <- function (x, ...) {
  cat ("* Results for TKMEANS algorithm: *\n")
  cat ("trim = ", x$par$alpha, ", k = ", x$k, "\n", sep = "")

  cat ("Classification (trimmed points are indicated by 0", "):\n")

  print (x$cluster)
  cat ("Means:\n")
  print (x$centers)
  if (x$obj < (-1e+20))
    warning ("The solution is not reliable. More iterations are probably needed.")
  cat ("\nTrimmed objective function: ", x$obj, "\n")

  ##    if (!is.null (x$restr.fact))
  ##      cat ("Selected restriction factor:", x$restr.fact, "\n")
  
  ##    cat (round (x$int$iter.converged / x$int$iter.successful* 100), 
  ##          "% of iterations converged successfully.\n", sep= "")

  invisible(x)
}

.restr.none <- function (iter, pa)
{
	iter$code = TRUE
	return (iter)
}

##	restricts the clusters' covariance matrices by averaging them
.restr.avgcov <- function (iter, pa)
{
	s.all <- matrix (0, pa$p, pa$p)
	for (k in 1:pa$K)
		s.all <- s.all + iter$sigma[,,k] * iter$csize[k]/sum(iter$csize)

	iter$sigma[,,] <- s.all			##

	iter$code = sum(diag (s.all)) > pa$zero.tol
	return (iter)
}

##	restricts the clusters' covariance matrices by restricting it's eigenvalues or determinants without restrics directions 
.restr.diffax <- function (iter, pa, f.restr.eigen = .restr2_eigenv, restr.deter, restr.fact = 12)
{
	if (pa$p == 1)								#	one - dimensional
		if (restr.fact == 1)					#	all variances should be equal -> use the simpler .restr.avgcov function instead
			return (.restr.avgcov (iter, pa))	#
		else									#
			restr.deter <- FALSE				#	else - if p == 1 always use the simpler eigen - restriction 

	if (!missing (restr.deter))					#	evaluating the appropriate restriction function
		f.restr.eigen <- if (restr.deter) .restr2_deter_ else .restr2_eigenv								#n

	u <- array (NA, c(pa$p, pa$p, pa$K))
	d <- array (NA, c(pa$p, pa$K))
	for (k in 1:pa$K)
	{
		ev <- eigen (iter$sigma[,,k])
		u [,,k] <- ev$vectors
		d [,k] <- ev$values
	}

	d [d < 0] <- 0		##	all eigenvalue < 0 are restricted to 0 BECAUSE: min (d) < 0 -> max (d) / min (d) < 0 which would always fit our eigenvalue - criterion!
						##	there is *NO* reason *AT ALL* for touching *ANY* eigenvalue > 0!! (e.g. 0 <= d < 1e-16!)

	d <- f.restr.eigen (d, iter$csize, restr.fact, pa$zero.tol)

						##	checking for singularity in all clusters.
	iter$code = max(d) > pa$zero.tol

	if (!iter$code)
		return (iter)

	for (k in 1:pa$K)	##	re-composing the sigmas
		iter$sigma[,,k] <- u[,,k] %*% diag (d[,k], nrow = pa$p) %*% t(u[,,k])
	return (iter)
}

.diag_matmult_y_x_yt <- function (x, y) diag (y %*% x %*% t(y))			#	a simple helper function for use with "apply"
.sum_diag_matmult_y_x <- function (x, y) sum (diag (y %*% x))			#	a simple helper function for use with "apply"

# Function for restricted principal components which allows restrictions type 0 and 1
.restr.dir <- function (iter, pa, restr.deter = FALSE, restr.fact = 12, n_inic = 10, n_iter1 = 10, n_iter2 = 10, ...)
{
	if (pa$p == 1 ||								# only one dim -> use the simpler .restr.diffax function instead, as directions make no sense here.
		(restr.fact == 1 && !restr.deter))			# all eigenvalues should be equal -> use the simpler .restr.diffax function instead!
		return (.restr.diffax (iter, pa, restr.fact = restr.fact))

													#	evaluating the appropriate restriction function
	f.restr.eigen <- if (restr.deter) .restr2_deter_ else .restr2_eigenv

	for (inic in 1:n_inic)
	{
		#initial solution for u
#o		ee <- matrix(runif(pa$p*pa$p, min=0, max=1),pa$p,pa$p)
		ee	<- matrix (rnorm (pa$p * (pa$p+1)), ncol=pa$p)					#n	as suggested by agustin 20110217
		w <- eigen(t(ee)%*%ee)
		u <- w$vectors

		#iterations
		for (iter1 in 1: n_iter1)
		{
			dd <- apply (iter$sigma, 3, .diag_matmult_y_x_yt , y = t(u))

			dd <- f.restr.eigen (dd, iter$csize, restr.fact, pa$zero.tol)

			u <- .optvectors (iter, pa, u, dd, n.iter = n_iter2, ...)
		}

		res <- 0

		for (i in 1:pa$K)
			res <- res + iter$csize[i] * sum (diag (u %*% diag (1 / dd[,i]) %*% t (u) %*% iter$sigma[,,i]))

										##	no need to store all solutions; only saving the best solution

		if (inic == 1 || res < best$res)	#n	store the first/best solution
			best <- list (u = u, dd = dd, res = res)
	}

	## reconstructing sgima
										
	iter$code <- max(best$dd) > pa$zero.tol

	if (!iter$code)						#	if function failed
		return (iter)
										#	calculate the new sigmas
	for (i in 1:pa$K)					#	using the best found solution
		iter$sigma[,, i] <- best$u %*% diag (best$dd[, i]) %*% t (best$u)

	return(iter)
}

# Function for vector optimization
.optvectors <- function (iter, pa, u, d, n.iter, ovv = 0)
{
	#iterations in each pair of directions

	for (iter2 in 1:n.iter)
	{
#t		for (j in 1:pa$p)
		for (j in 1:(pa$p - 1))
		{
#t			for (m in 1: pa$p)
			for (m in (j+1): pa$p)
			{
				a_mj <- matrix(0, pa$p, pa$p)
				for (i in 1:pa$K)
					a_mj <- a_mj + (d[j,i]   - d[m,i]  ) / (d[j,i]   * d[m,i]  ) * iter$csize[i] * iter$sigma[,,i]		## NOTE - this is actually equal for each iteration of iter2. Thus these values could be calculated in a 

				a2_mj <- t (u[,c(j,m)]) %*% a_mj %*% u[,c(j,m)]
				v <- eigen(a2_mj)
				uu <- u[,c(j,m)] %*% v$vectors

#cat ("values:\n")
#print (v$values)
#cat ("vecs:\n")
#print (v$vectors)
##browser ()

				if (ovv == 0)
				{
##	### NEW:	new part without checking for labels and signs.
					u[,j] <- uu[,1]
					u[,m] <- uu[,2]
				}
				else if (ovv == 1)
				{
##	### 2DELETE:	old part which checked the labels and signs
					#checking labels and signs in order to allow identifiability

					up1 <- up2 <- u
					up1[,j] <- up2[,m] <- uu[,1]
					up1[,m] <- up2[,j] <- uu[,2]


					resp1 <- resp2 <- 0
					for (i in 1:pa$K)
					{
						resp1 <- resp1 + iter$csize[i] * sum (diag (up1 %*% diag (1 / d[,i]) %*% t (up1) %*% iter$sigma[,,i]))	#n	extracted iter$csize[i] out of sum (diag (...))
						resp2 <- resp2 + iter$csize[i] * sum (diag (up2 %*% diag (1 / d[,i]) %*% t (up2) %*% iter$sigma[,,i]))	#n
					}

					less <- (resp1 - resp2) <= sqrt (pa$zero.tol)

					neg1 <- t (uu[, 1 + !less]) %*% u[, j] < 0		#	transforming the logical variable ("less") into the appropriate index
					neg2 <- t (uu[, 1 +  less]) %*% u[, m] < 0		#	

					u <- if (less) up1 else up2						#	storing the proper up1 or up2 matrix in u

					if (neg1) u[, j] <- -u[, j]						#	changing the sign of the proper columns (if necessary)
					if (neg2) u[, m] <- -u[, m]
				}
#	print (u)
#	cat ("\n")
			}
		}
	}
	return(u)
}

# Function for proporcionality case (only restrictions type 1 are allowed )
.restr.prop <- function (iter, pa, restr.fact = 12, n_inic = 10, n_iter1 = 10, n_iter2 = 10, ...)
{
	if (pa$p == 1)							## only one dim -> use the simpler .restr.diffax function instead!
		return (.restr.diffax (iter, pa, restr.fact = restr.fact))
	if (restr.fact == 1)					## all covmats should be exactly equal -> use the simpler .restr.avgcov function instead!
		return (.restr.avgcov (iter, pa))

	for (inic in 1:n_inic)
	{
		#initial solution
#o		ee	<- matrix (runif (pa$p * pa$p, min = 0, max = 1), pa$p, pa$p)
		ee	<- matrix (rnorm (pa$p * (pa$p+1)), ncol=pa$p)					#n	as suggested by agustin 20110217
		w	<- eigen (t (ee) %*% ee)
		u	<- w$vectors
		sf	<- w$values / (prod (w$values)^(1/pa$p))
				##	remark: if w$values[pa$p] is very small we might run in numerical problems here!

		#iterations
		for (iter1 in 1: n_iter1)
		{
			dd <- apply (iter$sigma, 3, .diag_matmult_y_x_yt , y = t(u))
			st <- colSums (dd / sf) / pa$p

			st <- .restr2_eigenv (t (st), iter$csize, restr.fact^(1 / pa$p), pa$zero.tol)

			sf <- colSums	(t (dd) * iter$csize / as.numeric (st))

			sf <- sf / prod(sf)^(1 / pa$p)

			#u estimation

			dd <- sf %*% st							#	st is a 1 x K matrix (from .restr2_eigenv)
			u <- .optvectors(iter, pa, u, dd, n.iter = n_iter2, ...)
		}

		tempSum <- apply (iter$sigma, 3, .sum_diag_matmult_y_x, y = u %*% diag (1/sf) %*% t (u))
		res <- sum (iter$csize * (log (st^pa$p) + tempSum / st))

		if (inic == 1 || res < best$res)
			best <- list (u = u, sf = sf, st = st, res = res)
	}

	iter$code <- max(best$sf) > pa$zero.tol

	if (!iter$code)								#	if function failed
		return (iter)
												# reconstruct the sigma - matrices
	for (i in 1:pa$K)
		iter$sigma[,,i] <- best$st[i] * best$u %*% diag (best$sf) %*% t (best$u)

#a	iter$sigma <- sapply (best$st, get ("*"), sigmaBase, simplify = FALSE) - iter$sigma	#a alternative: think about future representation of sigma as a list!

	return(iter)
}

##' Calculates Rand type Indices to compare two partitions
##' @param c1 labels of the first partition or contingency table. A numeric vector
##' or factor containining the class labels of the first partition or a 
##' 2-dimensional numeric matrix which contains the cross-tabulation of 
##' cluster assignments.
##' @param c2 labels of the second partition. A numeric vector or a factor 
##' containining the class labels of the second partition. The length of 
##' the vector \code{c2} must be equal to the length of the vector \code{c1}. 
##' The second parameter is required only if c1 is not a 2-dimensional numeric matrix.
##' @param noisecluster label or number associated to the 'noise class' or 'noise level'.
##' Number or character label which denotes the points which do not belong to any
##' cluster. These points are not takern into account for the computation of the 
##' Rand type indexes. The default is to consider all points.
##'
##' @return A list with Rand type indexes:
##' \itemize{
##'     \item AR Adjusted Rand index. A number between -1 and 1. The adjusted 
##'         Rand index is the corrected-for-chance version of the Rand index. 
##'     \item RI Rand index (unadjusted). A number between 0 and 1. Rand index 
##'         computes the fraction of pairs of objects for which both 
##'         classification methods agree. RI ranges from 0 (no pair classified 
##'         in the same way under both clusterings) to 1 (identical clusterings).
##'     \item MI Mirkin's index. A number between 0 and 1. Mirkin's index computes 
##'         the percentage of pairs of objects for which both classification 
##'         methods disagree. \code{MI=1-RI}.
##'     \item HI Hubert index. A number between -1 and 1. HI index is equal to the 
##'         fraction of pairs of objects for which both classification methods 
##'         agree minus the fraction of pairs of objects for which both classification 
##'         methods disagree. \code{HI= RI-MI}. 
##' }
##' @examples
##' ##  1. randindex with the contingency table as input.
##' T <- matrix(c(1, 1, 0, 1, 2, 1, 0, 0, 4), nrow=3)
##' (ARI <- randIndex(T))
##' 
##' ##  2. randindex with the two vectors as input.
##' c <- matrix(c(1, 1, 1, 2, 2, 1, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3), ncol=2, byrow=TRUE)
##' ## c1 = numeric vector containing the labels of the first partition
##' c1 <- c[,1]
##' ## c2 = numeric vector containing the labels of the second partition
##' c2 <- c[,2]
##' 
##' (ARI <- randIndex(c1,c2))
##' 
##' ##  3. Compare ARI for iris data (true classification against tclust classification)
##' library(tclust)
##' c1 <- iris$Species  # first partition c1 is the true partition
##' out <- tclust(iris[, 1:4], k=3, alpha=0, restr.fact=100)
##' c2 <- out$cluster   # second partition c2 is the output of tclust clustering procedure
##' 
##' randIndex(c1,c2)
##' 
##' ##  4. Compare ARI for iris data (exclude unassigned units from tclust).
##' 
##' c1 <- iris$Species      # first partition c1 is the true partition
##' out <- tclust(iris[,1:4], k=3, alpha=0.1, restr.fact=100)
##' c2 <- out$cluster       #  second partition c2 is the output of tclust clustering procedure
##' 
##' ## Units inside c2 which contain number 0 are referred to trimmed observations
##' noisecluster <- 0
##' randIndex(c1, c2, noisecluster=0)

randIndex <- function(c1, c2=NULL, noisecluster=NULL) {
  if(is.null(c2)) {
    if(ncol(as.matrix(c1)) < 2) {
      stop("RandIndex requires a contingency table with at least two columns")
    }
    C <- c1
  } else {
    if(min(dim(as.matrix(c1))) > 1 || min(dim(as.matrix(c2))) > 1) {
      stop("RandIndex requires two vector arguments")
    }
    
    if(!is.null(noisecluster)) {
      boo1 <- c1 == noisecluster
      boo2 <- c2 == noisecluster
      boo <- !(boo1 | boo2)
      c1 <- c1[boo]
      c2 <- c2[boo]
    }
    
    C <- table(c1, c2)  # Form contingency matrix
  }
  
  n <- sum(C)
  nis <- sum(rowSums(C)^2)  # Sum of squares of sums of rows
  njs <- sum(colSums(C)^2)  # Sum of squares of sums of columns
  
  totcomp <- choose(n, 2)   # Total number of pairs of entities
  t2 <- sum(C^2)            # Sum over rows and columns of nij^2
  t3 <- 0.5 * (nis + njs)
  
  # Expected value of the index (for adjustment)
  nc <- (n * (n^2 + 1) - (n + 1) * nis - (n + 1) * njs + 2 * (nis * njs) / n) / (2 * (n - 1))
  
  A <- totcomp + t2 - t3    # Total number of agreements
  D <- -t2 + t3             # Total number of disagreements
  
  if(totcomp == nc) {
    AR <- 0  # Avoid division by zero
  } else {
    AR <- (A - nc) / (totcomp - nc)  # Adjusted Rand Index
  }
  
  RI <- A / totcomp             # Rand Index
  MI <- D / totcomp             # Mirkin Index
  HI <- (A - D) / totcomp       # Hubert Index
  
  return(list(AR = AR, RI = RI, MI = MI, HI = HI))
}

discr_coords <- function (x, equal.weights)
{
    if (is.null (x$par$x))
        stop("Dataset not included in tclust object - cannot calculate discriminant coordinates.")
    
    if (missing(equal.weights))
        equal.weights <- x$par$equal.weights
    
    x.no <- x$par$x[drop = FALSE, x$cluster != 0,]
    
    p <- ncol (x.no)
    n <- nrow (x.no)
    
    W <- matrix (0, ncol = p, nrow = p)
    cl.num = x$k # sum (x$usek)
    
    if (is.null (x$cov))		# tk-means items don't have cov info
    {
        if (is.null (x$par$x))
            stop ("Cannot access the data matrix. Set store.x = TRUE.")
        x$cov <- array (dim = c (p, p, cl.num))
        for(i in 1:cl.num)
    	   x$cov[,,i] <- cov (x$par$x[x$cluster == i,])
    }
    
    for (i in 1:cl.num) {
        if (equal.weights)
            W <- W + (n - 1) / cl.num * .getsubmatrix (x$cov, i)
        else
            W <- W + (x$size[i] - 1) * .getsubmatrix (x$cov, i)
    }
    
    wm <- eigen(W, symmetric = TRUE)
    wm$values[wm$values < 1e-06] <- 1e-06
    Tm <- t (wm$vectors %*% diag (sqrt (wm$values)))
    
    #Tm <- tdecomp.hh(W)            ##  
    Tinv <- solve (Tm)
    
    S <- (n - 1) * cov (x.no)
    B <- S - W
    Z <- t (Tinv) %*% B %*% Tinv
    dc <- eigen (Z, symmetric = TRUE)
    units <- Tinv %*% dc$vectors * sqrt (n - cl.num)
    proj <- x$par$x %*% units
    
    return (proj)
}

.doEllipses <- function(eigval, eigvec, eigen, center, cov, n=100, size=1, ...) {
    if(!missing (cov))
        eigen <- base::eigen(cov)
    if(!missing(eigen)) {
        eigval <- eigen$values
        eigvec <- eigen$vectors
    }
    
    eigval[eigval < 0] <- 0

    ##  check dimensionality of eigenvalues
    if(!is.numeric(eigval) || !length(eigval) == 2)
        stop("argument eigval has to be a numeric vector of length 2.")

    ##  check dimensionality of center
    if(!is.numeric(center) || !length(center) == 2)
        stop("argument center has to be a numeric vector of length 2.")

    ##  check dimensionality of eigenvectors
    if(!is.matrix(eigvec) || any(dim(eigvec) != 2))
        stop("argument eigvec has to be a numeric mamtrix of dimension 2x2.")

    r <- seq(0, 2 * pi, length.out=n)   ##  create rad rep. of circle
    uc <- rbind(sin(r), cos(r))         ##  create a unit circle
    
    ##  "stretch" circle corresponding to ev & sizefact
    uc <- t(uc * sqrt(eigval) * size)
    
    ##  rotate resulting ellipses from PC into XY coords
    XY <- uc %*% t(eigvec)          

    XY <- t(t(XY) + center)     ##  move ellipses to the specified center

    lines(XY[,1], XY[,2], ...)  ##  draw ellipses
}

.getsubmatrix <- function(x, idx)  
    matrix (x[drop = FALSE,,,idx], nrow = dim (x)[1])

.vline <- function (x, yfact = 2, col = 1, lty = 1, lwd = 1, ...)
{
   ylim = par ("usr")[3:4]
   ylim <- ylim + diff (ylim) * (1 - 1 / yfact) / 2 * c(1,-1)
   
   col = rep (col, length (x))
   lty = rep (lty, length (x))
   lwd = rep (lwd, length (x))
   
   for (i in 1:length (x))
     lines (rep (x[i], 2), ylim, col = col[i], lty = lty [i], lwd = lwd [i]
             , ...)
}

#' Plot Method for \code{tclust} and \code{tkmeans} Objects
#'
#' The plot method for classes \code{tclust} and \code{tkmeans}.
#'
#' @name plot.tclust
#' @aliases plot.tkmeans
#' @method plot tclust

#' @description One and two dimensional structures are treated separately (e.g. tolerance 
#'  intervals/ellipses are displayed). Higher dimensional structures are displayed 
#'  by plotting the two first Fisher's canonical coordinates (evaluated by 
#'  \code{tclust::discr_coords}) and derived from the final cluster assignments 
#'  (trimmed observations are not taken into account). 
#'  \code{plot.tclust.Nd} can be called with one or two-dimensional \code{tclust}- or \code{tkmeans}-objects 
#'  too. The function fails, if \code{store.x = FALSE} is specified in the \code{tclust()} or \code{tkmeans()} call, 
#'  because the original data matrix is required here.
#'
#' @param x The \code{tclust} or \code{tkmeans} object to be displayed
#' @param \ldots Further (optional) arguments which specify the details of the 
#'    resulting plot (see section "Further Arguments").
#'
#' @section Further Arguments:
#'
#' \itemize{
#'  \item \code{xlab, ylab, xlim, ylim, pch, col} Arguments passed to \code{plot()}.
#'  \item \code{main} The title of the plot. Use \code{"/p"} for displaying the chosen parameters 
#'      \code{alpha} and \code{k} or \code{"/r"} for plotting the chosen restriction.
#'  \item \code{main.pre} An optional string which is added to the plot's caption.
#'  \item \code{sub} A string specifying the subtitle of the plot. Use \code{"/p"} (default) for 
#'  displaying the chosen parameters \code{alpha} and \code{k}, \code{"/r"} for plotting 
#'  the chosen restriction and \code{"/pr"} for both. 
#'  \item \code{sub1} A secondary (optional) subtitle.
#'  \item \code{labels} A string specifying the type of labels to be drawn. Either 
#'      \code{labels="none"} (default), \code{labels="cluster"} or \code{labels="observation"} 
#'      can be specified. If specified, parameter \code{pch} is ignored.
#'  \item \code{text} A vector of length n (the number of observations) containing 
#'      strings which are used as labels for each observation. If specified, 
#'      the parameters \code{labels} and \code{pch} are ignored.
#'  \item \code{by.cluster} Logical value indicating whether parameters 
#'      \code{pch} and \code{col} refer to observations (FALSE) or clusters (TRUE).
#'  \item \code{jitter.y} Logical value, specifying whether the drawn values shall be 
#'      jittered in y-direction for better visibility of structures in 1 dimensional data.
#'  \item \code{tol} The tolerance interval. 95\% tolerance ellipsoids (assuming normality) 
#'      are plotted by default.
#'  \item \code{tol.col, tol.lty, tol.lwd} Vectors of length k or 1 containing 
#'      the \code{col}, \code{lty} and \code{lwd} arguments for the tolerance 
#'      ellipses/lines.
#' }
#'
#' @examples
#'  #--- EXAMPLE 1------------------------------
#'  sig <- diag (2)
#'  cen <- rep (1, 2)
#'  x <- rbind(MASS::mvrnorm(360, cen * 0,   sig),
#'  	       MASS::mvrnorm(540, cen * 5,   sig * 6 - 2),
#'  	       MASS::mvrnorm(100, cen * 2.5, sig * 50))
#'  # Two groups and 10\% trimming level
#'  a <- tclust(x, k = 2, alpha = 0.1, restr.fact = 12)

#'  plot (a)
#'  plot (a, labels = "observation")
#'  plot (a, labels = "cluster")
#'  plot (a, by.cluster = TRUE)

#'  #--- EXAMPLE 2------------------------------
#'  sig <- diag (2)
#'  cen <- rep (1, 2)
#'  x <- rbind(MASS::mvrnorm(360, cen * 0,   sig),
#'  	       MASS::mvrnorm(540, cen * 5,   sig),
#'  	       MASS::mvrnorm(100, cen * 2.5, sig))
#'  # Two groups and 10\% trimming level
#'  a <- tkmeans(x, k = 2, alpha = 0.1)

#'  plot (a)
#'  plot (a, labels = "observation")
#'  plot (a, labels = "cluster")
#'  plot (a, by.cluster = TRUE)
#'

plot.tclust <-
function (x,  ...) {
  if (x$int$dim[2] == 1)
    .plot.tclust.1d (x, ...)
  else if (x$int$dim[2] == 2)
    .plot.tclust.2d (x, ...)
  else
    .plot.tclust.Nd (x, ...)
}

#' @rdname plot.tclust
#' @method plot tkmeans
plot.tkmeans <- function(x,  ...) {
	plot.tclust(x, ...)
}

#######################
##  .plot.tclust.1d  ##
#######################

.plot.tclust.1d <-
function (x, xlab, ylab, xlim, ylim, tol = 0.95, tol.lwd = 1, tol.lty = 3, tol.col,
          jitter.y = FALSE, ...)
{
  if (x$int$dim[2] != 1)
    stop ("tclust object of dimension 1 expected.")
  
  if (is.null (x$par$x))
    stop ("dataset not included in tclust object - cannot plot object.")

  if (missing (xlab))
  {
    dn <- dimnames (x$par$x)
    if (is.null (dn[[2]]))
      xlab <- "x"
    else
      xlab <- dn[[2]][1]
  }

  if (missing (ylab) && jitter.y)
    ylab <- "(random jitter)"

  n <- x$int$dim[1]

  if (jitter.y)
    y <- runif (n, min = -1) / 4
  else
    y <- rep (0, n)

  if (is.numeric (tol) && length (tol) == 1 &&  0 < tol && tol < 1)
    tol.fact = sqrt (qchisq(tol, 1))
  else
    tol.fact <- NULL

  x.c = as.numeric (x$centers)
  if (!is.null (x$cov))		## tkmeans- objects don't have cov-info
  {
	  x.sd = sqrt (as.numeric (x$cov))

	  if (missing (xlim))
	  {
		xlim <- range (x$par$x)
		if (!is.null (tol.fact))
		  xlim <- range (xlim, x.c + x.sd * tol.fact, x.c - x.sd * tol.fact)
	  }
  }
  else
  {
	if (missing (xlim))
		xlim <- range (x$par$x)
  }

  if (missing (ylim))
	ylim <- c (-1, 1)

  X <- cbind (x$par$x, y)
  .plot.tclust.0 (x = x, X = X, xlab = xlab, ylab = ylab, axes = 1,
                 xlim = xlim, ylim = ylim, ...)
 
  .vline (x.c, 3, lty = 2, col = 1 + (1:x$k))    ##  cluster centers

	if (!is.null (tol.fact) &&
		!is.null (x$cov))		## tkmeans- objects don't have cov-info
  {
    #tol.fact = sqrt(qchisq(tol, 1))
    if (missing (tol.col))
      tol.col <- (1:x$k) + 1
    else
      tol.col <- rep (tol.col, x$k)
      
    tol.lty <- rep (tol.lty , x$k)
    tol.lwd <- rep (tol.lwd , x$k)   
    .vline (x.c + x.sd * tol.fact, 2, col = tol.col, lwd = tol.lwd,
            lty = tol.lty)
    .vline (x.c - x.sd * tol.fact, 2, col = tol.col, lwd = tol.lwd,
            lty = tol.lty)
  }
}

#######################
##  .plot.tclust.2d  ##
#######################

.plot.tclust.2d <-
function (x, xlab, ylab, tol=0.95, tol.lwd=1, tol.lty=3, tol.col=1, ...) {
    if(nrow (x$centers) != 2)
        stop ("tclust object of dimension 2 expected.")

    if(is.null (x$par$x))
        stop ("dataset not included in tclust object - cannot plot object.")

    dn <- dimnames (x$par$x)
    if(is.list (dn) && length (dn[[2]]) == 2) {
        if(missing (xlab))
          xlab = dn[[2]][1]
        if(missing (ylab))
          ylab = dn[[2]][2]
    } else {
        if(missing(xlab))
          xlab = "x1"
        if (missing(ylab))
          ylab = "x2"
    }

    X <- cbind(x$par$x[, 1:2])
    .plot.tclust.0(x=x, X=X, xlab=xlab, ylab=ylab, axes=3, ...)

    if(!is.null(x$cov) && is.numeric(tol) && length(tol) == 1 &&  0 < tol && tol < 1)
    {
        tol.col <- rep(tol.col, x$k)
        tol.lty <- rep(tol.lty, x$k)
        tol.lwd <- rep(tol.lwd, x$k)
        
        tol.fact = sqrt(qchisq(tol, 2))  
        for(k in 1:x$k)
            .doEllipses(eigen=eigen(x$cov[,,k]), center=x$centers[,k],
                lwd=tol.lwd, lty=tol.lty[k], col=tol.col[k], size=tol.fact)
    }
}

#######################
##  .plot.tclust.Nd  ##
#######################

.plot.tclust.Nd <- 
function (x, xlab, ylab, ...)
{
  if (is.null (x$par$x))
    stop ("dataset not included in tclust object - cannot plot object.")

  if (missing (xlab))
    xlab <- "First discriminant coord."
  if (missing (ylab))
    ylab <- "Second discriminant coord."

  X <- discr_coords(x, x$par$equal.weights)

  .plot.tclust.0 (x = x, X = X, xlab = xlab, ylab = ylab, axes = 0, ...)
}

######################
##  .plot.tclust.0  ##
######################

.plot.tclust.0 <-
function (x, X, labels=c ("none", "cluster", "observation"), text,
          xlab, ylab, col, pch, by.cluster=TRUE, axes=3, xlim, ylim, ...)
{

    if(by.cluster) {
        maxassig <- max(x$cluster)
    
        if(missing(col))
            col <- 1:(x$k+1)
        else
            col <- rep(col, len=maxassig + 1)  
        
        if(missing(pch))
            pch <- 1:(x$k+1)          #rep (1, x$k+1)  #
        else
            pch <- rep(pch,  len=maxassig + 1 )
        col <- col[x$cluster + 1]    
        pch <- pch[x$cluster + 1]
    } else {
        if(missing(col))
            col <- x$cluster + 1
        if(missing(pch))
            pch <- 1
    }

    n <- x$int$dim[1]

    if(!missing(text))
        text <- rep(text, len=n)
    else if(!missing(labels))    {
        labels <- match.arg(labels)
        if(labels == "cluster")
            text = paste(x$cluster)
        else if(labels == "observation")
            text = paste(1:nrow (X))
    }

    plot.new()
    par(usr = .plot.tclust.calc.usr(X, xlim, ylim))

    if (missing (text))
        points(X[,1], X[,2], pch=pch, col=col)
    else
        text(X[,1], X[,2], labels=text, col=col)

    .plot.tclust.title(x, ...)

    axis.x <- axes %% 2        ## x axis 1 or 3
    axis.y <- axes >= 2        ## y axis 2 or 3
    
    cex <- par ("cex")
    if (!missing (xlab))
        mtext (side = 1, xlab, line = 1.5 + 1.5 * axis.x, cex = cex)
    if (!missing (ylab))
        mtext (side = 2, ylab, line = 1.5 + 1.5 * axis.y, cex = cex)

    box()
    if(axis.x)
        axis(1)
    if(axis.y)
        axis(2)
}

.plot.tclust.title <- function (x, main, main.pre, sub, sub1, ...) {
  sub.par <- TRUE
  sub.restr <- FALSE

  sub.ovr <- missing (sub)
  if (!missing (sub) && is.character (sub))   {
    sub.ovr <- TRUE
    if (sub == "/p")
      sub.par <- !(sub.restr <- FALSE)
    else if (sub ==  "/r")
      sub.par <- !(sub.restr <- TRUE)
    else if (sub ==  "/pr")
      sub.par <- sub.restr <- TRUE
    else
      sub.ovr <- FALSE
  }

	if (is.null (x$par$restr.C))
		sub.restr <- FALSE

  ralph <- round (x$par$alpha, 2)

  txt.par <- bquote(paste (k == .(x$par$k), ", ", alpha == .(ralph)))

  if (is.null(x$par$restr.C))
	txt.restr <- ""
  else if (x$par$restr.C == 2) ## this is the sigma - restriction, thus no restr.fact has to be printed.
    txt.restr <- paste ("restr = \"", x$par$restr, "\"", sep = "")
  else
    txt.restr <- paste ("restr = \"", x$par$restr, "\", restr.fact = ", x$par$restr.fact, sep = "")

  if (missing (main))
    main <- "Classification"  #"Cluster Assignment"
  else
    if (is.character (main))
      if (main == "/r" && !is.null (x$par$restr.C))
        main <- txt.restr
      if (main == "/p")
        main <- txt.par

  if (!missing (main.pre) && !is.null (main.pre))
    main <- paste (main.pre, main)

  if (sub.ovr)
    if (sub.par)
      sub  <- txt.par
    else if (!is.null (x$par$restr.C))
      sub <- txt.restr

  if (missing (sub1) && sub.restr && !(!sub.par && sub.ovr))
    sub1 <- txt.restr

  n.sub = .is.visible (sub) + .is.visible (sub1)    ##  number of subtitles to draw

  if (!is.null (main))
    title (main = main, line = ifelse (n.sub > 1, 2.3, 1.6))
  
  if (.is.visible (sub))
    mtext(sub, cex = 0.8, line = ifelse (n.sub > 1, 1.2, 0.3))

  if (.is.visible (sub1))
    mtext(sub1, cex = 0.8, line = ifelse (n.sub > 1, 0.1, 0.3))
}

.is.visible <- function(x) {
    if(missing(x))
    return(FALSE)

    !is.null(x) && x != ""
}

.plot.tclust.calc.usr <- function(X, xlim, ylim, fact=0.04)
{
    if(missing(xlim))
        xlim <- range(X[, 1])
    
    if(missing(ylim))
        ylim <- range(X[, 2])

    r <- cbind(xlim, ylim)
    rd <- apply(r, 2, diff)
    #  r + rd * fact * c(-1,1)
    as.numeric (r + (c(-1, 1) %*% t (rd)) * fact)
}

##  roxygen2::roxygenise("C:/users/valen/onedrive/myrepo/r/tclust", load_code=roxygen2:::load_installed)

#'
#' TKMEANS method for robust K-means clustering
#' 
#' @name tkmeans
#' @aliases print.tkmeans
#' @description This function searches for \code{k} (or less) spherical clusters 
#'  in a data matrix \code{x}, whereas the \code{ceiling(alpha n)} most outlying 
#'  observations are trimmed.
#' @param x A matrix or data.frame of dimension n x p, containing the observations (row-wise). 
#' @param k The number of clusters initially searched for.
#' @param alpha The proportion of observations to be trimmed.
#' @param nstart The number of random initializations to be performed.
#' @param niter1 The number of concentration steps to be performed for the nstart initializations.
#' @param niter2 The maximum number of concentration steps to be performed for the 
#'  \code{nkeep} solutions kept for further iteration. The concentration steps are 
#'  stopped, whenever two consecutive steps lead to the same data partition.
#' @param nkeep The number of iterated initializations (after niter1 concentration 
#'  steps) with the best values in the target function that are kept for further iterations
#' @param iter.max (deprecated, use the combination \code{nkeep, niter1 and niter2}) 
#'  The maximum number of concentration steps to be performed.
#'  The concentration steps are stopped, whenever two consecutive steps lead
#'  to the same data partition.
#' @param zero_tol The zero tolerance used. By default set to 1e-16.
#' @param points Optional initial mean vectors, \code{NULL} or a matrix with \code{k} 
#'  vectors used as means to initialize the algorithm. If initial mean vectors are 
#'  specified, \code{nstart} should be 1 (otherwise the same initial means are 
#'  used for all runs).
#' @param center Optional centering of the data: a function or a vector of length p 
#'  which can optionally be specified for centering x before calculation
#' @param scale Optional scaling of the data: a function or a vector of length p 
#'  which can optionally be specified for scaling x before calculation
#' @param store_x A logical value, specifying whether the data matrix \code{x} shall be 
#'  included in the result object. By default this value is set to \code{TRUE}, because 
#'  some of the plotting functions depend on this information. However, when big data 
#'  matrices are handled, the result object's size can be decreased noticeably 
#'  when setting this parameter to \code{FALSE}.
#' @param parallel A logical value, specifying whether the nstart initializations should be done in parallel.
#' @param n.cores The number of cores to use when paralellizing, only taken into account if parallel=TRUE.
#' @param drop.empty.clust Logical value specifying, whether empty clusters shall be 
#'  omitted in the resulting object. (The result structure does not contain center 
#'  estimates of empty clusters anymore. Cluster names are reassigned 
#'  such that the first l clusters (l <= k) always have at least one observation.
#' @param trace Defines the tracing level, which is set to 0 by default. Tracing level 1 
#'  gives additional information on the stage of the iterative process.
#'
#' @return The function returns the following values:
#' \itemize{
#'     \item cluster - A numerical vector of size \code{n} containing the cluster assignment 
#'          for each observation. Cluster names are integer numbers from 1 to k, 0 indicates 
#'          trimmed observations. Note that it could be empty clusters with no observations 
#'          when \code{equal.weights=FALSE}.
#'     \item obj - The value of the objective function of the best (returned) solution.
#'     \item size - An integer vector of size k, returning the number of observations contained by each cluster.
#'     \item centers - A matrix of size p x k containing the centers (column-wise) of each cluster. 
#'     \item code - A numerical value indicating if the concentration steps have 
#'          converged for the returned solution (2).
#'     \item cluster.ini - A matrix with nstart rows and number of columns equal to 
#'          the number of observations and where each row shows the final clustering 
#'          assignments (0 for trimmed observations) obtained after the \code{niter1} 
#'          iteration of the \code{nstart} random initializations.
#'     \item obj.ini - A numerical vector of length \code{nstart} containing the values 
#'          of the target function obtained after the \code{niter1} iteration of the 
#'          \code{nstart} random initializations.
#'     \item x - The input data set.
#'     \item k - The input number of clusters.
#'     \item alpha - The input trimming level.
#' }
#' 
#' @author Valentin Todorov, Luis Angel Garcia Escudero, Agustin Mayo Iscar.
#' 
#' @references 
#' 
#' Cuesta-Albertos, J. A.; Gordaliza, A. and Matrán, C. (1997), "Trimmed k-means: 
#'  an attempt to robustify quantizers". Annals of Statistics, Vol. 25 (2), 553-576. 
#' 
#' @export
#'
#' @examples
#' 
#'  \dontshow{
#'      set.seed(0)
#'  }
#'  ##--- EXAMPLE 1 ------------------------------------------
#'  sig <- diag(2)
#'  cen <- rep(1,2)
#'  x <- rbind(MASS::mvrnorm(360, cen * 0,   sig),
#'             MASS::mvrnorm(540, cen * 5,   sig),
#'             MASS::mvrnorm(100, cen * 2.5, sig))
#'  
#'  ## Two groups and 10\% trimming level
#'  (clus <- tkmeans(x, k = 2, alpha = 0.1))
#'
#'  plot(clus)
#'  plot(clus, labels = "observation")
#'  plot(clus, labels = "cluster")
#'
#'  #--- EXAMPLE 2 ------------------------------------------
#'  data(geyser2)
#'  (clus <- tkmeans(geyser2, k = 3, alpha = 0.03))
#'  plot(clus)
#'  
tkmeans <- function(x, k, alpha=0.05, nstart=500, niter1=3, niter2=20, nkeep=5, iter.max, points=NULL, 
                   center=FALSE, scale=FALSE, store_x=TRUE, 
                   parallel=FALSE, n.cores=-1, 
                   zero_tol=1e-16, drop.empty.clust=TRUE, trace=0)  {
    
    if(!missing(iter.max)) {
        warning("The parameter 'iter.max' is deprecated, please read the help and use the combination of 'niter1', 'niter2', 'nkeep'.")
        niter1 <- iter.max
    }
        

    if(!is.null(points)) {
        if(nstart > 1)
            warning("If initial mean vectors are specified, 'nstart' should be 1 (otherwise the same initial means are used for all runs)")
        nstart <- nkeep <- 1
        parallel <- FALSE
    }
        
	parlist <- list(k=k, alpha=alpha, nstart=nstart, niter1=niter1, niter2=niter2, nkeep=nkeep, 
        center=center, scale=scale,
        zero_tol=zero_tol, drop.empty.clust=drop.empty.clust, trace=trace, store_x=store_x,
        equal.weights=TRUE)
              
    # Initial checks
    
    if(is.data.frame(x))
        x <- (data.matrix(x))
    if(!is.matrix (x))
        x <- as.matrix(x)
    if(any(is.na(x)))
        stop ("x cannot contain NA")
    if(!is.numeric (x)) 
        stop ("Parameter x: numeric matrix/vector expected")
        
    scaled <- myscale(x, center=center, scale=scale)
    x <- scaled$x
    if(store_x)
        parlist$x <- x     
  
      if (k < 1) 
        stop ("Parameter k: must be >= 1")
      if (alpha < 0 || alpha > 1) 
        stop ("Parameter alpha: must be in [0,1]")
      if (niter1 < 0 || as.integer(niter1) != niter1) 
        stop ("Parameter niter1: must be an integer >= 0")
      if (niter2 < 0 || as.integer(niter2) != niter2) 
        stop ("Parameter niter2: must be an integer >= 0")
      if (nkeep < 0 || as.integer(nkeep) != nkeep) 
        stop ("Parameter nkeep: must be an integer >= 0")
      if (nkeep > nstart) 
        stop ("Parameter nkeep: must be <= nstart")
      if(!is.logical(parallel))
        stop ("Parameter parallel: must be a logical TRUE or FALSE")
      if (n.cores < -2 || as.integer(n.cores) != n.cores) 
        stop ("Parameter n.cores: must be an integer >= 0, -1 or -2")
      if(zero_tol < 0)
        stop ("Parameter zero_tol: must be >= 0")
      if(trace != 0 & trace !=1)
        stop ("Parameter trace: must be 0 or 1")

    ###
    # FIRST STEP: get nstart solutions with niter1 concentration steps
    ###
    if(trace){
        cat(paste("\nPhase 1: obtaining ", nstart, " solutions.\n", sep = ""))
        if(parallel) cat(paste("\n Parallelizing initializations using ", n.cores, 
            " cores. Progress bar will not display accurate information.\n", sep = ""))
        pb <- txtProgressBar(min = 0, max = nstart, style = 3)
    }
  
    if(!parallel) {
        cluster.ini <- vector("list", nstart)   ##	for containing the best values for the parameters after several random starts
        obj.ini <- rep(0, nstart)               ## for containing best objective values
    
        for(j in 1:nstart) {
            assig_obj <- tkmeans_c1(x, k, alpha, niter1, zero_tol, points)                                                                   # niter1 steps!
            cluster.ini[[j]] <- assig_obj$cluster
            obj.ini[j] <- assig_obj$obj
      
            if(trace){
                setTxtProgressBar(pb, j)
            }
        }
    } else {
    
        # Setup parallel cluster
        if(n.cores == -1){
            n.cores <- detectCores()
        } else if (n.cores == -2){
            n.cores <- detectCores() - 1
        }
        parclus <- makeCluster(n.cores)
        registerDoParallel(parclus)

        count <- 0
        comb <- function(...) { # Custom combination function for the foreach loop
            count <<- count + length(list(...)) - 1
            setTxtProgressBar(pb, count)
            flush.console()
            c(...) # this can feed into .combine option of foreach
        }
        
        init.results <- foreach(j = 1:nstart,
                                .packages = "tclust",
                                .combine = ifelse(trace, "comb", "c"),
                                .multicombine = TRUE,
                                .inorder = FALSE) %dopar% {
            assig_obj <- tkmeans_c1(x, k, alpha, niter1, zero_tol)
            list(assig_obj$cluster, assig_obj$obj)
        }

        stopCluster(parclus)

        cluster.ini <- init.results[0:(nstart-1) * 2 + 1]       # Impair positions of cluster.ini
        obj.ini <- unlist(init.results[1:nstart * 2])           # Pair positions of cluster.ini
    }
  
    ###
    # SECOND STEP: get nkeep best solutions so far
    ###
    if(trace){
        cat(paste("\n\nPhase 2: obtaining ", nkeep, " best solutions out of the intial ", nstart," solutions.\n", sep = ""))
    }
    best_index  <- order(obj.ini)[1:nkeep]
    best_assig_list <- cluster.ini[best_index]

  
    ###
    # THIRD STEP: apply niter2 concentration steps to nkeep best solutions, return the best solution
    ###
    if(trace){
        cat(paste("\nPhase 3: applying ", niter2, " concentration steps to each of the ", nkeep," best solutions.\n", sep = ""))
        pb2 <- txtProgressBar(min = 0, max = nkeep, style = 3)
    }
    
    best_iter <- NULL
    best_iter_obj <- Inf
  
    for(j in 1:nkeep){
        iter <- tkmeans_c2(x, k, best_assig_list[[j]], alpha, niter2, zero_tol=1e-16)
    
        if(iter$obj <= best_iter_obj){
            best_iter <- iter
            best_iter_obj <- iter$obj
        }
        
        if(trace){
            setTxtProgressBar(pb2, j)
        }
    }
    if(trace){
        cat("\n\n")
    }
  
    ## Adjust the returned object to be similar to 'tkmeans': 
    
    ## Handle empty clusters
    if(drop.empty.clust)
        idxuse <- which(best_iter$size > 0)
    else
        idxuse <- 1:k
    idxuse <- idxuse[order(best_iter$size[idxuse], decreasing = TRUE)]
    
    ClusterIDs <- rep(0, k + 1)
    ClusterIDs[idxuse + 1] <- 1:length(idxuse)

    k.real <- length(idxuse)
    
    ##  - A list of values internally used by functions related to tclust objects.
    int <- list(
    	iter.successful=0, iter.converged=0,
    	dim=dim(x))
    
    ## - Transpose the 'centers' matrix
    best_iter$centers <- t(best_iter$centers)
    best_iter$centers <- best_iter$centers[, idxuse, drop = FALSE]
    
    best_iter$cluster <- as.vector(best_iter$cluster)
    best_iter$cluster <- ClusterIDs[best_iter$cluster + 1]
    best_iter$size <- as.vector(best_iter$size)
    best_iter$size <- best_iter$size[idxuse]

    best_iter$cov <- NULL
    best_iter$posterior <- NULL

    best_iter$NlogL <- NULL     
    best_iter$disttom <- NULL
    
    ret <- c(best_iter, list(cluster.ini=matrix(unlist(cluster.ini), byrow=TRUE, nrow=length(cluster.ini)),
        obj.ini=obj.ini, int=int, par=parlist, k=sum(best_iter$size > 0)))

    ##  - Dimnames of center and cov
    dn.x <- dimnames(x)
    rownames(ret$centers) <- if(is.null(colnames(x))) paste ("X", 1:ncol(x)) else colnames (x)
    colnames(ret$centers) <- paste("C", 1:k.real)

    ## Back transform centers, cov and x
    cmm <- scaled$scale %*% t (scaled$scale)
    for(i in 1:ret$k) {
        ret$centers[,i] <- ret$centers[,i] * scaled$scale + scaled$center
    }
    x <- sweep(x, 2L, scaled$scale, `*`, check.margin = FALSE)
    x <- sweep(x, 2L, scaled$center, `+`, check.margin = FALSE)
    if(store_x)
        ret$par$x <- x
    
    class(ret) <- "tkmeans"    
    return(ret)
}

##  Multivariate normal density
.dmnorm <- function (X, mu, sigma)
{
  ((2 * pi)^(-length(mu) / 2)) *
  (det(sigma)^(-1/ 2)) *
  exp (-0.5 * .evmaha (X, mu, sigma))
}

.evmaha <- function (X, mu, sigma)    ##  calculate mahalanobis distances 
                                      ##  using the eigenvalues and eigenvectors. 
                                      ##  thus no singularity problems arise.
{                                     ##  Mahalanobis distance == Inf is possible.
    v <- eigen (sigma)
    Zt <- t (v$vectors) %*% (t (X) - mu)
    colSums ((Zt * v$values^(-.5))^2)
}


##	get a matrix object out of the sigma - tensor
.ssclmat <- function (x, k) as.matrix (x[,,k])

##	calculates the initial cluster sizes
.getini <- function (K, no.trim)
{
	if (K == 1)
		return (no.trim)

	pi.ini  <- runif(K)
	ni.ini <- sample(x = K, size = no.trim, replace = T, prob = pi.ini / sum (pi.ini))
	return (tabulate(ni.ini, nbins = K))
}

##	calculates the initial cluster assignment and parameter
.InitClusters <- function (X, iter, pa)
{
	for (k in 1:pa$K)
	{
		idx <- sample (1:pa$n, pa$p+1)
		X.ini = X [drop = F,idx,]#sample (1:pa$n, pa$p+1),]	##	selecting observations randomly for the current init - cluster
		iter$center [k,] <- colMeans (X.ini)				#n	(moved)	calculating the center

#n		iter$sigma[,,k] <- cov_fact (X.ini, iter$center [k,], pa$p/(pa$p+1))	#n	using the "new" covariance function, which is numerically more similiar to the C++ - version
		iter$sigma[,,k] <- (pa$p/(pa$p+1))*cov (X.ini)		##	calculating sigma (cc = current cov)
#o		iter $center [k,] <- colMeans (X.ini)				##	--> moved up
	}

	if (pa$equal.weights)									##	if we're considering equal weights, cw is set here **ONLY ONCE AND NEVER CHANGED**
		iter$csize <- rep (pa$no.trim / pa$K, pa$K)
	else
		iter$csize = .getini (pa$K, pa$no.trim)
	iter$cw <- iter$csize / pa$no.trim
									##		if we're considering different weights, calculate them, and they're gonna be recalculated every time in .estimClustPar
	return (iter)
}

.estimClustPar <- function (X, iter, pa)
{				

		for (k in 1:pa$K)
		{
			if (iter$csize[k] > pa$zero.tol)				##	this cluster's size is > 0
			{
				iter$center[k,] = (t(iter$z_ij[,k]) %*% X) / iter$csize[k]
				X.c <- (X - matrix (iter$center[k,], ncol = pa$p, nrow = pa$n, byrow = TRUE))
				iter$sigma[,,k] <- (t(X.c * iter$z_ij[,k]) %*% X.c) / iter$csize[k]
			}
			else											##	this cluster's size has decreased to 0
				iter$sigma[,,k] <- 0						##	do the same as in the non - fuzzy approach
		}

	return (iter)
}

.div_by0_set0 <- function (x, y)								##	a function which divides two elements, setting x/0 := 0
{
	ret <- x / y
	ret [y == 0] <- 0
	ret
}

.calcFuzzy_row <- function (x, pa)							#n	new function calculating a row of iter$Z, corresponding to a single observation
{															##	2DO: instead of <= 0 we might use pa$zero.tol ?!
    idx.max = which.max (x)

	if (x[idx.max] >= 1)
		return (as.numeric (1:pa$K == idx.max))						##	this gives an 0, 0, 1, 0, 0 vector for k = 5, idx.max = 3

	x.log <- -log (x)
	x.log [x <= 0] <- 0												##	avoid -Inf. Note, that x < 1. thus log (x) can never be 0!

	la2 <- matrix (x.log, ncol = pa$K, nrow = pa$K)
	la3 <- t (la2)
	z2_3d <- .div_by0_set0 (la3, la2)								##	this is 2d now actually...

	z2_ij <- colSums (z2_3d^(1/(pa$m-1)))
	z2_ij <- .div_by0_set0 (1, z2_ij)

#o	if (sum (z2_ij) <= 0)
#o		return (as.numeric (1:pa$K == idx.max))						##	this was a wrong solution to an issue discovered in the original version
	if (sum (z2_ij) <= pa$zero.tol)									#n	use zero.tol instead!
		return (rep (1/pa$K, pa$K))									#n	changed on 20110216

	return (z2_ij^pa$m)
}

.findClustAssig <- function (X, iter, pa)
{															##	finds the current cluster assignment based on the given center and sigma
	ll = matrix (NA, pa$n, pa$K)

	for (k in 1:pa$K)										##	calculating the probability (or density ?!) for each observation in each cluster.
		ll[,k] <- iter$cw[k] * .dmnorm(X, iter$center[k,], .ssclmat (iter$sigma,k)) ## dmvnorm could be used here...

	if (pa$fuzzy > 0)
	{
        old.z_ij <- iter$z_ij
		iter$assig <- apply(ll,1,which.max)

		if (pa$fuzzy >= 2)											#n	another possibility of calculating z_ij, which is closer related to the C++ implementation.
			iter$z_ij <- t (apply (ll, 1, .calcFuzzy_row, pa = pa))	#n apply the calculation for an observation (".calcFuzzy_row") for each row of the ll - vector.
		else
		{
	#n        iter$assig <- apply(ll, 1, select.clust, pa = pa)	#n	##	searching the cluster which fits best for each observation

			## to obtain FUZZY values of z_ij
			iter$z_ij <- matrix (0, ncol = pa$K, nrow = pa$n)	##	calculating "absolute z_ij" for !pa$fuzzy, which only contains 0 and 1

#o			z1_ij <-	matrix (0, ncol = pa$K, nrow = pa$n)	##  INITIALIZATION
#o			z2_ij <-	matrix (0, ncol = pa$K, nrow = pa$n)	##	INITIALIZATION
#o			pre.z_ij <-	matrix (0, ncol = pa$K, nrow = pa$n)	##	INITIALIZATION

			log.ll <- pre.z_ij <- z2_ij <- z1_ij <- iter$z_ij	#n  INITIALIZATION (these matrices are all the same at the beginning...)

			assig_ <- cbind (1:pa$n, iter$assig)
			z1_ij[assig_] <- 1
			max.ll <- apply (ll, 1, max)
			max.ll.greater1 <- max.ll >= 1
	#o		max.ll.greater1_ <-matrix (max.ll.greater1, nrow = pa$n, ncol = pa$K, byrow = FALSE)	#	not needed anymore, we use "max.ll.greater1" directly as column indices (e.g. z1_ij[max.ll.greater1, ])
	#o		log.ll <- ll * 0		##	initialized at the beginngin (-> "INITIALIZATION")

	#o		log.ll[ll > 0] <- -log(ll[ll > 0]) * (ll[ll > 0] < 1)
			ll.bzo <- ll > 0 & ll < 1							#n	ll between zero and one
			log.ll[ll.bzo] <- -log(ll[ll.bzo])					#n

	#o		ll2 <- log.ll %*% matrix (rep (diag (pa$K), pa$K), nrow=pa$K)	##	repeat matrix log.ll pa$K-times (columnwise)
	#i		ll2 <- matrix (log.ll, nrow = nrow (ll), ncol = ncol (ll) * pa$K)
			la2 <- array (log.ll, dim = c(pa$n, pa$K, pa$K))				#n	a (n x K x K) array. this is easier to handly

	#o		yy <- matrix (0, nrow = pa$K, ncol = pa$K * pa$K)
	#o		for (ii in 1:pa$K)
	#o			yy[ii,((ii-1)*pa$K+1):(ii*pa$K)]=rep(1,pa$K)
	#o		ll3 <- log.ll %*% yy
			la3 <- aperm (la2, c (1, 3, 2))									##	an array of same dimension as la2, but the 2nd and 3rd dimensions are swapped (this corresponds to yy) (this is done with function "aperm")

	#o		z2_ij__ <- (((ll3 * (ll2 > 0) / (ll2 + (ll2 == 0)))^(1/(pa$m-1))) %*% t(yy))	##	this has been split into the 3 following "#i" lines

	#i		z2_ij__ <- (ll3 / ll2)^(1/(pa$m-1))
	#i		z2_ij__ [ll2 == 0] <- 0
			z2_3d <- .div_by0_set0 (la3, la2)

	#i		z2_ij__ <- z2_ij__  %*% t(yy)
			z2_ij <- apply (z2_3d^(1/(pa$m-1)), c (1, 3), sum)

	#o		z2_ij <- 1*(z2_ij__  > 0) / (z2_ij__ + (z2_ij__ == 0))
	#i		z2_ij <- .div_by0_set0 (1, z2_ij__)
			z2_ij <- .div_by0_set0 (1, z2_ij)

	#o		sum_z2_ij = cbind (apply (z2_ij, 1, 'sum'))				##	we use rowSums instead
			sum_z2_ij <- rowSums (z2_ij)

	#o?		z2_ij[sum_z2_ij == 0,] <- assig_[sum_z2_ij == 0,]		##	is this really supposed to be assig_? or rather z1_ij ? note that assig_ contains values up to pa$n
	#o		z2_ij[sum_z2_ij == 0,] <- z1_ij[sum_z2_ij == 0,]		#i	i think it's supposed to be like this.. WRONG!!
			z2_ij[sum_z2_ij <= pa$zero.tol,] <- 1/pa$K				#n	this is the proper solution to this issue. changed on 20110216

	#o		pre.z_ij <- z1_ij * max.ll.greater1_ + z2_ij * (max.ll.greater1_ == 0)
			pre.z_ij[max.ll.greater1, ] <- z1_ij[max.ll.greater1, ]			#n
			pre.z_ij[!max.ll.greater1, ] <- z2_ij[!max.ll.greater1, ]		#n

			iter$z_ij <- pre.z_ij^pa$m
		}

        ##PUT TRIMMING in iter$assig and in iter$z_ij
        ll[ll < 0] <- 0
        log.ll_ <- log(ll)

		log.ll_[iter$z_ij == 0] <- 0							#n BUG !? this is a new line.. in order to avoid 0*-Inf division in iter$z_ij * log.ll_.
#o        disc <- apply (iter$z_ij * log.ll_, 1, sum)			#o use rowSums instead

        disc <- rowSums (iter$z_ij * log.ll_)

#o      idx.out <- rank (disc, ties.method = c ("random"))		#o	this is not an idx.out, but a rank - variable. let's use a "real" idx.out - variable
		idx.out <- which (rank (disc, ties.method = c ("random"))<=pa$trim)	#i	now this is a "real" idx.out - vector
#n		idx.out <- select.outliers (pa, disc)					#n	selecting the trimmed observations with special treatment of ties (2come)

#o		iter$assig [idx.out<=pa$trim] <- 0						##	setting outliers to class 0
#o		iter$z_ij[idx.out<=pa$trim,] <- 0

        iter$assig [idx.out] <- 0								#n	setting outliers to class 0
        iter$z_ij[idx.out,] <- 0								#n	using the "real" idx.out - vector

        ##CHECKING EQUALITY IN Z_IJ
        iter$code <- all (old.z_ij == iter$z_ij)				##	setting the code - parameter, signaling whether the assig - array is still the same

		iter$csize <- colSums (iter$z_ij)						##	calculating the cluster size
	} else {
		old.assig <- iter$assig
		iter$assig <- apply(ll,1,which.max)						#o	searching the cluster which fits best for each observation
#n		iter$assig <- apply(ll, 1, select.clust, pa = pa)		##	searching the cluster which fits best for each observation

		disc <-ll [cbind (1:pa$n, iter$assig)]

#o		idx.out=rank(disc,ties.method = c( "random"))			#o	this is not an idx.out, but a rank - variable. let's use a "real" idx.out - variable
		idx.out <- which (rank (disc, ties.method = c ("random"))<=pa$trim)	#i	now this is a "real" idx.out - vector
#n		idx.out <- select.outliers (pa, disc)					#n	selecting the trimmed observations with special treatment of ties (2come)

#o		iter$assig [idx.out<=pa$trim]=0							#o	setting outliers to class 0
		iter$assig [idx.out] <- 0								#n	using the "real" idx.out - vector	#	setting outliers to class 0

		iter$code <- all (old.assig == iter$assig)				##	setting the code - parameter, signaling whether the assig - array is still the same
		iter$csize <- tabulate (iter$assig, pa$K)
		iter$z_ij<- matrix (0, ncol = pa$K, nrow = pa$n)		##	calculating "absolute z_ij" for !pa$fuzzy, which only contains 0 and 1

#o		iter$z_ij[cbind ((1:pa$n)[idx.out>pa$trim], iter$assig[idx.out>pa$trim])] <- 1		

        ## VT::28.11.2023 - it will not work if alpha=0!
        if(length(idx.out) > 0)	
		  iter$z_ij[cbind (1:pa$n, iter$assig)[-idx.out, ]] <- 1	 #n	using the "real" idx.out - vector	
        else
		  iter$z_ij[cbind (1:pa$n, iter$assig)] <- 1	             #n	using the "real" idx.out - vector	
        

#n		idx.max <- cbind (1:pa$n, iter$assig)					##
#n		iter$z_ij[idx.max[,idx.out>pa$trim]] <- 1
	}

	if (!pa$equal.weights)										##	calculate cluster weights (if necessary)
		iter$cw <- iter$csize / sum(iter$csize)

	return (iter)
}

.calcobj <- function (X, iter, pa)
{			
		##	calculates the obj. functions value
#	iter$obj <- ifelse (pa$equal.weights, 0, sum ( (iter$csize *log(iter$cw)  )[iter$csize != 0] ) )
	iter$obj <- ifelse (pa$equal.weights, 0, sum ( (iter$csize *log(iter$cw)  )[iter$csize > pa$zero.tol] ) )

	for (k in 1:pa$K)
	{
		w <- .dmnorm(X,iter$center[k,],.ssclmat (iter$sigma,k))

		if (sum(iter$z_ij[  ,k]) >  pa$zero.tol)
#o			if (sum(iter$z_ij[!w,k]) <= pa$zero.tol)
			if (sum(iter$z_ij[w <= 0, k]) <= pa$zero.tol)
				iter$obj <- iter$obj + sum(iter$z_ij[w > 0,k] * log(w[w > 0]))
			else
			{
				iter$obj <- iter$obj -Inf
				return (iter)					#n	we can already return -Inf, no need to do any further calculation
			}
	}
	return (iter)
}

.TreatSingularity <- function (iter, pa)
{	
	warning ("After trimming, all points in the data set are concentrated in k subspaces.") ##  a single point is a subspace too.

	iter$code <- 2	# indicating the data's concentration in either k subspaces or points.
	return (iter)
}

#################						##	a development-function. This it takes a function argument (f.restr) for specifying the restriction.
##	.tclust.R  ##
#################

.tclust.R <- function (x, k = 3, alpha = 0.05, nstart = 50, iter.max = 20, f.restr = .restr.diffax, equal.weights = FALSE, fuzzy = FALSE, m = 2, zero.tol = 1e-16,  trace = 0, store.x = TRUE, f.hook.iter, f.hook.model, ...)
{
	if (is.data.frame(x))
		x <- data.matrix(x)
	else if (!is.matrix (x))
		x <- matrix(x, nrow = length(x), ncol = 1, dimnames = list (names(x), deparse(substitute(x))))
	if (!is.numeric (x))
		stop ("parameter x: numeric matrix/vector expected")

	parlist <- list (k = k, alpha = alpha, nstart = nstart, iter.max = iter.max, f.restr = f.restr, equal.weights = equal.weights, fuzzy = fuzzy, m = m, zero.tol = zero.tol, trace = trace, store.x = store.x, ...)

	if (store.x)
		parlist$x <- x

	n <- nrow (x)
	p <- ncol (x)
	no.trim <- floor(n * (1 - alpha))

	# preparing lot's of lists	
	pa <- list (									##	these are variables which all the iterations have in common, and will never change (pa for "params")
		n = n,													##	number of observations
		p = p,													##	number of dimensions
		no.trim = no.trim,										##	number of observations which are considered as to be not outlying
		trimm = n-no.trim,										##	number of observations which are considered as to be outlying
		K = k,													##	number of clusters to be searched for
		equal.weights = equal.weights,							##	wether equal weights shall be assumed for all clusters
		zero.tol = zero.tol,									##	zero tolerance				##	XXXS yet to be implemented
		trace = trace,											##	trace - level giving more information on the iterations
		fuzzy = fuzzy,											##	whether fuzzy clustering shall be performed
		m = m													##  ?? value specifying what ??
	)

	iter <- list (									##	these variables change each iteration - this object is passed to all the functions, modified and returned by them.
		obj = -Inf,												##	current objective value
		assig = array (0, n),									##	cluster assignment
		csize = array (NA, k),									##	cluster sizes
		cw = rep (NA, k),										##	cluster weights
		sigma = array (NA, c (p, p, k)),						##	cluster's sigmas
		center = array (NA, c(k, p)),							##	cluster's centers
		code = NA,												##	this is a return code supplied by functions like .findClustAssig
		z_ij = matrix (0, nrow = n, ncol = k )					##	z_ij ## -> what was it's old name - "ind" - right?
	)

#o	best.iter <- list (obj = -Inf)								##	empty iter - element, only containing the obj - value
	best.iter <- iter											#n	initializing the best.iter - structure


	for (j in 1:nstart)
	{
		iter <- .InitClusters (x, iter, pa)

		if (!missing (f.hook.iter))
			f.hook.iter (x, .findClustAssig (x, iter, pa), pa, parlist, j, 0)

		lastobj <- -Inf
		for (i in 0:iter.max)
		{
			iter <- f.restr (iter = iter, pa = pa, ...) ##	restricting the clusters' scatter structure

			if (!iter$code)
			{								##	all eigenvalues are zero / a singularity has been detected..
				if (i)
					return (.Parsetclust.Res (x, .TreatSingularity (.calcobj (x, iter, pa), pa), parlist))
				else
					iter$sigma[,,] = diag (pa$p)
            }

			iter <- .findClustAssig (x, iter, pa)				##		finding the cluster assignment on behalf of the current sigma & center information

			if (i && !missing (f.hook.iter))
				f.hook.iter (x, iter, pa, parlist, j, i)

			if (iter$code ||									##		if .findClustAssig returned 1, meaning that the cluster Assignment has not changed
				i == iter.max)									##		or we're in the last concentration step:
				break											##		break the for - loop - we finished this iteration! dont re-estimate cluster parameters this time

			iter <- .estimClustPar (x, iter, pa)				##		estimates the cluster's parameters (cov, center) based on the current iter$assig

			if (trace >= 2)
			{
				curobj <- .calcobj (x, iter, pa)$obj
				if (curobj < lastobj)
					cat ("obj. function dropped by", lastobj - curobj, "from", lastobj, "to", curobj, "\n")

				lastobj <- curobj
			}
		}

		iter <- .calcobj (x, iter, pa)							##	calculates the obj - value of struct iter
		iter$code = as.numeric (i == iter.max)					##	return code 1 : did not converge
																##				0 : converged

		if (!missing (f.hook.model))
			f.hook.model (x, iter, pa, parlist, j)

		if (j == 1 || iter$obj > best.iter$obj)
			best.iter = iter
	}

	# no more re-calculation of the cluster - assignment necessary, because the algorithm stopped at the right position (after calculating the cluster assignment)

	return (.Parsetclust.Res (x, best.iter, parlist))
}

####################						##	this function is supposed to act exactly as the final C implementation of tclust
##	.nr_tclust.R  ##
####################
	
.nr_tclust.R <- function (..., restr = c ("eigen", "deter", "prop", "dir.eigen", "dir.deter", "sigma"))
{
	restr <- match.arg (restr, c ("eigen", "deter", "prop", "dir.eigen", "dir.deter", "sigma"))

	if (restr == "eigen")
		return (.tclust.R (f.restr = .restr.diffax, restr.deter = FALSE, ...))
	else if (restr == "deter")
		return (.tclust.R (f.restr = .restr.diffax, restr.deter = TRUE, ...))
	else if (restr == "prop")
		return (.tclust.R (f.restr = .restr.prop, ...))
	else if (restr == "dir.eigen")
		return (.tclust.R (f.restr = .restr.dir, restr.deter = FALSE, ...))
	else if (restr == "dir.deter")	
		return (.tclust.R (f.restr = .restr.dir, restr.deter = TRUE, ...))
	else if (restr == "sigma")
		return (.tclust.R (f.restr = .restr.avgcov, ...))

## execution never comes here. (match.arg checks for unknown/invalid valuesof argument "restr")
}

########################
##	.Parsetclust.Res  ##
########################

.Parsetclust.Res <- function (x, iter, parlist)
{	##	converts the output of function .tclust.R to a "tclust" - object

	idx.clust <- order (iter$csize, decreasing = TRUE)
	idx.nz <- iter$csize[idx.clust] != 0
	idx.clust <- idx.clust [idx.nz]

	id.clust <- 0
	id.clust [1 + idx.clust] <- 1:length (idx.clust)

	int <- list (
		iter.successful = 0,
		iter.converged = 0,
		dim = dim (x)
		)

	ret <- list (
		centers = t (iter$center[idx.clust, , drop = FALSE]),
		cov = iter$sigma [,, idx.clust, drop = FALSE],
		cluster = id.clust [iter$assig + 1],
		par = parlist,
		k = length (idx.clust),
		obj = iter$obj,
		size = iter$csize [idx.clust],
		weights = iter$cw [idx.clust],
		ret.orig = iter,
		int = int
	)
	class (ret) <- "tclust"
	ret
}

##  roxygen2::roxygenise("C:/users/valen/onedrive/myrepo/r/tclust", load_code=roxygen2:::load_installed)

#'  LG5data data
#'
#' A data set in dimension 10 with three clusters around affine subspaces
#'  of common intrinsic dimension. A 10\% background noise is added uniformly 
#'  distributed in a rectangle containing the three main clusters. 
#' @name LG5data
#' @docType data
#' @usage data(LG5data)
#' @format The first 10 columns are the variables. The last column is the true 
#'  classification vector where symbol "0" stands for the contaminating data points. 
#' @examples 
#' #--- EXAMPLE 1 ------------------------------------------ 
#' data (LG5data)
#' x <- LG5data[, 1:10]
#' clus <- rlg(x, d = c(2,2,2), alpha=0.1, trace=TRUE)
#' plot(x, col=clus$cluster+1)
#' @keywords datasets
NULL

#'  M5data data
#'
#' A bivariate data set obtained from three normal bivariate distributions with 
#'  different scales and proportions 1:2:2. One of the components is very overlapped 
#'  with another one. A 10\% background noise is added uniformly distributed in a rectangle 
#'  containing the three normal components and not very overlapped with the three mixture 
#'  components. A precise description of the M5 data set can be found in 
#'  García-Escudero et al. (2008).
#'
#' @name M5data
#' @docType data
#' @usage data(M5data)
#' @format The first two columns are the two variables. The last column is the true 
#'  classification vector where symbol "0" stands for the contaminating data points. 
#' @source García-Escudero, L.A.; Gordaliza, A.; Matrán, C. and Mayo-Iscar, A. (2008), 
#'  "A General Trimming Approach to Robust Cluster Analysis". Annals of Statistics, 
#'  Vol.36, pp. 1324-1345.
#' @examples 
#' #--- EXAMPLE 1 ------------------------------------------ 
#' data (M5data) 
#' x <- M5data[, 1:2] 
#' clus <- tclust(x, k=3, alpha=0.1, nstart=200, niter1=3, niter2=17, 
#'    nkeep=10, opt="HARD", equal.weights=FALSE, restr.fact=50, trace=TRUE) 
#' plot (x, col=clus$cluster+1)
#' @keywords datasets
NULL

#' Wholesale customers dataset
#'
#' The data set refers to clients of a wholesale distributor. It includes the annual 
#'  spending in monetary units  on diverse product categories.
#'
#' @name wholesale
#' @docType data
#' @usage data(wholesale)
#' @format A data frame containing 440 observations in 8 variables (6 numerical and two categorical). 
#' The variables are as follows:
#'
#' \itemize{
#'      \item \code{Region} Customers' Region - Lisbon (coded as 1), Porto (coded as 2) or Other (coded as 3)
#'      \item \code{Fresh} Annual spending on fresh products
#'      \item \code{Milk} Annual spending on milk products
#'      \item \code{Grocery} Annual spending on grocery products
#'      \item \code{Frozen} Annual spending on frozen products
#'      \item \code{Detergents} Annual spending on detergents and paper products
#'      \item \code{Delicatessen} Annual spending on and delicatessen products
#'      \item \code{Channel}  Customers' Channel - Horeca (Hotel/Restaurant/Café) or 
#'              Retail channel. Horeca is coded as 1 and Retail channel is coded as 2
#'  }
#'
#' @source Abreu, N. (2011). Analise do perfil do cliente Recheio e desenvolvimento de 
#'  um sistema promocional. Mestrado em Marketing, ISCTE-IUL, Lisbon. 
#'  url={https://api.semanticscholar.org/CorpusID:124027622}
#'
#' @examples 
#' #--- EXAMPLE 1 ------------------------------------------ 
#' data (wholesale) 
#' x <- wholesale[, -c(1, ncol(wholesale))] 
#' clus <- tclust(x, k=3, alpha=0.1, nstart=200, niter1=3, niter2=17, 
#'    nkeep=10, opt="HARD", equal.weights=FALSE, restr.fact=50, trace=TRUE) 
#'  plot (x, col=clus$cluster+1)
#'  plot(clus)
#' @keywords datasets
NULL

#' Pinus nigra dataset
#'
#' To study the growth of the wood mass in a cultivated forest of \emph{Pinus nigra} 
#'  located in the north of Palencia (Spain), a sample of 362 trees was studied. 
#'  The data set is made of measurements of heights (in meters), in variable "HT", 
#'  and diameters (in millimetres), in variable "Diameter", of these trees. 
#'  The presence of three linear groups can be guessed apart from a small group 
#'  of trees forming its own cluster with larger heights and diameters one isolated 
#'  tree with the largest diameter but small height. More details on the 
#'  interpretation of this dataset in García-Escudero et al (2010).
#'
#' @name pine
#' @docType data
#' @usage data(pine)
#' @format A data frame containing 362 observations in 2 variables. 
#' The variables are as follows:
#'
#' \itemize{
#'      \item \code{Diameter} Diameter
#'      \item \code{HT} Height
#'  }
#'
#' @references García-Escudero, L. A., Gordaliza, A., Mayo-Iscar, A., and San Martín, R. (2010). 
#'  Robust clusterwise linear regression through trimming. 
#'  \emph{Computational Statistics & Data Analysis}, 54(12), 3057--3069.
#'
#' @keywords datasets
NULL

#' Old Faithful Geyser Data
#'
#' A bivariate data set obtained from the Old Faithful Geyser, containing the 
#'  eruption length and the length of the previous eruption for 271 eruptions 
#'  of this geyser in minutes.
#'
#' @name geyser2
#' @docType data
#' @usage data(geyser2)
#' @format A data frame containing 272 observations in 2 variables. 
#' The variables are as follows:
#'
#' \itemize{
#'      \item \code{Eruption length} The eruption length in minutes. 
#'      \item \code{Previous eruption length} The length of the previous eruption in minutes. 
#'  }
#'
#' @source
#'  This particular data structure can be obtained by applying the following code
#'  to the "Old Faithful Geyser" (\code{faithful} data set (Härdle 1991) in the 
#'  package \code{datasets}):
#'  \cr
#'  \code{f1 <- faithful[,1]}\cr
#'  \code{geyser2 <- cbind(f1[-length(f1)], f1[-1])}\cr
#'  \code{colnames(geyser2) <- c("Eruption length",}\cr
#'  \code{                      "Previous eruption length")}
#
#' @references 
#'  García-Escudero, L.A. and Gordaliza, A. (1999). 
#'  Robustness properties of k-means and trimmed k-means. \emph{Journal of the American Statistical Assoc.}, Vol.94, No.447, 956--969.
#'
#'  Härdle, W. (1991). \emph{Smoothing Techniques with Implementation in S.}, New York: Springer.
#'
#' @keywords datasets
NULL

#' Swiss banknotes data
#'
#' Six variables measured on 100 genuine and 100 counterfeit old Swiss 1000-franc 
#'  bank notes (Flury and Riedwyl, 1988).
#'
#' @name swissbank
#' @docType data
#' @usage data(swissbank)
#' @format A data frame containing 200 observations in 6 variables. 
#' The variables are as follows:
#'
#' \itemize{
#'      \item \code{Length} Length of the bank note 
#'      \item \code{Ht_Left} Height of the bank note, measured on the left 
#'      \item \code{Ht_Right} Height of the bank note, measured on the right 
#'      \item \code{IF_Lower} Distance of inner frame to the lower border 
#'      \item \code{IF_Upper} Distance of inner frame to the upper border 
#'      \item \code{Diagonal} Length of the diagonal 
#'  }
#'
#' @details 
#'  Observations 1--100 are the genuine bank notes and the other 100 observations are the counterfeit bank notes. 
#'
#' @source
#'  Flury, B. and Riedwyl, H. (1988). \emph{Multivariate Statistics, A Practical Approach}, Cambridge University Press. 
#
#' @keywords datasets
NULL
#' Flea
#'
#' Flea-beetle measurements
#'
#' @name flea
#' @docType data
#' @usage data(flea)
#' @format A data frame with 74 rows and 7 variables: six explanatory and one response variable - \code{species}.
#' The variables are as follows:
#'
#' \itemize{
#'   \item tars1: width of the first joint of the first tarsus in microns (the sum of measurements for both tarsi)
#'   \item tars2: the same for the second joint
#'   \item head: the maximal width of the head between the external edges of the eyes in 0.01 mm
#'   \item ade1: the maximal width of the aedeagus in the fore-part in microns
#'   \item ade2: the front angle of the aedeagus ( 1 unit = 7.5 degrees)
#'   \item ade3: the aedeagus width from the side in microns
#'   \item species, which species is being examined - \code{Concinna}, \code{Heptapotamica}, \code{Heikertingeri}
#' }
#'
#'
#' @references A. A. Lubischew (1962), On the Use of Discriminant Functions in Taxonomy, \emph{Biometrics}, \bold{18}4 pp.455--477.
#'
#' @examples
#'  data(flea)
#'  head(flea)
#'
#' @keywords datasets
NULL
#' mixsym
#'
#' Simulated bivariate data set
#'
#' @name mixsym
#' @docType data
#' @usage data(mixsym)
#' @description A simulated bivariate data set of size \code{n = 100} and 
#'  dimension \code{p = 2} from a \code{k = 3} component mixture obtained
#' by applying the \code{MixSim} method of Maitra and Melnykov (2010), 
#'  as extended by Riani et al. (2015) and incorporated into the FSDA toolbox 
#'  of Matlab (Riani et al., 2012). The data set has been generated by imposing 
#'  an average cluster overlap (defined as a sum of pairwise misclassification 
#'  probabilities) equal to 0.04 and a maximum eigenvalue ratio for the scatters 
#'  matrices equal to 5.
#' @format A data frame with 10 rows and 3 variables: 2 numerical and one categorical - the thrue cluster.
#' The variables are as follows:
#'
#' \itemize{
#'   \item X1: first numerical variable
#'   \item X2: second numerical variable
#'   \item class: the true cluster
#' }
#'
#'
#' @references 
#' Maitra, R. and Melnykov, V. (2010). Simulating data to study performance of finite mixture
#'  modeling and clustering algorithms. J. Comput. Graph. Stat., 19:354– 376.
#'
#' Riani, M., Cerioli, A., Perrotta, D., and Torti, F. (2015). Simulating mixtures of multivariate 
#'  data with fixed cluster overlap in FSDA library. Adv. Data Anal. Classif., 9:2015.
#'
#' Riani, M., Perrotta, D., and Torti, F. (2012). FSDA: a matlab toolbox for robust analysis
#'  and interactive data exploration,. Chemometr. Intell. Lab. Syst., 116:17–32.
#'
#' @examples
#'  data(mixsym)
#'  head(mixsym)
#'
#' @keywords datasets
NULL

.restr2_eigenv <- function (autovalues, ni.ini, restr.fact, zero.tol)
{
    ev <- autovalues

###### function parameters:
###### ev: matrix containin eigenvalues									#n proper naming - changed autovalue to ev (eigenvalue)
###### ni.ini: current sample size of the clusters						#n proper naming - changed ni.ini to cSize (cluster size)
###### factor: the factor parameter in tclust program
###### init: 1 if we are applying restrictions during the smart inicialization
######       0 if we are applying restrictions during the C steps execution

###### some inicializations

	if (!is.matrix (ev))												#n	checking for malformed ev - argument (e.g. a vector of determinants instead of a matrix)
		if (is.atomic (ev))												#n
			ev <- t (as.numeric (ev))									#n
		else															#n
			ev <- as.matrix (ev)										#n

	stopifnot (ncol (ev) == length (ni.ini))							#n	check wether the matrix autovalues and ni.ini have the right dimension.

	d <- t (ev)

    p <- nrow (ev)
	K <- ncol (ev)	

	n <- sum(ni.ini)

	nis <- matrix(data=ni.ini, nrow=K, ncol=p)
																		#m	MOVED: this block has been moved up a bit. see "old position of "block A" for it's old occurrence
	idx.nis.gr.0 <- nis > zero.tol										#n	as nis is in R we have to be carefull when checking against 0
	used.ev <- ni.ini > zero.tol										#n
	ev.nz <- ev[,used.ev]												#n	non-zero eigenvalues
																		#m
#o	if ((max (d[nis > 0]) <= zero.tol))									#m
#i	if ((max (d[idx.nis.gr.0]) <= zero.tol))							#n	"idx.nis.gr.0" instead of (nis > 0)
	if ((max (ev.nz) <= zero.tol))										#n	simplify syntax
		return (matrix (0, nrow = p, ncol = K))							#m
																		#m
	###### we check if the  eigenvalues verify the restrictions			#m
																		#m
#o	if (max (d[nis > 0]) / min (d[nis > 0]) <= restr.fact)				#m
#i	if (max (d[idx.nis.gr.0]) / min (d[idx.nis.gr.0]) <= restr.fact)	#n	"idx.nis.gr.0" instead of (nis > 0)
	if (max (ev.nz) / min (ev.nz) <= restr.fact)						#n	simplify syntax

	{																	#m
#o		d[!idx.nis.gr.0] <- mean (d[idx.nis.gr.0])						#n	"idx.nis.gr.0" instead of (nis > 0)
		ev[,!used.ev] <- mean (ev.nz)									#n	simplify syntax
		return (ev)														#m
	}																	#m

	###### d_ is the ordered set of values in which the restriction objective function change the definition
	###### points in d_ correspond to  the frontiers for the intervals in which this objective function has the same definition
	###### ed is a set with the middle points of these intervals

#o	d_ <- sort (c (d, d / restr.fact))
	d_ <- sort (c (ev, ev / restr.fact))								#n	using ev instead of d
	dim <- length (d_)													##2do: continue here cleaning up .restr2_eigenv
	d_1 <- d_
	d_1[dim+1] <- d_[dim] * 2
	d_2 <- c (0, d_)
	ed <- (d_1 + d_2) / 2
	dim <- dim + 1;

##o
##o	old position of "block A"
##o

	###### the only relevant eigenvalues are those belong to a clusters with sample size greater than 0.
	###### eigenvalues corresponding to a clusters with 0 individuals has no influence in the objective function
	###### if all the eigenvalues are 0 during the smart initialization we assign to all the eigenvalues the value 1

	###### we build the sol array
	###### sol[1],sol[2],.... this array contains the critical values of the interval functions which defines the m objective function
	###### we use the centers of the interval to get a definition for the function in each interval
	###### this set with the critical values (in the array sol) contains the optimum m value

	t <- s <- r <- array(0, c(K, dim))
	sol <- sal <- array(0, c(dim))

	for (mp_ in 1:dim)
	{
		for (i in 1:K)
		{
			r[i,mp_] <- sum ((d[i,] < ed[mp_])) + sum((d[i,] > ed[mp_]*restr.fact))
			s[i,mp_] <- sum (d[i,]*(d[i,] < ed[mp_]))
			t[i,mp_] <- sum (d[i,]*(d[i,] > ed[mp_] * restr.fact))
		}

		sol[mp_] <- sum (ni.ini / n * (s[,mp_] + t[,mp_] / restr.fact)) / (sum(ni.ini /n * (r[, mp_])))

		e <-	sol[mp_] * (d < sol[mp_]) +
				d * (d >= sol[mp_]) * (d <= restr.fact * sol[mp_]) +
				(restr.fact*sol[mp_]) * (d > restr.fact * sol[mp_])
		o <- -1/2 * nis / n * (log(e) + d / e)

		sal[mp_] <- sum(o)
	}

	###### m is the optimum value for the eigenvalues procedure
#o	eo <- which.max (c (sal))						## remove c ()
#	m <- sol[eo]
	m <- sol[which.max (sal)]						#n
	###### based on the m value we get the restricted eigenvalues

	t (m * (d < m) + d * (d >= m) * (d <= restr.fact * m) + (restr.fact * m) * (d > restr.fact * m))	##	the return value
}

.restr2_deter_old <- function(autovalues, ni.ini, restr.fact, zero.tol=1e-16)
{
    ###### function parameters:
    ###### autovalues: matrix containing eigenvalues
    ###### ni.ini: current sample size of the clusters
    ###### factor: the factor parameter in tclust program
    ###### some initializations

	p = nrow (autovalues)
	K = ncol (autovalues)

	if(p == 1)
		return (.restr2_eigenv (autovalues, ni.ini, restr.fact, zero.tol))

	es = apply (autovalues, 2, prod)

	idx.ni.ini.gr.0 <- ni.ini > zero.tol										#n	as ni.ini is in R we have to be carefull when checking against 0

				######	we check if all the determinants in no empty populations are 0
#o	if (max(es[ni.ini > 0]) <= zero.tol)	##	all eigenvalues are somehow zero.
	if (max(es[idx.ni.ini.gr.0]) <= zero.tol)	#n	"idx.ni.ini.gr.0" instead of (ni.ini > 0)
		return (matrix(0, p, K))										 		##		-> return zero mat

				######  we put in d the determinants of the populations (converting es into a matrix of dim 1 x K)
	d = t(es)	#### --> dim (d) = 1 x K (has once been "d <- matrix (es, nrow = 1)")
																				#n	put this block into a function (for improved readability)
	autovalues_det <- .HandleSmallEv (autovalues, zero.tol)						##	handling close to zero eigenvalues here

print(autovalues)
print(autovalues_det)

#cat ("\n1d^(1/p):\t", d^(1/p), "\n")

					######	we check if all the determinants verify the restrictions
#o	if (max (d[ni.ini > 0]) / min (d[ni.ini > 0]) <= restr.fact)
	if (max (d[idx.ni.ini.gr.0]) / min (d[idx.ni.ini.gr.0]) <= restr.fact)		#n	"idx.ni.ini.gr.0" instead of (ni.ini > 0)
	{
#o		d [ni.ini == 0] <-  mean (d[ni.ini > 0])								## and get the mean - determinants for all clusters without observations.
		d [!idx.ni.ini.gr.0] <-  mean (d[idx.ni.ini.gr.0])						#n	"idx.ni.ini.gr.0" instead of (ni.ini > 0)

		dfin <- d^(1/p)
	}
	else {
		dfin <- .restr2_eigenv(d^(1/p), ni.ini, restr.fact^(1/p), zero.tol)
        cat("\ndfin coming from restr2_eigenv()...\n")
        print(dfin)
    }
					######  we apply the restriction to the determinants by using the .restr2_eigenv function
					######  In order to apply this function is neccessary to transform d and factor with the power (1/p)

#cat ("\nfin:\t", dfin, "\n")


	return(autovalues_det %*% diag(as.numeric(dfin))) 							## autovalues_det %*% diag (dfin)
}

.HandleSmallEv <- function (autovalues, zero.tol)							    #n	a part of .restr2_deter_, which handles almost zero eigenvalues
{	

    ##	handling close to zero eigenvalues here
    ######  populations with one eigenvalue close to 0 are very close to be contained in a hyperplane
    ######  autovalues2 is equal to autovalues except for the columns corresponding to populations close to singular
    ######  for these populations we put only one eigenvalue close to 0 and the rest far from 0   	
	
	p <- nrow (autovalues)													#n
	autovalues[autovalues <= zero.tol] <- zero.tol							#n	"<= zero.tol" for checking for zero

	mi <- apply(autovalues, 2, min)											##	the minimum eigenvalue of each cluster
	ma <- apply(autovalues, 2, max)											##	the maximum eigenvalue of each cluster

	idx.iter <- which(mi/ma <= zero.tol)									#n	making more obvious for what to check!

    ##  cat("\nmi/ma\n")
    ##  print(mi/ma)
    ##  cat("\nmi/zero.tol\n")
    ##  print(mi/zero.tol)
    ##  cat("\nidx.iter\n")
    ##  print(idx.iter)
    
    ##	for each of these clusters. set all "normal" eigenvalues to a high value
	for(i in idx.iter)	{
        ##  cat("\nTo modify index: ", which(autovalues[,i] > mi[i] / zero.tol), "\n")    
        autovalues[autovalues[,i] > mi[i] / zero.tol, i] <- mi[i] / zero.tol
    }
    
    ##  cat("\nModified autovalues\n")
    ##  print(autovalues)

	det = apply(autovalues, 2, prod)											#n	the new determinants

	######	autovalues_det contains the autovalues corrected by the determinant
	######	the product of the eigenvalues of each column in autovalues_det is equal to 1

	autovalues_det <- autovalues %*% diag(det^(-1/p))  					
	return (autovalues_det)
}

.restr2_deter_ <- function(autovalues, ni.ini, restr.factor, cshape=1e10, zero.tol = 1e-16) {
    ## autovalues: matrix containing eigenvalues
    ## ni.ini: current sample size of the clusters
    ## implicitly we are using two restriction parameters: 
    ##  - restr.factor  - constraint level for the determinant constraints
    ##  - cshape        - constraint level for the eigenvalues constraints, default is 1e10


    p <- nrow(autovalues)
    K <- ncol(autovalues)

    autovalues[autovalues < 1e-16] <- 0
    autovalues_ <- autovalues

    if(p == 1)
        return(.restr2_eigenv(autovalues, ni.ini, restr.factor, zero.tol))

    for(k in 1:K)                                
        autovalues_[,k] <- .restr2_eigenv(autovalues=cbind(autovalues[,k]), ni.ini=1, restr.fact=cshape, zero.tol)

    ##  cat("\nautovalues_\n"); print(autovalues_)
    
    es <- apply(autovalues_, 2, prod)
    es[es==0] <- 1
    
    ##  cat("\nes\n"); print(es)
    
    ##  cat("\nesmat\n"); print(matrix((es^(1/p)), ncol=K, nrow=p, byrow=TRUE))

    gm <- autovalues_/matrix((es^(1/p)), ncol=K, nrow=p, byrow=TRUE)

    ##  cat("\ngm\n"); print(gm)

    d <- rbind(apply(autovalues/gm, 2, sum)/p)
    d[is.nan(d)] <- 0

    dfin <- .restr2_eigenv(d, ni.ini, restr.factor^(1/p), zero.tol)
    dout <- matrix(dfin, ncol=K, nrow=p, byrow=TRUE) * (gm * (gm>0) + 1 * (gm==0))  
    
    return (dout)
}
    

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

rlg_c1 <- function(x, d, alpha = 0.05, niter1 = 3L) {
    .Call(`_tclust_rlg_c1`, x, d, alpha, niter1)
}

rlg_c2 <- function(x, d, cluster, alpha = 0.05, niter2 = 20L) {
    .Call(`_tclust_rlg_c2`, x, d, cluster, alpha, niter2)
}

#' Function to perform the E-step for a Gaussian mixture distribution
#' @name estepRR
#' @description Compute the log PDF for each observation, the posterior probabilities
#'     and the objective function (total log-likelihood) for a Gaussian 
#'     mixture distribution
#' @param ll Rcpp::NumericMatrix, n-by-k where \code{n} is the number of 
#'  observations and \code{k} is the number of clusters.
#' @return The function returns a list with the following elements:
#' \itemize{
#'    \item obj The value of the objective function (total log-likelihood)
#'    \item postprob an \code{n-by-k} matrix with the posterior probablilities
#'    \item logpdf a vector of length \code{n} containing the log PDF for 
#'         each observation
#' }
#' @details Formally a mixture model corresponds to the mixture distribution that
#'   represents the probability distribution of observations in the overall
#'   population. Mixture models are used
#'   to make statistical inferences about the properties of the
#'   sub-populations given only observations on the pooled population, without
#'   sub-population-identity information.
#'   Mixture modeling approaches assume that data at hand $\eqn{y_1, ..., y_n} in
#'   \eqn{R^p} come from a probability distribution with density given by the sum of k components
#'   \deqn{\sum_{j=1}^k  \pi_j \phi( \cdot, \theta_j)}
#'   with \eqn{\phi( \cdot, \theta_j)} being the
#'   \eqn{p}-variate  (generally multivariate normal) densities with parameters
#'   \eqn{\theta_j}, \eqn{j=1, \ldots, k}. Generally \eqn{\theta_j= (\mu_j, \Sigma_j)}
#'   where \eqn{\mu_j} is the population mean  and   \eqn{\Sigma_j} is the covariance
#'   matrix for component \eqn{j}.
#'   \eqn{\pi_j} is the (prior) probability of component \eqn{j}.
#'   The objective function is obj is equal to
#'    \deqn{ obj = \log   \left( \prod_{i=1}^n  \sum_{j=1}^k \pi_j \phi (y_i; \; \theta_j)    \right) 
#'     }
#'
#'    or
#'  
#'     \deqn{ obj =  \sum_{i=1}^n  \log   \left( \sum_{j=1}^k \pi_j \phi (y_i; \; \theta_j)    \right)
#'     }
#'     where \eqn{k} is the number of components of the mixture and \eqn{\pi_j} are the 
#'     component probabilitites and \eqn{\theta_j} are the  parameters of the \eqn{j}-th 
#'     mixture component.
#'
#'
#' @references
#'
#'   McLachlan, G.J.; Peel, D. (2000). Finite Mixture Models. Wiley. ISBN 0-471-00626-2
#'
#' @examples
#'##      Generate two Gaussian normal distributions
#'##      and do not produce plots
#'
#'        mu1 = c(1,2)
#'        sigma1 = matrix(c(2, 0, 0, .5), nrow=2, byrow=TRUE)    #[2 0; 0 .5];
#'        mu2 = c(-3, -5)
#'        sigma2 = matrix(c(1, 0, 0, 1), nrow=2, byrow=TRUE)
#'        n1 = 100
#'        n2 = 200
#'        Y = rbind(MASS::mvrnorm(n1, mu1, sigma1), 
#'                  MASS::mvrnorm(n2, mu2, sigma2))
#'        k = 2
#'        pi = c(1/3, 2/3)
#'        mu = rbind(mu1, mu2)
#'        sigma = array(0, dim=c(2,2,2))
#'        sigma[,,1] = sigma1
#'        sigma[,,2] = sigma2
#'        
#'        ll = matrix(0, nrow=n1+n2, ncol=2)
#'        for(j in 1:k)
#'            ll[,j] = log(pi[j]) +  tclust:::dmvnrm(Y, mu[j,], sigma[,,j])
#'
#'        dd = tclust:::estepRR(ll)
#'        dd$obj
#'        dd$logpdf
#'        dd$postprob
NULL

tclust_c2 <- function(x, k, cluster, alpha = 0.05, restrC = 0L, deterC = FALSE, restr_fact = 12, cshape = 1e10, niter2 = 20L, opt = "HARD", equal_weights = FALSE, zero_tol = 1e-16) {
    .Call(`_tclust_tclust_c2`, x, k, cluster, alpha, restrC, deterC, restr_fact, cshape, niter2, opt, equal_weights, zero_tol)
}

tclust_c1 <- function(x, k, alpha = 0.05, restrC = 0L, deterC = FALSE, restr_fact = 12, cshape = 1e10, niter1 = 3L, opt = "HARD", equal_weights = FALSE, zero_tol = 1e-16) {
    .Call(`_tclust_tclust_c1`, x, k, alpha, restrC, deterC, restr_fact, cshape, niter1, opt, equal_weights, zero_tol)
}

tclust_restr2_eigenv <- function(autovalues, ni_ini, factor_e = 12, zero_tol = 1e-16) {
    .Call(`_tclust_tclust_restr2_eigenv`, autovalues, ni_ini, factor_e, zero_tol)
}

tclust_restr2_deter_old <- function(autovalues, ni_ini, factor_e = 12, zero_tol = 1e-16) {
    .Call(`_tclust_tclust_restr2_deter_old`, autovalues, ni_ini, factor_e, zero_tol)
}

tclust_restr2_deter <- function(autovalues, ni_ini, restr_factor = 12, cshape = 1e10, zero_tol = 1e-16) {
    .Call(`_tclust_tclust_restr2_deter`, autovalues, ni_ini, restr_factor, cshape, zero_tol)
}

tclust_HandleSmallEv <- function(autovalues, zero_tol = 1e-16) {
    .Call(`_tclust_tclust_HandleSmallEv`, autovalues, zero_tol)
}

dmvnrm <- function(x, mean, cov) {
    .Call(`_tclust_dmvnrm`, x, mean, cov)
}

estepRR <- function(ll) {
    .Call(`_tclust_estepRR`, ll)
}

tkmeans_c1 <- function(x, k, alpha = 0.05, niter1 = 3L, zero_tol = 1e-16, points = NULL) {
    .Call(`_tclust_tkmeans_c1`, x, k, alpha, niter1, zero_tol, points)
}

tkmeans_c2 <- function(x, k, cluster, alpha = 0.05, niter2 = 20L, zero_tol = 1e-16) {
    .Call(`_tclust_tkmeans_c2`, x, k, cluster, alpha, niter2, zero_tol)
}

## The parameters from a tclust object that we need:
##  - x, n, p, alpha, k, size, centers, cov
##  TO DO:
##  - x can be also object different from tclust: tkmeans or rlg?
##  - The optimization method used for computing x can be either HARD or MIXT
##
##  roxygen2::roxygenise("C:/users/valen/onedrive/myrepo/rrdev/robClus", load_code=roxygen2:::load_installed)

#'
#' Discriminant Factor analysis for \code{tclust} objects
#' 
#' @name DiscrFact
#' @aliases print.DiscrFact
#' @description Analyzes a \code{tclust}-object by calculating discriminant factors 
#'  and comparing the quality of the actual cluster assignments to that of the second best 
#'  possible assignment for each observation. Cluster assignments of observations 
#'  with large discriminant factors are considered "doubtful" decisions. Silhouette 
#'  plots give a graphical overview of the discriminant factors distribution 
#'  (see \code{\link{plot.DiscrFact}}). More details can be found in García-Escudero et al. (2011).
#' @param x A \code{tclust} object.
#' @param threshold  A cluster assignment or a trimming decision for an observation with a 
#'  discriminant factor larger than \code{log(threshold)} is considered a "doubtful" decision. 
#' @return The function returns an S3 object of type \code{DiscrFact} containing the following components:
#'	\itemize{
#'      \item \code{x} A \code{tclust} object. 
#'      \item \code{ylimmin} A minimum y-limit calculated for plotting purposes. 
#'	    \item \code{ind} The actual cluster assignment. 
#'	    \item \code{ind2} The second most likely cluster assignment for each observation. 
#'	    \item \code{lik} The (weighted) likelihood of the actual cluster assignment of each observation. 
#'	    \item \code{lik2} The (weighted) likelihood of the second best cluster assignment of each observation. 
#'	    \item \code{assignfact} The factor \code{log(disc/disc2)}. 
#'	    \item \code{threshold} The threshold used for deciding whether \code{assignfact} indicates a "doubtful" assignment. 
#'	    \item \code{mean.DiscrFact} A vector of length \code{k + 1} containing the mean discriminant 
#'      factors for each cluster (including the outliers). 
#'  }
#' @references 
#'    \enc{García}{Garcia}-Escudero, L.A.; Gordaliza, A.; \enc{Matrán}{Matran}, C. and Mayo-Iscar, A. (2011), 
#'    "Exploring the number of groups in robust model-based clustering." \emph{Statistics and Computing}, \bold{21} 
#'    pp. 585-599, <doi:10.1007/s11222-010-9194-z>
#'
#' 
DiscrFact <- function(x, threshold=1/10)
{
    if (!any (class (x) == "tclust"))
      stop ("parameter x: expected object of type \x22tclust\x22")

    p <- x$int$dim[2]
    alpha <- x$par$alpha
    n <- nrow(x$par$x)
    
    ll <- matrix(nrow=n, ncol=x$k)
    lik <- lik2 <- ind <- array (NA, n)

    no.trim = floor(n*(1-alpha))

    for(k in 1:x$k)
      ll[,k] <- (x$size[k] / no.trim) * dmvnrm(x$par$x, x$centers[,k], as.matrix(x$cov[,,k]))

    llo <- apply(-ll, 1, order)         # the row-wise order of matrix ll 
    if(!is.matrix(llo))
      llo <- matrix(llo, ncol=nrow(ll)) # -> transposed..

    lik <- ll[cbind (1:n, llo[1, ])]
    ind <- llo[1,]

    if(nrow(llo) >= 2) {
      lik2 <- ll[cbind(1:n, llo[2, ])]
      ind2 <- llo[2,]
    } else {
      ind2 <- rep (0, n)
      lik2 <- lik * threshold
     }

    mropt <- sort(lik)[n - floor (n * (1 - alpha)) + 1] 
    idx.out <- lik < mropt

    ind2[idx.out] <- ind[idx.out]
    lik2[idx.out] <- lik[idx.out]
    lik[idx.out] <- mropt
    ind[idx.out] <- 0

    assignfact <- log(lik2 / lik)
    idx.fin <- is.finite(assignfact)
    ylimmin <- min(assignfact[idx.fin]) * 1.5
    assignfact[!idx.fin] <- ylimmin * 2

    mean.DiscrFact <- array(,x$k)
    for (i in 0:x$k)
      mean.DiscrFact[i + 1] = mean(assignfact[ind == i])
      
    names(mean.DiscrFact) <- c ("O", 1:x$k)

    ret <- list(x=x, ylimmin=ylimmin, ind=ind, ind2=ind2, assignfact=assignfact, 
            lik=lik, lik2=lik2, threshold=log(threshold), 
            mean.DiscrFact=mean.DiscrFact)
    class (ret) <- "DiscrFact"
    ret
  }

#' @export

print.DiscrFact <- function (x, ...) {
  cat ("Mean overall discriminant factor:", mean (x$assignfact), "\n")
  cat ("Mean discriminant factor per cluster:\n")
  print (x$mean.DiscrFact)

  idx <- x$assignfact > x$threshold

  if(!sum(idx))
    cat("No decision is considered as doubtful\n")
  else
    cat(sum(idx), "decisions are considered as doubtful\n")

  invisible(x)
}

#' @name plot.DiscrFact
#' @title The \code{plot} method for objects of class \code{DiscrFact}
#' @rdname plot.DiscrFact
#' @description The \code{plot} method for class \code{DiscrFact}: Next to a plot of the \code{tclust} 
#'  object which has been used for creating the \code{DiscrFact} object, a silhouette plot 
#'  indicates the presence of groups with a large amount of doubtfully assigned 
#'  observations. A third plot similar to the standard \code{tclust} plot serves 
#'  to highlight the identified doubtful observations.
#'
#' @param x An object of class \code{DiscrFact} as returned from DiscrFact()
#' @param enum.plots A logical value indicating whether the plots shall be enumerated 
#'  in their title ("(a)", "(b)", "(c)").
#' @param xlab,ylab,xlim Arguments passed to funcion \code{plot.tclust()}
#' @param print.DiscrFact A logical value indicating whether each clusters mean discriminant factor shall be plotted
#' @param col.nodoubt Color of all observations not considered as to be assigned doubtfully.
#' @param by.cluster Logical value indicating whether optional parameters pch and col 
#'  (if present) refer to observations (FALSE) or clusters (TRUE)
#' @param \ldots Arguments to be passed to or from other methods
#'
#' @details \code{plot_DiscrFact_p2} displays a silhouette plot based on the discriminant 
#'  factors of the observations. A solution with many large discriminant factors is 
#'  not reliable. Such clusters can be identified with this silhouette plot. 
#'  Thus \code{plot_DiscrFact_p3} displays the dataset, highlighting observations with 
#'  discriminant factors greater than the given threshold. The function \code{plot.DiscrFact()} 
#'  combines the standard plot of a \code{tclust} object, and the two plots introduced here.
#'
#' @references 
#'    \enc{García}{Garcia}-Escudero, L.A.; Gordaliza, A.; \enc{Matrán}{Matran}, C. and Mayo-Iscar, A. (2011), 
#'    "Exploring the number of groups in robust model-based clustering." \emph{Statistics and Computing}, \bold{21} 
#'    pp. 585-599, <doi:10.1007/s11222-010-9194-z>
#'
#' @examples
#'  sig <- diag (2)
#'  cen <- rep (1, 2)
#'  x <- rbind(MASS::mvrnorm(360, cen * 0,   sig),
#'  	       MASS::mvrnorm(540, cen * 5,   sig * 6 - 2),
#'  	       MASS::mvrnorm(100, cen * 2.5, sig * 50))
#'
#'  clus.1 <- tclust(x, k = 2, alpha=0.1, restr.fact=12)
#'  clus.2 <- tclust(x, k = 3, alpha=0.1, restr.fact=1)
#'
#'  dsc.1 <- DiscrFact(clus.1)
#'  plot(dsc.1)
#'
#'  dsc.2 <- DiscrFact(clus.2)
#'  plot(dsc.2)
#'
plot.DiscrFact <- function (x, enum.plots = FALSE, 
    xlab="Discriminant Factor", ylab="Clusters", print.DiscrFact=TRUE, xlim, 
    col.nodoubt= grey(0.8), by.cluster=FALSE, ...)
{
  if(enum.plots)
    main.pre <-  c ("(a)", "(b)", "(c)")
  else
    main.pre <- NULL

  old.par <- par (mfrow = c (1,3))

  plot(x$x, main.pre = main.pre[1], ...)
  plot_DiscrFact_p2 (x, main.pre = main.pre[2], xlab=xlab, ylab=ylab, print.DiscrFact=print.DiscrFact, xlim=xlim, ...)
  plot_DiscrFact_p3 (x, main.pre = main.pre[3], col.nodoubt=col.nodoubt, by.cluster=by.cluster, ...)  

  par (old.par)
}

plot_DiscrFact_p2 <- function(x, xlab="Discriminant Factor", ylab="Clusters", main, xlim,
          print.DiscrFact=TRUE, main.pre, ...)
{
  n <- x$x$int$dim[1]

  if(missing (main))
    main <- "Silhouette Plot"  
    
  if(!missing(main.pre) && !is.null(main.pre))
    main <- paste(main.pre, main)

  if(missing (xlim))
      xlim = c(x$ylimmin,0)
      
  plot(0, 0, xlim=xlim, ylim=c(1,n), type="n", xlab=xlab, ylab=ylab,
        main=main, axes=FALSE, ...)
  

  axis(side=1)

  cs <- c(0, cumsum(c(x$x$int$dim[1] - sum(x$x$size), x$x$size)))

  {
    ylines <- cs[-1]
    ylines <- ylines[-length (ylines)]
    abline (h = ylines, lty = 2)
  }

  cs = (cs[-1] + cs[-length(cs)]) / 2
  axis (side = 2, at = cs, labels = c ("O", 1:x$x$k))
  box ()

  cury = 0
  for (k in 0:x$x$k)
  {
    grupo.k <- sort (x$assignfact[x$ind == k])
    gs = length (grupo.k)
    if (gs > 0)
      polygon (c (0, grupo.k, 0), c (1, 1:gs, gs) + cury, border = 0, col =
              k + 1)
    
    {  
      ll <- cury
      ul <- cury + gs

      if (k == 0)
        ll <- par ("usr")[3]
      if (k == x$x$k)
        ul <- par ("usr")[4]

    }
    cury = cury + gs
  }

  xpos = sum (par ("usr")[1:2] * c (1, 4)) / 5
  
  if(print.DiscrFact)
    legend ("topleft", legend = format (x$mean.DiscrFact[(x$x$k + 1):1],
            digits = 4), inset = 0.04, col = 1 + (x$x$k:0), pch = 15,
            title = "Mean Discriminant Factors", box.lwd = 0, bty = "n")
  abline (v = x$threshold + 1, lty = 2)

}

plot_DiscrFact_p3 <- function(x, main="Doubtful Assignments", col, pch, 
          col.nodoubt= grey(0.8), by.cluster=FALSE, ...)
{
  idxplot <- x$assignfact > x$threshold
  n <- length(x$x$cluster)

  if(missing(col)) {
    col <- rep(col.nodoubt, n)
    col[idxplot] <- x$ind[idxplot] + 1 
  }

  if(missing (pch))
    pch <- x$x$cluster + 1
  
  plot(x$x, by.cluster=FALSE, col=col, pch=pch, main=main, sub="",sub1="", ...)
}


#' @name summary.DiscrFact
#' @title The \code{summary} method for objects of class \code{DiscrFact}
#' @rdname summary.DiscrFact
#' @description The summary method for class \code{DiscrFact}.
#'
#' @param object An object of class \code{DiscrFact} as returned from \code{DiscrFact()}.
#' @param hide.emtpy A logical value specifying whether clusters without doubtful 
#'  assignment shall be hidden.
#' @param show.clust A logical value specifying whether the number of doubtful 
#'  assignments per cluster shall be displayed.
#' @param show.alt A logical value specifying whether the alternative cluster 
#'  assignment shall be displayed.
#' @param \ldots Arguments passed to or from other methods.
#' @references 
#'    \enc{García}{Garcia}-Escudero, L.A.; Gordaliza, A.; \enc{Matrán}{Matran}, C. and Mayo-Iscar, A. (2011), 
#'    "Exploring the number of groups in robust model-based clustering." \emph{Statistics and Computing}, \bold{21} 
#'    pp. 585-599, <doi:10.1007/s11222-010-9194-z>
#'
#' @examples
#'  sig <- diag (2)
#'  cen <- rep (1, 2)
#'  x <- rbind(MASS::mvrnorm(360, cen * 0,   sig),
#'  	       MASS::mvrnorm(540, cen * 5,   sig * 6 - 2),
#'  	       MASS::mvrnorm(100, cen * 2.5, sig * 50)
#'  )
#'
#'  clus.1 <- tclust(x, k = 2, alpha=0.1, restr.fact=12)
#'  clus.2 <- tclust(x, k = 3, alpha=0.1, restr.fact=1)
#'
#'  dsc.1 <- DiscrFact(clus.1)
#'  summary(dsc.1)
#'
#'  dsc.2 <- DiscrFact(clus.2)
#'  summary(dsc.2)
#'
summary.DiscrFact <- function(object, hide.emtpy=TRUE, show.clust, show.alt, ...) {
  idx <- object$assignfact > object$threshold
  k <- object$x$k
  
  if(missing(show.clust))
    show.clust <- k <= 20
  if(missing(show.alt))
    show.alt <- k <= 5

  k1 <- k + 1

  dn <- c ("O", 1:k)

  if(show.clust) {
    cat ("\nNumber of doubtful assignments in clusters:\n")
    facts1 <- object$ind[idx] + 1
    cha <- as.array(tabulate(facts1, nbins = k1))
    dimnames(cha) <- list(dn)
    if(hide.emtpy)
      cha <- cha[cha != 0]
    print(cha)
  }
  
  if(show.alt)  {
    facts2 <- object$ind[idx] * k1 + object$ind2[idx]  + 1

    cat("\nObservations with doubtful decision are alternatively assigned to clusters:\n")  
    chma <- matrix(tabulate(facts2, nbins = k1 * k1), ncol=k1, nrow=k1)
    dimnames(chma) <- list(dn, dn)
    if(hide.emtpy) {
      chma <- chma[apply(chma, 1, sum) != 0, , drop = FALSE]
      chma <- chma[, apply(chma, 2, sum) != 0, drop = FALSE]
    }
    print(chma)
  }  
}

## Fix drop[i,j], out[i,j]  and unrestr.fact[i,j]
##

##  roxygen2::roxygenise("C:/users/valen/onedrive/myrepo/r/tclust", load_code=roxygen2:::load_installed)

#'
#' Classification Trimmed Likelihood Curves
#' 
#' @name ctlcurves
#' @aliases print.ctlcurves
#' @description The function applies \code{\link{tclust}} several times on a given dataset while parameters 
#'  \code{alpha} and \code{k} are altered. The resulting object gives an idea of the optimal 
#'  trimming level and number of clusters considering a particular dataset.
#' @details These curves show the values of the trimmed classification (log-)likelihoods 
#'  when altering the trimming proportion \code{alpha} and the number of clusters \code{k}. 
#'  The careful examination of these curves provides valuable information for choosing 
#'  these parameters in a clustering problem. For instance, an appropriate \code{k} 
#'  to be chosen is one that we do not observe a clear increase in the trimmed classification 
#'  likelihood curve for k with respect to the k+1 curve for almost all the range 
#'  of alpha values. Moreover, an appropriate choice of parameter alpha may be derived 
#'  by determining where an initial fast increase of the trimmed classification 
#'  likelihood curve stops for the final chosen k. A more detailed explanation can 
#'  be found in García-Escudero et al. (2011). 
#' @param x A matrix or data frame of dimension n x p, containing the observations (row-wise).
#' @param k A vector of cluster numbers to be checked. By default cluster numbers from 1 to 5 are examined.
#' @param alpha A vector containing the alpha levels to be checked. By default \code{alpha} 
#'  levels from 0 to 0.2 (continuously increased by 0.01), are checked.
#' @param restr.fact The restriction factor passed to \code{\link{tclust}}.
#' @param parallel A logical value, to be passed further to \code{tclust()}.
#' @param \ldots Further arguments (as e.g. \code{restr}), passed to \code{\link{tclust}} 
#' @param trace Defines the tracing level, which is set to \code{1} by default. 
#'  Tracing level \code{2} gives additional information on the current iteration.
#'  Tracing level \code{} suppresses all trace messages.
#' @return The function returns an S3 object of type \code{ctlcurves} containing the following components:
#'	\itemize{
#'  \item \code{par} A list containing all the parameters passed to this function 
#'	\item \code{obj}  An array containing the objective functions values of each computed cluster-solution 
#'	\item \code{min.weights} An array containing the minimum cluster weight of each computed cluster-solution 
#'  }
#' @references 
#'    \enc{García}{Garcia}-Escudero, L.A.; Gordaliza, A.; \enc{Matrán}{Matran}, C. and Mayo-Iscar, A. (2011), 
#'    "Exploring the number of groups in robust model-based clustering." \emph{Statistics and Computing}, \bold{21} 
#'    pp. 585-599, <doi:10.1007/s11222-010-9194-z>
#'
#' @examples
#'
#' \donttest{
#'
#'  #--- EXAMPLE 1 ------------------------------------------
#'
#'  sig <- diag (2)
#'  cen <- rep (1, 2)
#'  x <- rbind(MASS::mvrnorm(108, cen * 0,   sig),
#'  	       MASS::mvrnorm(162, cen * 5,   sig * 6 - 2),
#'  	       MASS::mvrnorm(30, cen * 2.5, sig * 50))
#'
#'  ctl <- ctlcurves(x, k = 1:4)
#'  ctl
#'
#'    ##  ctl-curves 
#'  plot(ctl)  ##  --> selecting k = 2, alpha = 0.08
#'
#'    ##  the selected model 
#'  plot(tclust(x, k = 2, alpha = 0.08, restr.fact = 7))
#'
#'  #--- EXAMPLE 2 ------------------------------------------
#'
#'  data(geyser2)
#'  ctl <- ctlcurves(geyser2, k = 1:5)
#'  ctl
#'  
#'    ##  ctl-curves 
#'  plot(ctl)  ##  --> selecting k = 3, alpha = 0.08
#'
#'    ##  the selected model
#'  plot(tclust(geyser2, k = 3, alpha = 0.08, restr.fact = 5))
#'
#'
#'  #--- EXAMPLE 3 ------------------------------------------
#'  
#'  data(swissbank)
#'  ctl <- ctlcurves(swissbank, k = 1:5, alpha = seq (0, 0.3, by = 0.025))
#'  ctl
#'  
#'    ##  ctl-curves 
#'  plot(ctl)  ##  --> selecting k = 2, alpha = 0.1
#'  
#'    ##  the selected model
#'  plot(tclust(swissbank, k = 2, alpha = 0.1, restr.fact = 50))
#'  
#' }
#'
ctlcurves <- function(x, k=1:4, alpha=seq(0, 0.2, len=6), restr.fact=50, parallel=FALSE, trace=1, ...) {

## disabled parameter:
  mah.alpha <- 0.05


  stopifnot (length (restr.fact) == 1)

  if(trace >= 1)
  {
    cat ("Depending on arguments x, k and alpha, ",
         "this function needs some time to compute.\n",
         "(Remove this message by setting \"trace = 0\")\n", sep = "")
     flush.console()
  }

  rs <- array (NA, c (length (k), length (alpha)))
  dimnames (rs) <- list (k = k, alpha = round (alpha, 2))
  ndoubt <- drop <- out <- obj <- min.weights <- unrestr.fact <- rs

#  stab <-
#  stab[,] <- TRUE

  p <- ncol (x)
  n.k <- length (k)
  n.alpha <- length (alpha)
  n <- nrow (x)

  for (i in 1:n.k) {
    for (j in 1:n.alpha)   {
      cur.alpha <- alpha[j]

      if (trace >= 2)
        cat ("k =", k[i], "; alpha =", alpha[j])

      clus <- tclust(x, k=k[i], alpha=alpha[j], restr.fact=restr.fact, parallel=parallel, trace=0, ...)

      out[i, j] <- sum(clus$mah > qchisq(1-mah.alpha, p), na.rm = TRUE)

      drop[i, j] <- FALSE   # clus$warnings$drop | clus$warnings$size | clus$warnings$sizep

      dsc <- DiscrFact(clus)
      ndoubt[i, j] <- sum (dsc$assignfact > dsc$threshold)

      unrestr.fact[i, j] <- clus$unrestr.fact
      obj[i,j] <- clus$obj                      ##  the objective function criterion
      min.weights[i,j] <- min(clus$weights)     ##  weights of the smallest group

      if (trace >= 2)
        cat ("; obj = ", clus$obj, ";min (weights) =", min(clus$weights), "\n")
    }
  }

  par <- list(x=x, k=k, alpha=alpha, mah.alpha=mah.alpha, restr.fact=restr.fact)

  ret <- list (obj=obj, min.weights=min.weights, par=par,
    unrestr.fact=unrestr.fact, ndoubt=ndoubt, out=out, drop=drop)

  class (ret) <- "ctlcurves"
  ret
}

#' @name plot.ctlcurves
#' @title The \code{plot} method for objects of class \code{ctlcurves}
#' @rdname plot.ctlcurves
#' @description The \code{plot} method for class \code{ctlcurves}: This function implements 
#'  a series of plots, which display characteristic values 
#'  of the each model, computed with different values for \code{k} and \code{alpha}.
#' @param x The ctlcurves object to be shown
#' @param what A string indicating which type of plot shall be drawn. See the details section for more information.
#' @param main A character-string containing the title of the plot.
#' @param xlab,ylab,xlim,ylim Arguments passed to plot().
#' @param col A single value or vector of line colors passed to \code{\link[graphics]{lines}}.
#' @param lty A single value or vector of line colors passed to \code{\link[graphics]{lines}}.
#' @param \ldots Arguments to be passed to or from other methods.
#' @details These curves show the values of the trimmed classification (log-)likelihoods 
#'  when altering the trimming proportion \code{alpha} and the number of clusters \code{k}.
#'  The careful examination of these curves provides valuable information for choosing these 
#'  parameters in a clustering problem. For instance, an appropriate \code{k} to be chosen 
#'  is one that we do not observe a clear increase in the trimmed classification likelihood
#'  curve for \code{k} with respect to the \code{k+1} curve for almost all the range of
#'  \code{alpha} values. Moreover, an appropriate choice of parameter \code{alpha} may
#'  be derived by determining where an initial fast increase of the trimmed classification 
#'  likelihood curve stops for the final chosen \code{k}. A more detailed explanation 
#'  can be found in García-Escudero et al. (2011).
#'	  
#'  This function implements a series of plots, which display characteristic values 
#'  of the each model, computed with different values for \code{k} and \code{alpha}.
#   The plot type is selected by setting argument \code{what} to one of the following values:
#'  \describe{
#'    \item{\code{"obj"}}{Objective function values.}
#'    \item{\code{"min.weights"}}{The minimum cluster weight found for each computed model. 
#'      This plot is intended to spot spurious clusters, which in 
#'	     general yield quite small weights.}
#'    \item{\code{"doubtful"}}{The number of "doubtful" decisions identified by \code{\link{DiscrFact}}.}
#'  }
#'
#' @references 
#'    \enc{García}{Garcia}-Escudero, L.A.; Gordaliza, A.; \enc{Matrán}{Matran}, C. and Mayo-Iscar, A. (2011), 
#'    "Exploring the number of groups in robust model-based clustering." \emph{Statistics and Computing}, \bold{21} 
#'    pp. 585-599, <doi:10.1007/s11222-010-9194-z>
#'
#' @examples
#'
#'  #--- EXAMPLE 1 ------------------------------------------
#'
#'  \donttest{
#'  sig <- diag (2)
#'  cen <- rep (1, 2)
#'  x <- rbind(MASS::mvrnorm(108, cen * 0,   sig),
#'  	       MASS::mvrnorm(162, cen * 5,   sig * 6 - 2),
#'  	       MASS::mvrnorm(30, cen * 2.5, sig * 50))
#'
#'  (ctl <- ctlcurves(x, k = 1:4))
#'
#'  plot(ctl)
#'  }
#'
plot.ctlcurves <- function(x, what=c("obj", "min.weights", "doubtful"),
         main, xlab, ylab, xlim, ylim, col, lty=1, ...)
{
  mark.art = FALSE

  what <- match.arg(what[1], eval(formals ()$what))

  n.k <- length (x$par$k)
  idxb.use <- rep (TRUE, n.k)
  if(what == "obj") {
    dat <- x$obj
    set.ylab <- "Objective Function Value"
    set.main <- "CTL-Curves"
  } else if(what == "out") {
    dat <- x$out
    set.ylab <- "Number of Outliers among Clusters"
    set.main <- "Outlier-Curves"
  } else if(what == "doubtful") {
    dat <- x$ndoubt
    set.ylab <- "Number of Doubtful Decision"
    set.main <- "Doubtfull Decision-Curves"
  } else if(what == "min.weights") {
    idxb.use <- x$par$k != 1       # k == 1 -> min.weights == 1 -> we're not interested in that.
    dat <- x$min.weights
    set.ylab <- "Minimum Weigths"
    set.main <- "Minimum Weight-Curves"
  }

  if (missing (ylim)) ylim <- range (dat[idxb.use,])
  if (missing (xlim)) xlim <- range (x$par$alpha)
  if (missing (xlab)) xlab <- expression (alpha)
  if (missing (ylab)) ylab <- set.ylab
  if (missing (main)) main <- set.main

  n.alpha <- length (x$par$alpha)
  lty <- rep (lty, length (x$par$k))

  n.use.k <- sum (idxb.use)

  if (missing (col))
    col <- 1 + (1:n.k)
  else
    col <- rep (col, len = n.k)

  plot (0, type = "n", ylim = ylim, xlim = xlim ,
        main = main, xlab = xlab, ylab = ylab)
  mtext (paste ("Restriction Factor =", x$par$restr.fact), line = 0.25)

  pch <- rownames (dat)

#  idx.grey <- !x$stable
  if (mark.art)
    idx.grey <- x$par$restr.fact < x$unrestr.fact
  else
    idx.grey <- FALSE

  for (j in n.k:1)
  {
    if (!idxb.use[j])
    next

  cur.col <- rep (col[j], len = n.alpha)
  if (any (idx.grey))
      cur.col [idx.grey[j,]] <- 8

#    lines (x$par$alpha, dat[j,], type="b", col = cur.col, lty = lty[j],
#           pch = as.character (pch[j]))
    lines (x$par$alpha, dat[j,], type = "b", col = col[j], lty = lty[j],
           pch = "  ")

    text (x = x$par$alpha, y = dat[j,], col = cur.col, labels = pch[j])

  }

  if (what == "out")
    lines (x$par$alpha, nrow (x$par$x) * (1-x$par$alpha) * x$par$mah.alpha, lty = 2)
}

#' @export

print.ctlcurves <- function (x, ...) {

  cat ("Computed ", length (x$par$k) * length (x$par$alpha),
       " solutions (chosen restr.fact = ", x$par$restr.fact, ").\n\n",
     sep = "")

  idx.ar <- x$par$restr.fact < x$unrestr.fact

  if (any (idx.ar | x$drop))
  {
    uf <- array ("", dim = dim (x$unrestr.fact))
    uf[idx.ar] <- "*"
    uf[x$drop] <- paste (uf[x$drop], "k", sep = "")

    attributes (uf) <- attributes (x$unrestr.fact)

    print (uf, justify = "right", quote = FALSE)

    if (any (idx.ar))
        cat ("\n(*) Identified ", sum (idx.ar), " artificially restricted solutions.", sep = "")
    if (any (x$drop))
        cat ("\n(k) Identified ", sum (x$drop), " solutions with very small/dropped clusters.", sep = "")
  }
  else
    cat ("\nNo artificially restricted solutions or dropped clusters found.")

 cat ("\n")

 invisible(x)
}

##  roxygen2::roxygenise("C:/users/valen/onedrive/myrepo/r/tclust", load_code=roxygen2:::load_installed)

#'
#' TCLUST method for robust clustering
#' 
#' @name tclust
#' @aliases print.tclust
#' @description This function searches for \code{k} (or less) clusters with 
#'  different covariance structures in a data matrix \code{x}. Relative cluster 
#'  scatter can be restricted when \code{restr="eigen"} by constraining the ratio 
#'  between the largest and the smallest of the scatter matrices eigenvalues 
#'  by a constant value \code{restr.fact}. Relative cluster scatters can be also 
#'  restricted with \code{restr="deter"} by constraining the ratio between the 
#'  largest and the smallest of the scatter matrices' determinants. 
#'
#'  For robustifying the estimation, a proportion \code{alpha} of observations is trimmed. 
#'  In particular, the trimmed k-means method is represented by the \code{tclust()} method,
#'  by setting parameters \code{restr.fact=1}, \code{opt="HARD"} and \code{equal.weights=TRUE}. 
#'
#' @param x A matrix or data.frame of dimension n x p, containing the observations (row-wise). 
#' @param k The number of clusters initially searched for.
#' @param alpha The proportion of observations to be trimmed.
#' @param nstart The number of random initializations to be performed.
#' @param niter1 The number of concentration steps to be performed for the nstart initializations.
#' @param niter2 The maximum number of concentration steps to be performed for the 
#'  \code{nkeep} solutions kept for further iteration. The concentration steps are 
#'  stopped, whenever two consecutive steps lead to the same data partition.
#' @param nkeep The number of iterated initializations (after niter1 concentration 
#'  steps) with the best values in the target function that are kept for further iterations
#' @param iter.max (deprecated, use the combination \code{nkeep, niter1 and niter2}) 
#'  The maximum number of concentration steps to be performed.
#'  The concentration steps are stopped, whenever two consecutive steps lead
#'  to the same data partition.
#' @param equal.weights A logical value, specifying whether equal cluster weights 
#'  shall be considered in the concentration and assignment steps.
#' @param restr Restriction type to control relative cluster scatters. 
#'  The default value is \code{restr="eigen"}, so that the maximal ratio between 
#'  the largest and the smallest of the scatter matrices eigenvalues is constrained 
#'  to be smaller then or equal to \code{restr.fact} 
#'  (Garcia-Escudero, Gordaliza, Matran, and Mayo-Iscar, 2008). 
#'  Alternatively, \code{restr="deter"} imposes that the maximal ratio between 
#'  the largest and the smallest of the scatter matrices determinants is smaller 
#'  or equal than \code{restr.fact} (see Garcia-Escudero, Mayo-Iscar and Riani, 2020) 
#'
#' @param restr.fact The constant \code{restr.fact >= 1} constrains the allowed 
#'  differences among group scatters in terms of eigenvalues ratio
#'  (if \code{restr="eigen"}) or determinant ratios (if \code{restr="deter"}). Larger values 
#'  imply larger differences of group scatters, a value of 1 specifies the 
#'  strongest restriction.
#' @param cshape constraint to apply to the shape matrices, \code{cshape >= 1}, 
#'  (see Garcia-Escudero, Mayo-Iscar and Riani, 2020)). 
#'  This options only works if \code{restr=='deter'}. In this case the default 
#'  value is \code{cshape=1e10} to ensure the procedure is (virtually) affine equivariant. 
#'  On the other hand, \code{cshape} values close to 1 would force the clusters to 
#'  be almost spherical (without necessarily the same scatters if \code{restr.fact} 
#'  is strictly greater than 1).
#' @param zero_tol The zero tolerance used. By default set to 1e-16.
#' @param center Optional centering of the data: a function or a vector of length p 
#'  which can optionally be specified for centering x before calculation
#' @param scale Optional scaling of the data: a function or a vector of length p 
#'  which can optionally be specified for scaling x before calculation
#' @param store_x A logical value, specifying whether the data matrix \code{x} shall be 
#'  included in the result object. By default this value is set to \code{TRUE}, because 
#'  some of the plotting functions depend on this information. However, when big data 
#'  matrices are handled, the result object's size can be decreased noticeably 
#'  when setting this parameter to \code{FALSE}.
#' @param parallel A logical value, specifying whether the nstart initializations should be done in parallel.
#' @param n.cores The number of cores to use when paralellizing, only taken into account if \code{parallel=TRUE}.
#' @param opt Define the target function to be optimized. A classification likelihood 
#'  target function is considered if \code{opt="HARD"} and a mixture classification 
#'  likelihood if \code{opt="MIXT"}.
#' @param drop.empty.clust Logical value specifying, whether empty clusters shall be 
#'  omitted in the resulting object. (The result structure does not contain center 
#'  and covariance estimates of empty clusters anymore. Cluster names are reassigned 
#'  such that the first l clusters (l <= k) always have at least one observation.
#' @param trace Defines the tracing level, which is set to 0 by default. Tracing level 1 
#'  gives additional information on the stage of the iterative process.
#'
#' @return The function returns the following values:
#' \itemize{
#'     \item cluster - A numerical vector of size \code{n} containing the cluster assignment 
#'          for each observation. Cluster names are integer numbers from 1 to k, 0 indicates 
#'          trimmed observations. Note that it could be empty clusters with no observations 
#'          when \code{equal.weights=FALSE}.
#'     \item obj - The value of the objective function of the best (returned) solution.
#'     \item NlogL - A value related to the classification log-likelihood of the best (returned) solution. 
#'          If \code{opt=="HARD"}, \code{NlogL = -2*obj}.
#'     \item size - An integer vector of size k, returning the number of observations contained by each cluster.
#'     \item weights - Vector of Cluster weights
#'     \item centers - A matrix of size p x k containing the centers (column-wise) of each cluster. 
#'     \item cov - 	An array of size p x p x k containing the covariance matrices of each cluster. 
#'     \item code - A numerical value indicating if the concentration steps have 
#'          converged for the returned solution (2).
#'     \item posterior - A matrix with k columns that contains the posterior 
#'          probabilities of membership of each observation (row-wise) to the \code{k} 
#'          clusters. This posterior probabilities are 0-1 values in the 
#'          \code{opt="HARD"} case. Trimmed observations have 0 membership probabilities 
#'          to all clusters.
#'      \item{MIXMIX} - BIC which based on the parameters estimated through the mixture log-likelihood and 
#'          the maximized mixture likelihood as goodness of fit measure. This output 
#'          is present only  if \code{opt="MIXT"}.
#'      \item{MIXMIX} - BIC which uses the classification likelihood based on
#'          parameters estimated through the mixture likelihood (In some books 
#'          this quantity is called ICL). This output 
#'          is present only  if \code{opt="MIXT"}.
#'     \item{CLACLA} - BIC which uses the classification likelihood based on
#'          parameters estimated using the classification likelihood. This output 
#'          is present only  if \code{opt="HARD"}.
#'     \item cluster.ini - A matrix with nstart rows and number of columns equal to 
#'          the number of observations and where each row shows the final clustering 
#'          assignments (0 for trimmed observations) obtained after the \code{niter1} 
#'          iteration of the \code{nstart} random initializations.
#'     \item obj.ini - A numerical vector of length \code{nstart} containing the values 
#'          of the target function obtained after the \code{niter1} iteration of the 
#'          \code{nstart} random initializations.
#'     \item x - The input data set.
#'     \item k - The input number of clusters.
#'     \item alpha - The input trimming level.
#' }
#' 
#' @details The procedure allows to deal with robust clustering with an \code{alpha}
#'  proportion of trimming level and searching for \code{k} clusters. We are considering 
#'  classification trimmed likelihood when using \code{opt=”HARD”} so that “hard” or “crisp” 
#'  clustering assignments are done. On the other hand, mixture trimmed likelihood 
#'  are applied when using \code{opt=”MIXT”} so providing a kind of clusters “posterior” 
#'  probabilities for the observations. 

#'  Relative cluster scatter can be restricted when \code{restr="eigen"} by constraining 
#'  the ratio between the largest and the smallest of the scatter matrices eigenvalues 
#'  by a constant value \code{restr.fact}. Setting \code{restr.fact=1}, yields the 
#'  strongest restriction, forcing all clusters to be spherical and equally scattered. 
#'  Relative cluster scatters can be also restricted with \code{restr="deter"} by 
#'  constraining the ratio between the largest and the smallest of the scatter 
#'  matrices' determinants. 
#'
#'  This iterative algorithm performs "concentration steps" to improve the current 
#'  cluster assignments. For approximately obtaining the global optimum, the procedure 
#'  is randomly initialized \code{nstart} times and \code{niter1} concentration steps are performed for 
#'  them. The \code{nkeep} most “promising” iterations, i.e. the \code{nkeep} iterated solutions with 
#'  the initial best values for the target function, are then iterated until convergence 
#'  or until \code{niter2} concentration steps are done. 
#'
#' The parameter \code{restr.fact} defines the cluster scatter matrices restrictions, 
#'  which are applied on all clusters during each concentration step. It restricts 
#'  the ratio between the maximum and minimum eigenvalue of 
#'  all clusters' covariance structures to that parameter. Setting \code{restr.fact=1}, 
#'  yields the strongest restriction, forcing all clusters to be spherical and equally scattered. 
#'
#' Cluster components with similar sizes are favoured when considering \code{equal.weights=TRUE} 
#'  while \code{equal.weights=FALSE} admits possible different prior probabilities for 
#'  the components and it can easily return empty clusters when the number of 
#'  clusters is greater than apparently needed.
#' 
#' @author Javier Crespo Guerrero, Luis Angel Garcia Escudero, Agustin Mayo Iscar.
#' 
#' @references 
#' 
#' Fritz, H.; Garcia-Escudero, L.A.; Mayo-Iscar, A. (2012), "tclust: An R Package 
#'  for a Trimming Approach to Cluster Analysis". Journal of Statistical Software, 
#'  47(12), 1-26. URL http://www.jstatsoft.org/v47/i12/
#' 
#' Garcia-Escudero, L.A.; Gordaliza, A.; Matran, C. and Mayo-Iscar, A. (2008), 
#'  "A General Trimming Approach to Robust Cluster Analysis". Annals of Statistics, 
#'  Vol.36, 1324--1345.  
#'
#' García-Escudero, L. A., Gordaliza, A. and Mayo-Íscar, A. (2014). A constrained 
#'  robust proposal for mixture modeling avoiding spurious solutions. 
#'  Advances in Data Analysis and Classification, 27--43. 
#' 
#' García-Escudero, L. A., and Mayo-Íscar, A. and Riani, M. (2020). Model-based 
#'  clustering with determinant-and-shape constraint. Statistics and Computing, 
#'  30, 1363--1380.] 
#' @export
#'
#' @examples
#' 
#'  \dontshow{
#'      set.seed (0)
#'  }
#'  ##--- EXAMPLE 1 ------------------------------------------
#'  sig <- diag(2)
#'  cen <- rep(1,2)
#'  x <- rbind(MASS::mvrnorm(360, cen * 0,   sig),
#'             MASS::mvrnorm(540, cen * 5,   sig * 6 - 2),
#'             MASS::mvrnorm(100, cen * 2.5, sig * 50))
#'  
#'  ## Two groups and 10\% trimming level
#'  clus <- tclust(x, k = 2, alpha = 0.1, restr.fact = 8)
#'  
#'  plot(clus)
#'  plot(clus, labels = "observation")
#'  plot(clus, labels = "cluster")
#'  
#'  ## Three groups (one of them very scattered) and 0\% trimming level
#'  clus <- tclust(x, k = 3, alpha=0.0, restr.fact = 100)
#'  
#'  plot(clus)
#'  
#'  ##--- EXAMPLE 2 ------------------------------------------
#'  data(geyser2)
#'  (clus <- tclust(geyser2, k = 3, alpha = 0.03))
#'  
#'  plot(clus)
#'  
#' \donttest{
#'
#'  ##--- EXAMPLE 3 ------------------------------------------
#'  data(M5data)
#'  x <- M5data[, 1:2]
#'  
#'  clus.a <- tclust(x, k = 3, alpha = 0.1, restr.fact =  1,
#'                    restr = "eigen", equal.weights = TRUE)
#'  clus.b <- tclust(x, k = 3, alpha = 0.1, restr.fact =  50,
#'                     restr = "eigen", equal.weights = FALSE)
#'  clus.c <- tclust(x, k = 3, alpha = 0.1, restr.fact =  1,
#'                    restr = "deter", equal.weights = TRUE)
#'  clus.d <- tclust(x, k = 3, alpha = 0.1, restr.fact = 50,
#'                    restr = "deter", equal.weights = FALSE)
#'  
#'  pa <- par(mfrow = c (2, 2))
#'  plot(clus.a, main = "(a)")
#'  plot(clus.b, main = "(b)")
#'  plot(clus.c, main = "(c)")
#'  plot(clus.d, main = "(d)")
#'  par(pa)
#'  
#'  ##--- EXAMPLE 4 ------------------------------------------
#'
#'  data (swissbank)
#'  ## Two clusters and 8\% trimming level
#'  (clus <- tclust(swissbank, k = 2, alpha = 0.08, restr.fact = 50))
#'  
#'  ## Pairs plot of the clustering solution
#'  pairs(swissbank, col = clus$cluster + 1)
#'  ## Two coordinates
#'  plot(swissbank[, 4], swissbank[, 6], col = clus$cluster + 1,
#'       xlab = "Distance of the inner frame to lower border",
#'       ylab = "Length of the diagonal")
#'  plot(clus)
#'  
#'  ## Three clusters and 0\% trimming level
#'  clus<- tclust(swissbank, k = 3, alpha = 0.0, restr.fact = 110)
#'  
#'  ## Pairs plot of the clustering solution
#'  pairs(swissbank, col = clus$cluster + 1)
#'  
#'  ## Two coordinates
#'  plot(swissbank[, 4], swissbank[, 6], col = clus$cluster + 1, 
#'        xlab = "Distance of the inner frame to lower border", 
#'        ylab = "Length of the diagonal")
#'  
#'  plot(clus)
#'  
#'  ##--- EXAMPLE 5 ------------------------------------------
#'   data(M5data)
#'   x <- M5data[, 1:2]
#'   
#'   ## Classification trimmed likelihood approach
#'   clus.a <- tclust(x, k = 3, alpha = 0.1, restr.fact =  50,
#'                      opt="HARD", restr = "eigen", equal.weights = FALSE)
#'  ## Mixture trimmed likelihood approach
#'   clus.b <- tclust(x, k = 3, alpha = 0.1, restr.fact =  50,
#'                      opt="MIXT", restr = "eigen", equal.weights = FALSE)
#'  
#'  ## Hard 0-1 cluster assignment (all 0 if trimmed unit)
#'  head(clus.a$posterior)
#'  
#'  ## Posterior probabilities cluster assignment for the
#'  ##  mixture approach (all 0 if trimmed unit)
#'  head(clus.b$posterior)
#'  
#' }
#'

tclust <- function(x, k, alpha=0.05, nstart=500, niter1=3, niter2=20, nkeep=5, iter.max,
                   equal.weights=FALSE, restr=c("eigen", "deter"), restr.fact=12, cshape=1e10, 
                   opt=c("HARD", "MIXT"),
                   center=FALSE, scale=FALSE, store_x=TRUE, 
                   parallel=FALSE, n.cores=-1, 
                   zero_tol=1e-16, drop.empty.clust=TRUE, trace=0)  {
    
    restr <- match.arg(restr)
    opt <- match.arg(opt)
    restrC <- 0
    deterC <- restr == "deter"

    if(!missing(iter.max)) {
        warning("The parameter 'iter.max' is deprecated, please read the help and use the combination of 'niter1', 'niter2', 'nkeep'.")
        niter1 <- iter.max
    }
        
    if(equal.weights && opt == "MIXT") {
        warning("The parameter 'equalweights' cannot be TRUE if mixture model approach is assumed (opt='MIXT')")
        equal.weights <- FALSE
    }

	parlist <- list(k=k, alpha=alpha, nstart=nstart, niter1=niter1, niter2=niter2, nkeep=nkeep, 
        restr=restr, restr.C=restrC, deter.C=deterC, restr.fact=restr.fact, cshape=cshape, opt=opt,
        equal.weights=equal.weights, center=center, scale=scale,
#           fuzzy=fuzzy, m=m, 
        zero_tol=zero_tol, drop.empty.clust=drop.empty.clust, trace=trace, store_x=store_x)
              
    ## Initial checks
    
    if(is.data.frame(x))
        x <- (data.matrix(x))
    if(!is.matrix (x))
        x <- as.matrix(x)
    if(any(is.na(x)))
        stop ("x cannot contain NA")
    if(!is.numeric (x)) 
        stop ("Parameter x: numeric matrix/vector expected")

    parlist$n <- n <- nrow(x)
    parlist$p <- p <- ncol(x)
            
    scaled <- myscale(x, center=center, scale=scale)
    x <- scaled$x
    if(store_x)
        parlist$x <- x     
  
      if (!k >= 1) 
        stop ("Parameter k: must be >= 1")
      if (alpha < 0 || alpha > 1) 
        stop ("Parameter alpha: must be in [0,1]")
      if (niter1 < 0 || as.integer(niter1) != niter1) 
        stop ("Parameter niter1: must be an integer >= 0")
      if (niter2 < 0 || as.integer(niter2) != niter2) 
        stop ("Parameter niter2: must be an integer >= 0")
      if (nkeep < 0 || as.integer(nkeep) != nkeep) 
        stop ("Parameter nkeep: must be an integer >= 0")
      if (nkeep > nstart) 
        stop ("Parameter nkeep: must be <= nstart")
      if(!is.logical(equal.weights))
        stop ("Parameter equal.weights: must be a logical TRUE or FALSE")
      if(!is.logical(parallel))
        stop ("Parameter parallel: must be a logical TRUE or FALSE")
      if (n.cores < -2 || as.integer(n.cores) != n.cores) 
        stop ("Parameter n.cores: must be an integer >= 0, -1 or -2")
      if(zero_tol < 0)
        stop ("Parameter zero_tol: must be >= 0")
      if(trace != 0 & trace !=1)
        stop ("Parameter trace: must be 0 or 1")
  
  ###
  # FIRST STEP: get nstart solutions with niter1 concentration steps
  ###
  if(trace){
    cat(paste("\nPhase 1: obtaining ", nstart, " solutions.\n", sep = ""))
    if(parallel) cat("\n Running parallel initializations.", 
        "\nProgress bar will not display accurate information.\n")
    pb <- txtProgressBar(min = 0, max = nstart, style = 3)
  }
  
  if(!parallel){
    cluster.ini <- vector("list", nstart)   ##	for containing the best values for 
                                            ##      the parameters after several random starts
    obj.ini <- rep(0, nstart)               ## for containing best objective values
    
    for(j in 1:nstart) {
      assig_obj <- tclust_c1(x, k, alpha, restrC=restrC, deterC=deterC, restr.fact, cshape=cshape,
        niter1, opt, equal.weights, zero_tol)                                                                   # niter1 steps!
      cluster.ini[[j]] <- assig_obj$cluster
      obj.ini[j] <- assig_obj$obj
      
      if(trace){
        setTxtProgressBar(pb, j)
      }
    }
  } else {
    
    # Setup parallel cluster
    if(n.cores == -1){
      n.cores <- detectCores()
    } else if (n.cores == -2){
      n.cores <- detectCores() - 1
    }
    parclus <- makeCluster(n.cores)
    registerDoParallel(parclus)
    
    count <- 0
    comb <- function(...) { # Custom combination function for the foreach loop
      count <<- count + length(list(...)) - 1
      setTxtProgressBar(pb, count)
      flush.console()
      c(...) # this can feed into .combine option of foreach
    }
    
    init.results <- foreach(j = 1:nstart,
                            .packages = "tclust",
                            .combine = ifelse(trace, "comb", "c"),
                            .multicombine = TRUE,
                            .inorder = F) %dopar% {
      assig_obj <- tclust_c1(x, k, alpha, restrC=restrC, deterC=deterC, restr.fact, cshape=cshape, 
        niter1, opt, equal.weights, zero_tol)
      
      list(assig_obj$cluster, assig_obj$obj)
    }
    
    stopCluster(parclus)
    
    cluster.ini <- init.results[0:(nstart-1) * 2 + 1]       # Impair positions of cluster.ini
    obj.ini <- unlist(init.results[1:nstart * 2])           # Pair positions of cluster.ini
  }
  
  
  ###
  # SECOND STEP: get nkeep best solutions so far
  ###
  if(trace){
    cat(paste("\n\nPhase 2: obtaining ", nkeep, " best solutions out of the intial ", nstart," solutions.\n", sep = ""))
  }
  best_index  <- order(obj.ini, decreasing = TRUE)[1:nkeep]
  best_assig_list <- cluster.ini[best_index]
  
  ###
  # THIRD STEP: apply niter2 concentration steps to nkeep best solutions, return the best solution
  ###
  if(trace){
    cat(paste("\nPhase 3: applying ", niter2, " concentration steps to each of the ", nkeep," best solutions.\n", sep = ""))
    pb2 <- txtProgressBar(min = 0, max = nkeep, style = 3)
  }
  
    best_iter <- NULL
    best_iter_obj <- -Inf
    
    for(j in 1:nkeep){
        iter <- tclust_c2(x, k, best_assig_list[[j]], alpha, restrC=restrC, deterC=deterC, restr.fact, cshape=cshape,
            niter2, opt, equal.weights, zero_tol=1e-16)
        
        if(iter$obj > best_iter_obj){
            best_iter <- iter
            best_iter_obj <- iter$obj
        }
    
        if(trace){
            setTxtProgressBar(pb2, j)
        }
    }

    if(trace){
        cat("\n\n")
    }
  
    ## Adjust the returned object to be similar to 'tclust': 
    
    ## Handle empty clusters
    if(drop.empty.clust)
        idxuse <- which(best_iter$size > 0)
    else
        idxuse <- 1:k
    idxuse <- idxuse[order(best_iter$size[idxuse], decreasing = TRUE)]
    
    ClusterIDs <- rep(0, k + 1)
    ClusterIDs[idxuse + 1] <- 1:length(idxuse)

    k.real <- length(idxuse)
    
    ##  - A list of values internally used by functions related to tclust objects.
    int <- list(
    	iter.successful=0, iter.converged=0,
    	dim=dim(x))
    
    ## - Transpose the 'centers' matrix
    best_iter$centers <- t(best_iter$centers)
    best_iter$centers <- best_iter$centers[, idxuse, drop = FALSE]
 
    best_iter$cov <- best_iter$cov[, , idxuse, drop = FALSE]
   
    best_iter$cluster <- as.vector(best_iter$cluster)
    best_iter$cluster <- ClusterIDs[best_iter$cluster + 1]
    
    best_iter$size <- as.vector(best_iter$size)
    best_iter$size <- best_iter$size[idxuse]
    best_iter$weights <- as.vector(best_iter$weights)
    best_iter$weights <- best_iter$weights[idxuse]
     
    best_iter$disttom <- NULL       # remove disttom which is used only in tkmeans objects
    
    ret <- c(best_iter, list(cluster.ini=matrix(unlist(cluster.ini), byrow=TRUE, nrow=length(cluster.ini)),
                obj.ini=obj.ini, int=int, par=parlist, k=sum(best_iter$size > 0)))

    ##  - Dimnames of center and cov
    dn.x <- dimnames(x)
    rownames(ret$centers) <- if(is.null(colnames(x))) paste ("X", 1:ncol(x)) else colnames (x)
    colnames(ret$centers) <- paste("C", 1:k.real)
    dimnames(ret$cov) <- dimnames(ret$centers)[c (1, 1, 2)]

	## calculate the "unrestr.fact"
    get.Mm <- if(parlist$deter.C) .get.Mm.det else .get.Mm.eigen
    EV <- sapply(1:ret$k, get.Mm, ret=ret, x=x)
    if(all(is.na(EV)))  ret$unrestr.fact <- 1
    else	            ret$unrestr.fact <- ceiling(max(EV, na.rm=TRUE)/min(EV, na.rm=TRUE))

    ## Back transform centers, cov and x
    cmm <- scaled$scale %*% t (scaled$scale)
    for(i in 1:ret$k) {
        ret$centers[,i] <- ret$centers[,i] * scaled$scale + scaled$center
        ret$cov[,,i] <- ret$cov[,,i] * cmm
    }
    x <- sweep(x, 2L, scaled$scale, `*`, check.margin = FALSE)
    x <- sweep(x, 2L, scaled$center, `+`, check.margin = FALSE)
    if(store_x)
        ret$par$x <- x
    
    ## Calculate mahalanobis distances
    ret$mah <- array (dim = nrow(x))
    ret$mah[!ret$cluster] <- NA

    for(i in 1:ret$k) {
        idx <- ret$cluster == i
        ret$mah[idx] <- mahalanobis(x[idx, , drop=FALSE], center=ret$centers[, i], cov=ret$cov[,, i])
    }

    ## VT::25.09.2024
    ## Compute information criteria
    ic <- .getIC(ret)
    if(opt == "HARD") 
        ret$CLACLA <- ic$CLACLA
    else {
        ret$MIXMIX <- ic$MIXMIX
        ret$MIXCLA <- ic$MIXCLA
    }
    
    class(ret) <- "tclust"    
    return(ret)
}

.get.Mm.eigen <- function(ii, ret, x) {
	idx <- which(ii==ret$cluster)
	if(length(idx) <= 1)
		return(rep(NA, ncol(x)))
	eigen(cov(x[idx, , drop=FALSE]))$value
}

.get.Mm.det <- function (ii, ret, x) {
	idx <- which(ii==ret$cluster)
	if(length(idx) <= 1)
		return(NA)
	det(cov(x[idx, , drop=FALSE]))
}

.getIC <- function(obj) {

    n <- obj$par$n
    p <- obj$par$p
    MIXMIX <- MIXCLA <- CLACLA <- NULL
    h <- floor((1 - obj$par$alpha) * n)
    
    NlogL <- obj$NlogL
    NlogLmixt <- if(obj$par$opt == "HARD") NULL else -2*obj$obj
    npar <- p * obj$k            # p * k
    if(!obj$par$equal.weights)   # if equalweights = false the k-1 mixture proportions parameters must be added
        npar = npar + (obj$k-1)
    
    if(obj$par$restr == "eigen")
        nParam <- npar + 0.5 * p * (p-1) * obj$k + (p * obj$k - 1) * (1 - 1/obj$par$restr.fact) +1
    else
        nParam=   npar + 0.5 * p * (p-1) * obj$k  + (obj$k - 1) * (1 - 1/(obj$par$restr.fact^(1/p))) + 1 + obj$k * (p-1) * (1 - 1/obj$par$cshape)
    
    logh <- log(h)
    if(obj$par$opt == "HARD")
        CLACLA <- NlogL + nParam * logh
    else {
        MIXCLA <- NlogL + nParam * logh
        MIXMIX <- NlogLmixt + nParam * logh
    }

    ##  cat("\nNlogL=", NlogL, " nParam=", nParam, " h=", h, " logh=", logh, "CLALA=", CLACLA, "\n")
    
    list(nParam=nParam, CLACLA=CLACLA, MIXMIX=MIXMIX, MIXCLA=MIXCLA)
}

##  roxygen2::roxygenise("C:/users/valen/onedrive/myrepo/r/tclust", load_code=roxygen2:::load_installed)

#' Plot Method for \code{tclustICsol} Objects
#'
#' The plot method for class \code{tclustICsol}.
#'
#' @name plot.tclustICsol
#' @method plot tclustICsol

#' @description Displays one of the solutions, selected by the argument 'sol'.
#'  The default display is a scatterplot matrix of the data using
#'  colors and symbols of the observations to identify the groups in the 
#'  selected solution. If the argument 'choice' is specified and its length 
#'  is two, a simple scatterplot will be shown.
#'  The function fails, if \code{store.x = FALSE} is specified in 
#'  the \code{tclustICsol()} call because the original data matrix is 
#'  required here.
#'
#' @param x The \code{tclustICsol} object to be displayed
#' @param whichIC A character value which specifies which information criteria 
#'  to use. Fpr the possible values for \code{whichIC} see the help of \code{tclustIC}.
#' @param sol Which solution to display - a number between 1 and the \code{nsol}
#'  argument of \code{tclustICsol}.
#' @param col optional colors to identify the groups. If not specified default 
#'  values will be selected.
#' @param pch optional symbols to identify the groups. If not specified default 
#'  values will be selected.
#' @param main optional title. The default title shows the information criteria, 
#'  the solution number, number of groups and restriction factor c used and 
#'  indication wheather the solution is true or spurios.
#' @param sub1 an optional subtitle. The default subtitle shows the type of the 
#'  displaued solution: 'Best in' and 'Stable in'.
#' @param choice a numeric vector of length between 2 and the number of 
#'  variables in the input data matrix. If missing, a scatterplot matrix with 
#'  all variables will be shown. If two variables are selected, a simple 
#'  scatterplot of the two selected variables is shown.
#' @param \ldots Further (optional) graphical arguments
#' 
#' @examples
#'
#'  #--- EXAMPLE 1 ------------------------------------------
#'  \donttest{
#'  data(geyser2)
#'  (out <- tclustIC(geyser2, whichIC="MIXMIX", alpha=0.1))
#'
#'  ##  Show the first two best solutions using as Information criterion MIXMIX
#'  cat("\nBest solutions using MIXMIX\n")
#'  outsol <- tclust::tclustICsol(out, whichIC="MIXMIX", nsol=2)
#'  plot(outsol)
#'  plot(outsol, choice=c(1,2))
#'  plot(outsol, choice=c(1,2), xlab="XLAB", ylab="YLAB")
#'  }
#'
plot.tclustICsol <- function(x, whichIC, sol=1, col, pch, main, sub1, choice, ...) {

    ## The requested information criterion whichIC should correspond 
    ##  to those of the 'tclustICsol' object 
    if(missing(whichIC)) {
        whichIC <- x$whichIC
        if(whichIC == "ALL")
            whichIC <- "MIXMIX"  
    } else {
        if(x$whichIC == "ALL")
            choices <- c("MIXMIX", "MIXCLA", "CLACLA")
        else if(x$whichIC == "MIXMIX")
            choices <- c("MIXMIX")
        else if(x$whichIC == "MIXCLA")
            choices <- c("MIXCLA")
        else if(x$whichIC == "CLACLA")
            choices <- c("CLACLA")
            
        whichIC <- match.arg(whichIC, choices)
    }

    df <- if(whichIC == "MIXMIX") x$MIXMIXbs else if(whichIC=="CLACLA") x$CLACLAbs else x$MIXCLAbs
    idx <- if(whichIC == "MIXMIX") x$MIXMIXbsIDX else if(whichIC=="CLACLA") x$CLACLAbsIDX else x$MIXCLAbsIDX
    
    ## The selected solution 'sol' should be between 1 and the number of solutions in 
    ##  the input 'tclustICsol' object (i.e. the number of rows of 
    ##  the chosen MIXMIXbs or CLACLAbs or MIXCLAbs matrix. Defaults to 1.
    ## The values of 'k', 'c' and IDX will be chosen according to 'sol'
    if(sol < 1 || sol > nrow(df))
        stop(paste("'sol' must be between 1 and ", nrow$df))
    k <- df[[sol,1]]
    c <- df[[sol,2]]
    id <- idx[,sol]

    ## Select colors and symbols for plotting according to the number of groups
    maxassig <- max(id)     # number of groups
    if(missing(col))
        col <- 1:(k+1)
    else
        col <- rep(col, len=maxassig + 1)  
    
    if(missing(pch))
        pch <- 1:(k+1)
    else
        pch <- rep(pch,  len=maxassig + 1)
    col <- col[id + 1]    
    pch <- pch[id + 1]

    ## Select the type of solution: 'Best in' and 'Stable in'. These will be 
    ##  used to form the subtitle.
    best <- df[[sol, 3]]
    stable <- sort(c(best, df[[sol, 4]]))
    type <- df[sol, 5]
    xbest <- if(length(best) > 1) paste("Best in c=", best[1], "-", best[length(best)], sep="") 
        else if(length(best) == 1) paste("Best in c=", best[1], sep="") else ""
    xstable <- if(length(stable) > 1) paste("Stable in c=", stable[1], "-", stable[length(stable)], sep="") 
        else if(length(stable) == 1) paste("Stable in c=", stable[1], sep="") else ""

    if(missing(main))
        main <- paste(whichIC, " (solution ", sol, ", ", type, "): k=", k, " c=", c, sep="")
    if(missing(sub1))
        sub1 <- paste(xbest, "", xstable)

    ## Chose the type of plot according to the parameter 'choice'. 
    ##  - If missing 'choice', independent of the number of variables 
    ##      a scatterplot matrix will be plotted. Same, if the length of 
    ##      'choice' is greater than 2.
    ##  - If two components are selected a simple scatterplot of the two 
    ##      selected variables will be shown
    ##  Select the corresponding columns from the data matrix.
    data <- x$x
    if(!missing(choice)) {
        if(length(choice) < 2 || length(choice) > ncol(data))
            stop(paste("The length of 'choice' must be greater than 2 and smaller than ", ncol(data)))
         if(any(choice < 0) || any(choice > ncol(data)))
            stop(paste("'chice' must  be between 1 and ", ncol(data)))
        if(length(choice) != length(unique(choice)))
            stop(paste("There are duplicated columns in 'choice'!"))
         
        data <- data[, choice]
    }

    if(missing(choice) || ncol(data) > 2) {
        ## Matrix scatterplot when p > 2
        .plot.tclustICsol.pairs(data, ngroups=k, groups=id, col=col, pch=pch, main=main, sub1=sub1, ...)
    } else
        .plot.tclustICsol.2d(data, ngroups=k, groups=id, col=col, pch=pch, main=main, sub1=sub1, ...)
    
    return(invisible(x))
}

##  Draw pairwise scatter plots for the data set 'x'
##  - upper triangle - scatter plot with classical and robust 0.975-ellipses
##  - histograms on the diagonal
##  - lower triangle - robust (MCD) and classical correlations
##
##  - x     - data
##  - main  - caption of the plot
##
.plot.tclustICsol.pairs <- function(x, ngroups, groups, main="", sub1="", ...){
    
    panel.density <- function(x,...)
    {
        usr <- par("usr"); on.exit(par(usr=usr))
        par(usr = c(usr[1:2], 0, 1.5) )

        args <- list(...)
        if("col" %in% names(args)) {
            col <- sort(unique(args[["col"]]))
            if(is.null(col))
                col <- (1:ngroups) + 1
        } else
            col <- (1:ngroups) + 1

        for(i in 1:ngroups) {
            xx <- x[groups==i]
            tryd <- try(d <- density(xx, na.rm=TRUE, bw="nrd", adjust=1.2), silent=TRUE)

            ## VT::11.08.2022: fix error "Found if() conditions comparing class() to string"
            ##  if(class(tryd) != "try-error")
            if(!is(tryd, "try-error")) {
                d$y <- d$y/max(d$y)
                lines(d, col=col[i])
            }
        }
    }

    panel.boxplot <- function(x,...)
    {
        xr <- range(x, na.rm = TRUE)
        usr <- par("usr"); on.exit(par(usr=usr))

        ff <- 0.1
        dd <- ff * (xr[2] - xr[1])
        xr[1] <- xr[1] - dd
        xr[2] <- xr[2] + dd
        par(usr=c(xr, 0, 1.5))

       
## cat("\nxr=", xr, "\n")
## cat("\nminmax=", c(min(x), max(x)), "\n")
## cat("\nusr=", par("usr"), "\n")

        args <- list(...)
        if("col" %in% names(args)) {
            col <- sort(unique(args[["col"]]))
            if(is.null(col))
                col <- (1:ngroups) + 1
        } else
            col <- (1:ngroups) + 1

##  cat("\nargs=", "col" %in% names(args), ", ngroups=", ngroups, ", col=", col, "\n")

##        if(length(groups == 0) > 0) {   # there are trimmed observations, remove them from the boxplots
##            x <- x[groups > 0]
##            groups <- groups[groups > 0]
##            if(length(col) > length(unique(groups)))
##                col <- col[-1]
##        }

##  cat("\nngroups=", ngroups, ", length(table(groups))=", length(table(groups)), "Classes: ", table(groups), "\n")
        ngroups <- length(table(groups))

        boxplot(x ~ groups, 
              add=TRUE, 
              horizontal=TRUE,
              at=seq(0.2, 0.8, length.out=ngroups), 
              axes=FALSE, 
              border=col, col=0, 
              pars=list(boxwex = 0.15)) # Adjust box thickness
    }

    panel.ellipse <- function(x, y, ...) {
        usr <- par("usr"); on.exit(par(usr=usr))

        xmin <- min(x)
        xmax <- max(x)
        ymin <- min(y)
        ymax <- max(y)

        ff <- 0.1
        xmin <- xmin - ff*(xmax-xmin); #print(ff*(xmax-xmin))
        xmax <- xmax + ff*(xmax-xmin); #print(ff*(xmax-xmin))
        ymin <- ymin - ff*(ymax-ymin); #print(ff*(ymax-ymin))
        ymax <- ymax + ff*(ymax-ymin); #print(ff*(ymax-ymin))
        par(usr = c(xmin, xmax, ymin, ymax))

        points(x,y, ...)
    }

##    cat("\nngroups=", ngroups, ", length(table(groups))=", length(table(groups)), "Classes: ", table(groups), " Not trimmed: ", length(table(groups[groups > 0])), "\n")
##    print(table(groups))

    if(ngroups > length(table(groups[groups > 0]))) {     ## there are empty clusters
        warning("There are empty clusters: ", ngroups," requested ", length(table(groups[groups > 0])), " available!")
    }
    
    pairs(x, 
        lower.panel=panel.ellipse,
        diag.panel=panel.boxplot,
        upper.panel=panel.ellipse,
        labels=names(x),
        ...)

    if(!is.null(main))
        title(main=main, line=3)
  
    if(!is.null(sub1))
        title(sub=sub1)      
}

.plot.tclustICsol.2d <- function(x, ngroups, groups, xlab, ylab, axes=3, xlim, ylim, col=col, pch=pch, main, sub1, ...){

    if(missing(xlab))
        xlab <- colnames(x)[1]
    if(missing(ylab))
        ylab <- colnames(x)[2]
        
    if(missing(xlim))
        xlim <- range(x[, 1])
    
    if(missing(ylim))
        ylim <- range(x[, 2])

    fact <- 0.04
    r <- cbind(xlim, ylim)
    rd <- apply(r, 2, diff)
    usr <- as.numeric (r + (c(-1, 1) %*% t (rd)) * fact)
    
    plot.new()
    par(usr = usr)

    points(x[,1], x[,2], pch=pch, col=col)

    if(!is.null(main))
        title(main=main)
  
    if(!is.null(sub1))
        title(sub=sub1)      

    axis.x <- axes %% 2        ## x axis 1 or 3
    axis.y <- axes >= 2        ## y axis 2 or 3
    
    cex <- par ("cex")
    if (!missing (xlab))
        mtext (side = 1, xlab, line = 1.5 + 1.5 * axis.x, cex = cex)
    if (!missing (ylab))
        mtext (side = 2, ylab, line = 1.5 + 1.5 * axis.y, cex = cex)

    box()
    if(axis.x)
        axis(1)
    if(axis.y)
        axis(2)
}

##  roxygen2::roxygenise("C:/users/valen/onedrive/myrepo/r/tclust", load_code=roxygen2:::load_installed)

#'
#' Performs cluster analysis by calling \code{\link{tclust}} for different
#'  number of groups \code{k} and restriction factors \code{c}
#' 
#' @name tclustIC
#' @aliases print.tclustIC
#' @description Computes the values of BIC (MIXMIX), ICL (MIXCLA) or CLA (CLACLA),
#'  for different values of \code{k} (number of groups) and different values of \code{c}
#'  (restriction factor), for a prespecified level of trimming (the last two letters in the name
#'  stand for 'Information Criterion'). 
#'
#' @param x A matrix or data frame of dimension n x p, containing the observations (row-wise).
#' @param kk an integer vector specifying the number of mixture components (clusters) 
#'  for which the information criteria are be calculated. By default \code{kk=1:5}.
#' @param cc an  vector specifying the values of the restriction factor which have to 
#'  be considered. By default \code{cc=c(1, 2, 4, 8, 16, 32, 64, 128)}.
#' @param whichIC A character value which specifies which information criteria must be computed
#'  for each \code{k} (number of groups) and each value of the restriction factor 
#'  \code{c}. Possible values for \code{whichIC} are:
#'  \itemize{
#'   \item "MIXMIX": a mixture model is fitted and for computing the information criterion
#'      the mixture likelihood is used. This option corresponds to the use of the Bayesian
#'      Information criterion (BIC). In output just the matrix \code{MIXMIX} is given.
#'  \item "MIXCLA": a mixture model is fitted but to compute the information criterion
#'      the classification likelihood is used. This option corresponds to the use of the
#'      Integrated Complete Likelihood (ICL). In the output just the matrix \code{MIXCLA} is given.
#'  \item "CLACLA": everything is based on the classification likelihood. This information
#'      criterion will be called CLA. In the output just the matrix \code{CLACLA} is given.
#'  \item "ALL": both classification and mixture likelihood are used. In this case all
#'      three information criteria CLA, ICL and BIC are computed. In the output all
#'      three matrices \code{MIXMIX}, \code{MIXCLA} and \code{CLACLA} are given.
#'  }
#'
#' @param alpha The proportion of observations to be trimmed.
#' @param parallel A logical value, specifying whether the calls to \code{\link{tclust}} should be done in parallel.
#' @param n.cores The number of cores to use when paralellizing, only taken into account if \code{parallel=TRUE}.
#' @param trace Whether to print intermediate results. Default is \code{trace=FALSE}.
#' @param \ldots Further arguments (as e.g. \code{restr}), passed to \code{\link{tclust}} 
#' @return The functions \code{print()} and \code{summary()} are used to obtain and print a
#'  summary of the results. The function returns an S3 object of type 
#'  \code{tclustIC} containing the following components:
#'	\itemize{
#'  \item call the matched call
#'  \item kk a vector containing the values of \code{k} (number of components) which have been considered.
#'      This vector is identical to the optional argument \code{kk} (default is \code{kk=1:5}.
#'  \item cc a vector containing the values of \code{c} (values of the restriction factor) which
#'      have been considered. This vector is identical to the optional argument \code{cc} (defalt is \code{cc=c(1, 2, 4, 8, 16, 32, 64, 128)}.
#'  \item alpha trimming level
#'  \item whichIC Information criteria used 
#'  \item CLACLA a matrix of size \code{length(kk)-times-length(cc)} containinig the value of
#'      the penalized classification likelihood. This output is present only if \code{whichIC="CLACLA"} or  \code{whichIC="ALL"}.
#'  \item IDXCLA a matrix of lists of size \code{length(kk)-times-length(cc)} containinig the assignment of each unit
#'      using the classification model. This output is present only if \code{whichIC="CLACLA"} or  \code{whichIC="ALL"}.
#'  \item MIXMIX a matrix of size \code{length(kk)-times-length(cc)} containinig the value of
#'      the penalized mixtrue likelihood. This output is present only if \code{whichIC="MIXMIX"} or  \code{whichIC="ALL"}.
#'  \item IDXMIX a matrix of lists of size \code{length(kk)-times-length(cc)} containinig the assignment of each unit
#'      using the classification model. This output is present only if \code{whichIC="MIXMIX"} or  \code{whichIC="ALL"}.
#'  \item MIXCLA a matrix of size \code{length(kk)-times-length(cc)} containinig the value of
#'      the ICL criterion. This output is present only if \code{whichIC="MIXCLA"} or  \code{whichIC="ALL"}.
#'  \item x the input data matrix of size \code{length(n)-times-length(p)} with which 
#'      the Information Criteria were computed.
#'  }
#' @references
#'      Cerioli, A., Garcia-Escudero, L.A., Mayo-Iscar, A. and Riani M. (2017).
#'      Finding the Number of Groups in Model-Based Clustering via Constrained Likelihoods,
#'      \emph{Journal of Computational and Graphical Statistics}, pp. 404-416,
#'      https://doi.org/10.1080/10618600.2017.1390469.
#'
#' @seealso \code{\link{tclust}}
#' @examples
#'
#'  #--- EXAMPLE 1 ------------------------------------------
#'  \donttest{
#'  data(geyser2)
#'  (out <- tclustIC(geyser2, whichIC="MIXMIX", alpha=0.1))
#'  summary(out)
#'  ## Find the smallest value inside the table and write the corresponding
#'  ## values of k (number of groups) and c (restriction factor)
#'  inds <- which(out$MIXMIX == min(out$MIXMIX), arr.ind=TRUE)
#'  vals <- out$MIXMIX[inds]
#'  cat("\nThe smallest value of the IC is ", vals, 
#'      " and takes place for k=", out$kk[inds[1]], " and c=",   
#'      out$cc[inds[2]], "\n")
#'  }
#'
#'  #--- EXAMPLE 2 ------------------------------------------
#'  \donttest{
#'  data(flea)
#'  Y <- as.matrix(flea[, 1:(ncol(flea)-1)])    # select only the numeric variables
#'  rownames(Y) <- 1:nrow(Y)
#'  head(Y)
#'
#'  (out <- tclustIC(Y, whichIC="CLACLA", alpha=0.1))
#'  summary(out)
#'  ## Find the smallest value inside the table and write the corresponding
#'  ## values of k (number of groups) and c (restriction factor)
#'  inds <- which(out$CLACLA == min(out$CLACLA), arr.ind=TRUE)
#'  vals <- out$CLACLA[inds]
#'  cat("\nThe Smallest value of the IC is ", vals, 
#'      " and takes place for k=", out$kk[inds[1]], " and c=",   
#'      out$cc[inds[2]], "\n")
#'  }
#'
#'  #--- EXAMPLE 3 ------------------------------------------
#'  \donttest{
#'  data(swissbank)
#'  (out <- tclustIC(swissbank, whichIC="ALL"))
#'  
#'  plot(out)  ##  --> selecting k=3, c=128
#'  
#'  ##  the selected model
#'  plot(tclust(swissbank, k = 3, alpha = 0.1, restr.fact = 128))
#'  
#'  }
#'
tclustIC <- function(x, kk=1:5, cc=c(1, 2, 4, 8, 16, 32, 64, 128), alpha=0.05, 
    whichIC=c("ALL", "MIXMIX", "MIXCLA", "CLACLA"), parallel=FALSE, n.cores=-1, trace=FALSE, ...) {

    ## Check the dots ...
    args <- list(...)
    if("opt" %in% names(args))
        stop("Argument 'opt' cannot be used with tclustIC()!")
    if("restr.fact" %in% names(args))
        stop("Argument 'restr.fact' cannot be used with tclustIC()!")
    
    rlang::check_dots_used()
    
    whichIC <- match.arg(whichIC)
    if(trace) {
        cat ("Depending on arguments x, kk and cc, ",
             "this function needs some time to compute.\n",
             "(Remove this message by setting \"trace=FALSE\")\n", sep = "")
         flush.console()
    }

    n <- nrow (x)
    p <- ncol (x)
    nkk <- length(kk)
    ncc <- length(cc)
    aa <- cbind(rep(1:nkk, each=ncc), rep(1:ncc, nkk))

    if(trace){
        if(parallel) {
            cat("\n Running in parallel.", 
                "\nProgress bar will not display accurate information.\n")
        }
        pb <- txtProgressBar(min = 0, max = nkk*ncc, style = 3)
    }
    
    if(!parallel) {
        IClist <- vector("list", length=nrow(aa))
        for(ii in 1:nrow(aa)) {
            ik <- aa[ii, 1]; ic <- aa[ii, 2]
            
            ##  if(trace)
            ##      cat("k =", kk[ik], "; c=", cc[ic], "\n")
            if(trace) {
                setTxtProgressBar(pb, ii)
                flush.console()
            }
            
            ax <- list(ik=ik, ic=ic, CLACLA=NULL, MIXMIX=NULL, MIXCLA=NULL, IDXCLA=NULL, IDXMIX=NULL)
            if(whichIC == "CLACLA" || whichIC == "ALL") {        
                out <- tclust(x, k=kk[ik], alpha=alpha, restr.fact=cc[ic], parallel=FALSE, trace=0, opt="HARD", ...)
                ax$CLACLA <- out$CLACLA
                ax$IDXCLA <- out$cluster
            } 
            if(whichIC == "MIXMIX" || whichIC == "MIXCLA" || whichIC == "ALL") {
                out <- tclust(x, k=kk[ik], alpha=alpha, restr.fact=cc[ic], parallel=FALSE, trace=0, opt="MIXT", ...)
                
                if(whichIC == "MIXMIX" || whichIC == "ALL") {
                    ax$MIXMIX <- out$MIXMIX
                    ax$IDXMIX <- out$cluster
                }
                if(whichIC == "MIXCLA" || whichIC == "ALL") {
                    ax$MIXCLA <- out$MIXCLA
                }
            }
            IClist[[ii]] <- ax
            
        }
    } else {
                    
        ## Setup parallel cluster
        if(n.cores == -1) {
            n.cores <- detectCores()
        } else if(n.cores == -2){
            n.cores <- detectCores() - 1
        }
        
        parclus <- makeCluster(n.cores)
        registerDoParallel(parclus)
        

        comb <- function(results, x) { 
            if(trace) {
                setTxtProgressBar(pb, length(results))
                flush.console()
            }
            
            return(append(results, list(x)))
        }
  
        IClist <- foreach(ii = 1:nrow(aa),
                                .packages = "tclust",
                                .combine = "comb",
                                .init = list(),
                                ##.multicombine = TRUE,
                                .inorder = FALSE) %dopar% {

            ik <- aa[ii, 1]; ic <- aa[ii, 2]
          
            ax <- list(ik=ik, ic=ic, CLACLA=NULL, MIXMIX=NULL, MIXCLA=NULL, IDXCLA=NULL, IDXMIX=NULL)
            if(whichIC == "CLACLA" || whichIC == "ALL") {        
                out <- tclust(x, k=kk[ik], alpha=alpha, restr.fact=cc[ic], parallel=FALSE, trace=0, opt="HARD", ...)
                ax$CLACLA <- out$CLACLA
                ax$IDXCLA <- out$cluster
            } 
            if(whichIC == "MIXMIX" || whichIC == "MIXCLA" || whichIC == "ALL") {
                out <- tclust(x, k=kk[ik], alpha=alpha, restr.fact=cc[ic], parallel=FALSE, trace=0, opt="MIXT", ...)
                
                if(whichIC == "MIXMIX" || whichIC == "ALL") {
                    ax$MIXMIX <- out$MIXMIX
                    ax$IDXMIX <- out$cluster
                }
                if(whichIC == "MIXCLA" || whichIC == "ALL") {
                    ax$MIXCLA <- out$MIXCLA
                }
            }
            ax
        }
        stopCluster(parclus)
    }

    if(trace)
        close(pb)
    
    CLACLA <- matrix(NA, nrow=length(kk), ncol=length(cc))
    MIXMIX <- matrix(NA, nrow=length(kk), ncol=length(cc))
    MIXCLA <- matrix(NA, nrow=length(kk), ncol=length(cc))
    cla <- list()
    mix <- list()
    for(ii in 1:(nkk*ncc)){
        ik <- IClist[[ii]]$ik
        ic <- IClist[[ii]]$ic
        
        if(whichIC == "CLACLA" || whichIC == "ALL") {        
            CLACLA[ik, ic] <- IClist[[ii]]$CLACLA
            cla[[(ik-1)*ncc + ic]] <- IClist[[ii]]$IDXCLA
        }
        if(whichIC == "MIXMIX" || whichIC == "MIXCLA" || whichIC == "ALL") {
            if(whichIC == "MIXMIX" || whichIC == "ALL") {
                MIXMIX[ik, ic] <- IClist[[ii]]$MIXMIX
                mix[[(ik-1)*ncc + ic]] <- IClist[[ii]]$IDXMIX
            } 
            if(whichIC == "MIXCLA" || whichIC == "ALL") {
                MIXCLA[ik, ic] <- IClist[[ii]]$MIXCLA
            }
        }
    }   

    ret <- list(call=match.call(), kk=kk, cc=cc, alpha=alpha, whichIC=whichIC, x=x)
    xkk <- paste0("k=", kk)
    xcc <- paste0("c=", cc)

    if(whichIC == "CLACLA" || whichIC == "ALL") {
        ret$CLACLA <- CLACLA
        dimnames(ret$CLACLA) <- list(xkk, xcc)
        ret$IDXCLA <- matrix(cla, nrow=length(kk), byrow=TRUE)
        dimnames(ret$IDXCLA) <- list(xkk, xcc)
    }
    if(whichIC == "MIXMIX" || whichIC == "ALL") {
        ret$MIXMIX <- MIXMIX
        dimnames(ret$MIXMIX) <- list(xkk, xcc)
        ret$IDXMIX <- matrix(mix, nrow=length(kk), byrow=TRUE)
        dimnames(ret$IDXMIX) <- list(xkk, xcc)
    }
    if(whichIC == "MIXCLA" || whichIC == "ALL") {
        ret$MIXCLA <- MIXCLA
        dimnames(ret$MIXCLA) <- list(xkk, xcc)
    }
    
    class(ret) <- "tclustIC"
    
    ret
}

#' @export
print.tclustIC <- function(x, digits = max(3, getOption("digits") - 3), ...) {
    cat("\nCall:\n", deparse(x$call), "\n", sep = "")
    cat("\nInformation criteria for TCLUST:", x$whichIC, "\n", paste0("Trimming = ", x$alpha))
    cat("\nNumber of mixture components (clusters):", x$kk)
    cat("\nvalues of the restriction factor:", x$cc, "\n")
    if(!is.null(x$MIXMIX))
    {
        cat("\n\nPenalized mixture likelihood:\n")
        print.default(format(x$MIXMIX, digits = digits), print.gap = 2, quote = FALSE)
    }
    if(!is.null(x$CLACLA))
    {
        cat("\n\nPenalized classification likelihood:\n")
        print.default(format(x$CLACLA, digits = digits), print.gap = 2, quote = FALSE)
    }
    if(!is.null(x$MIXCLA))
    {
        cat("\n\nICL criterion:\n")
        print.default(format(x$MIXCLA, digits = digits), print.gap = 2, quote = FALSE)
    }

     invisible(x)
}

#' @export
summary.tclustIC <- function (object, ...) {
    ans <- list(tclustobj=object)
    class(ans) <- "summary.tclustIC"
    ans
}

#' @export
print.summary.tclustIC <- function(x, digits = max(3, getOption("digits") - 3), ...) {
    cat("\nCall:\n",
	paste(deparse(x$tclustobj$call), sep = "\n", collapse = "\n"), "\n", sep = "")

    cat("\nInformation criteria for TCLUST:", x$tclustobj$whichIC, "\n", paste0("Trimming = ", x$tclustobj$alpha))
    cat("\nNumber of mixture components (clusters):", x$tclustobj$kk)
    cat("\nValues of the restriction factor:", x$tclustobj$cc, "\n")

    invisible(x)
}

#' @name plot.tclustIC
#' @title The \code{plot} method for objects of class \code{tclustIC}
#' @rdname plot.tclustIC
#' @description The \code{plot} method for class \code{tclustIC}: This function implements 
#'  a series of plots, which display characteristic values 
#'  of each model, computed with different values for \code{k} and \code{c} for a fixed \code{alpha}.
#' @param x The \code{tclustIC} object to be shown
#' @param whichIC A string indicating which information criterion will be used. See the details section for more information.
#' @param cc choose which curves to plot (for which restriction factors \code{c}).
#'  If missing, by default all curves will be printed.
#' @param main A character-string containing the title of the plot.
#' @param xlab,ylab,xlim,ylim Arguments passed to plot().
#' @param col A single value or vector of line colors passed to \code{\link[graphics]{lines}}.
#' @param lty A single value or vector of line types passed to \code{\link[graphics]{lines}}.
#' @param \ldots Arguments to be passed to or from other methods.
#'
#' @references
#'      Cerioli, A., Garcia-Escudero, L.A., Mayo-Iscar, A. and Riani M. (2017).
#'      Finding the Number of Groups in Model-Based Clustering via Constrained Likelihoods,
#'      \emph{Journal of Computational and Graphical Statistics}, pp. 404-416,
#'      https://doi.org/10.1080/10618600.2017.1390469.
#'
#' @examples
#'
#'  \donttest{
#'  sig <- diag (2)
#'  cen <- rep (1, 2)
#'  x <- rbind(MASS::mvrnorm(108, cen * 0,   sig),
#'  	       MASS::mvrnorm(162, cen * 5,   sig * 6 - 2),
#'  	       MASS::mvrnorm(30, cen * 2.5, sig * 50))
#'
#'  (out <- tclustIC(x, whichIC="ALL"))
#'
#'  plot(out)
#'  }
#'
plot.tclustIC <- function(x, whichIC, cc, main, xlab, ylab, xlim, ylim, col, lty, ...)
{
   
    nkk <- length(x$kk)

    if(missing(whichIC)) {
        whichIC <- x$whichIC
        if(whichIC == "ALL")
            whichIC <- "MIXMIX"  
    } else {
        if(x$whichIC == "ALL")
            choices <- c("MIXMIX", "MIXCLA", "CLACLA")
        else if(x$whichIC == "MIXMIX")
            choices <- c("MIXMIX")
        else if(x$whichIC == "MIXCLA")
            choices <- c("MIXCLA")
        else if(x$whichIC == "CLACLA")
            choices <- c("CLACLA")
            
        whichIC <- match.arg(whichIC, choices)
    }
    
    if(whichIC == "CLACLA") {
        dat <- x$CLACLA
        set.ylab <- "CLACLA"
        set.main <- "Penalized mixture likelihood"
    } else if(whichIC == "MIXMIX") {
        dat <- x$MIXMIX
        set.ylab <- "MIXMIX"
        set.main <- "Penalized mixture likelihood"
    } else if(whichIC == "MIXCLA") {
        dat <- x$MIXCLA
        set.ylab <- "MIXCLA"
        set.main <- "ICL criterion"
    } 
  

    if(missing(ylim)) ylim <- range(dat)
    if(missing(xlim)) xlim <- range(x$kk)
    if(missing(xlab)) xlab <- "Number of groups"
    if(missing(ylab)) ylab <- set.ylab
    if(missing(main)) main <- set.main
    
    ncc <- length(x$cc)
    
    if(missing(lty))
        lty <- 1 + (1:ncc)
    else
        lty <- rep(lty, len=ncc)
    
    if(missing(col))
        col <- 1 + (1:ncc)
    else
        col <- rep(col, len=ncc)
    
    pch <- 1 + (1:ncc)
    
    if(missing(cc))
    {
        plot(0, type="n", ylim=ylim, xlim=xlim, main=main, xlab=xlab, ylab=ylab)
        mtext(paste("Trimming =", x$alpha), line = 0.25)
        for(j in ncc:1) {    
            lines(x$kk, dat[,j], type = "b", col=col[j], lty=lty[j], pch=pch[j])   
        }
        
        legend("topright", legend=colnames(dat), lty=lty, pch=pch, col=col, ...)
    } else {    

        if(length(which(x$cc %in% cc)) != length(cc))
            stop(paste("All elements of cc must be in ", paste(x$cc, collapse=","), ".", sep=""))

        xcc <- if(length(cc) == 1) paste("c=", cc) else paste("c=[", paste(cc, collapse=","), "]", sep="")
        plot(0, type="n", ylim=ylim, xlim=xlim, main=xcc, xlab=xlab, ylab=ylab)
        for(j in which(x$cc %in% cc)) {    
            lines(x$kk, dat[,j], type = "b", col=col[j], lty=lty[j], pch=pch[j])   
        }
    }    
    
    invisible(x)
}

##  VT::08.11.2023 This is an improved version of doScale() from robustbase
##
myscale <- function (x, center, scale)
{
    stopifnot(is.numeric(p <- ncol(x)))
    ## MM: follow standard R's	scale.default() as much as possible

	if(missing(center) || is.null(center) || (is.logical(center) && !center)) 
            center <- rep(0, p)
    else {
        if(is.logical(center) && center)
            center <- mean
        
        centerFn <- is.function(center)
        doIt <- if(centerFn) {
            centerName <- deparse(substitute(center)) # "median" typically
    	   center <- apply(x, 2L, center)
    	   TRUE
        } else {
        	if(length(center) == p && is.numeric(center))
        	    TRUE
        	else if(missing(center) || is.null(center)) {
                    center <- 0; FALSE
    	} else
    	    stop(gettextf("'%s' must be a function, numeric vector of length p, or NULL",
                              "center"), domain=NA)
        }
        if(doIt)
    	   x <- sweep(x, 2L, center, `-`, check.margin=FALSE)
    }
    
	if(missing(scale) || is.null(scale) || (is.logical(scale) && !scale)) 
            scale <- rep(1, p)
    else {

        if(is.logical(scale) && scale)
            scale <- sd

        scaleFn <- is.function(scale)
        doIt <- if(scaleFn) {
    	   scale <- apply(x, 2L, scale)
    	   TRUE
        } else {
        	if(length(scale) == p && is.numeric(scale))
        	    TRUE
        	else if(missing(scale) || is.null(scale)) {
        	    scale <- 1
        	    FALSE
        	} else
        	    stop(gettextf("'%s' must be a function, numeric vector of length p, or NULL",
                                  "scale"), domain=NA)
            }
            
            if(doIt) {
                if(any(is.na(scale)) || any(scale < 0))
                    stop("provide better scale; must be all positive")
                if(any(s0 <- scale == 0)) {
        ## FIXME:
        ### Better and easier alternative (and as "FAST MCD"): return "singular cov.matrix"
        ### since scale 0 ==>  more than 50% points are on hyperplane x[,j] == const.
                    ## find scale if there is any variation; otherwise use s := 1
                    S <- if(centerFn && centerName == "median")
                             abs else function(.) abs(. - median(.))
                    non0Q <- function(u) {
                        alph <- c(10:19, 19.75)/20 # not all the way to '1' {=> finite qnorm()}
                        qq <- quantile(S(u), probs=alph, names=FALSE)
                        if(any(pos <- qq != 0)) { ## the first non-0 if there is one
                            i <- which.max(pos)
                            qq[i] / qnorm((alph[i] + 1)/2)
                        } else 1
                    }
                    scale[s0] <- apply(x[,s0, drop=FALSE], 2L, non0Q)
                }
                x <- sweep(x, 2L, scale, `/`, check.margin = FALSE)
            }
        }
    
    ## return
    list(x=x, center=center, scale=scale)
}

#' Simulate contaminated data set for applying TCLUST
#' 
#' @description Simulate 10\% contaminated data set for applying TCLUST
#' 
#' @param n number of observations
#' @param p dimension (p>=2 and p>q)
#' @param k number of clusters (only k=3 and k=6 are allowed!!!)
#' @param type 1 (spherical for rest.fact=1) or 2 (elliptical for rest.fact=9^2)
#' @param balanced 1 (all clusters equal size) or 2 [proportions (25,30,35)\% if k=3 and (12.5,15,17.5,12.5,15,17.5)\% if k=6]
#' 
#' @return a list with the following items
#' \itemize{
#'     \item x - The generated dataset
#'     \item true - The true classification
#' }
#' 
#' @export
#' 
#' @examples 
#' res <- simula.tclust(n=400,k=3,p=8,type=2,balanced=1)
#' plot(res$x,col=res$true+1)
#'
simula.tclust <- function(n,p=4,k=3,type=2,balanced=1){
  
  if (k!=3 & k!=6) stop("Invalid k")
  if (p<2) stop("Invalid p")
  
  if(k==6){nn <- n/2}else{nn <- n}
  
  if (balanced==1){ nn1=nn2=nn3=ceiling(nn*0.3)}else{
              nn1=ceiling(nn*0.25)
              nn2=ceiling(nn*0.3)
              nn3=ceiling(nn*0.35)}
  
  nh <- nn1+nn2+nn3

  Y <- matrix(NA,ncol=p,nrow=n)
  
  if (type==1){
    ### type=1
    if(k==3){
        Y[1:nn1,] <- MASS::mvrnorm(nn1,c(4,4),matrix(c(1,0,0,1),nrow=2))
        Y[(nn1+1):(nn1+nn2),] <- MASS::mvrnorm(nn2,c(-4,4),matrix(c(1,0,0,1),nrow=2))
        Y[(nn1+nn2+1):(nn1+nn2+nn3),] <- MASS::mvrnorm(nn3,c(0,0),matrix(c(1,0,0,1),nrow=2))
        Y[(nh+1):n,] <- matrix(runif(p*(n-nh),-20,20),ncol=p)
        if(p>2){Y[1:n,3:p] <- MASS::mvrnorm(n,rep(0,p-2),diag(p-2))}
      }
      else
      {
      Y[1:nn1,] <- MASS::mvrnorm(nn1,c(4,4),matrix(c(1,0,0,1),nrow=2))
      Y[(nn1+1):(nn1+nn2),] <- MASS::mvrnorm(nn2,c(-4,4),matrix(c(1,0,0,1),nrow=2))
      Y[(nn1+nn2+1):(nn1+nn2+nn3),] <- MASS::mvrnorm(nn3,c(0,0),matrix(c(1,0,0,1),nrow=2))
      Y[(nh+1):(nh+nn1),] <- MASS::mvrnorm(nn1,c(24,0),matrix(c(1,0,0,1),nrow=2))
      Y[(nh+nn1+1):(nh+nn1+nn2),] <- MASS::mvrnorm(nn2,c(16,0),matrix(c(1,0,0,1),nrow=2))
      Y[(nh+nn1+nn2+1):(nh+nn1+nn2+nn3),] <- MASS::mvrnorm(nn3,c(20,4),matrix(c(1,0,0,1),nrow=2))
      Y[(2*nh+1):n,] <- matrix(runif((n-2*nh)*p,-20,40),ncol=p)
      if(p>2){Y[1:n,3:p] <- MASS::mvrnorm(n,rep(0,p-2),diag(p-2))}  
    }
  }
  
  if (type==2){
    ### type=2
    alp1 <- runif(1,0,2*pi)
    u1 <- matrix(c(cos(alp1),sin(alp1),-sin(alp1),cos(alp1)),ncol=2)
    alp2 <- runif(1,0,2*pi)
    u2 <- matrix(c(cos(alp2),sin(alp2),-sin(alp2),cos(alp2)),ncol=2)
  
    if(k==3){
        Y[1:nn1,] <- MASS::mvrnorm(nn1,c(20,20),t(u1)%*%diag(c(1,9^2))%*%u1)
        Y[(nn1+1):(nn1+nn2),] <- MASS::mvrnorm(nn2,c(-20,-20),t(u1)%*%diag(c(9^2,1))%*%u1)
        Y[(nn1+nn2+1):(nn1+nn2+nn3),] <- MASS::mvrnorm(nn3,c(0,0),t(u1)%*%diag(c(3^2,3^2))%*%u1)
        Y[(nn1+nn2+nn3+1):n,] <- matrix(runif(p*(n-nh),-50,50),ncol=p)
        if(p>2){Y[1:n,3:p] <- MASS::mvrnorm(n,rep(0,p-2),diag(p-2))}
      }
      else
      {
        Y[1:nn1,] <- MASS::mvrnorm(nn1,c(20,20),t(u1)%*%diag(c(1,9^2))%*%u1)
        Y[(nn1+1):(nn1+nn2),] <- MASS::mvrnorm(nn2,c(-20,-20),t(u1)%*%diag(c(9^2,1))%*%u1)
        Y[(nn1+nn2+1):(nn1+nn2+nn3),] <- MASS::mvrnorm(nn3,c(0,0),t(u1)%*%diag(c(3^2,3^2))%*%u1)
        Y[(nh+1):(nh+nn1),] <-  MASS::mvrnorm(nn1,c(40,20),t(u2)%*%diag(c(1,9^2))%*%u2)
        Y[(nh+nn1+1):(nh+nn1+nn2),] <- MASS::mvrnorm(nn2,c(0,-20),t(u2)%*%diag(c(9^2,1))%*%u2)
        Y[(nh+nn1+nn2+1):(nh+nn1+nn2+nn3),] <- MASS::mvrnorm(nn3,c(20,0),t(u2)%*%diag(c(3^2,3^2))%*%u2)
        Y[(2*nh+1):n,] <- matrix(runif((n-2*nh)*p,-60,80),ncol=p)
        if(p>2){Y[1:n,3:p] <- MASS::mvrnorm(n,rep(0,p-2),diag(p-2))}  
      }
    }
  
  # true
  true <- rep(0,n)
  if(k==3){true[1:nh] <- rep(1:3,c(nn1,nn2,nn3))}else{true[1:(2*nh)] <- rep(1:6,c(nn1,nn2,nn3,nn1,nn2,nn3))}

  # Return
  return(list(x=Y,true=true))
}

##  roxygen2::roxygenise("C:/users/valen/onedrive/myrepo/rrdev/robClus", load_code=roxygen2:::load_installed)

#' Robust Linear Grouping
#' 
#' @description The function \code{rlg()} searches for clusters around affine subspaces of dimensions given by 
#'  vector \code{d} (the length of that vector is the number of clusters). For instance \code{d=c(1,2)} 
#'  means that we are clustering around a line and a plane. For robustifying the estimation, 
#'  a proportion \code{alpha} of observations is trimmed. In particular, the trimmed k-means 
#'  method is represented by the rlg method, if \code{d=c(0,0,..0)} (a vector of length 
#'  \code{k} with zeroes).
#'
#' @param x A matrix or data.frame of dimension n x p, containing the observations (rowwise).
#' @param d A numeric vector of length equal to the number of clusters to be detected. 
#'  Each component of vector \code{d} indicates the intrinsic dimension of the affine subspace 
#'  where observations on that cluster are going to be clustered. All the elements 
#'  of vector \code{d} should be smaller than the problem dimension minus 1.
#' @param alpha The proportion of observations to be trimmed. 
#' @param nstart The number of random initializations to be performed.
#' @param niter1 The number of concentration steps to be performed for the nstart initializations.
#' @param nkeep The number of iterated initializations (after niter1 concentration 
#'  steps) with the best values in the target function that are kept for further iterations
#' @param niter2 The maximum number of concentration steps to be performed for 
#'  the nkeep solutions kept for further iteration. The concentration steps 
#'  are stopped, whenever two consecutive steps lead to the same data partition.
#' @param scale A robust centering and scaling (using the median and MAD) is done if TRUE.
#' @param parallel A logical value, specifying whether the nstart initializations should be done in parallel.
#' @param n.cores The number of cores to use when paralellizing, only taken into account if parallel=T.
#' @param trace Defines the tracing level, which is set to 0 by default. Tracing level 1 gives additional information on the stage of the iterative process.
#'
#' @return Returns an object of class \code{rlg} which is basically a list with the following elements:
#' \itemize{
#'     \item centers - A matrix of size p x k containing the location vectors (column-wise) of each cluster. 
#'     \item U - A list with k elements where each element is p x d_j matrix whose d_j columns are unitary and orthogonal vectors generating the affine subspace (after subtracting the corresponding cluster’s location parameter in centers). d_j is the intrinsic dimension of the affine subspace approximation in the j-th cluster, i.e., the elements of vector d.
#'     \item cluster - A numerical vector of size n containing the cluster assignment 
#'          for each observation. Cluster names are integer numbers from 1 to k, 
#'          0 indicates trimmed observations.
#'     \item obj - The value of the objective function of the best (returned) solution.
#'     \item cluster.ini - A matrix with nstart rows and number of columns equal to the number of observations and where each row shows the final clustering assignments (0 for trimmed observations) obtained after the niter1 iteration of the nstart random initializations.
#'     \item obj.ini -A numerical vector of length nstart containing the values of the target function obtained after the niter1 iteration of the nstart random initializations.
#'     \item x - The input data set. 
#'     \item dimensions - The input d vector with the intrinsic dimensions. The number of clusters is the length of that vector. 
#'     \item alpha - The input trimming level.
#' }
#' 
#' @details The procedure allows to deal with robust clustering around affine subspaces 
#'    with an alpha proportion of trimming level by minimizing the trimmed sums of squared 
#'    orthogonal residuals. Each component of vector \code{d} indicates the intrinsic dimension of 
#'    the affine subspace where observations on that cluster are going to be clustered. 
#'    Therefore a component equal to 0 on that vector implies clustering around centres, 
#'    equal to 1 around lines, equal to 2 around planes and so on. The procedure so 
#'    allows simultaneous clustering and dimensionality reduction. 
#'
#' This iterative algorithm performs "concentration steps" to improve the current 
#'  cluster assignments. For approximately obtaining the global optimum, the procedure 
#'  is randomly initialized \code{nstart} times and \code{niter1} concentration steps are performed 
#'  for them. The \code{nkeep} most “promising” iterations, i.e. the \code{nkeep} iterated solutions 
#'  with the initial best values for the target function, are then iterated until 
#'  convergence or until \code{niter2} concentration steps are done.
#' 
#' @author Javier Crespo Guerrero, Jesús Fernández Iglesias, Luis Angel Garcia Escudero, Agustin Mayo Iscar.
#' 
#' @references
#' 
#' García‐Escudero, L. A., Gordaliza, A., San Martin, R., Van Aelst, S., & Zamar, R. (2009). 
#'  Robust linear clustering. Journal of the Royal Statistical Society: 
#'  Series B (Statistical Methodology), 71, 301-318. 
#' 
#' @export
#'
#' @examples
#' ##--- EXAMPLE 1 ------------------------------------------
#' data (LG5data)
#' x <- LG5data[, 1:10]
#' clus <- rlg(x, d = c(2,2,2), alpha=0.1)
#' plot(x, col=clus$cluster+1)
#' plot(clus, which="eigenvalues") 
#' plot(clus, which="scores") 
#'
#' ##--- EXAMPLE 2 ------------------------------------------
#'  data (pine) 
#'  clus <- rlg(pine, d = c(1,1,1), alpha=0.035)
#'  plot(pine, col=clus$cluster+1)
#'  


rlg <- function(x, d, alpha=0.05, nstart=500, niter1=3, niter2=20, nkeep=5, scale=FALSE, 
    parallel=FALSE, n.cores=-1, trace=FALSE){
  x <- as.matrix(x)
  if(is.null(nstart)){nstart <- sum(d)*40}
  
  if(scale){
    standard <- function(x){(x-median(x,na.rm=TRUE))/mad(x,na.rm=TRUE)}
    x <- apply(x,2,standard)
  }
  
  clusters<-matrix(nrow = nstart, ncol = dim(x)[1])
  ecms<-rep(0,nstart)
  
  if(trace){
    cat(paste("\nPhase 1: obtaining ", nstart, " solutions.\n", sep = ""))
    if(parallel) cat(paste("\n Parallelizing initializations using ", n.cores, 
        " cores. Progress bar will not display accurate information.\n", sep = ""))
    pb <- txtProgressBar(min = 0, max = nstart, style = 3)
  }
  
  if(!parallel){
    for(inicializacion in 1:nstart){
      fit<-rlg_c1(x,d,alpha,niter1=niter1)
      clusters[inicializacion,]<-fit$cluster
      ecms[inicializacion]<-fit$obj
      
      if(trace){
        setTxtProgressBar(pb, inicializacion)
      }
    }
  } else {
    # Setup parallel cluster
    if(n.cores == -1){
      n.cores <- detectCores()
    } else if (n.cores == -2){
      n.cores <- detectCores() - 1
    }
    parclus <- makeCluster(n.cores)
    registerDoParallel(parclus)
    
    count <- 0
    comb <- function(...) { # Custom combination function for the foreach loop
      count <<- count + length(list(...)) - 1
      setTxtProgressBar(pb, count)
      flush.console()
      c(...) # this can feed into .combine option of foreach
    }
    
    init.results <- foreach(j = 1:nstart,
                            .packages = "tclust",
                            .combine = ifelse(trace, "comb", "c"),
                            .multicombine = T,
                            .inorder = F) %dopar% {
                              fit <- rlg_c1(x,d,alpha,niter1=niter1)
                              list(fit$cluster, fit$obj)
                            }
    stopCluster(parclus)
    
    clusters <- init.results[0:(nstart-1) * 2 + 1] # Impair positions of cluster.ini
    clusters <- do.call(rbind, clusters) # Turn list into matrix
    ecms <- unlist(init.results[1:nstart * 2]) # Pair positions of cluster.ini
  }
  
  if(trace){
    cat(paste("\n\nPhase 2: obtaining ", nkeep, " best solutions out of the intial ", 
        nstart," solutions.\n", sep = ""))
  }
  num<-min(nkeep,length(na.omit(ecms)))
  ecms_ord<-sort(ecms,decreasing = FALSE)
  candidatos<-ecms_ord[1:num]
  
  grupos_optimos<-matrix(nrow = 0, ncol = dim(x)[1])
  minecms<-c()
  
  for(i in 1:num){
    for(j in 1:length(ecms)){
      if(!is.na(ecms[j])){
        if(candidatos[i] == ecms[j]){
          if(sum(unique(clusters[j,]) != 0) == length(d)){
            grupos_optimos<-rbind(grupos_optimos,clusters[j,])
            minecms<-c(minecms,ecms[j])
          }
          break
        }
      }
    }
  }
  
  n_optimos<-length(minecms)
  resultado_ecm<-c()
  resultado_grupo<-matrix(nrow = 0, ncol = dim(x)[1])
  
  if(trace){
    cat(paste("\nPhase 3: applying ", niter2, " concentration steps to each of the ", 
        n_optimos," best solutions.\n", sep = ""))
    pb2 <- txtProgressBar(min = 0, max = nkeep, style = 3)
  }
  
  if(n_optimos == 1){
    fit<-rlg_c2(x,d,grupos_optimos,alpha,niter2=niter2)
    
    if(trace){
      setTxtProgressBar(pb2, n_optimos)
    }
    
    grupos <- fit$cluster
    optimo <- fit$obj
    centers <- matrix(NA,nrow=dim(x)[2],ncol=length(d))
    u <- vector("list", length(d))
    
    for (k in 1:length(d)){
      nk <-  dim(x[grupos==k,])[1]
      centers[,k] <- apply(x[grupos==k,],2,mean)
      u[[k]] <- svd(t( x[grupos==k,]-rep(1,nk)%*%t(centers[,k])),nu=d[k])$u
    }
    
    ret <- list(obj = optimo, cluster = grupos , cluster.ini = clusters, obj.ini = ecms , dimensions = d , centers=centers , U = u, x=x, alpha=alpha)
    
    
    return(ret)
  }else{
    for(omega in 1:n_optimos){
      fit<-rlg_c2(x,d,grupos_optimos[omega,],alpha,niter2=niter2)
      resultado_grupo<-rbind(resultado_grupo,fit$cluster)
      resultado_ecm<-c(resultado_ecm,fit$obj)
      
      if(trace){
        setTxtProgressBar(pb2, omega)
      }
    }
  }
  if(trace){
    cat("\n\n")
  }
  optimo<-min(resultado_ecm)
  flag<-which.min(resultado_ecm)
  grupos<-resultado_grupo[flag,]
  
  centers <- matrix(NA,nrow=dim(x)[2],ncol=length(d))
  u <- vector("list", length(d))
  for (k in 1:length(d)){
    nk <-  dim(x[grupos==k,])[1]
    centers[,k] <- apply(x[grupos==k,],2,mean)
    #u[[k]] <- princomp(x[grupos==k,])$loadings[,1:d[k],drop=F]
    u[[k]] <- svd(t( x[grupos==k,]-rep(1,nk)%*%t(centers[,k])),nu=d[k])$u
  }
  ret <- list(obj = optimo, cluster = grupos , cluster.ini = clusters, obj.ini = ecms , dimensions = d , centers=centers , U = u, x=x, alpha=alpha)
  class(ret) <- "rlg"
  
  return(ret)
}

\documentclass[nojss,shortnames]{jss}

\usepackage{amsfonts}
\usepackage{graphicx}
\usepackage{amsmath}
\usepackage{natbib}
\usepackage{float}
\usepackage{color}
%\input colornam.sty

%\VignetteIndexEntry{A Trimming Approach to Cluster Analysis}
%\VignetteDepends{tclust}
%\VignetteKeywords{Model-based clustering, trimming, heterogeneous clusters}
%\VignettePackage{tclust}

\newcommand{\rounding}[1]{\lceil#1\rceil}   %% new rounding symbol



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% declarations for jss.cls %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% almost as usual
\author{Heinrich Fritz\\Department of Statistics\\ and Probability Theory\\ Vienna University of Technology \And
        Luis A. Garc\'{\i}a-Escudero\\Departamento de Estad\'{\i}stica\\ e Investigaci\'{o}n Operativa\\ Universidad de Valladolid \And
        Agust\'{\i}n Mayo-Iscar\\Departamento de Estad\'{\i}stica\\ e Investigaci\'{o}n Operativa\\ Universidad de Valladolid }
\title{\pkg{tclust}: An \proglang{R} Package for a Trimming Approach to Cluster Analysis }

%% for pretty printing and a nice hypersummary also set:
\Plainauthor{Heinrich Fritz, Luis A. Garc\'{\i}a-Escudero, Agust\'{\i}n Mayo-Iscar}
\Plaintitle{tclust: An R Package for a Trimming Approach to Cluster Analysis}
%\Shorttitle{\pkg{tclust}: An \proglang{R} package for a trimming approach to cluster analysis} %% a short title (if necessary) % this is optional. no use if not different from "real" title

%% an abstract and keywords
\Abstract{This introduction to the \proglang{R} package \pkg{tclust} is a 
significantly modified version of \cite{FriG12}, published in the
\emph{Journal of Statistical Software}. The most essential changes came after 
releasing version \code{2-0.0} of the package.

Outlying data can heavily influence standard clustering
methods. At the same time, clustering principles can be useful when
robustifying statistical procedures. These two reasons motivate the
development of feasible robust model-based clustering approaches.
With this in mind, an R package for performing non-hierarchical
robust clustering, called \pkg{tclust}, is presented here. Instead
of trying to ``fit'' noisy data, a proportion $\alpha$ of the most
outlying observations is trimmed. The \pkg{tclust} package
efficiently handles different cluster scatter constraints. Graphical
exploratory tools are also provided to help the user make sensible
choices for the trimming proportion as well as the number of
clusters to search for.} \Keywords{Model-based clustering, trimming,
heterogeneous clusters}
\Plainkeywords{Model-based clustering, trimming, heterogeneous clusters} %% without formatting
%% at least one keyword must be supplied

%% publication information
%% NOTE: Typically, this can be left commented and will be filled out by the technical editor
%% \Volume{13}
%% \Issue{9}
%% \Month{September}
%% \Year{2004}
%% \Submitdate{2004-09-29}
%% \Acceptdate{2004-09-29}

%% The address of (at least) one author should be given
%% in the following format:
\Address{
  Luis A. Garc\'{\i}a-Escudero\\
  Departamento de Estad\'{\i}stica e Investigaci\'{o}n Operativa. Universidad de Valladolid\\
  Prado de la Magdalena s/n, 47005 Valladolid, SPAIN\\
  E-mail: \email{lagarcia@eio.uva.es}\\
  %URL: \url{http://www.eio.uva.es/infor/personas/langel.html}
}

%% It is also possible to add a telephone and fax number
%% before the e-mail in the following format:
%% Telephone: +43/1/31336-5053
%% Fax: +43/1/31336-734

%% for those who use Sweave please include the following line (with % symbols):
%% need no \usepackage{Sweave.sty}

%% end of declarations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Note that you should use the \pkg{}, \proglang{} and \code{} commands.
\begin{document}

\section{Introduction to robust clustering and tclust}\label{sec4:int}

Methods for cluster analysis attempt to detect homogeneous clusters
with large heterogeneity among them. As happens with other
(non-robust) statistical procedures, clustering methods may be
heavily influenced by even a small fraction of outlying data. For
instance, two or more clusters might be joined artificially, due to
outlying observations, or ``spurious'' non-informative clusters may
be composed of only a few outlying observations \citep[see,
e.g.,][]{GarG99,GarG10}. Therefore, the application of robust methods
in this context is very advisable, especially in fully automatic
clustering (unsupervised learning) problems. Certain relations
between cluster analysis and robust methods
\citep{RocW02,HarR04,GarG03,WooR04} also motivate interest in robust
clustering techniques. For example, robust clustering techniques can
be used to handle ``clusters'' of highly concentrated outliers which
are especially dangerous in (non-robust) estimation. \cite{GarG10}
provides a recent survey of robust clustering methods.

The \pkg{tclust} package for the \proglang{R} environment for
statistical computing \citep{R} implements different robust
non-hierarchical clustering algorithms where trimming plays a key
role. This package is available at
\texttt{http://CRAN.R-project.org/package=tclust}. When trimming
allows the removal of a fraction $\alpha$ of the ``most outlying''
data, the strong influence of outlying observations can be avoided.
This trimming approach to clustering has been introduced in
\cite{CueG97}, \cite{GalM02}, \cite{GalR05} and \cite{GarG08}.
Trimming also serves to identify potentially interesting anomalous
observations.

Trimming is not a new concept in statistics. For instance, the
trimmed mean for one-dimensional data removes a proportion
$\alpha/2$ each of the largest and smallest observations before
computing the mean. However, it is not straightforward to extend
this philosophy to cluster analysis, because most of these problems
are of multivariate nature. Moreover, it is often the case that
``bridge points'' lying between clusters ought to be trimmed. Instead
of forcing the statistician to define the regions to be trimmed in
advance, the procedures implemented in \pkg{tclust} take the whole
data structure into account in order to decide which parts of the
sample should be discarded. By considering this type of trimming,
these procedures are even able to trim outlying bridge points. The
``self-trimming'' philosophy behind these procedures is exactly the
same as adopted by some well-known high breakdown-point methods
\citep[see, e.g.,][]{RouL87}.

As a first example of this trimming approach, let us consider the
trimmed $k$-means method introduced in \cite{CueG97}. The function
\code{tkmeans} from the \pkg{tclust} package implements this method.
In the following example, this function is applied to a bivariate
data set based on the Old Faithful geyser called \code{geyser2} that
accompanies the \pkg{tclust} package. The code given below creates
Figure~\ref{fig4:f1}.
<<label=intro, echo=false, results=hide>>=

options(prompt = "R> ", continue = "+  ", width = 70, useFancyQuotes = FALSE)

## standard margins
mmar <- c(5.1, 4.1, 4.1, 1.1)
og <- grey(0)   ## color for outliers
@

<<label=fig-1, fig=TRUE, include=FALSE, width=7.14, height=7.8>>=
library(tclust)
set.seed(100)
data(geyser2)
clus <- tkmeans(geyser2, k = 3, alpha=0.03)
plot(clus, col = c(og, 2:4), tol.lwd = 1, tol.lty = 2)
@

In the data set \code{geyser2}, we are searching for $k=3$ clusters
and a proportion $\alpha=0.03$ of the data is trimmed. The
clustering results are shown in Figure~\ref{fig4:f1}. Among this $3\%$ of
trimmed data, we can see 6 anomalous ``short followed by short''
eruptions lengths. Notice that an observation situated between the
clusters is also trimmed.

\begin{figure}[t!]
\centering
\includegraphics{tclust-fig-1}
\caption{Trimmed $k$-means results with $k=3$ and $\alpha=0.03$ for
the bivariate Old Faithful Geyser data. Trimmed observations are
denoted by the symbol ``$\circ$'' (a convention followed in all the
figures in this work).} \label{fig4:f1}
\end{figure}

The package presented here adopts a ``crisp'' clustering approach,
meaning that each observation is either trimmed or fully assigned to
a cluster. In comparison, mixture approaches estimate a cluster
pertinence probability for each observation. Robust mixture
alternatives have also been proposed where noisy data is tried to be
fitted through additional mixture components. For instance, package
\pkg{mclust} \citep{FraR12, BanR93,FraR98} and the Fortran program
\pkg{emmix} \citep{EMMIX99, LacP00} implement such robust mixture fitting
approaches. Mixture fitting results can be easily converted into a
``crisp'' clustering result by converting the cluster pertinence
probabilities into 0-1 probabilities. Contrary to these mixture
fitting approaches, the procedures implemented in the \pkg{tclust}
package simply remove outlying observations and do not intend to fit
them at all. Package \pkg{tlemix} \citep[see][]{NeyF07, NeyF12} also
implements a closely related trimming approach. As described in
Section~\ref{sec4:cons}, the \pkg{tclust} package focuses on
offering adequate cluster scatter matrix constraints leading to a
wide range of clustering procedures depending on the chosen
constraint, and avoiding the occurrence of spurious non-interesting
clusters.

The outline of the paper is as follows: In Section~\ref{sec4:tri} we
briefly review the so-called ``spurious outliers'' model and show how
to derive two different clustering criteria from it. Different
constraints on the cluster scatter matrices and their implementation
in the \pkg{tclust} package are addressed in Section~\ref{sec4:cons}.
Section~\ref{sec4:num} presents the numerical
output returned by this package. Section~\ref{sec4:alg} provides
some brief comments concerning the algorithms implemented, and a
comparison of \pkg{tclust} and several other robust clustering
approaches are given in Section~\ref{sec4:comp}.
Section~\ref{sec4:kal} shows some graphical outputs that help advise the
choice of the number of clusters and the trimming proportion. Other
useful plots summarizing the robust clustering results are shown in
Section~\ref{sec4:gra}. Finally, Section~\ref{sec4:exa} applies the
\pkg{tclust} package to a well-know real data set.


\section{Trimming and the spurious outliers model}\label{sec4:tri}
\cite{GalM02} and \cite{GalR05} propose the ``spurious outliers
model'' as a probabilistic framework for robust crisp clustering. Let
$f(\cdot;\mu,\Sigma)$ denote the probability density function of the
$p$-variate normal distribution with mean $\mu$ and covariance
matrix $\Sigma$. The ``spurious-outlier model'' is defined through
``likelihoods'' like
\begin{equation}\label{eqn4:spu}
  \bigg[\prod_{j=1}^{k}\prod_{i\in R_j}f(x_i;\mu_j,\Sigma_j)\bigg]\bigg[\prod_{i \in R_0}g_i(x_i)\bigg]
\end{equation}
with $\{R_0,...,R_k\}$ being a partition of the set of indices
$\{1,2,...,n\}$ such that $\#R_0=\rounding{n\alpha}$. $R_0$ are the
indices of the ``non-regular'' observations  generated by other (not
necessarily normal) probability density functions $g_i$.
``Non-regular'' observations can be clearly considered as ``outliers''
if we assume certain sensible assumptions for the $g_i$ \citep[see
details in][]{GalM02,GalR05}. Under these assumptions, the search of
a partition $\{R_0,...,R_k\}$ with $\#R_0=\rounding{n\alpha}$,
vectors $\mu_j$ and positive definite matrices $\Sigma_j$ maximizing
(\ref{eqn4:spu}) can be simplified to the same search (of a
partition, vectors and positive definite matrices) by just
maximizing
\begin{equation}\label{eqn4:e3}
 \sum_{j=1}^{k}\sum_{i\in R_j}\log f(x_i;\mu_j,\Sigma_j).
\end{equation}
Notice that observations $x_i$ with $i\in R_0$ are not taken into
account in (\ref{eqn4:e3}). Maximizing (\ref{eqn4:e3}) with $k=1$ yields the
Minimum Covariance Determinant (MCD) estimator \citep{RouP85}.

Unfortunately, the direct maximization of (\ref{eqn4:e3}) is not a
well-defined problem when $k>1$. It is easy to see that (\ref{eqn4:e3})
is unbounded without any constraint on the cluster scatter matrices
$\Sigma_j$. The \code{tclust} function from the \pkg{tclust} package
approximately maximizes (\ref{eqn4:e3}) under different cluster scatter
matrix constraints which will be shown in Section~\ref{sec4:cons}.

The maximization of (\ref{eqn4:e3}) implicitly assumes equal cluster
weights. In other words, we are ideally searching for clusters with
equal sizes. The function \code{tclust} provides this option by
setting the argument \code{equal.weights = TRUE}. The use of this
option does not guarantee that all resulting clusters exactly
contain the same number of observations, but the method hence
prefers this type of solutions.

Alternatively, different cluster sizes or cluster weights can be
considered by searching for a partition $\{R_0,...,R_k\}$ (with
$\#R_0=\rounding{n\alpha}$), vectors $\mu_j$, positive definite
matrices $\Sigma_j$ and weights $\pi_j\in[0,1]$ maximizing
\begin{equation}\label{eqn4:e4}
 \sum_{j=1}^{k}\sum_{i\in R_j}(\log \pi_j + \log f(x_i;\mu_j,\Sigma_j)).
\end{equation}
The (default) option \code{equal.weights = FALSE} is used in this
case. Again, the scatter matrices also have to be constrained such
that the maximization of (\ref{eqn4:e4}) becomes a well-defined
problem. Note that (\ref{eqn4:e4}) simplifies to (\ref{eqn4:e3})
when assuming \code{equal.weights = TRUE} and all weights are
equally set to $\pi_j=1/k$.

\newpage
\section{Constraints on the cluster scatter matrices}\label{sec4:cons}

As already mentioned, the function \code{tclust} implements
different algorithms aimed at approximately maximizing
(\ref{eqn4:e3}) and (\ref{eqn4:e4}) under different types of
constraints which can be applied on the scatter matrices $\Sigma_j$.
The type of constraint is specified by the argument \code{restr} of
the \code{tclust} function. Table~\ref{tab4:app.ovv} gives an
overview of the different clustering approaches implemented by the
\code{tclust} function depending on the chosen type of constraint.

\begin{table}[t!]
  \centering
\begin{tabular}{||l|l|l||}
  \hline\hline
  &\code{equal.weights = TRUE}&\code{equal.weights = FALSE}\\ \hline
 \code{restr = "eigen"} & \hspace{-1.8mm}$\begin{array}{l} k\textit{-means} \\ \text{\cite{CueG97}} \end{array}$ & \cite{GarG08} \\\hline
 \code{restr = "deter"} & \text{\cite{GalM02}} & \text{This work}\\\hline
 \hline\hline
\end{tabular}
  \caption{\label{tab4:app.ovv}Clustering methods handled by \pkg{tclust}. Names in cursive letters are untrimmed ($\alpha=0$) methods.}
\end{table}

Imposing constraints is compulsory because maximizing
(\ref{eqn4:e3}) or (\ref{eqn4:e4}) without any restriction is not a
well-defined problem. Notice that an almost degenerated scatter
matrix $\Sigma_j$ would cause trimmed log-likelihoods
(\ref{eqn4:e3}) and (\ref{eqn4:e4}) to tend to infinity. This issue
can cause a (robust) clustering algorithm of this type to end up
finding ``spurious'' clusters almost lying in lower-dimensional
subspaces. Moreover, the resulting clustering solutions might
heavily depend on the chosen constraint. The strength of the
constraint is controlled by the argument \code{restr.fact} $\geq 1$
in the \code{tclust} function. The larger \code{restr.fact} is
chosen, the looser is the restriction on the scatter matrices,
allowing for more heterogeneity among the clusters. On the contrary,
small values of \code{restr.fact} close to 1 imply very ``equally
scattered'' clusters. This idea of constraining cluster scatters to
avoid spurious solutions goes back to \cite{HatR85}, who proposed it
in mixture fitting problems.

Also arising from the spurious outlier model, other types of
constraints have recently been introduced by \cite{GalR09,GalR10}.
These (closely related) constraints also serve to avoid degeneracy
of trimmed likelihoods but they are not implemented in the current
version of the \pkg{tclust} package.

\subsection{Constraints on the eigenvalues}\label{sec4:consteig}
Based on the eigenvalues of the cluster scatter matrices, a scatter
similarity constraint may be defined. With $\lambda_l(\Sigma_j)$ as
the eigenvalues of the cluster scatter matrices $\Sigma_j$ and
\begin{equation}\label{eqn4:e5}
    M_n=\max_{j=1,...,k}\,\max_{l=1,...,p}\lambda_l(\Sigma_j)\text{
and } m_n=\min_{j=1,...,k}\,\min_{l=1,...,p}\lambda_l(\Sigma_j)
\end{equation}
as the maximum and minimum eigenvalues, the restriction
\texttt{restr = "eigen"} constrains the ratio $M_n/m_n$  to be
smaller or equal than a fixed value \code{restr.fact} $\geq 1$. A
theoretical study of the properties of this approach with
\code{equal.weights = FALSE} can be found in \cite{GarG08}.

This type of constraint limits the relative size of the axes of the
equidensity ellipsoids defined through the obtained $\Sigma_j$ when
assuming normality. This way we are simultaneously controlling the
relative group sizes and also the deviation from sphericity in each
cluster.

Setting \code{equal.weights = TRUE}, \code{restr = "eigen"} and
\code{restr.fact = 1} implies the most constrained case. In this
case, the \code{tclust} function tries to solve the trimmed
$k$-means problem as introduced by \cite {CueG97}. This problem
simplifies to the well-known $k$-means clustering criterion when no
trimming is done (i.e., \code{alpha = 0}). The \code{tkmeans}
function directly implements this most constrained application of
the \code{tclust} function.

\subsection{Constraints on the determinants}\label{sec4:constdet}
Another way of restricting cluster scatter matrices is constraining
their determinants. Thus, if
$$
M_n=\max_{j=1,...,k}|\Sigma_j|\text{ and
}m_n=\min_{j=1,...,k}|\Sigma_j|
$$
are the maximum and minimum
determinants, we attempt to maximize (\ref{eqn4:e3}) or (\ref{eqn4:e4}) by
constraining the ratio $M_n/m_n$ to be smaller or equal than a fixed
value \code{restr.fact}. This is done in the function \code{tclust}
by using the option \code{restr = "deter"}.

Now, this type of constraint limits the relative volumes of the
mentioned equidensity ellipsoids, but not the cluster shapes. The
use of this type of constraint is particularly advisable when affine
equivariance is required because this property is satisfied when
\code{restr = "deter"}.

The untrimmed case \code{alpha = 0}, \code{restr = "deter"} and
\code{restr.fact = 1} was already outlined in \cite{MarJ74}, as the
only sensible way to avoid (Mahalanobis distance modified) $k$-means
type algorithms to return clusters of a few almost collinear
observations. The possibility of trimming data was also considered
in \cite{GalM02} who implicitly assumed $|\Sigma_1|=...=|\Sigma_k|$
(and so \code{restr.fact = 1}). The package presented here extends
her approach to more general cases (\code{restr.fact} $>1$).

\subsection{Example}
\begin{figure}[t!]
\centering
%\includegraphics[width= 0.65 \textwidth]{Plots_R-chunk2}
%\includegraphics[width=10cm]{Plots_R-chunk2a}
<<fig=TRUE,echo=false,results=hide,label=chunk2,width=7.14,height=7.8>>=

################
##  Figure 2  ##
################

data(M5data)
cl <- M5data[, "cluster"]
plot(M5data[, 1:2], col = cl + 1, pch = cl + 1, main = "The M5data data set")

@
\caption{ A scatter plot of the \code{M5data} data set. Different
symbols are used for the data points generated by each of the three
bivariate normal components and ``$\circ$'' for the added
outliers.}\label{f2a}
\end{figure}
In this example, we examine the influence of different constraints
by applying the function \code{tclust} to the so-called
\code{M5data} data set. This data set, which accompanies the
\pkg{tclust} package, has been generated following the simulation
scheme M5 introduced in \cite{GarG08}. Thus it is a bivariate
mixture of three simulated gaussian components with very different
scatters and a clear overlap between two of these components.
\begin{figure}[t!] \centering
%\includegraphics[width= 0.77 \textwidth]{Plots_R-chunk3}
<<fig=TRUE,echo=false,results=hide,label=chunk3,width=8,height=9>>=

################
##  Figure 3  ##
################

data(M5data)
x <- M5data[, 1:2]
set.seed(100)

## applying tclust with restr.fact = 1, restr= "eigen"
res.a <- tclust(x, k = 3, alpha=0.1, restr.fact = 1, restr= "eigen",
                 equal.weights = TRUE)

## applying tclust with restr.fact = 50, restr= "eigen"
res.b <- tclust(x, k = 3, alpha=0.1, restr.fact = 50, restr= "eigen",
                 equal.weights = FALSE)

## applying tclust with restr.fact = 1, restr= "deter"
res.c <- tclust(x, k = 3, alpha=0.1, restr.fact = 1, restr= "deter",
                 equal.weights = TRUE)

## applying tclust with restr.fact = 50, restr= "deter"
res.d <- tclust(x, k = 3, alpha=0.1, restr.fact = 50, restr= "deter",
                 equal.weights = TRUE)

op <- par(mfrow = c (2, 2), mar = c (2.1, 2.1, 3.6, 1.1))
col <- c(og, 4, 2, 3)
pch <- c(1, 4, 2, 3)

  ## plotting results..
plot(res.a, col = col, pch = pch, tol.lty = 2, xlab = "",
     ylab = "", main.pre ="(a)", main = "/r")

plot(res.b, col = col, pch = pch, tol.lty = 2, xlab = "",
     ylab = "", main.pre ="(b)", main = "/r")

plot(res.c, col = col, pch = pch, tol.lty = 2, xlab = "",
     ylab = "", main.pre ="(c)", main = "/r")
plot(res.d, col = col, pch = pch, tol.lty = 2, xlab = "",
     ylab = "", main.pre ="(d)", main = "/r")
par(op)


@

\caption{Results of the clustering processes for the \code{M5data}
data set for different constraints on the cluster scatter matrices
and the parameters $\alpha = 0.1$ and $k= 3$. Different colors and
symbols represent each observation's individual cluster assignment.}
\label{fig4:f2}
\end{figure}
A $10\%$ proportion of outliers is also added in the outer region of
the bounding rectangle enclosing the three gaussian components. See
Figure~\ref{f2a} for a graphical representation and \cite{GarG08}
for more details on the structure of this \code{M5data} data set.
Executing the following code yields Figure~\ref{fig4:f2}.
\begin{Code}
R > data ("M5data")
R > x <- M5data[, 1:2]
R > res.a <- tclust(x, k = 3, alpha = 0.1, restr.fact = 1,  restr = "eigen",
    + equal.weights = TRUE)
R > res.b <- tclust(x, k = 3, alpha = 0.1, restr.fact = 50, restr = "eigen",
    + equal.weights = FALSE)
R > res.c <- tclust(x, k = 3, alpha = 0.1, restr.fact = 1,  restr = "deter",
    + equal.weights = TRUE)
R > res.d <- tclust(x, k = 3, alpha = 0.1, restr.fact = 50, restr = "deter",
    + equal.weights = FALSE)
R > plot(res.a, main = "/r")
R > plot(res.b, main = "/r")
R > plot(res.c, main = "/r")
R > plot(res.d, main = "/r")
\end{Code}
Although different constraints are imposed, we are searching for
$k=3$ clusters and the trimming proportion is set to $\alpha=0.1$ in
all the cases. Note that only the clustering procedure introduced in
\cite{GarG08}, shown in Figure~\ref{fig4:f2},(d), with a sufficiently
large value of \code{restr.fact} approximately returns the three
original clusters in spite of the very different cluster scatters
and different cluster sizes. Moreover, this clustering procedure
adequately handles the severe overlap of two clusters. The value
\code{restr.fact = 50} has been chosen in this case because the
eigenvectors of the covariance matrices of the three gaussian
components satisfy restriction (\ref{eqn4:e5}) for this value. Due to
their underlying assumptions, the other three clustering methods
(trimmed $k$-means in Figure~\ref{fig4:f2},(a), \cite{GalR05} in (b),
\cite{GalM02} in (c)) return rather similarly structured clusters.
In fact, we found spherical clusters in (a), clusters with the same
scatter matrix in (b) and clusters with the same cluster scatter
matrix determinant in (c). The \code{M5data} is perhaps a very
``extreme'' situation and restriction settings in (a), (b) and (c)
can be useful (and easier to be interpreted) with not so extreme
data sets and where the assumptions implied by these restriction
settings hold.

\section{Numerical output}\label{sec4:num}

The function \code{tclust} returns an S3 object containing the
cluster centers $\mu_j$ by columns (\code{\$centers}), scatter
matrices $\Sigma_j$ as an array (\code{\$cov}), the weights
(\code{\$weights}), the number of observations in each group
(\code{\$size}) and the maximum value found for the trimmed
log-likelihood objective function (\ref{eqn4:e3}) or (\ref{eqn4:e4})
(\code{\$obj}). The vector \code{\$cluster} provides the cluster
assignment of each observation, whereas an artificial cluster
``\code{0}'' (without location and scatter information) is introduced
which holds all trimmed data points.


\begin{figure}[t!]
\centering
%\includegraphics[width= 0.65 \textwidth]{Plots_R-chunk4}
<<fig=TRUE,echo=false,results=hide,label=chunk4,width=7.14,height=7.8>>=

################
##  Figure 4  ##
################

  ## data generation
library(MASS)
set.seed (10)
x <- rbind(MASS::mvrnorm(200, c (0, 0), diag (2)),
           MASS::mvrnorm(200, c (5, 0), diag (2)))

  ## applying tclust
clus.4 <- tclust(x, k = 3, alpha = 0.0, restr.fact = 1)

  ## plotting results..
plot(clus.4, xlim = c (-3, 9), ylim = c (-6, 6))


@
\caption{Applying \code{tclust} with $k = 3$ and $\alpha = 0$ on a
simulated data set which originally consists of 2 clusters when
\code{equal.weights = FALSE}.} \label{fig4:f3a}
\end{figure}

Sometimes, (\ref{eqn4:e3}) and (\ref{eqn4:e4}) maximize with some
clusters remaining empty (see Figure~\ref{fig4:f3a}). In this case,
only information on the non-empty groups is returned. Notice that,
if we are searching for $k$ clusters, empty clusters can be found
when a clustering solution for a number of clusters strictly smaller
than $k$ attains a higher value for (\ref{eqn4:e3}) or
(\ref{eqn4:e4}) than the best solution found with $k$ clusters. In
this case, artificial empty clusters may be defined by considering
sufficiently remote centers $\mu_j$ and scatter matrices $\Sigma_j$
satisfying the specific constraints that are assigned to these empty
clusters. They are chosen such that $f(\cdot;\mu_j,\Sigma_j)$ gives
almost null density to all the observations in the sample. These
artificially added centers and scatter matrices are not returned as
output by the \code{tclust} function and a warning is issued. For
instance, let us consider the following code
\begin{Code}
R > set.seed(10)
R > x <- rbind(MASS::mvrnorm(200, c (0, 0), diag (2)),
    +          MASS::mvrnorm(200, c (5, 0), diag (2)))
R > clus <- tclust(x, k = 3, alpha = 0, restr.fact = 1)
R > plot(clus)
\end{Code}
Although we are searching for $k=3$ clusters, Figure~\ref{fig4:f3a} and
the issued warning show that only 2 clusters are found. Notice that
$k=2$ is surely a more sensible choice for the number of clusters
than $k=3$ for this generated data set. Therefore, the detection of
empty clusters, or clusters with few data points, can be helpful,
providing valuable tools for making sensible choices for $k$ as we
will see in Section~\ref{sec4:kal}. On the other hand, the detection
of empty clusters is very unlikely to happen when the argument
\code{equal.weights = TRUE} is provided in the call to
\code{tclust}.

\section{Algorithms}\label{sec4:alg}
The maximization of (\ref{eqn4:e3}) or (\ref{eqn4:e4}) considering different
cluster scatter matrix constraints is not straightforward because of
the combinatorial nature of the associated maximization problems.

The algorithm presented in \cite{GarG08} can be adapted to
approximately solve all these problems. The methods implemented in
\pkg{tclust} could be seen as Classification EM algorithms
\citep{Sch76,CelG92}, whereas a certain type of ``concentration''
steps \citep[see the fast-MCD algorithm in][]{RouD98} is also
applied. In fact, the concentration steps applied by the package
\pkg{tclust} can be considered as an extension of those applied by
the batch-mode $k$-means algorithm \citep{Ste56,For65}. It can be
seen that the target function always increases throughout the
application of concentration steps, whereas several random start
configurations are needed in order to avoid ending trapped in local
maxima. Therefore, \code{nstart} random initializations and
\code{iter.max} concentration steps are considered. The probability
that the algorithm converges close to the global optimum maximizing
(\ref{eqn4:e3}) or (\ref{eqn4:e4}) increases with larger values of
\code{nstart} and \code{iter.max}. The drawback of high values of
\code{nstart} and \code{iter.max} obviously is the increasing
computational effort.

In the concentration step, the centers and scatter matrices are
updated by considering the cluster sample means and cluster sample
covariance matrices. New cluster assignments are obtained by
gathering the ``closest'' observations to the new centers.
Mahalanobis distances, based on the computed cluster sample
covariance matrices, are used in order to decide which are the
closest observations to each center. If needed, in the updating
step, the cluster sample covariance matrices are modified as little
as possible but in such a way that they satisfy the desired
constraints \citep{GarG08}. The main idea behind these
``constrained'' concentration steps is to replace the eigenvalues of
the sample covariance matrices by optimally truncated eigenvalues,
which satisfy the desired constraint. A more detailed description of
the algorithm applied by \code{tclust} and the way the restrictions
are forced onto the cluster scatter matrices can be found in
\cite{FriG11}.

\section{Comparison with other robust clustering
proposals}\label{sec4:comp}

In this section, we briefly compare the performance of the
clustering procedures implemented in the \pkg{tclust} package with
respect to other robust clustering proposals in the literature.

The Partitioning Around Medoids (PAM) clustering method
\citep{KauR90} has been proposed as a robust alternative to
$k$-means clustering. It can be seen that the effect of the 6
anomalous ``short followed by short'' eruptions lengths in the lower
left corner of Figure~\ref{fig4:f1} do not affect the position of the
$k$-medoid centers (see Figure~\ref{fig4:pam},(a)) too much, nor the
resulting clusters. However, in Figure~\ref{fig4:pam},(b), we see
that the clustering results with $k=3$ are strongly affected when
moving these 6 anomalous points toward a more distant position. On
the other hand, in Figure~\ref{fig4:pam},(c), we can see that these
outlying data points do not affect the trimmed $k$-means based
clustering at all once that they are trimmed.

\begin{figure}[t!]
\centering
%\includegraphics[width= 1 \textwidth]{plots_R-chunk5}
<<fig=TRUE,echo=false,results=hide,label=chunk5,width=9,height=5>>=
################
##  Figure 5  ##
################

library(cluster)
data(geyser2)

set.seed(0)

op <- par(mfrow = c (1, 3), mar = c (3.1, 3.1, 3.1, 2.1))

xlab <- "" #names (geyser2)[1]
ylab <- "" #names (geyser2)[2]

  ## applying PAM on the geyser2 data set
clus <- pam(geyser2, 3)
plot(geyser2, xlab = xlab, ylab = ylab, col = clus$clustering + 1,
      main = "(a) PAM with Geyser")
points(clus$medoids, pch = "X", cex = 1.4)

  ## modifying the geyser2 data set
geyser2.mod <- geyser2
idx <- c(16,  21, 36, 171, 236, 265)
geyser2.mod[idx,] <- geyser2[idx,] - 12

  ## applying PAM on the modified geyser2 data set
clus <- pam(geyser2.mod, 3)
plot(geyser2.mod, xlab = xlab, ylab = ylab, col = clus$clustering + 1,
      main = "(b) PAM with modified Geyser")

  ## applying tkmeans on the modified geyser2 data set
clus <- tkmeans(geyser2.mod, k = 3, alpha = 0.03)
plot(clus, xlab = xlab, ylab = ylab,
      main = "(c) TCLUST with modified Geyser", sub = "")
par(op)

@
\caption{PAM's clustering results for the \code{geyser2} data with
$3$-medoids denoted by the symbol ``$\times$'' in (a). PAM's
clustering results for a modified \code{geyser2} data set in (b) and
when applying \code{tkmeans} with $k=3$ and $\alpha=0.03$ in (c).}
\label{fig4:pam}
\end{figure}

In fact, only one single outlier placed in a very remote position is
able to completely break down the PAM method \citep{GarG99}. This
also happens when applying \pkg{emmix}, which has a breakdown point
of zero \citep{Hen04}.  The \pkg{emmix} approach is able to obtain
appropriate clustering results for the two data sets made of
mixtures of symmetric and asymmetric $t$ components as those shown
in Figure~\ref{fig4:elt}. These two data sets contain three main
clusters with some distant observations in the heavy tails of these
$t$ components, which would be considered as outliers when assuming
normality. When applying the classification EM algorithm without
trimming to these data sets, we are not able to find the three
cluster structures and two main clusters are artificially joined
together. However, when considering $\alpha =0.05$, \pkg{tclust}
perfectly avoids the harmful effect of the observations in the tails
and still discovers the three clusters. In fact, almost all
non-trimmed observations are correctly clustered in both, symmetric
and asymmetric, cases. Any  $\alpha >0$ discarding the most outlying
observations would give similar results. Moreover, it may be seen
that the shape of the elliptical clusters are essentially discovered
in the case of the symmetric-elliptical $t$ components. In this
example, we see that applying \pkg{tclust} to data sets including
non-normally distributed components as those in
Figure~\ref{fig4:elt} may result in proper clustering solutions,
this however cannot be guaranteed if the underlying distributions
differ too much from normality. The closely related \pkg{tlemix}
package allows to consider other non-normal models by taking
advantage of the flexibility provided by the \pkg{FlexMix} package
\citep{LeiF04}. On the other hand, \pkg{tclust} focuses on normally
distributed components and on the implementation of appropriate
cluster scatter matrix constraints while \pkg{tlemix} does not. The
\pkg{tlemix} mainly controls the minimum number of observations in a
cluster.

\begin{figure}[t!]
\centering
%\includegraphics[width= 0.9 \textwidth]{plots_R-chunk6}
<<fig=TRUE,echo=false,results=hide,label=chunk6,width=9,height=6.5>>=
################
##  Figure 6  ##
################

fig.elt <- list()

library(sn)

op <- par(mfrow = c (2, 3), mar = c (3.1, 2, 3.1, 2))

set.seed (3)

n <- 200
xlab <- "" #"x1"
ylab <- "" #"x2"

## data generation: Mixture of elliptical t's
xi <- c (0.5, -1)
Omega <- diag (2) / 2 + 0.5
alpha <- c (0, 0)
rnd1 <- rmst (n, xi, Omega, alpha, nu = 2)
xi <- c (6, -3)
Omega <- matrix (c (4, 1, 1, 2), nrow = 2, ncol = 2)
alpha <- c (0, 0)
rnd2 <- rmst (n, xi, Omega, alpha, nu = 3)
xi <- c (-3, 3)
Omega <- matrix (c (2, 1, 1, 4), nrow = 2, ncol = 2)
alpha <- c (0, 0)
rnd3 <- rmst (n, xi, Omega, alpha, nu = 4)
rnd.1 <- rbind (rnd1, rnd2, rnd3)
real.clus1 <- c (rep (1, n), rep (2, n), rep (3, n))

ylim <- c (-25, 15)

 ## Plotting the real clusters
plot(rnd.1, xlim = c (-15, 80), ylim = ylim, xlab = xlab, ylab = ylab,
      col = real.clus1 + 1, pch = real.clus1 + 1)
title("(a) Elliptical t components", line = 1.6)

  ## applying tclust without trimming
clus <- tclust(rnd.1, k = 3, alpha = 0.0)
plot(clus, xlim = c (-15, 80), ylim = ylim, xlab = xlab, ylab = ylab,
      main.pre = "(b)")

 ## applying tclust with trimming (5%)
clus <- tclust(rnd.1, k = 3, alpha = 0.05)
plot(clus, xlim = c (-15, 80), ylim = ylim, xlab = xlab, ylab = ylab,
      main.pre = "(c)")

## data generation: Mixture of non-elliptical t's
xi <- c (0.5, -1)
Omega <- diag (2) / 2 + 0.5
alpha <- c (2, 80)
rnd1 <- rmst (n, xi, Omega, alpha, nu = 2)
xi <- c (6, -3)
Omega <- matrix (c (4, 1, 1, 2), nrow = 2, ncol = 2)
alpha <- c (2, 2)
rnd2 <- rmst (n, xi, Omega, alpha, nu = 3)
xi <- c (-3, 3)
Omega <- matrix (c (2, 1, 1, 4), nrow = 2, ncol = 2)
alpha <- c (4, 4)
rnd3 <- rmst (n, xi, Omega, alpha, nu = 4)

rnd.2 <- rbind (rnd1, rnd2, rnd3)
real.clus2 <- c (rep (1, n), rep (2, n), rep(3, n))

ylim <- c (-10, 20)
  ## Plotting the real clusters
plot (rnd.2, xlim = c (-10, 20), ylim = ylim, xlab = xlab, ylab = ylab,
      col = real.clus2 + 1, pch=real.clus2 + 1)
title ("(d) Non-elliptical t components", line = 1.6)

 ## applying tclust without trimming
clus <- tclust(rnd.2, k = 3, alpha = 0.0)
plot(clus, xlim = c (-10, 20), ylim = ylim, xlab = xlab, ylab = ylab,
      main.pre = "(e)")

 ## applying tclust with trimming (5%)
clus <- tclust(rnd.2, k = 3, alpha = 0.05)
plot(clus, xlim = c (-10, 20), ylim = ylim, xlab = xlab, ylab = ylab,
      main.pre = "(f)")

par(op)


@

\caption{A data set made up of three elliptical $t$ components is
shown in (a) and three asymmetrical $t$ components in (d). The
associated clustering results when applying the \code{tclust}
function with $k=3$ and $\alpha=0$ are shown in (b) and (e) and with
$k=3$ and $\alpha=0.05$ in (c) and (f).} \label{fig4:elt}
\end{figure}

The widely used \pkg{mclust} package considers a uniformly
distributed component for explaining outlying data points. As we can
see, this uniform component successfully accommodates the $10\%$
``background noise'' as seen in Figure~\ref{fig4:mclust},(a).
However, it is not able to cope with a more structured noise pattern
like the ``helix'' in Figure~\ref{fig4:mclust},(c) which also
accounts for $10\%$ of the data, although the information of a
$10\%$ contamination level was passed to \pkg{mclust}.
Alternatively, the \pkg{tclust} package with $k = 2$ and $\alpha =
0.1$ properly discovers the outlying data points without trying to
fit them.

\begin{figure}[t!]
\centering
%\includegraphics[width= 0.7 \textwidth]{plots_R-chunk7}
<<fig=TRUE,echo=false,results=hide,label=chunk7,width=7.14,height=7.8>>=

################
##  Figure 7  ##
################

library(mclust)

  ## function for plotting mclust-objects in tclust-style.
mclustplottcluststyle2d <- function (x, clus, clus.perm, tol.lty = 3,
                                     size = sqrt (qchisq (0.95, 2)), ...)
{
  if (missing (clus.perm))
    cuse <- clus$classification
  else
    cuse <- clus.perm [clus$classification + 1]

  plot (x, col = cuse + 1, pch = cuse + 1, ...)

  k <- ncol (clus$parameters$mean)

  for (i in 1:k)
    tclust:::.doEllipses(cov = clus$parameters$variance$sigma[,, i],
                          center = clus$parameters$mean[, i], size = size,
                          lty = tol.lty)
}

op <- par (mfrow = c (2, 2), mar = c (3.1, 3.1, 3.1, 2.1))

  ## example 1 - uniformly distributed noise

  ## data generation

set.seed (0)
x <- rbind (
            MASS::mvrnorm(360, c (0.0,  0), matrix (c ( 1,  0,  0, 1), ncol = 2)),
            MASS::mvrnorm(540, c (5.0, 10), matrix (c ( 6, -2, -2, 6), ncol = 2)),
            cbind (runif (100, -5, 15), runif (100, -10, 20)))

  ## applying mclust
noiseInit <- sample (c (TRUE, FALSE), size = nrow (x),
                     replace = TRUE, prob = c (0.1, 0.9))
clus <- Mclust(x, initialization = list (noise = noiseInit), G = 2)
mclustplottcluststyle2d (x, clus, c (0, 2, 1), xlab = "", ylab = "",
                         main = "(a) mclust", tol.lty = 2)

  ##  applying tclust
clus <- tclust(x, k = 2, alpha = 0.1)
plot (clus, xlab = "", ylab = "", main = "(b) tclust", sub = "", tol.lty = 2)


  ## example 2 - structured noise

  ## data generation
set.seed (0)
v <- runif (100, -2 * pi, 2 * pi)
noise <- cbind (100 + 25 * sin (v), 10 + 5 * v)

x <- rbind (
            MASS::mvrnorm(360, c (0.0,  0), matrix (c (1,  0,  0, 1), ncol = 2)),
            MASS::mvrnorm(540, c (5.0, 10), matrix (c (6, -2, -2, 6), ncol = 2)),
            noise)

  ##  applying mclust
noiseInit <- sample (c (TRUE, FALSE), size = nrow (x),
                     replace = TRUE, prob = c (0.1, 0.9))
clus <- Mclust (x, initialization = list (noise = noiseInit), G = 2)
mclustplottcluststyle2d (x, clus, c (0, 2, 1), xlab = "", ylab = "",
                         main = "(c) mclust", tol.lty = 2)

  ##  applying tclust
clus <- tclust(x, k = 2, alpha = 0.1)
plot (clus, xlab = "", ylab = "", main = "(d) tclust", sub = "", tol.lty = 2)

par (op)


@

\caption{Clustering results for two simulated data sets when
applying \pkg{mclust} with $k=2$ in (a) and (c) and \pkg{tclust}
with $k=2$ and $\alpha=0.1$ in (b) and (d).} \label{fig4:mclust}
\end{figure}

Since groups of outliers may be considered as further clusters, it
could be argued that robust clustering problems can always be solved
by increasing the number of groups we are searching for. However, as
explained in \cite{GarG10}, this is not necessarily the best
strategy. Firstly, sometimes the researcher fixes the number of
clusters in advance, not being aware of the presence of a small
amount of outlying observations. Secondly, it could lead to a quite
large number of clusters when very scattered outliers are present in
the data set.

A clear limitation of \pkg{tclust} is that it is not applicable on
high-dimensional data sets, as the method in its current definition
definitely needs a data set containing more observations than
dimensions.

\section{Selecting the number of groups and the trimming size}\label{sec4:kal}

Perhaps one of the most complex problems when applying cluster
analysis is the choice of the number of clusters, $k$. In some cases
one might have an idea of the number of clusters in advance, but
usually $k$ is completely unknown. Moreover, in the approach
proposed here, the trimming proportion $\alpha$ has also to be
chosen without knowing the true contamination level.

As we will see through the following example, the choices for $k$
and $\alpha$ are related problems that should be addressed
simultaneously. It is important to see that a particular trimming
level implies a specific number of clusters and vice versa. This
dependency can be explained as entire clusters tend to be trimmed
completely when increasing $\alpha$. On the other hand, when
choosing $\alpha$ too low, groups of outliers might form new
spurious clusters and thus it appears that the number of clusters
found in the data set is higher. Moreover, the simultaneous choice
of $k$ and $\alpha$ depends on the  type of clusters we are
searching for and on the allowed differences between cluster sizes.
These two aspects can be controlled by the choice of arguments
\code{restr} and \code{restr.fact}.

To demonstrate the relation between $\alpha$, $k$ and
\code{restr.fact}, let us consider \code{restr = "eigen"} and the
data set in Figure~\ref{fig4:f3} which could either be interpreted as a
mixture of three components (a) or a mixture of two components (b)
with a $10\%$ outlier proportion. Both clustering solutions shown in
Figure~\ref{fig4:f3} are perfectly sensible and the final choice of
$\alpha$ and $k$ only depends on the value given to
\code{restr.fact}. The code used to obtain Figure~\ref{fig4:f3} is the
following:
\begin{Code}
R > sigma1 <- diag(2)          ## EigenValues: 1, 1
R > sigma2 <- diag(2) * 8 - 2  ## EigenValues: 8, 4
R > sigma3 <- diag(2) * 50     ## EigenValues: 50, 50
R > mixt <- rbind(
    + MASS::mvrnorm(360, mean = c (0.0,  0), sigma = sigma1),
    + MASS::mvrnorm(540, mean = c (5.0, 10), sigma = sigma2),
    + MASS::mvrnorm(100, mean = c (2.5,  5), sigma = sigma3))
R > plot(tclust(mixt, k = 3, alpha = 0.00, restr.fact = 50))
R > plot(tclust(mixt, k = 2, alpha = 0.05, restr.fact = 12))
\end{Code}

\begin{figure}[t!]
\centering
%\includegraphics[width= 0.9 \textwidth]{Plots_R-chunk8}
<<fig=TRUE,echo=false,results=hide,label=chunk8,width=10,height=5.45>>=

################
##  Figure 8  ##
################

  ## data generation
set.seed (100)
mixt <- rbind(MASS::mvrnorm(360, c (  0,  0), matrix (c (1,  0,  0,  1), ncol = 2)),
              MASS::mvrnorm(540, c (  5, 10), matrix (c (6, -2, -2,  6), ncol = 2)),
              MASS::mvrnorm(100, c (2.5,  5), matrix (c (50, 0,  0, 50), ncol = 2)))

  ## applying tclust
set.seed (100)
clus.1 <- tclust(mixt, k = 3, alpha =  0.0, restr.fact = 50)
clus.2 <- tclust(mixt, k = 2, alpha = 0.05, restr.fact =  8)

  ## plotting results
op <- par (mfrow = c (1, 2))
plot(clus.1, by.clust = TRUE, col = c (og, 2, 3, 4), pch = c (1, 2, 3, 4),
     tol.lty = 2, main.pre = "(a)")
plot(clus.2, by.clust = TRUE, col = c (og, 2, 3), pch = c (1, 2, 3),
     tol.lty = 2, main.pre = "(b)")
par (op)


@
\caption{Clustering results for the simulated data set \code{mixt}
with $k=3$, $\alpha=0$ and \code{restr.fact} $= 50$ (a) and $k=2$,
$\alpha=0.1$ and \code{restr.fact} $=8$ (b).} \label{fig4:f3}
\end{figure}

Considering \code{sigma1} and \code{sigma2}, the quotient of the
largest and smallest eigenvalue is 8, whereas the maximal quotient
is 50 if we consider \code{sigma1}, \code{sigma2} and \code{sigma3}.
Thus \code{restr.fact} $=8$ would allow to consider two clusters
while \code{restr.fact} $=50$ would also allow to assume three
groups there. Although the proportion of ``contaminated'' data is
equal to $10\%$, the trimming level must be reduced to $5\%$, when
considering $k=2$, because the third (more scattered) gaussian
component partially overlaps with the other two components.

Let us assume first that \code{restr} and \code{restr.fact} have
been fixed in advance by the researcher who applies the robust
clustering method. Even with this information and assuming
$\alpha=0$, choosing the appropriate number of clusters is not an
easy task. The careful monitoring of the maximum value attained by
log-likelihoods like those in (\ref{eqn4:e3}) and (\ref{eqn4:e4}) while
changing $k$ has traditionally been applied as a method for choosing
the number of clusters when $\alpha=0$. Moreover \cite{BryP91}
stated that the use of ``weighted'' log-likelihoods (\ref{eqn4:e4}) is
preferred to the use of log-likelihoods assuming equal weights
(\ref{eqn4:e3}). Notice that increasing $k$ always causes the maximized
log-likelihood (\ref{eqn4:e3}) to increase too, and this could lead to
``overestimate'' the appropriate number of clusters
\citep[see][]{GarG11}.

In this trimming framework, let us consider
$\mathcal{L}_{\texttt{restr.fact}}^{\Pi}(\alpha,k)$ as the maximum
value reached by (\ref{eqn4:e4}) for each combination of a given set of
values for $k$ and $\alpha$. \cite{GarG11} propose to monitor the
``classification trimmed likelihoods'' functionals
$$(\alpha,k)\mapsto \mathcal{L}_{\texttt{restr.fact}}^{\Pi}(\alpha,k)$$
while altering $\alpha$ and $k$, which yields an exploratory
graphical tool for making sensible choices for parameters $\alpha$
and $k$. In fact, it is proposed to choose the number of clusters as
the smallest value of $k$ such that
\begin{equation}\label{eqn4:diff}
    \mathcal{L}_{\texttt{restr.fact}}^{\Pi}(\alpha,k+1)-\mathcal{L}_{\texttt{restr.fact}}^{\Pi}(\alpha,k)
\end{equation}
is (close to) 0 except for small values of $\alpha$. Once the number
of clusters is fixed, a good choice for the trimming level is the
first $\alpha_0$ such that (\ref{eqn4:diff}) is (close to) 0 for every
$\alpha\geq \alpha_0$. Although we are convinced that monitoring the
classification trimmed likelihoods functionals is very informative,
no theoretical statistical procedures are available yet for
determining when (\ref{eqn4:diff}) can be formally considered as ``close
to 0''.

The function \code{ctlcurves} in package \pkg{tclust} approximates
the classification trimmed likelihoods by successively applying the
\code{tclust} function for a sequence of values of $k$ and $\alpha$.
A default value  \code{restr.fact = 50} is considered but, if
desired, other values of \code{restr.fact} can be passed to
\code{tclust} via \code{ctlcurves} too.

For instance, the following code applied to the previously simulated
\code{mixt} data set
\begin{Code}
R > plot (ctlcurves (mixt, k = 1:4, alpha = seq (0, 0.2, by = 0.05)))
\end{Code}
results in Figure~\ref{fig4:f4}. This figure shows that increasing
$k$ from 2 to 3 is needed when $\alpha=0$, as the objective
functions value differs noticeably between $k = 2$ and $k = 3$. On
the other hand, increasing $k$ from 2 to 3 is not needed anymore as
the third (more scattered) ``cluster'' vanishes when trimming 5\% of
the most outlying observations. Thus, there is no discernable
difference of the objective functions value with $\alpha \geq
\alpha_0=0.05$ and $k \geq 2$. Increasing $k$ from 3 to 4 is not
needed in any case.

\begin{figure}[t!]
\centering
%\includegraphics[width= 0.65 \textwidth]{Plots_R-chunk9}
<<fig=TRUE,echo=false,results=hide,label=chunk9,width=7.14,height=6.25>>=

################
##  Figure 9  ##
################

  ## computing ctlcurves
ctl <- ctlcurves(mixt, restr.fact = 50, alpha = seq (0, 0.2, by = 0.05), trace=0)

op <- par(mfrow = c (1, 1))
plot (ctl)
par (op)

@

\caption{Classification trimmed likelihoods with $k=1,...,4$,
$\alpha=0,$ $0.05$, ..., $0.2$ and \code{restr.fact} $=50$ for the
\code{mixt} data set in Figure~\ref{fig4:f3}.} \label{fig4:f4}
\end{figure}

The previously described procedure for making sensible choices for
parameters $k$ and $\alpha$ requires an active role from the
researcher. The type of restriction and the allowed restriction
factor, which do not necessarily depend on the given data set, must
be specified in advance. For instance, some specific clustering
applications like ``location-facilities'' problems require almost
spherical clusters that can be obtained by setting \code{restr =
"eigen"} and a \code{restr.fact} close to 1. The researcher's
decision on the restriction consequently modifies the proper
determination of parameters $k$ and $\alpha$.


Due to the important role of the statement of \code{restr} and
\code{restr.fact}, some general guideline for fixing them will be
given here. For instance, as already commented, fixing \code{restr =
"deter"} is recommended when only the relative cluster sizes shall
be constrained, or when affine equivariance is clearly needed. On
the other hand, using \code{restr = "eigen"} is advised when we want
to simultaneously constrain relative cluster sizes and shapes.

With respect to the choice of \code{restr.fact}, we recommend to
initially use large values when applying \code{ctlcurves}, thus,
providing high flexibility to the clustering method. The default
value \code{restr.fact = 50} is suggested for \code{ctlcurves}, as
it worked well with a lot of data sets (especially if the variables
have been properly standardized through the \code{scale} argument in
the \code{tclust} function). The so obtained ``sensible'' values for
$k$ and $\alpha$ and their associated clustering solutions must be
explored carefully. For instance, \code{tclust} issues a warning
when the returned clustering solution has been ``artificially
restricted'' by the algorithm, as shown in Section~\ref{sec4:exa}.
This means, that the values $M_n$ and $m_n$ (see
Section~\ref{sec4:consteig} and Section~\ref{sec4:constdet}) derived
from the returned scatter matrices satisfy $M_n/m_n =$
\code{restr.fact}, because the algorithm has forced the chosen
constraint, since the (unconstrained) group sample covariance
matrices do not satisfy $M_n/m_n \leq$ \code{restr.fact}. In this
situation, if no specific constraints are required,
\code{restr.fact} may be increased stepwise until this warning
disappears. Moreover, printing the object returned by the
\code{ctlcurves} function points out all ``artificially restricted''
solutions for each computed combination of parameters $k$ and
$\alpha$. In this way, if desired, we can easily search for
clustering solutions which are not artificially restricted and do
not contain spurious clusters. Finally, the exploratory tools in
Section~\ref{sec4:gra} also help to evaluate whether all these
parameters are reasonably chosen.

Note that arguments \code{nstart} and \code{iter.max} may be
provided in the call to \code{ctlcurves} and they are internally
passed to function \code{tclust}.

The curves presented in \cite{GarG03} can be considered as
precedents of those we obtain by using the \code{ctlcurves}
function. Trimmed likelihoods have also been taken into account in
\cite{NeyF07} for choosing $k$ and $\alpha$ by using a BIC
criterion.

\section{Graphical displays}\label{sec4:gra}
As seen in previous examples, the package \pkg{tclust} provides
functions for visualizing the computed cluster assignments.
One-dimensional, two-dimensional and higher-dimensional cases are
visualized differently:
\begin{description}
\item[$p = 1$:] The one-dimensional data set with the corresponding cluster assignments is
 displayed along the $x$-axis. Setting the argument \code{jitter = TRUE} jitters
  the data along the $y$-axis in order to increase the visibility of the actual
 data structure. Additionally, a (robust) scatter
 estimation of each cluster is also displayed.
\item[$p = 2$:] Tolerance ellipsoids are plotted additionally in order to
visualize the estimated cluster scatter matrices.
\item[$p > 2$:] The first two Fisher's canonical coordinates are displayed in
this case, which are computed based on the estimated cluster scatter
matrices. Notice that trimmed observations are not taken into
account when computing these coordinates, since they have been
completely discarded. The implementation of these canonical
coordinates is derived from the function \code{discrcoord} as
implemented in the package \pkg{fpc} \citep{HenC10}.
\end{description}

A simple example demonstrates how the \code{plot} function works in
different dimensions. The code:
\begin{Code}
R > geyser1 <- geyser2[, 1, drop = FALSE]
R > geyser3 <- cbind (geyser2, rnorm (nrow (geyser2)))
R > plot (tkmeans (geyser1, k = 2, alpha = 0.03), jitter = TRUE)
R > plot (tkmeans (geyser3, k = 3, alpha = 0.03))
\end{Code}
yields Figure~\ref{fig4:f5a}. For demonstrating the different plotting
modes, we have selected one single variable from \code{geyser2} to
obtain a one-dimensional data set (\code{geyser1}), and, added an
additional normally distributed variable to \code{geyser2}, yielding
a three-dimensional data set (\code{geyser3}). Figure~\ref{fig4:f5a}
plots the results of the trimmed $k$-means robust clustering method
for these two generated data sets.

\begin{figure}[t!]
\centering
%\includegraphics[width= 0.9 \textwidth]{Plots_R-chunk10}
<<label=fig-10, fig=TRUE, echo=false, results=hide, width=10, height=5.45>>=
#################
##  Figure 10  ##
#################

data (geyser2)
op <- par (mfrow = c (1, 2))
set.seed (10)

  ## creating a one dimensional data "matrix"
geyser1 <- geyser2[, 1, drop = FALSE]
  ## applying tkmeans
plot(tkmeans(geyser1, k = 2, alpha = 0.03), jitter = TRUE, tol.lwd = 2,
      main.pre = "(a)")

  ## adding a random dimension to geyser2
geyser3 <- cbind(geyser2, rnorm (nrow (geyser2)))
  ## applying tkmeans
plot(tkmeans(geyser3, k = 3, alpha = 0.03), main.pre = "(b)")

par(op)

@
\caption{Trimmed $k$-means clustering results for \code{geyser1}
(one-dimensional) in (a) and for \code{geyser3} (three-dimensional)
in (b). These two data sets are based on \code{geyser2}. $k=2$ is
fixed in (a) and $k = 3$ in (b) while $\alpha=0.03$ is fixed in both
cases.} \label{fig4:f5a}
\end{figure}

Given a \code{tclust} object, some additional exploratory graphical
tools can be applied in order to evaluate the quality of the cluster
assignments and the trimming decisions. This is done by applying the
function \code{DiscrFact}.

Let $\widehat{R}=\{\widehat{R_0},\widehat{R_1},...,\widehat{R_k}\}$,
$\widehat{\theta}=(\widehat{\theta_1},...,\widehat{\theta_k})$ and
$\widehat{\pi}=(\widehat{\pi_1},...,\widehat{\pi_k})$ be the values
obtained by maximizing  (\ref{eqn4:e3}) or (\ref{eqn4:e4}) (we set
$\widehat{\pi_j}=1/k$ when maximizing (\ref{eqn4:e3})).
$D_j(x_i;\widehat{\theta},\widehat{\pi})=\widehat{\pi_j}\phi(x_i,\widehat{\theta_j})$
is a measure of the degree of affiliation of observation $x_i$ with
cluster $j$. These values can be ordered as
$D_{(1)}(x_i;\widehat{\theta},\widehat{\pi}) \leq ... \leq
D_{(k)}(x_i;\widehat{\theta},\widehat{\pi})$. Thus the quality of
the assignment decision of a non trimmed observation $x_i$ to the
cluster $j$ with
$D_{(k)}(x_i;\widehat{\theta},\widehat{\pi})=D_{j}(x_i;\widehat{\theta},\widehat{\pi})$
can be evaluated by comparing its degree of affiliation with cluster
$j$ to the best second possible assignment. That is
$$\text{DF}(i)=\log\big(D_{(k-1)}(x_i;\widehat{\theta},\widehat{\pi})/D_{(k)}(x_i;\widehat{\theta},\widehat{\pi})\big)
\text{ for }x_i\text{ not trimmed}.$$

Let $x_{(1)},... ,x_{(n)}$ be the observations in the sample after
being sorted according to their
$D_{(k)}(\cdot;\widehat{\theta},\widehat{\pi})$ values, i.e., $
D_{(k)}(x_{(1)};\widehat{\theta},\widehat{\pi}) \leq ... \leq
D_{(k)}(x_{(n)};\widehat{\theta},\widehat{\pi}).$ It is not
difficult to see that $x_{(1)},... , x_{(\rounding{n\alpha})}$ are
the trimmed observations which are not assigned to any cluster.
Nevertheless, it is possible to compute the degree of affiliation
$D_{(k)}(x_i;\widehat{\theta},\widehat{\pi})$ of a trimmed
observation $x_i$ to its nearest cluster. Thus, the quality of the
trimming decision on this observation can be evaluated by comparing
$D_{(k)}(x_i;\widehat{\theta},\widehat{\pi})$ to
$D_{(k)}(x_{(\rounding{n\alpha}+1)};\widehat{\theta},\widehat{\pi})$,
with $x_{(\rounding{n\alpha}+1)}$ being the non-trimmed observation
with smallest value of
$D_{(k)}(\cdot;\widehat{\theta},\widehat{\pi})$. That is
$$\text{DF}(i)=\log\big(
D_{(k)}(x_i;\widehat{\theta},\widehat{\pi})/D_{(k)}(x_{(\rounding{n\alpha}+1)};\widehat{\theta},\widehat{\pi})\big)\text{
for }x_i\text{ trimmed}.
$$
Hence, discriminant factors $\text{DF}(i)$ $\leq 0$ are obtained for
every observation in the data set, whether trimmed or not.

\begin{figure}[t!]
\centering
%\includegraphics[width= 0.9 \textwidth]{Plots_R-chunk11}
<<label=fig-11, fig=TRUE, echo=false, results=hide, width=9, height=6.5>>=

#################
##  Figure 11  ##
#################

## data generation
set.seed (100)
mixt2 <- rbind (
  MASS::mvrnorm(360, c (0,   0), matrix (c (1,  0,  0,  1), ncol = 2)),
  MASS::mvrnorm(540, c (5,  10), matrix (c (6, -2, -2,  6), ncol = 2)),
  MASS::mvrnorm(100, c (2.5, 5), matrix (c (50, 0,  0, 50), ncol = 2)))

  ## applying tclust
set.seed (100)
clus.w <- tclust(mixt2, k = 3, alpha = 0.1, restr.fact = 1,
                  equal.weights = TRUE)

  ## applying DiscrFact
discr.clus.w <- DiscrFact(clus.w)

  ## plotting results
op <- par(mfrow = c(1, 3), mar = mmar)

plot(clus.w, col = c(og, 2, 3, 4), tol.lty = 2, main.pre = "(a)")
tclust:::plot_DiscrFact_p2(discr.clus.w, xlim = c (-70, 0), main.pre = "(b)")
tclust:::plot_DiscrFact_p3(discr.clus.w, tol.lty = 2, main.pre = "(c)")

par(op)
@
\caption{Graphical displays based on the $\text{DF}(i)$ values for a
\code{tclust} cluster solution obtained with $k=3$, $\alpha=0.1$,
\code{restr.fact} $= 1$ and \code{equal.weights = TRUE} for the
\code{mixt} data set.} \label{fig4:f5}
\end{figure}

Observations with large $\text{DF}(i)$ values (i.e., values close to
zero) indicate doubtful assignments or trimming decisions. The use
of this type of discriminant factors was already suggested in
\cite{AelW06} in a clustering problem without trimming.
``Silhouette'' plots \citep{RouP87} can be used for summarizing the
obtained ordered discriminant factors. Clusters in the silhouette
plot with many large $\text{DF}(i)$ values indicate the existence of
not very ``well-determined'' clusters. The most ``doubtful''
assignments with $\text{DF}(i)$ larger than a
$\log(\texttt{threshold})$ value are highlighted by the function
\code{DiscrFact}.

Figure~\ref{fig4:f5} shows the result of applying the \code{DiscrFact}
function to a clustering solution found for the \code{mixt} data set
appearing in Figure~\ref{fig4:f3}. The following code is used to obtain
this figure:

\begin{Code}
R > clus.w <- tclust (mixt, k = 3, alpha = 0.1, restr.fact = 1,
    + equal.weights = TRUE)
R > discr.clus.w <- DiscrFact (clus.w, threshold = 0.1)
R > plot (discr.clus.w)
\end{Code}

The choice \code{threshold} $= 0.1$ means that a decision on a
particular observation $x_i$ is considered as doubtful, if the
quality of the second best possible decision
($D_{(k-1)}(x_i;\widehat{\theta},\widehat{\pi})$ or
$D_{(k)}(x_{(\rounding{n\alpha}+1)};\widehat{\theta},\widehat{\pi})$
for trimmed observations) is larger than one tenth of the quality
of the actually made decision
($D_{(k)}(x_i;\widehat{\theta},\widehat{\pi})$).

Although Figure~\ref{fig4:f4} suggests to choose $k = 2$, $k$ has been
increased to $3$ in order to show how such a change leads to
doubtful cluster assignment decisions which can be visualized by
\code{DiscrFact}. Figure~\ref{fig4:f5},(a) simply illustrates the cluster
assignments and trimming decisions. The mentioned silhouette plot is
presented in (b), whereas the doubtful decisions are marked in (c).
All observations with $\text{DF}(i) \geq \log(0.1)$ are highlighted
as they are plotted darker/in color. Most of the doubtful decisions
are located in the overlapping area of the two artificially found
clusters (highlighted symbols ``$\times$'' and ``$+$''). Some
doubtfully trimmed observations (highlighted symbol ``$\circ$'') are
located in the boundaries of these two clusters.

\section{Swiss Bank notes data}\label{sec4:exa}
\begin{figure}[t!]
\centering
%\includegraphics[width= 0.65 \textwidth]{plots_R-chunk12}
<<label=fig12, fig=TRUE, echo=false, results=hide, width=7.14, height=6.25>>=
#################
##  Figure 12  ##
#################

data (swissbank)
set.seed (0)
fig6.ctl <- ctlcurves(swissbank, k = 1:4, alpha = seq (0, 0.3, by = 0.025), restr.fact = 50, trace=0)

op <- par (mfrow = c (1, 1), mar = mmar)
plot (fig6.ctl)

par (op)
@
\caption{Classification trimmed likelihoods for $k=1,...,4$ and
$\alpha=0,$ $.025$, ..., $.3$ when \code{restr.fact} $=50$ for the
``Swiss Bank notes'' data set.} \label{fig4:f6}
\end{figure}
The well-known ``Swiss Bank notes'' data set includes 6 numerical
measurements (six-dimensi\-onal data set) made on 100 genuine and
100 counterfeit old Swiss 1000-franc bank notes \citep{FluR88}. The
following code can be used to obtain the classification trimmed
likelihoods shown in Figure~\ref{fig4:f6}.
\begin{Code}
R > data ("swissbank")
R > plot (ctlcurves (swissbank, k = 1:4, alpha = seq (0, 0.3, by = 0.025)))
\end{Code}
This figure indicates the clear existence of $k=2$ main clusters
(``genuine'' and ``forged'' bills). Moreover, considering the clear
difference between $\mathcal{L}_{\texttt{50}}^{\Pi}(0,3)$ and
$\mathcal{L}_{\texttt{50}}^{\Pi}(0,2)$, we can see that a further
cluster, i.e., $k=3$, is needed when no trimming is allowed. This
extra cluster can be justified by the heterogeneity of the group of
forgeries (perhaps due to the presence of different sources of
forged bills).

\begin{figure}[t!]
\centering
%\includegraphics[width= 0.9 \textwidth]{plots_R-chunk13}

<<label=fig13, fig=TRUE, echo=false, results=hide, width=9, height=6.5>>=
#################
##  Figure 13  ##
#################

fig7.clus <- tclust(swissbank, k = 2, alpha = .1, restr.fact = 50)
fig7.discrfact <- DiscrFact(fig7.clus, threshold = .000125)

op <- par (mfrow = c (1, 3), mar = mmar)
plot (fig7.discrfact, enum = TRUE)
par (op)
@
\caption{Clustering results with $k=2$, $\alpha = 0.1$ and
\code{restr.fact} $=50$ summarized by the use of \code{DiscrFact}
function for the ``Swiss Bank notes'' data set. The threshold value
is chosen in order to highlight the 7 most doubtful cluster
assignments.} \label{fig4:f7}
\end{figure}

Considering Figure~\ref{fig4:f6}, the choice $k=2$ and a value of
$\alpha$ close to 0.1 also seem sensible. Notice that
$\mathcal{L}_{50}^{\Pi}(\alpha,3)$ is clearly larger than
$\mathcal{L}_{50}^{\Pi}(\alpha,2)$ for $\alpha < 0.1$ while these
differences are not so big when $\alpha \geq 0.1$. We can even see
smaller differences in the classification trimmed likelihood curves
when increasing $k$ from 3 to 4. However, these differences are less
significant than those previously commented. More spurious clusters
can be surely found but they have less entity and importance.

Figure~\ref{fig4:f7} shows the clustering results with $k=2$, $\alpha =
0.1$ and \code{restr.fact = 50} obtained by executing the code:
\begin{Code}
R > clus <- tclust (swissbank, k = 2, alpha = 0.1, restr.fact = 50)
R > plot (DiscrFact (clus, threshold = 0.0001))
\end{Code}

Notice that, in this example, we did not want to impose a specific
constraint on the solution. Thus, the default parameter
\code{restr.fact = 50} has initially been used in \code{ctlcurves}.
After choosing the combination $\alpha = 0.10$ and $k = 2$, we could
try to reduce the restriction factor which resulted in a warning:

\begin{Code}
R > tclust(swissbank, k = 2, alpha = 0.1, restr.fact = 40)
In .tclust.warn(warnings, ret):
  The result is artificially constrained due to restr.fact = 40.
\end{Code}

Thus the choice \code{restr.fact = 50} seems appropriate as it does
not artificially restrict the result, whereas a slightly smaller
restriction factor (40) does. By examining the sizes of the obtained
groups, we see that no spurious groups are found with
\code{restr.fact = 50}:
\begin{Code}
R > clus$size
  [1] 95 85
\end{Code}

We have used \code{restr = "eigen"} in this example, but \code{restr
= "deter"} can be also successfully applied  with smaller values of
\code{restr.fact}.

We also use the function \code{DiscrFact} to summarize the obtained
clustering results. The two first Fisher's canonical coordinates
derived from the final cluster assignments are plotted. The
threshold value 0.0001 is chosen in order to highlight the 7 most
doubtful decisions.

Finally, Figure~\ref{fig4:f8} shows a scatterplot of the fourth
(``Distance of the inner frame to lower border'') against the sixth
variable (``Length of the diagonal'') with the corresponding cluster
assignments. We use the symbols ``\code{G}'' for the genuine bills
and ``\code{F}'' for the forged ones. The 7 most doubtful decisions
(i.e., the observations with largest $\text{DF}(i)$ values that were
highlighted in Figure~\ref{fig4:f7},(c)) are surrounded by circles in
this figure.
\begin{figure}[t!]
\centering
%\includegraphics[clip,width= 0.9 \textwidth]{plots_R-chunk14}
<<label=fig14, fig=TRUE, echo=false, results=hide, width=9, height=6.5>>=

#################
##  Figure 14  ##
#################


clus <- tclust(swissbank, k = 2, alpha=.1, restr.fact=50)
discrfact <- DiscrFact(clus)
pch <- c (rep ("G", 100), rep ("F", 100))
condition <- discrfact$assignfact > log (.000125)
cl <- clus$cluster
data (swissbank)
op <- par (mfrow = c (1, 3), mar = mmar)

xlab <- "Distance of the inner frame to lower border"
ylab <- "Length of the diagonal"

plot(swissbank[, 4], swissbank[, 6], col = "darkgrey", pch = pch,
      main = "(a) Cluster1", xlab = xlab, ylab = ylab)

cl1 <- cl == 1
points(swissbank[cl1, 4], swissbank[cl1, 6], pch = pch[cl1], col = 2)
idx <- (cl1) & condition
points(swissbank[idx, 4], swissbank[idx, 6], pch = 1, cex = 4, col = "blue")

plot(swissbank[, 4], swissbank[, 6], col = "darkgrey", pch = pch,
      main = "(b) Cluster2", xlab = xlab, ylab = ylab)
cl2 <- cl == 2
points(swissbank[cl2, 4], swissbank[cl2, 6], pch = pch[cl2], col = 3)
idx <- (cl2) & condition
points(swissbank[idx, 4], swissbank[idx, 6], pch = 1, cex = 4, col = "blue")

cl0 <- cl == 0
plot(swissbank[, 4], swissbank[, 6], col = "darkgrey", pch = pch,
      main = "(c) Trimmed", xlab = xlab, ylab = ylab)

points(swissbank[cl0, 4], swissbank[cl0, 6], pch = pch[cl0])
idx <- (cl0) & condition
points(swissbank[idx, 4], swissbank[idx, 6], pch = 1, cex = 4, col = "blue")
par(op)


@
\caption{Clustering results with $k=2$, $\alpha=.1$ and 
    \code{restr.fact} $=50$ for the ``Swiss Bank notes'' data set. 
    Only the fourth and sixth variables are plotted. The 7 most doubtful 
    decisions are rounded by a circle symbol.} \label{fig4:f8}
\end{figure}
We can see that ``Cluster 1'' essentially includes most of the
``forged'' bills while ``Cluster 2'' includes most of the ``genuine''
ones. Among the trimmed observations, we can find a subset of 15
forged bills following a clearly different forgery pattern that has
been previously commented by other authors \citep[see,
e.g.,][]{FluR88,CooP99}. These most doubtful assignments include 5
``genuine'' bills that have perhaps been wrongly trimmed.

\section{Conclusion}\label{sec4:con}
This paper presents a package called \pkg{tclust} for robust
(non-hierarchical) clustering. The implementation is flexible, so
only the restrictions on the cluster scatters have to be changed in
order to carry out different robust clustering algorithms.
Robustness is achieved by trimming a specific proportion of
observations which are identified as the ``most outlying'' ones.

Although this \proglang{R}-package implements robust clustering
approaches which have already been described in the literature, some
of these approaches have been extended to provide increased
flexibility. The package also provides some graphical tools which on
the one hand help to chose appropriate parameters (\code{ctlcurves})
and on the other hand help to estimate the adequacy of a particular
clustering solution (\code{DiscrFact}).

Future work on this package focuses on implementing additional types
of scatter restrictions, making the algorithm even more flexible,
and on providing numerical tools for automatically choosing the
number of clusters and the trimming proportion.

\bibliography{tclust}

\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simul_tclust.R
\name{simula.tclust}
\alias{simula.tclust}
\title{Simulate contaminated data set for applying TCLUST}
\usage{
simula.tclust(n, p = 4, k = 3, type = 2, balanced = 1)
}
\arguments{
\item{n}{number of observations}

\item{p}{dimension (p>=2 and p>q)}

\item{k}{number of clusters (only k=3 and k=6 are allowed!!!)}

\item{type}{1 (spherical for rest.fact=1) or 2 (elliptical for rest.fact=9^2)}

\item{balanced}{1 (all clusters equal size) or 2 [proportions (25,30,35)\% if k=3 and (12.5,15,17.5,12.5,15,17.5)\% if k=6]}
}
\value{
a list with the following items
\itemize{
    \item x - The generated dataset
    \item true - The true classification
}
}
\description{
Simulate 10\% contaminated data set for applying TCLUST
}
\examples{
res <- simula.tclust(n=400,k=3,p=8,type=2,balanced=1)
plot(res$x,col=res$true+1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.tclustICsol.R
\name{plot.tclustICsol}
\alias{plot.tclustICsol}
\title{Plot Method for \code{tclustICsol} Objects}
\usage{
\method{plot}{tclustICsol}(x, whichIC, sol = 1, col, pch, main, sub1, choice, ...)
}
\arguments{
\item{x}{The \code{tclustICsol} object to be displayed}

\item{whichIC}{A character value which specifies which information criteria 
to use. Fpr the possible values for \code{whichIC} see the help of \code{tclustIC}.}

\item{sol}{Which solution to display - a number between 1 and the \code{nsol}
argument of \code{tclustICsol}.}

\item{col}{optional colors to identify the groups. If not specified default 
values will be selected.}

\item{pch}{optional symbols to identify the groups. If not specified default 
values will be selected.}

\item{main}{optional title. The default title shows the information criteria, 
the solution number, number of groups and restriction factor c used and 
indication wheather the solution is true or spurios.}

\item{sub1}{an optional subtitle. The default subtitle shows the type of the 
displaued solution: 'Best in' and 'Stable in'.}

\item{choice}{a numeric vector of length between 2 and the number of 
variables in the input data matrix. If missing, a scatterplot matrix with 
all variables will be shown. If two variables are selected, a simple 
scatterplot of the two selected variables is shown.}

\item{\ldots}{Further (optional) graphical arguments}
}
\description{
Displays one of the solutions, selected by the argument 'sol'.
 The default display is a scatterplot matrix of the data using
 colors and symbols of the observations to identify the groups in the 
 selected solution. If the argument 'choice' is specified and its length 
 is two, a simple scatterplot will be shown.
 The function fails, if \code{store.x = FALSE} is specified in 
 the \code{tclustICsol()} call because the original data matrix is 
 required here.
}
\details{
The plot method for class \code{tclustICsol}.
}
\examples{

 #--- EXAMPLE 1 ------------------------------------------
 \donttest{
 data(geyser2)
 (out <- tclustIC(geyser2, whichIC="MIXMIX", alpha=0.1))

 ##  Show the first two best solutions using as Information criterion MIXMIX
 cat("\nBest solutions using MIXMIX\n")
 outsol <- tclust::tclustICsol(out, whichIC="MIXMIX", nsol=2)
 plot(outsol)
 plot(outsol, choice=c(1,2))
 plot(outsol, choice=c(1,2), xlab="XLAB", ylab="YLAB")
 }

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{pine}
\alias{pine}
\title{Pinus nigra dataset}
\format{
A data frame containing 362 observations in 2 variables. 
The variables are as follows:

\itemize{
     \item \code{Diameter} Diameter
     \item \code{HT} Height
 }
}
\usage{
data(pine)
}
\description{
To study the growth of the wood mass in a cultivated forest of \emph{Pinus nigra} 
 located in the north of Palencia (Spain), a sample of 362 trees was studied. 
 The data set is made of measurements of heights (in meters), in variable "HT", 
 and diameters (in millimetres), in variable "Diameter", of these trees. 
 The presence of three linear groups can be guessed apart from a small group 
 of trees forming its own cluster with larger heights and diameters one isolated 
 tree with the largest diameter but small height. More details on the 
 interpretation of this dataset in García-Escudero et al (2010).
}
\references{
García-Escudero, L. A., Gordaliza, A., Mayo-Iscar, A., and San Martín, R. (2010). 
 Robust clusterwise linear regression through trimming. 
 \emph{Computational Statistics & Data Analysis}, 54(12), 3057--3069.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tkmeans.R
\name{tkmeans}
\alias{tkmeans}
\alias{print.tkmeans}
\title{TKMEANS method for robust K-means clustering}
\usage{
tkmeans(
  x,
  k,
  alpha = 0.05,
  nstart = 500,
  niter1 = 3,
  niter2 = 20,
  nkeep = 5,
  iter.max,
  points = NULL,
  center = FALSE,
  scale = FALSE,
  store_x = TRUE,
  parallel = FALSE,
  n.cores = -1,
  zero_tol = 1e-16,
  drop.empty.clust = TRUE,
  trace = 0
)
}
\arguments{
\item{x}{A matrix or data.frame of dimension n x p, containing the observations (row-wise).}

\item{k}{The number of clusters initially searched for.}

\item{alpha}{The proportion of observations to be trimmed.}

\item{nstart}{The number of random initializations to be performed.}

\item{niter1}{The number of concentration steps to be performed for the nstart initializations.}

\item{niter2}{The maximum number of concentration steps to be performed for the 
\code{nkeep} solutions kept for further iteration. The concentration steps are 
stopped, whenever two consecutive steps lead to the same data partition.}

\item{nkeep}{The number of iterated initializations (after niter1 concentration 
steps) with the best values in the target function that are kept for further iterations}

\item{iter.max}{(deprecated, use the combination \code{nkeep, niter1 and niter2}) 
The maximum number of concentration steps to be performed.
The concentration steps are stopped, whenever two consecutive steps lead
to the same data partition.}

\item{points}{Optional initial mean vectors, \code{NULL} or a matrix with \code{k} 
vectors used as means to initialize the algorithm. If initial mean vectors are 
specified, \code{nstart} should be 1 (otherwise the same initial means are 
used for all runs).}

\item{center}{Optional centering of the data: a function or a vector of length p 
which can optionally be specified for centering x before calculation}

\item{scale}{Optional scaling of the data: a function or a vector of length p 
which can optionally be specified for scaling x before calculation}

\item{store_x}{A logical value, specifying whether the data matrix \code{x} shall be 
included in the result object. By default this value is set to \code{TRUE}, because 
some of the plotting functions depend on this information. However, when big data 
matrices are handled, the result object's size can be decreased noticeably 
when setting this parameter to \code{FALSE}.}

\item{parallel}{A logical value, specifying whether the nstart initializations should be done in parallel.}

\item{n.cores}{The number of cores to use when paralellizing, only taken into account if parallel=TRUE.}

\item{zero_tol}{The zero tolerance used. By default set to 1e-16.}

\item{drop.empty.clust}{Logical value specifying, whether empty clusters shall be 
omitted in the resulting object. (The result structure does not contain center 
estimates of empty clusters anymore. Cluster names are reassigned 
such that the first l clusters (l <= k) always have at least one observation.}

\item{trace}{Defines the tracing level, which is set to 0 by default. Tracing level 1 
gives additional information on the stage of the iterative process.}
}
\value{
The function returns the following values:
\itemize{
    \item cluster - A numerical vector of size \code{n} containing the cluster assignment 
         for each observation. Cluster names are integer numbers from 1 to k, 0 indicates 
         trimmed observations. Note that it could be empty clusters with no observations 
         when \code{equal.weights=FALSE}.
    \item obj - The value of the objective function of the best (returned) solution.
    \item size - An integer vector of size k, returning the number of observations contained by each cluster.
    \item centers - A matrix of size p x k containing the centers (column-wise) of each cluster. 
    \item code - A numerical value indicating if the concentration steps have 
         converged for the returned solution (2).
    \item cluster.ini - A matrix with nstart rows and number of columns equal to 
         the number of observations and where each row shows the final clustering 
         assignments (0 for trimmed observations) obtained after the \code{niter1} 
         iteration of the \code{nstart} random initializations.
    \item obj.ini - A numerical vector of length \code{nstart} containing the values 
         of the target function obtained after the \code{niter1} iteration of the 
         \code{nstart} random initializations.
    \item x - The input data set.
    \item k - The input number of clusters.
    \item alpha - The input trimming level.
}
}
\description{
This function searches for \code{k} (or less) spherical clusters 
 in a data matrix \code{x}, whereas the \code{ceiling(alpha n)} most outlying 
 observations are trimmed.
}
\examples{

 \dontshow{
     set.seed(0)
 }
 ##--- EXAMPLE 1 ------------------------------------------
 sig <- diag(2)
 cen <- rep(1,2)
 x <- rbind(MASS::mvrnorm(360, cen * 0,   sig),
            MASS::mvrnorm(540, cen * 5,   sig),
            MASS::mvrnorm(100, cen * 2.5, sig))
 
 ## Two groups and 10\\% trimming level
 (clus <- tkmeans(x, k = 2, alpha = 0.1))

 plot(clus)
 plot(clus, labels = "observation")
 plot(clus, labels = "cluster")

 #--- EXAMPLE 2 ------------------------------------------
 data(geyser2)
 (clus <- tkmeans(geyser2, k = 3, alpha = 0.03))
 plot(clus)
 
}
\references{
Cuesta-Albertos, J. A.; Gordaliza, A. and Matrán, C. (1997), "Trimmed k-means: 
 an attempt to robustify quantizers". Annals of Statistics, Vol. 25 (2), 553-576.
}
\author{
Valentin Todorov, Luis Angel Garcia Escudero, Agustin Mayo Iscar.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/randIndex.R
\name{randIndex}
\alias{randIndex}
\title{Calculates Rand type Indices to compare two partitions}
\usage{
randIndex(c1, c2 = NULL, noisecluster = NULL)
}
\arguments{
\item{c1}{labels of the first partition or contingency table. A numeric vector
or factor containining the class labels of the first partition or a 
2-dimensional numeric matrix which contains the cross-tabulation of 
cluster assignments.}

\item{c2}{labels of the second partition. A numeric vector or a factor 
containining the class labels of the second partition. The length of 
the vector \code{c2} must be equal to the length of the vector \code{c1}. 
The second parameter is required only if c1 is not a 2-dimensional numeric matrix.}

\item{noisecluster}{label or number associated to the 'noise class' or 'noise level'.
Number or character label which denotes the points which do not belong to any
cluster. These points are not takern into account for the computation of the 
Rand type indexes. The default is to consider all points.}
}
\value{
A list with Rand type indexes:
\itemize{
    \item AR Adjusted Rand index. A number between -1 and 1. The adjusted 
        Rand index is the corrected-for-chance version of the Rand index. 
    \item RI Rand index (unadjusted). A number between 0 and 1. Rand index 
        computes the fraction of pairs of objects for which both 
        classification methods agree. RI ranges from 0 (no pair classified 
        in the same way under both clusterings) to 1 (identical clusterings).
    \item MI Mirkin's index. A number between 0 and 1. Mirkin's index computes 
        the percentage of pairs of objects for which both classification 
        methods disagree. \code{MI=1-RI}.
    \item HI Hubert index. A number between -1 and 1. HI index is equal to the 
        fraction of pairs of objects for which both classification methods 
        agree minus the fraction of pairs of objects for which both classification 
        methods disagree. \code{HI= RI-MI}. 
}
}
\description{
Calculates Rand type Indices to compare two partitions
}
\examples{
##  1. randindex with the contingency table as input.
T <- matrix(c(1, 1, 0, 1, 2, 1, 0, 0, 4), nrow=3)
(ARI <- randIndex(T))

##  2. randindex with the two vectors as input.
c <- matrix(c(1, 1, 1, 2, 2, 1, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3), ncol=2, byrow=TRUE)
## c1 = numeric vector containing the labels of the first partition
c1 <- c[,1]
## c2 = numeric vector containing the labels of the second partition
c2 <- c[,2]

(ARI <- randIndex(c1,c2))

##  3. Compare ARI for iris data (true classification against tclust classification)
library(tclust)
c1 <- iris$Species  # first partition c1 is the true partition
out <- tclust(iris[, 1:4], k=3, alpha=0, restr.fact=100)
c2 <- out$cluster   # second partition c2 is the output of tclust clustering procedure

randIndex(c1,c2)

##  4. Compare ARI for iris data (exclude unassigned units from tclust).

c1 <- iris$Species      # first partition c1 is the true partition
out <- tclust(iris[,1:4], k=3, alpha=0.1, restr.fact=100)
c2 <- out$cluster       #  second partition c2 is the output of tclust clustering procedure

## Units inside c2 which contain number 0 are referred to trimmed observations
noisecluster <- 0
randIndex(c1, c2, noisecluster=0)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FowlkesMallowsIndex.R
\name{FowlkesMallowsIndex}
\alias{FowlkesMallowsIndex}
\alias{print.FowlkesMallowsIndex}
\title{Computes the Fowlkes and Mallows index}
\usage{
FowlkesMallowsIndex(c1, c2 = NULL, noisecluster = NULL)
}
\arguments{
\item{c1}{Labels of the first partition or contingency table. A numeric or 
character vector containing the class labels of the first partition or 
a 2-dimensional numeric matrix which contains the cross-tabulation 
of cluster assignments.}

\item{c2}{Labels of the second partition. A numeric or character vector 
containing the class labels of the second partition. 
The length of vector c2 must be equal to the length of vector c1. 
This second input is required only if \code{c1} is not a 2-dimensional numeric matrix.}

\item{noisecluster}{Label or number associated to the \emph{noise class} or \emph{noise level}.
Number or character label which denotes the points which do not belong to any cluster.
These points are not takern into account for the  computation of the Fowlkes and Mallows index}
}
\value{
A list containing the following components:
 \item{ABK}{Adjusted Fowlkes and Mallows index. A number between -1 and 1. The adjusted 
 Fowlkes and Mallows index is the corrected-for-chance version of the Fowlkes and Mallows index.}
 \item{BK}{Value of the Fowlkes and Mallows index. A number between 0 and 1. }
 \item{EBk}{Expected value of the index computed under the null hypothesis of no-relation.}
 \item{VarBk}{Variance of the Fowlkes and Mallows index. Variance of the index computed under the null
     hypothesis of no-relation.}
}
\description{
Fowlkes-Mallows index is an external evaluation method
 that is used to determine the similarity between two clusterings
 (clusters obtained after a clustering algorithm). This measure of
 similarity could be either between two hierarchical clusterings or a
 clustering and a benchmark classification. A higher the value for the
 Fowlkes-Mallows index indicates a greater similarity between the clusters
 and the benchmark classifications.
 This index can be used to compare either two cluster label sets or a
 cluster label set with a true label set. The formula of the
 adjusted Fowlkes-Mallows index (ABk) is given in the details.
}
\details{
The formula of the adjusted Fowlkes-Mallows index (ABk) is as follows:

 \deqn{ABk=\frac{Bk-Expected~value~of~Bk}{Max~Index - Expected~value~of~Bk}}
}
\examples{

 ##  1. FowlkesMallowsIndex (adjusted) with the two vectors as input
 c <- matrix(c(1, 1, 1, 2, 2, 1,2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3), 
         ncol=2, byrow=TRUE)

 ##  c1 - numeric vector containing the labels of the first partition
 c1 <- c[, 1]

 ##  c2 - numeric vector containing the labels of the second partition
 c2 <- c[, 2]

 (FM <- FowlkesMallowsIndex(c1, c2))

 ##  2. FM index (adjusted) with the contingency table as input.
 T <- matrix(c(1, 1, 0, 1, 2, 1, 0, 0, 4), ncol=3, byrow=TRUE)
 (FM <- FowlkesMallowsIndex(T))


 ##  3. Compare FM (unadjusted) for iris data (true classification against 
 ##      tclust classification).

 ##  First partition c1 is the true partition
 c1 <- iris$Species

 ##  Second partition c2 is the output of tclust clustering procedure
 out <- tclust(iris[, 1:4], k=3, alpha=0, restr.fact=100)
 c2<- out$cluster

 (FM <- FowlkesMallowsIndex(c1, c2))

 ##  4. Compare FM index (unadjusted) for iris data (exclude unassigned units from tclust).

 ##  First partition c1 is the true partition
 c1 <- iris$Species

 ##  Second partition c2 is the output of tclust clustering procedure
 out <- tclust(iris[, 1:4], k=3, alpha=0.1, restr.fact=100)
 c2<- out$cluster

 ##  Units inside c2 which contain number 0 are referred to trimmed observations
 noisecluster <- 0
 (FM <- FowlkesMallowsIndex(c1, c2, noisecluster=noisecluster))

}
\references{
Fowlkes, E.B. and Mallows, C.L. (1983), A Method for Comparing Two Hierarchical Clusterings, 
 \emph{Journal of the American Statistical Association}, Vol. 78, pp. 553--569.
}
\seealso{
\code{\link{randIndex}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DiscrFact.R
\name{summary.DiscrFact}
\alias{summary.DiscrFact}
\title{The \code{summary} method for objects of class \code{DiscrFact}}
\usage{
\method{summary}{DiscrFact}(object, hide.emtpy = TRUE, show.clust, show.alt, ...)
}
\arguments{
\item{object}{An object of class \code{DiscrFact} as returned from \code{DiscrFact()}.}

\item{hide.emtpy}{A logical value specifying whether clusters without doubtful 
assignment shall be hidden.}

\item{show.clust}{A logical value specifying whether the number of doubtful 
assignments per cluster shall be displayed.}

\item{show.alt}{A logical value specifying whether the alternative cluster 
assignment shall be displayed.}

\item{\ldots}{Arguments passed to or from other methods.}
}
\description{
The summary method for class \code{DiscrFact}.
}
\examples{
 sig <- diag (2)
 cen <- rep (1, 2)
 x <- rbind(MASS::mvrnorm(360, cen * 0,   sig),
 	       MASS::mvrnorm(540, cen * 5,   sig * 6 - 2),
 	       MASS::mvrnorm(100, cen * 2.5, sig * 50)
 )

 clus.1 <- tclust(x, k = 2, alpha=0.1, restr.fact=12)
 clus.2 <- tclust(x, k = 3, alpha=0.1, restr.fact=1)

 dsc.1 <- DiscrFact(clus.1)
 summary(dsc.1)

 dsc.2 <- DiscrFact(clus.2)
 summary(dsc.2)

}
\references{
\enc{García}{Garcia}-Escudero, L.A.; Gordaliza, A.; \enc{Matrán}{Matran}, C. and Mayo-Iscar, A. (2011), 
   "Exploring the number of groups in robust model-based clustering." \emph{Statistics and Computing}, \bold{21} 
   pp. 585-599, <doi:10.1007/s11222-010-9194-z>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.tclust.R
\name{plot.tclust}
\alias{plot.tclust}
\alias{plot.tkmeans}
\title{Plot Method for \code{tclust} and \code{tkmeans} Objects}
\usage{
\method{plot}{tclust}(x, ...)

\method{plot}{tkmeans}(x, ...)
}
\arguments{
\item{x}{The \code{tclust} or \code{tkmeans} object to be displayed}

\item{\ldots}{Further (optional) arguments which specify the details of the 
resulting plot (see section "Further Arguments").}
}
\description{
One and two dimensional structures are treated separately (e.g. tolerance 
 intervals/ellipses are displayed). Higher dimensional structures are displayed 
 by plotting the two first Fisher's canonical coordinates (evaluated by 
 \code{tclust::discr_coords}) and derived from the final cluster assignments 
 (trimmed observations are not taken into account). 
 \code{plot.tclust.Nd} can be called with one or two-dimensional \code{tclust}- or \code{tkmeans}-objects 
 too. The function fails, if \code{store.x = FALSE} is specified in the \code{tclust()} or \code{tkmeans()} call, 
 because the original data matrix is required here.
}
\details{
The plot method for classes \code{tclust} and \code{tkmeans}.
}
\section{Further Arguments}{


\itemize{
 \item \code{xlab, ylab, xlim, ylim, pch, col} Arguments passed to \code{plot()}.
 \item \code{main} The title of the plot. Use \code{"/p"} for displaying the chosen parameters 
     \code{alpha} and \code{k} or \code{"/r"} for plotting the chosen restriction.
 \item \code{main.pre} An optional string which is added to the plot's caption.
 \item \code{sub} A string specifying the subtitle of the plot. Use \code{"/p"} (default) for 
 displaying the chosen parameters \code{alpha} and \code{k}, \code{"/r"} for plotting 
 the chosen restriction and \code{"/pr"} for both. 
 \item \code{sub1} A secondary (optional) subtitle.
 \item \code{labels} A string specifying the type of labels to be drawn. Either 
     \code{labels="none"} (default), \code{labels="cluster"} or \code{labels="observation"} 
     can be specified. If specified, parameter \code{pch} is ignored.
 \item \code{text} A vector of length n (the number of observations) containing 
     strings which are used as labels for each observation. If specified, 
     the parameters \code{labels} and \code{pch} are ignored.
 \item \code{by.cluster} Logical value indicating whether parameters 
     \code{pch} and \code{col} refer to observations (FALSE) or clusters (TRUE).
 \item \code{jitter.y} Logical value, specifying whether the drawn values shall be 
     jittered in y-direction for better visibility of structures in 1 dimensional data.
 \item \code{tol} The tolerance interval. 95\% tolerance ellipsoids (assuming normality) 
     are plotted by default.
 \item \code{tol.col, tol.lty, tol.lwd} Vectors of length k or 1 containing 
     the \code{col}, \code{lty} and \code{lwd} arguments for the tolerance 
     ellipses/lines.
}
}

\examples{
 #--- EXAMPLE 1------------------------------
 sig <- diag (2)
 cen <- rep (1, 2)
 x <- rbind(MASS::mvrnorm(360, cen * 0,   sig),
 	       MASS::mvrnorm(540, cen * 5,   sig * 6 - 2),
 	       MASS::mvrnorm(100, cen * 2.5, sig * 50))
 # Two groups and 10\\% trimming level
 a <- tclust(x, k = 2, alpha = 0.1, restr.fact = 12)
 plot (a)
 plot (a, labels = "observation")
 plot (a, labels = "cluster")
 plot (a, by.cluster = TRUE)
 #--- EXAMPLE 2------------------------------
 sig <- diag (2)
 cen <- rep (1, 2)
 x <- rbind(MASS::mvrnorm(360, cen * 0,   sig),
 	       MASS::mvrnorm(540, cen * 5,   sig),
 	       MASS::mvrnorm(100, cen * 2.5, sig))
 # Two groups and 10\\% trimming level
 a <- tkmeans(x, k = 2, alpha = 0.1)
 plot (a)
 plot (a, labels = "observation")
 plot (a, labels = "cluster")
 plot (a, by.cluster = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rlg.R
\name{rlg}
\alias{rlg}
\title{Robust Linear Grouping}
\usage{
rlg(
  x,
  d,
  alpha = 0.05,
  nstart = 500,
  niter1 = 3,
  niter2 = 20,
  nkeep = 5,
  scale = FALSE,
  parallel = FALSE,
  n.cores = -1,
  trace = FALSE
)
}
\arguments{
\item{x}{A matrix or data.frame of dimension n x p, containing the observations (rowwise).}

\item{d}{A numeric vector of length equal to the number of clusters to be detected. 
Each component of vector \code{d} indicates the intrinsic dimension of the affine subspace 
where observations on that cluster are going to be clustered. All the elements 
of vector \code{d} should be smaller than the problem dimension minus 1.}

\item{alpha}{The proportion of observations to be trimmed.}

\item{nstart}{The number of random initializations to be performed.}

\item{niter1}{The number of concentration steps to be performed for the nstart initializations.}

\item{niter2}{The maximum number of concentration steps to be performed for 
the nkeep solutions kept for further iteration. The concentration steps 
are stopped, whenever two consecutive steps lead to the same data partition.}

\item{nkeep}{The number of iterated initializations (after niter1 concentration 
steps) with the best values in the target function that are kept for further iterations}

\item{scale}{A robust centering and scaling (using the median and MAD) is done if TRUE.}

\item{parallel}{A logical value, specifying whether the nstart initializations should be done in parallel.}

\item{n.cores}{The number of cores to use when paralellizing, only taken into account if parallel=T.}

\item{trace}{Defines the tracing level, which is set to 0 by default. Tracing level 1 gives additional information on the stage of the iterative process.}
}
\value{
Returns an object of class \code{rlg} which is basically a list with the following elements:
\itemize{
    \item centers - A matrix of size p x k containing the location vectors (column-wise) of each cluster. 
    \item U - A list with k elements where each element is p x d_j matrix whose d_j columns are unitary and orthogonal vectors generating the affine subspace (after subtracting the corresponding cluster’s location parameter in centers). d_j is the intrinsic dimension of the affine subspace approximation in the j-th cluster, i.e., the elements of vector d.
    \item cluster - A numerical vector of size n containing the cluster assignment 
         for each observation. Cluster names are integer numbers from 1 to k, 
         0 indicates trimmed observations.
    \item obj - The value of the objective function of the best (returned) solution.
    \item cluster.ini - A matrix with nstart rows and number of columns equal to the number of observations and where each row shows the final clustering assignments (0 for trimmed observations) obtained after the niter1 iteration of the nstart random initializations.
    \item obj.ini -A numerical vector of length nstart containing the values of the target function obtained after the niter1 iteration of the nstart random initializations.
    \item x - The input data set. 
    \item dimensions - The input d vector with the intrinsic dimensions. The number of clusters is the length of that vector. 
    \item alpha - The input trimming level.
}
}
\description{
The function \code{rlg()} searches for clusters around affine subspaces of dimensions given by 
 vector \code{d} (the length of that vector is the number of clusters). For instance \code{d=c(1,2)} 
 means that we are clustering around a line and a plane. For robustifying the estimation, 
 a proportion \code{alpha} of observations is trimmed. In particular, the trimmed k-means 
 method is represented by the rlg method, if \code{d=c(0,0,..0)} (a vector of length 
 \code{k} with zeroes).
}
\details{
The procedure allows to deal with robust clustering around affine subspaces 
   with an alpha proportion of trimming level by minimizing the trimmed sums of squared 
   orthogonal residuals. Each component of vector \code{d} indicates the intrinsic dimension of 
   the affine subspace where observations on that cluster are going to be clustered. 
   Therefore a component equal to 0 on that vector implies clustering around centres, 
   equal to 1 around lines, equal to 2 around planes and so on. The procedure so 
   allows simultaneous clustering and dimensionality reduction. 

This iterative algorithm performs "concentration steps" to improve the current 
 cluster assignments. For approximately obtaining the global optimum, the procedure 
 is randomly initialized \code{nstart} times and \code{niter1} concentration steps are performed 
 for them. The \code{nkeep} most “promising” iterations, i.e. the \code{nkeep} iterated solutions 
 with the initial best values for the target function, are then iterated until 
 convergence or until \code{niter2} concentration steps are done.
}
\examples{
##--- EXAMPLE 1 ------------------------------------------
data (LG5data)
x <- LG5data[, 1:10]
clus <- rlg(x, d = c(2,2,2), alpha=0.1)
plot(x, col=clus$cluster+1)
plot(clus, which="eigenvalues") 
plot(clus, which="scores") 

##--- EXAMPLE 2 ------------------------------------------
 data (pine) 
 clus <- rlg(pine, d = c(1,1,1), alpha=0.035)
 plot(pine, col=clus$cluster+1)
 
}
\references{
García‐Escudero, L. A., Gordaliza, A., San Martin, R., Van Aelst, S., & Zamar, R. (2009). 
 Robust linear clustering. Journal of the Royal Statistical Society: 
 Series B (Statistical Methodology), 71, 301-318.
}
\author{
Javier Crespo Guerrero, Jesús Fernández Iglesias, Luis Angel Garcia Escudero, Agustin Mayo Iscar.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{wholesale}
\alias{wholesale}
\title{Wholesale customers dataset}
\format{
A data frame containing 440 observations in 8 variables (6 numerical and two categorical). 
The variables are as follows:

\itemize{
     \item \code{Region} Customers' Region - Lisbon (coded as 1), Porto (coded as 2) or Other (coded as 3)
     \item \code{Fresh} Annual spending on fresh products
     \item \code{Milk} Annual spending on milk products
     \item \code{Grocery} Annual spending on grocery products
     \item \code{Frozen} Annual spending on frozen products
     \item \code{Detergents} Annual spending on detergents and paper products
     \item \code{Delicatessen} Annual spending on and delicatessen products
     \item \code{Channel}  Customers' Channel - Horeca (Hotel/Restaurant/Café) or 
             Retail channel. Horeca is coded as 1 and Retail channel is coded as 2
 }
}
\source{
Abreu, N. (2011). Analise do perfil do cliente Recheio e desenvolvimento de 
 um sistema promocional. Mestrado em Marketing, ISCTE-IUL, Lisbon. 
 url={https://api.semanticscholar.org/CorpusID:124027622}
}
\usage{
data(wholesale)
}
\description{
The data set refers to clients of a wholesale distributor. It includes the annual 
 spending in monetary units  on diverse product categories.
}
\examples{
#--- EXAMPLE 1 ------------------------------------------ 
data (wholesale) 
x <- wholesale[, -c(1, ncol(wholesale))] 
clus <- tclust(x, k=3, alpha=0.1, nstart=200, niter1=3, niter2=17, 
   nkeep=10, opt="HARD", equal.weights=FALSE, restr.fact=50, trace=TRUE) 
 plot (x, col=clus$cluster+1)
 plot(clus)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tclustIC.R
\name{tclustIC}
\alias{tclustIC}
\alias{print.tclustIC}
\title{Performs cluster analysis by calling \code{\link{tclust}} for different
 number of groups \code{k} and restriction factors \code{c}}
\usage{
tclustIC(
  x,
  kk = 1:5,
  cc = c(1, 2, 4, 8, 16, 32, 64, 128),
  alpha = 0.05,
  whichIC = c("ALL", "MIXMIX", "MIXCLA", "CLACLA"),
  parallel = FALSE,
  n.cores = -1,
  trace = FALSE,
  ...
)
}
\arguments{
\item{x}{A matrix or data frame of dimension n x p, containing the observations (row-wise).}

\item{kk}{an integer vector specifying the number of mixture components (clusters) 
for which the information criteria are be calculated. By default \code{kk=1:5}.}

\item{cc}{an  vector specifying the values of the restriction factor which have to 
be considered. By default \code{cc=c(1, 2, 4, 8, 16, 32, 64, 128)}.}

\item{alpha}{The proportion of observations to be trimmed.}

\item{whichIC}{A character value which specifies which information criteria must be computed
for each \code{k} (number of groups) and each value of the restriction factor 
\code{c}. Possible values for \code{whichIC} are:
\itemize{
 \item "MIXMIX": a mixture model is fitted and for computing the information criterion
    the mixture likelihood is used. This option corresponds to the use of the Bayesian
    Information criterion (BIC). In output just the matrix \code{MIXMIX} is given.
\item "MIXCLA": a mixture model is fitted but to compute the information criterion
    the classification likelihood is used. This option corresponds to the use of the
    Integrated Complete Likelihood (ICL). In the output just the matrix \code{MIXCLA} is given.
\item "CLACLA": everything is based on the classification likelihood. This information
    criterion will be called CLA. In the output just the matrix \code{CLACLA} is given.
\item "ALL": both classification and mixture likelihood are used. In this case all
    three information criteria CLA, ICL and BIC are computed. In the output all
    three matrices \code{MIXMIX}, \code{MIXCLA} and \code{CLACLA} are given.
}}

\item{parallel}{A logical value, specifying whether the calls to \code{\link{tclust}} should be done in parallel.}

\item{n.cores}{The number of cores to use when paralellizing, only taken into account if \code{parallel=TRUE}.}

\item{trace}{Whether to print intermediate results. Default is \code{trace=FALSE}.}

\item{\ldots}{Further arguments (as e.g. \code{restr}), passed to \code{\link{tclust}}}
}
\value{
The functions \code{print()} and \code{summary()} are used to obtain and print a
 summary of the results. The function returns an S3 object of type 
 \code{tclustIC} containing the following components:
\itemize{
 \item call the matched call
 \item kk a vector containing the values of \code{k} (number of components) which have been considered.
     This vector is identical to the optional argument \code{kk} (default is \code{kk=1:5}.
 \item cc a vector containing the values of \code{c} (values of the restriction factor) which
     have been considered. This vector is identical to the optional argument \code{cc} (defalt is \code{cc=c(1, 2, 4, 8, 16, 32, 64, 128)}.
 \item alpha trimming level
 \item whichIC Information criteria used 
 \item CLACLA a matrix of size \code{length(kk)-times-length(cc)} containinig the value of
     the penalized classification likelihood. This output is present only if \code{whichIC="CLACLA"} or  \code{whichIC="ALL"}.
 \item IDXCLA a matrix of lists of size \code{length(kk)-times-length(cc)} containinig the assignment of each unit
     using the classification model. This output is present only if \code{whichIC="CLACLA"} or  \code{whichIC="ALL"}.
 \item MIXMIX a matrix of size \code{length(kk)-times-length(cc)} containinig the value of
     the penalized mixtrue likelihood. This output is present only if \code{whichIC="MIXMIX"} or  \code{whichIC="ALL"}.
 \item IDXMIX a matrix of lists of size \code{length(kk)-times-length(cc)} containinig the assignment of each unit
     using the classification model. This output is present only if \code{whichIC="MIXMIX"} or  \code{whichIC="ALL"}.
 \item MIXCLA a matrix of size \code{length(kk)-times-length(cc)} containinig the value of
     the ICL criterion. This output is present only if \code{whichIC="MIXCLA"} or  \code{whichIC="ALL"}.
 \item x the input data matrix of size \code{length(n)-times-length(p)} with which 
     the Information Criteria were computed.
 }
}
\description{
Computes the values of BIC (MIXMIX), ICL (MIXCLA) or CLA (CLACLA),
 for different values of \code{k} (number of groups) and different values of \code{c}
 (restriction factor), for a prespecified level of trimming (the last two letters in the name
 stand for 'Information Criterion').
}
\examples{

 #--- EXAMPLE 1 ------------------------------------------
 \donttest{
 data(geyser2)
 (out <- tclustIC(geyser2, whichIC="MIXMIX", alpha=0.1))
 summary(out)
 ## Find the smallest value inside the table and write the corresponding
 ## values of k (number of groups) and c (restriction factor)
 inds <- which(out$MIXMIX == min(out$MIXMIX), arr.ind=TRUE)
 vals <- out$MIXMIX[inds]
 cat("\nThe smallest value of the IC is ", vals, 
     " and takes place for k=", out$kk[inds[1]], " and c=",   
     out$cc[inds[2]], "\n")
 }

 #--- EXAMPLE 2 ------------------------------------------
 \donttest{
 data(flea)
 Y <- as.matrix(flea[, 1:(ncol(flea)-1)])    # select only the numeric variables
 rownames(Y) <- 1:nrow(Y)
 head(Y)

 (out <- tclustIC(Y, whichIC="CLACLA", alpha=0.1))
 summary(out)
 ## Find the smallest value inside the table and write the corresponding
 ## values of k (number of groups) and c (restriction factor)
 inds <- which(out$CLACLA == min(out$CLACLA), arr.ind=TRUE)
 vals <- out$CLACLA[inds]
 cat("\nThe Smallest value of the IC is ", vals, 
     " and takes place for k=", out$kk[inds[1]], " and c=",   
     out$cc[inds[2]], "\n")
 }

 #--- EXAMPLE 3 ------------------------------------------
 \donttest{
 data(swissbank)
 (out <- tclustIC(swissbank, whichIC="ALL"))
 
 plot(out)  ##  --> selecting k=3, c=128
 
 ##  the selected model
 plot(tclust(swissbank, k = 3, alpha = 0.1, restr.fact = 128))
 
 }

}
\references{
Cerioli, A., Garcia-Escudero, L.A., Mayo-Iscar, A. and Riani M. (2017).
     Finding the Number of Groups in Model-Based Clustering via Constrained Likelihoods,
     \emph{Journal of Computational and Graphical Statistics}, pp. 404-416,
     https://doi.org/10.1080/10618600.2017.1390469.
}
\seealso{
\code{\link{tclust}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tclust.R
\name{tclust}
\alias{tclust}
\alias{print.tclust}
\title{TCLUST method for robust clustering}
\usage{
tclust(
  x,
  k,
  alpha = 0.05,
  nstart = 500,
  niter1 = 3,
  niter2 = 20,
  nkeep = 5,
  iter.max,
  equal.weights = FALSE,
  restr = c("eigen", "deter"),
  restr.fact = 12,
  cshape = 1e+10,
  opt = c("HARD", "MIXT"),
  center = FALSE,
  scale = FALSE,
  store_x = TRUE,
  parallel = FALSE,
  n.cores = -1,
  zero_tol = 1e-16,
  drop.empty.clust = TRUE,
  trace = 0
)
}
\arguments{
\item{x}{A matrix or data.frame of dimension n x p, containing the observations (row-wise).}

\item{k}{The number of clusters initially searched for.}

\item{alpha}{The proportion of observations to be trimmed.}

\item{nstart}{The number of random initializations to be performed.}

\item{niter1}{The number of concentration steps to be performed for the nstart initializations.}

\item{niter2}{The maximum number of concentration steps to be performed for the 
\code{nkeep} solutions kept for further iteration. The concentration steps are 
stopped, whenever two consecutive steps lead to the same data partition.}

\item{nkeep}{The number of iterated initializations (after niter1 concentration 
steps) with the best values in the target function that are kept for further iterations}

\item{iter.max}{(deprecated, use the combination \code{nkeep, niter1 and niter2}) 
The maximum number of concentration steps to be performed.
The concentration steps are stopped, whenever two consecutive steps lead
to the same data partition.}

\item{equal.weights}{A logical value, specifying whether equal cluster weights 
shall be considered in the concentration and assignment steps.}

\item{restr}{Restriction type to control relative cluster scatters. 
The default value is \code{restr="eigen"}, so that the maximal ratio between 
the largest and the smallest of the scatter matrices eigenvalues is constrained 
to be smaller then or equal to \code{restr.fact} 
(Garcia-Escudero, Gordaliza, Matran, and Mayo-Iscar, 2008). 
Alternatively, \code{restr="deter"} imposes that the maximal ratio between 
the largest and the smallest of the scatter matrices determinants is smaller 
or equal than \code{restr.fact} (see Garcia-Escudero, Mayo-Iscar and Riani, 2020)}

\item{restr.fact}{The constant \code{restr.fact >= 1} constrains the allowed 
differences among group scatters in terms of eigenvalues ratio
(if \code{restr="eigen"}) or determinant ratios (if \code{restr="deter"}). Larger values 
imply larger differences of group scatters, a value of 1 specifies the 
strongest restriction.}

\item{cshape}{constraint to apply to the shape matrices, \code{cshape >= 1}, 
(see Garcia-Escudero, Mayo-Iscar and Riani, 2020)). 
This options only works if \code{restr=='deter'}. In this case the default 
value is \code{cshape=1e10} to ensure the procedure is (virtually) affine equivariant. 
On the other hand, \code{cshape} values close to 1 would force the clusters to 
be almost spherical (without necessarily the same scatters if \code{restr.fact} 
is strictly greater than 1).}

\item{opt}{Define the target function to be optimized. A classification likelihood 
target function is considered if \code{opt="HARD"} and a mixture classification 
likelihood if \code{opt="MIXT"}.}

\item{center}{Optional centering of the data: a function or a vector of length p 
which can optionally be specified for centering x before calculation}

\item{scale}{Optional scaling of the data: a function or a vector of length p 
which can optionally be specified for scaling x before calculation}

\item{store_x}{A logical value, specifying whether the data matrix \code{x} shall be 
included in the result object. By default this value is set to \code{TRUE}, because 
some of the plotting functions depend on this information. However, when big data 
matrices are handled, the result object's size can be decreased noticeably 
when setting this parameter to \code{FALSE}.}

\item{parallel}{A logical value, specifying whether the nstart initializations should be done in parallel.}

\item{n.cores}{The number of cores to use when paralellizing, only taken into account if \code{parallel=TRUE}.}

\item{zero_tol}{The zero tolerance used. By default set to 1e-16.}

\item{drop.empty.clust}{Logical value specifying, whether empty clusters shall be 
omitted in the resulting object. (The result structure does not contain center 
and covariance estimates of empty clusters anymore. Cluster names are reassigned 
such that the first l clusters (l <= k) always have at least one observation.}

\item{trace}{Defines the tracing level, which is set to 0 by default. Tracing level 1 
gives additional information on the stage of the iterative process.}
}
\value{
The function returns the following values:
\itemize{
    \item cluster - A numerical vector of size \code{n} containing the cluster assignment 
         for each observation. Cluster names are integer numbers from 1 to k, 0 indicates 
         trimmed observations. Note that it could be empty clusters with no observations 
         when \code{equal.weights=FALSE}.
    \item obj - The value of the objective function of the best (returned) solution.
    \item NlogL - A value related to the classification log-likelihood of the best (returned) solution. 
         If \code{opt=="HARD"}, \code{NlogL = -2*obj}.
    \item size - An integer vector of size k, returning the number of observations contained by each cluster.
    \item weights - Vector of Cluster weights
    \item centers - A matrix of size p x k containing the centers (column-wise) of each cluster. 
    \item cov - 	An array of size p x p x k containing the covariance matrices of each cluster. 
    \item code - A numerical value indicating if the concentration steps have 
         converged for the returned solution (2).
    \item posterior - A matrix with k columns that contains the posterior 
         probabilities of membership of each observation (row-wise) to the \code{k} 
         clusters. This posterior probabilities are 0-1 values in the 
         \code{opt="HARD"} case. Trimmed observations have 0 membership probabilities 
         to all clusters.
     \item{MIXMIX} - BIC which based on the parameters estimated through the mixture log-likelihood and 
         the maximized mixture likelihood as goodness of fit measure. This output 
         is present only  if \code{opt="MIXT"}.
     \item{MIXMIX} - BIC which uses the classification likelihood based on
         parameters estimated through the mixture likelihood (In some books 
         this quantity is called ICL). This output 
         is present only  if \code{opt="MIXT"}.
    \item{CLACLA} - BIC which uses the classification likelihood based on
         parameters estimated using the classification likelihood. This output 
         is present only  if \code{opt="HARD"}.
    \item cluster.ini - A matrix with nstart rows and number of columns equal to 
         the number of observations and where each row shows the final clustering 
         assignments (0 for trimmed observations) obtained after the \code{niter1} 
         iteration of the \code{nstart} random initializations.
    \item obj.ini - A numerical vector of length \code{nstart} containing the values 
         of the target function obtained after the \code{niter1} iteration of the 
         \code{nstart} random initializations.
    \item x - The input data set.
    \item k - The input number of clusters.
    \item alpha - The input trimming level.
}
}
\description{
This function searches for \code{k} (or less) clusters with 
 different covariance structures in a data matrix \code{x}. Relative cluster 
 scatter can be restricted when \code{restr="eigen"} by constraining the ratio 
 between the largest and the smallest of the scatter matrices eigenvalues 
 by a constant value \code{restr.fact}. Relative cluster scatters can be also 
 restricted with \code{restr="deter"} by constraining the ratio between the 
 largest and the smallest of the scatter matrices' determinants. 

 For robustifying the estimation, a proportion \code{alpha} of observations is trimmed. 
 In particular, the trimmed k-means method is represented by the \code{tclust()} method,
 by setting parameters \code{restr.fact=1}, \code{opt="HARD"} and \code{equal.weights=TRUE}.
}
\details{
The procedure allows to deal with robust clustering with an \code{alpha}
 proportion of trimming level and searching for \code{k} clusters. We are considering 
 classification trimmed likelihood when using \code{opt=”HARD”} so that “hard” or “crisp” 
 clustering assignments are done. On the other hand, mixture trimmed likelihood 
 are applied when using \code{opt=”MIXT”} so providing a kind of clusters “posterior” 
 probabilities for the observations. 
 Relative cluster scatter can be restricted when \code{restr="eigen"} by constraining 
 the ratio between the largest and the smallest of the scatter matrices eigenvalues 
 by a constant value \code{restr.fact}. Setting \code{restr.fact=1}, yields the 
 strongest restriction, forcing all clusters to be spherical and equally scattered. 
 Relative cluster scatters can be also restricted with \code{restr="deter"} by 
 constraining the ratio between the largest and the smallest of the scatter 
 matrices' determinants. 

 This iterative algorithm performs "concentration steps" to improve the current 
 cluster assignments. For approximately obtaining the global optimum, the procedure 
 is randomly initialized \code{nstart} times and \code{niter1} concentration steps are performed for 
 them. The \code{nkeep} most “promising” iterations, i.e. the \code{nkeep} iterated solutions with 
 the initial best values for the target function, are then iterated until convergence 
 or until \code{niter2} concentration steps are done. 

The parameter \code{restr.fact} defines the cluster scatter matrices restrictions, 
 which are applied on all clusters during each concentration step. It restricts 
 the ratio between the maximum and minimum eigenvalue of 
 all clusters' covariance structures to that parameter. Setting \code{restr.fact=1}, 
 yields the strongest restriction, forcing all clusters to be spherical and equally scattered. 

Cluster components with similar sizes are favoured when considering \code{equal.weights=TRUE} 
 while \code{equal.weights=FALSE} admits possible different prior probabilities for 
 the components and it can easily return empty clusters when the number of 
 clusters is greater than apparently needed.
}
\examples{

 \dontshow{
     set.seed (0)
 }
 ##--- EXAMPLE 1 ------------------------------------------
 sig <- diag(2)
 cen <- rep(1,2)
 x <- rbind(MASS::mvrnorm(360, cen * 0,   sig),
            MASS::mvrnorm(540, cen * 5,   sig * 6 - 2),
            MASS::mvrnorm(100, cen * 2.5, sig * 50))
 
 ## Two groups and 10\\% trimming level
 clus <- tclust(x, k = 2, alpha = 0.1, restr.fact = 8)
 
 plot(clus)
 plot(clus, labels = "observation")
 plot(clus, labels = "cluster")
 
 ## Three groups (one of them very scattered) and 0\\% trimming level
 clus <- tclust(x, k = 3, alpha=0.0, restr.fact = 100)
 
 plot(clus)
 
 ##--- EXAMPLE 2 ------------------------------------------
 data(geyser2)
 (clus <- tclust(geyser2, k = 3, alpha = 0.03))
 
 plot(clus)
 
\donttest{

 ##--- EXAMPLE 3 ------------------------------------------
 data(M5data)
 x <- M5data[, 1:2]
 
 clus.a <- tclust(x, k = 3, alpha = 0.1, restr.fact =  1,
                   restr = "eigen", equal.weights = TRUE)
 clus.b <- tclust(x, k = 3, alpha = 0.1, restr.fact =  50,
                    restr = "eigen", equal.weights = FALSE)
 clus.c <- tclust(x, k = 3, alpha = 0.1, restr.fact =  1,
                   restr = "deter", equal.weights = TRUE)
 clus.d <- tclust(x, k = 3, alpha = 0.1, restr.fact = 50,
                   restr = "deter", equal.weights = FALSE)
 
 pa <- par(mfrow = c (2, 2))
 plot(clus.a, main = "(a)")
 plot(clus.b, main = "(b)")
 plot(clus.c, main = "(c)")
 plot(clus.d, main = "(d)")
 par(pa)
 
 ##--- EXAMPLE 4 ------------------------------------------

 data (swissbank)
 ## Two clusters and 8\\% trimming level
 (clus <- tclust(swissbank, k = 2, alpha = 0.08, restr.fact = 50))
 
 ## Pairs plot of the clustering solution
 pairs(swissbank, col = clus$cluster + 1)
 ## Two coordinates
 plot(swissbank[, 4], swissbank[, 6], col = clus$cluster + 1,
      xlab = "Distance of the inner frame to lower border",
      ylab = "Length of the diagonal")
 plot(clus)
 
 ## Three clusters and 0\\% trimming level
 clus<- tclust(swissbank, k = 3, alpha = 0.0, restr.fact = 110)
 
 ## Pairs plot of the clustering solution
 pairs(swissbank, col = clus$cluster + 1)
 
 ## Two coordinates
 plot(swissbank[, 4], swissbank[, 6], col = clus$cluster + 1, 
       xlab = "Distance of the inner frame to lower border", 
       ylab = "Length of the diagonal")
 
 plot(clus)
 
 ##--- EXAMPLE 5 ------------------------------------------
  data(M5data)
  x <- M5data[, 1:2]
  
  ## Classification trimmed likelihood approach
  clus.a <- tclust(x, k = 3, alpha = 0.1, restr.fact =  50,
                     opt="HARD", restr = "eigen", equal.weights = FALSE)
 ## Mixture trimmed likelihood approach
  clus.b <- tclust(x, k = 3, alpha = 0.1, restr.fact =  50,
                     opt="MIXT", restr = "eigen", equal.weights = FALSE)
 
 ## Hard 0-1 cluster assignment (all 0 if trimmed unit)
 head(clus.a$posterior)
 
 ## Posterior probabilities cluster assignment for the
 ##  mixture approach (all 0 if trimmed unit)
 head(clus.b$posterior)
 
}

}
\references{
Fritz, H.; Garcia-Escudero, L.A.; Mayo-Iscar, A. (2012), "tclust: An R Package 
 for a Trimming Approach to Cluster Analysis". Journal of Statistical Software, 
 47(12), 1-26. URL http://www.jstatsoft.org/v47/i12/

Garcia-Escudero, L.A.; Gordaliza, A.; Matran, C. and Mayo-Iscar, A. (2008), 
 "A General Trimming Approach to Robust Cluster Analysis". Annals of Statistics, 
 Vol.36, 1324--1345.  

García-Escudero, L. A., Gordaliza, A. and Mayo-Íscar, A. (2014). A constrained 
 robust proposal for mixture modeling avoiding spurious solutions. 
 Advances in Data Analysis and Classification, 27--43. 

García-Escudero, L. A., and Mayo-Íscar, A. and Riani, M. (2020). Model-based 
 clustering with determinant-and-shape constraint. Statistics and Computing, 
 30, 1363--1380.]
}
\author{
Javier Crespo Guerrero, Luis Angel Garcia Escudero, Agustin Mayo Iscar.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{M5data}
\alias{M5data}
\title{M5data data}
\format{
The first two columns are the two variables. The last column is the true 
 classification vector where symbol "0" stands for the contaminating data points.
}
\source{
García-Escudero, L.A.; Gordaliza, A.; Matrán, C. and Mayo-Iscar, A. (2008), 
 "A General Trimming Approach to Robust Cluster Analysis". Annals of Statistics, 
 Vol.36, pp. 1324-1345.
}
\usage{
data(M5data)
}
\description{
A bivariate data set obtained from three normal bivariate distributions with 
 different scales and proportions 1:2:2. One of the components is very overlapped 
 with another one. A 10\% background noise is added uniformly distributed in a rectangle 
 containing the three normal components and not very overlapped with the three mixture 
 components. A precise description of the M5 data set can be found in 
 García-Escudero et al. (2008).
}
\examples{
#--- EXAMPLE 1 ------------------------------------------ 
data (M5data) 
x <- M5data[, 1:2] 
clus <- tclust(x, k=3, alpha=0.1, nstart=200, niter1=3, niter2=17, 
   nkeep=10, opt="HARD", equal.weights=FALSE, restr.fact=50, trace=TRUE) 
plot (x, col=clus$cluster+1)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{swissbank}
\alias{swissbank}
\title{Swiss banknotes data}
\format{
A data frame containing 200 observations in 6 variables. 
The variables are as follows:

\itemize{
     \item \code{Length} Length of the bank note 
     \item \code{Ht_Left} Height of the bank note, measured on the left 
     \item \code{Ht_Right} Height of the bank note, measured on the right 
     \item \code{IF_Lower} Distance of inner frame to the lower border 
     \item \code{IF_Upper} Distance of inner frame to the upper border 
     \item \code{Diagonal} Length of the diagonal 
 }
}
\source{
Flury, B. and Riedwyl, H. (1988). \emph{Multivariate Statistics, A Practical Approach}, Cambridge University Press.
}
\usage{
data(swissbank)
}
\description{
Six variables measured on 100 genuine and 100 counterfeit old Swiss 1000-franc 
 bank notes (Flury and Riedwyl, 1988).
}
\details{
Observations 1--100 are the genuine bank notes and the other 100 observations are the counterfeit bank notes.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DiscrFact.R
\name{plot.DiscrFact}
\alias{plot.DiscrFact}
\title{The \code{plot} method for objects of class \code{DiscrFact}}
\usage{
\method{plot}{DiscrFact}(
  x,
  enum.plots = FALSE,
  xlab = "Discriminant Factor",
  ylab = "Clusters",
  print.DiscrFact = TRUE,
  xlim,
  col.nodoubt = grey(0.8),
  by.cluster = FALSE,
  ...
)
}
\arguments{
\item{x}{An object of class \code{DiscrFact} as returned from DiscrFact()}

\item{enum.plots}{A logical value indicating whether the plots shall be enumerated 
in their title ("(a)", "(b)", "(c)").}

\item{xlab, ylab, xlim}{Arguments passed to funcion \code{plot.tclust()}}

\item{print.DiscrFact}{A logical value indicating whether each clusters mean discriminant factor shall be plotted}

\item{col.nodoubt}{Color of all observations not considered as to be assigned doubtfully.}

\item{by.cluster}{Logical value indicating whether optional parameters pch and col 
(if present) refer to observations (FALSE) or clusters (TRUE)}

\item{\ldots}{Arguments to be passed to or from other methods}
}
\description{
The \code{plot} method for class \code{DiscrFact}: Next to a plot of the \code{tclust} 
 object which has been used for creating the \code{DiscrFact} object, a silhouette plot 
 indicates the presence of groups with a large amount of doubtfully assigned 
 observations. A third plot similar to the standard \code{tclust} plot serves 
 to highlight the identified doubtful observations.
}
\details{
\code{plot_DiscrFact_p2} displays a silhouette plot based on the discriminant 
 factors of the observations. A solution with many large discriminant factors is 
 not reliable. Such clusters can be identified with this silhouette plot. 
 Thus \code{plot_DiscrFact_p3} displays the dataset, highlighting observations with 
 discriminant factors greater than the given threshold. The function \code{plot.DiscrFact()} 
 combines the standard plot of a \code{tclust} object, and the two plots introduced here.
}
\examples{
 sig <- diag (2)
 cen <- rep (1, 2)
 x <- rbind(MASS::mvrnorm(360, cen * 0,   sig),
 	       MASS::mvrnorm(540, cen * 5,   sig * 6 - 2),
 	       MASS::mvrnorm(100, cen * 2.5, sig * 50))

 clus.1 <- tclust(x, k = 2, alpha=0.1, restr.fact=12)
 clus.2 <- tclust(x, k = 3, alpha=0.1, restr.fact=1)

 dsc.1 <- DiscrFact(clus.1)
 plot(dsc.1)

 dsc.2 <- DiscrFact(clus.2)
 plot(dsc.2)

}
\references{
\enc{García}{Garcia}-Escudero, L.A.; Gordaliza, A.; \enc{Matrán}{Matran}, C. and Mayo-Iscar, A. (2011), 
   "Exploring the number of groups in robust model-based clustering." \emph{Statistics and Computing}, \bold{21} 
   pp. 585-599, <doi:10.1007/s11222-010-9194-z>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DiscrFact.R
\name{DiscrFact}
\alias{DiscrFact}
\alias{print.DiscrFact}
\title{Discriminant Factor analysis for \code{tclust} objects}
\usage{
DiscrFact(x, threshold = 1/10)
}
\arguments{
\item{x}{A \code{tclust} object.}

\item{threshold}{A cluster assignment or a trimming decision for an observation with a 
discriminant factor larger than \code{log(threshold)} is considered a "doubtful" decision.}
}
\value{
The function returns an S3 object of type \code{DiscrFact} containing the following components:
\itemize{
     \item \code{x} A \code{tclust} object. 
     \item \code{ylimmin} A minimum y-limit calculated for plotting purposes. 
    \item \code{ind} The actual cluster assignment. 
    \item \code{ind2} The second most likely cluster assignment for each observation. 
    \item \code{lik} The (weighted) likelihood of the actual cluster assignment of each observation. 
    \item \code{lik2} The (weighted) likelihood of the second best cluster assignment of each observation. 
    \item \code{assignfact} The factor \code{log(disc/disc2)}. 
    \item \code{threshold} The threshold used for deciding whether \code{assignfact} indicates a "doubtful" assignment. 
    \item \code{mean.DiscrFact} A vector of length \code{k + 1} containing the mean discriminant 
     factors for each cluster (including the outliers). 
 }
}
\description{
Analyzes a \code{tclust}-object by calculating discriminant factors 
 and comparing the quality of the actual cluster assignments to that of the second best 
 possible assignment for each observation. Cluster assignments of observations 
 with large discriminant factors are considered "doubtful" decisions. Silhouette 
 plots give a graphical overview of the discriminant factors distribution 
 (see \code{\link{plot.DiscrFact}}). More details can be found in García-Escudero et al. (2011).
}
\references{
\enc{García}{Garcia}-Escudero, L.A.; Gordaliza, A.; \enc{Matrán}{Matran}, C. and Mayo-Iscar, A. (2011), 
   "Exploring the number of groups in robust model-based clustering." \emph{Statistics and Computing}, \bold{21} 
   pp. 585-599, <doi:10.1007/s11222-010-9194-z>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{LG5data}
\alias{LG5data}
\title{LG5data data}
\format{
The first 10 columns are the variables. The last column is the true 
 classification vector where symbol "0" stands for the contaminating data points.
}
\usage{
data(LG5data)
}
\description{
A data set in dimension 10 with three clusters around affine subspaces
 of common intrinsic dimension. A 10\% background noise is added uniformly 
 distributed in a rectangle containing the three main clusters.
}
\examples{
#--- EXAMPLE 1 ------------------------------------------ 
data (LG5data)
x <- LG5data[, 1:10]
clus <- rlg(x, d = c(2,2,2), alpha=0.1, trace=TRUE)
plot(x, col=clus$cluster+1)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{geyser2}
\alias{geyser2}
\title{Old Faithful Geyser Data}
\format{
A data frame containing 272 observations in 2 variables. 
The variables are as follows:

\itemize{
     \item \code{Eruption length} The eruption length in minutes. 
     \item \code{Previous eruption length} The length of the previous eruption in minutes. 
 }
}
\source{
This particular data structure can be obtained by applying the following code
 to the "Old Faithful Geyser" (\code{faithful} data set (Härdle 1991) in the 
 package \code{datasets}):
 \cr
 \code{f1 <- faithful[,1]}\cr
 \code{geyser2 <- cbind(f1[-length(f1)], f1[-1])}\cr
 \code{colnames(geyser2) <- c("Eruption length",}\cr
 \code{                      "Previous eruption length")}
}
\usage{
data(geyser2)
}
\description{
A bivariate data set obtained from the Old Faithful Geyser, containing the 
 eruption length and the length of the previous eruption for 271 eruptions 
 of this geyser in minutes.
}
\references{
García-Escudero, L.A. and Gordaliza, A. (1999). 
 Robustness properties of k-means and trimmed k-means. \emph{Journal of the American Statistical Assoc.}, Vol.94, No.447, 956--969.

 Härdle, W. (1991). \emph{Smoothing Techniques with Implementation in S.}, New York: Springer.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simul_rlg.R
\name{simula.rlg}
\alias{simula.rlg}
\title{Simulate contaminated data set for applying rlg}
\usage{
simula.rlg(q = 2, p = 10, n = 200, var = 0.01, sep.means = 0, alpha = 0.05)
}
\arguments{
\item{q}{intrinsic dimension}

\item{p}{dimension (\code{p >= 2} and \code{p > q})}

\item{n}{number of observations}

\item{var}{The smaller 'var' the smaller the scatter around the lower dimensional space}

\item{sep.means}{Parameter controlling the location vectors separation}

\item{alpha}{contamination level}
}
\value{
a list with the following items
\itemize{
    \item x - The generated dataset
    \item true - The true classification
}
}
\description{
Simulate alpha*100\% contaminated data set for applying rlg by 
 generating a k=3 components with equal size  and            # 
 common underlying dimension q_1=q_2=q_3=q
}
\examples{
 res <- simula.rlg(q=5, p=200, n=150, var=0.01, sep.means=0.00)
 plot(res$x,col=res$true+1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mixsym}
\alias{mixsym}
\title{mixsym}
\format{
A data frame with 10 rows and 3 variables: 2 numerical and one categorical - the thrue cluster.
The variables are as follows:

\itemize{
  \item X1: first numerical variable
  \item X2: second numerical variable
  \item class: the true cluster
}
}
\usage{
data(mixsym)
}
\description{
A simulated bivariate data set of size \code{n = 100} and 
 dimension \code{p = 2} from a \code{k = 3} component mixture obtained
by applying the \code{MixSim} method of Maitra and Melnykov (2010), 
 as extended by Riani et al. (2015) and incorporated into the FSDA toolbox 
 of Matlab (Riani et al., 2012). The data set has been generated by imposing 
 an average cluster overlap (defined as a sum of pairwise misclassification 
 probabilities) equal to 0.04 and a maximum eigenvalue ratio for the scatters 
 matrices equal to 5.
}
\details{
Simulated bivariate data set
}
\examples{
 data(mixsym)
 head(mixsym)

}
\references{
Maitra, R. and Melnykov, V. (2010). Simulating data to study performance of finite mixture
 modeling and clustering algorithms. J. Comput. Graph. Stat., 19:354– 376.

Riani, M., Cerioli, A., Perrotta, D., and Torti, F. (2015). Simulating mixtures of multivariate 
 data with fixed cluster overlap in FSDA library. Adv. Data Anal. Classif., 9:2015.

Riani, M., Perrotta, D., and Torti, F. (2012). FSDA: a matlab toolbox for robust analysis
 and interactive data exploration,. Chemometr. Intell. Lab. Syst., 116:17–32.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ctlcurves.R
\name{plot.ctlcurves}
\alias{plot.ctlcurves}
\title{The \code{plot} method for objects of class \code{ctlcurves}}
\usage{
\method{plot}{ctlcurves}(
  x,
  what = c("obj", "min.weights", "doubtful"),
  main,
  xlab,
  ylab,
  xlim,
  ylim,
  col,
  lty = 1,
  ...
)
}
\arguments{
\item{x}{The ctlcurves object to be shown}

\item{what}{A string indicating which type of plot shall be drawn. See the details section for more information.}

\item{main}{A character-string containing the title of the plot.}

\item{xlab, ylab, xlim, ylim}{Arguments passed to plot().}

\item{col}{A single value or vector of line colors passed to \code{\link[graphics]{lines}}.}

\item{lty}{A single value or vector of line colors passed to \code{\link[graphics]{lines}}.}

\item{\ldots}{Arguments to be passed to or from other methods.}
}
\description{
The \code{plot} method for class \code{ctlcurves}: This function implements 
 a series of plots, which display characteristic values 
 of the each model, computed with different values for \code{k} and \code{alpha}.
}
\details{
These curves show the values of the trimmed classification (log-)likelihoods 
 when altering the trimming proportion \code{alpha} and the number of clusters \code{k}.
 The careful examination of these curves provides valuable information for choosing these 
 parameters in a clustering problem. For instance, an appropriate \code{k} to be chosen 
 is one that we do not observe a clear increase in the trimmed classification likelihood
 curve for \code{k} with respect to the \code{k+1} curve for almost all the range of
 \code{alpha} values. Moreover, an appropriate choice of parameter \code{alpha} may
 be derived by determining where an initial fast increase of the trimmed classification 
 likelihood curve stops for the final chosen \code{k}. A more detailed explanation 
 can be found in García-Escudero et al. (2011).
  
 This function implements a series of plots, which display characteristic values 
 of the each model, computed with different values for \code{k} and \code{alpha}.
 \describe{
   \item{\code{"obj"}}{Objective function values.}
   \item{\code{"min.weights"}}{The minimum cluster weight found for each computed model. 
     This plot is intended to spot spurious clusters, which in 
     general yield quite small weights.}
   \item{\code{"doubtful"}}{The number of "doubtful" decisions identified by \code{\link{DiscrFact}}.}
 }
}
\examples{

 #--- EXAMPLE 1 ------------------------------------------

 \donttest{
 sig <- diag (2)
 cen <- rep (1, 2)
 x <- rbind(MASS::mvrnorm(108, cen * 0,   sig),
 	       MASS::mvrnorm(162, cen * 5,   sig * 6 - 2),
 	       MASS::mvrnorm(30, cen * 2.5, sig * 50))

 (ctl <- ctlcurves(x, k = 1:4))

 plot(ctl)
 }

}
\references{
\enc{García}{Garcia}-Escudero, L.A.; Gordaliza, A.; \enc{Matrán}{Matran}, C. and Mayo-Iscar, A. (2011), 
   "Exploring the number of groups in robust model-based clustering." \emph{Statistics and Computing}, \bold{21} 
   pp. 585-599, <doi:10.1007/s11222-010-9194-z>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{flea}
\alias{flea}
\title{Flea}
\format{
A data frame with 74 rows and 7 variables: six explanatory and one response variable - \code{species}.
The variables are as follows:

\itemize{
  \item tars1: width of the first joint of the first tarsus in microns (the sum of measurements for both tarsi)
  \item tars2: the same for the second joint
  \item head: the maximal width of the head between the external edges of the eyes in 0.01 mm
  \item ade1: the maximal width of the aedeagus in the fore-part in microns
  \item ade2: the front angle of the aedeagus ( 1 unit = 7.5 degrees)
  \item ade3: the aedeagus width from the side in microns
  \item species, which species is being examined - \code{Concinna}, \code{Heptapotamica}, \code{Heikertingeri}
}
}
\usage{
data(flea)
}
\description{
Flea-beetle measurements
}
\examples{
 data(flea)
 head(flea)

}
\references{
A. A. Lubischew (1962), On the Use of Discriminant Functions in Taxonomy, \emph{Biometrics}, \bold{18}4 pp.455--477.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.rlg.R
\name{plot.rlg}
\alias{plot.rlg}
\title{Plot an 'rlg' object}
\usage{
\method{plot}{rlg}(
  x,
  which = c("all", "scores", "loadings", "eigenvalues"),
  sort = TRUE,
  ask = (which == "all" && dev.interactive(TRUE)),
  ...
)
}
\arguments{
\item{x}{An \code{rlg} object to plot.}

\item{which}{Select the required plot.}

\item{sort}{Whether to sort.}

\item{ask}{if \code{TRUE}, the user is \emph{ask}ed before each plot, see \code{par(ask=.)}. 
Default is \code{ask = which=="all" && dev.interactive()}.}

\item{\dots}{Other parameters to be passed to the lower level functions.}
}
\description{
Different plots for the results of 'rlg' analysis, stored in an 
   \code{rlg} object, see Details.
}
\examples{
 data (LG5data)
 x <- LG5data[, 1:10]
 clus <- rlg(x, d = c(2,2,2), alpha=0.1)
 plot(clus, which="eigenvalues") 
 plot(clus, which="scores") 

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tclustICsol.R
\name{tclustICsol}
\alias{tclustICsol}
\alias{print.tclustICsol}
\title{Extracts a set of best relevant solutions obtained by \code{\link{tclustIC}}}
\usage{
tclustICsol(
  obj,
  whichIC = c("ALL", "MIXMIX", "MIXCLA", "CLACLA"),
  nsol = 5,
  index = c("Rand", "FM"),
  thresholdRI = 0.7,
  trace = FALSE
)
}
\arguments{
\item{obj}{An S3 object of class \code{\link{tclustIC}}
(output of \code{\link{tclustIC}}) containing the values
of the information criteria BIC (MIXMIX), ICL (MIXCLA) or CLA (CLACLA),
for different values of \code{k} (number of groups) and different
values of \code{c} (restriction factor), for a prespecified level of trimming.}

\item{whichIC}{A character value which Specifies the information criterion to 
use to extract best solutions. Possible values for \code{whichIC} are:
\itemize{
 \item "MIXMIX": a mixture model is fitted and for computing the information criterion
    the mixture likelihood is used. This option corresponds to the use of the Bayesian
    Information criterion (BIC). In output just the matrix \code{MIXMIX} is given.
\item "MIXCLA": a mixture model is fitted but to compute the information criterion
    the classification likelihood is used. This option corresponds to the use of the
    Integrated Complete Likelihood (ICL). In the output just the matrix \code{MIXCLA} is given.
\item "CLACLA": everything is based on the classification likelihood. This information
    criterion will be called CLA. In the output just the matrix \code{CLACLA} is given.
\item "ALL": both classification and mixture likelihood are used. In this case all
    three information criteria CLA, ICL and BIC are computed. In the output all
    three matrices \code{MIXMIX}, \code{MIXCLA} and \code{CLACLA} are given.
}}

\item{nsol}{Number of best solutions to extract from BIC/ICL matrix.
The default value of NumberOfBestSolutions is 5}

\item{index}{Index to use to compare partitions. If \code{index=Rand} (default) 
the adjusted Rand index is used, else, \code{index="FM"}, the adjusted 
Fowlkes and Mallows index is used}

\item{thresholdRI}{Threshold to identify spurious solutions - the threshold
 of the adjusted Rand index to use to consider two solutions as equivalent.
The default value of ThreshRandIndex is 0.7}

\item{trace}{Whether to print intermediate results. Default is \code{trace=FALSE}.}
}
\value{
The function returns an S3 object of type \code{tclustICsol} containing the following components:
 \item{call}{the matched call}
 \item{kk}{a vector containing the values of \code{k} (number of components) which have been considered.
     This vector is identical to the optional argument \code{kk} (default is \code{kk=1:5}.}
 \item{cc}{a vector containing the values of \code{c} (values of the restriction factor) which
     have been considered. This vector is identical to the optional argument \code{cc}
  (defalt is \code{cc=c(1, 2, 4, 8, 16, 32, 64, 128)}.}
 \item{alpha}{trimming level}
 \item{whichIC}{Information criteria used}
 \item{MIXMIXbs}{a matrix of lists of size \code{NumberOfBestSolutions-times-5} which
 contains the details of the best solutions for MIXMIX (BIC). Each row refers to a
 solution. The information which is stored in the columns is as follows.
 \itemize{
 \item 1st col = value of k for which solution takes place
 \item 2nd col = value of c for which solution takes place;
 \item 3rd col = a vector of length \code{d} which contains the values of \code{c}
     for which the solution is uniformly better.
 \item 4th col = a vector of length \code{d + r} which contains the values of \code{c}
     for which the solution is considered stable (i.e. for which the value
     of the adjusted Rand index (or the adjusted Fowlkes and Mallows index)
     does not go below the threshold defined in input option \code{ThreshRandIndex}).
 \item 5th col = string which contains 'true' or 'spurious'. The solution is labelled
     spurious if the value of the adjusted Rand index with the previous solutions
     is greater than ThreshRandIndex.
 }

 Remark: the field MIXMIXbs is present only if \code{whichIC=ALL} or \code{whichIC="MIXMIX"}.
 }
 \item{MIXMIXbsari}{a matrix of adjusted Rand indexes (or Fowlkes and Mallows indexes)
     associated with the best solutions for MIXMIX. A matrix of size \code{NumberOfBestSolutions-times-NumberOfBestSolutions}
     whose \code{i,j}-th entry contains the adjusted Rand index between classification produced by solution
     \code{i} and solution \code{j}, \code{i,j=1,2, ...,NumberOfBestSolutions}.

 Remark: the field \code{MIXMIXbsari} is present only if \code{whichIC=ALL} or \code{whichIC="MIXMIX"}.
 }
 \item{ARIMIX}{a matrix of adjusted Rand indexes between two consecutive value of \code{c}.
     Matrix of size \code{k-by-length(cc)-1}. The first column contains the ARI indexes
     between \code{cc[2]} and \code{cc[1]} given \code{k}.
     The second column contains the the ARI indexes between \code{cc[3]} and \code{cc[2]} given \code{k}.

 Remark: the field \code{ARIMIX} is present only if \code{whichIC=ALL} or \code{whichIC="MIXMIX"} or \code{whichIC="MIXCLA"}.
 }
 \item{MIXCLAbs}{has the same structure as \code{MIXMIXbs} but referres to MIXCLA.

 Remark: the field MIXCLAbs is present only if \code{whichIC=ALL} or \code{whichIC="MIXCLA"}.
 }
 \item{MIXCLAbsari}{has the same structure as \code{MIXMIXbsari} but referres to MIXCLA.

 Remark: the field \code{MIXMIXbsari} is present only if \code{whichIC=ALL} or \code{whichIC="MIXCLA"}.
 }
 \item{CLACLAbs}{has the same structure as \code{MIXMIXbs} but referres to CLACLA.

 Remark: the field CLACLAbs is present only if \code{whichIC=ALL} or \code{whichIC="CLACLA"}.
 }
 \item{CLACLAbsari}{has the same structure as \code{MIXMIXbsari} but referres to CLACLA.

 Remark: the field \code{CLACLAbsari} is present only if \code{whichIC=ALL} or \code{whichIC="CLACLA"}.
 }
 \item{ARICLA}{a matrix of adjusted Rand indexes between two consecutive value of \code{c}.
     Matrix of size \code{k-by-length(cc)-1}. The first column contains the ARI indexes
     between \code{cc[2]} and \code{cc[1]} given \code{k}.
     The second column contains the the ARI indexes between \code{cc[3]} and \code{cc[2]} given \code{k}.

 Remark: the field \code{ARICLA} is present only if \code{whichIC=ALL} or \code{whichIC="CLACLA"}.
 }
 \item{x}{the input data matrix of size \code{n-times-p} with which
     the Information Criteria were computed.}
}
\description{
The function \code{tclustICsol()} takes as input an object of class
 \code{\link{tclustIC}}, the output
 of function \code{\link{tclustIC}} (that is a series of matrices which contain
 the values of the information criteria BIC/ICL/CLA for different values of \code{k}
 and \code{c}) and extracts the first best solutions. Two solutions are considered
 equivalent if the value of the adjusted Rand index (or the adjusted Fowlkes and
 Mallows index) is above a certain threshold. For each tentative solution the program
 checks the adjacent values of \code{c} for which the solution is stable.
 A matrix with adjusted Rand indexes is given for the extracted solutions.
}
\examples{

 #--- EXAMPLE 1 ------------------------------------------
 \donttest{
 data(geyser2)
 (out <- tclustIC(geyser2, whichIC="MIXMIX", alpha=0.1))

 ##  Show the first two best solutions using as Information criterion MIXMIX
 cat("\nBest solutions using MIXMIX\n")
 outsol <- tclust::tclustICsol(out, whichIC="MIXMIX", nsol=2)
 print(outsol$MIXMIXbs)
 }

 #--- EXAMPLE 2 ------------------------------------------
 \donttest{
 data(flea)
 Y <- as.matrix(flea[, 1:(ncol(flea)-1)])    # select only the numeric variables
 rownames(Y) <- 1:nrow(Y)
 head(Y)

 (out <- tclustIC(Y, whichIC="CLACLA", alpha=0.1))
 ## Find the smallest value inside the table and write the corresponding
 ## values of k (number of groups) and c (restriction factor)
 inds <- which(out$CLACLA == min(out$CLACLA), arr.ind=TRUE)
 vals <- out$CLACLA[inds]
 cat("\nThe Smallest value of the IC is ", vals, 
     " and takes place for k=", out$kk[inds[1]], " and c=",   
     out$cc[inds[2]], "\n")

 ##  Show the first two best solutions using as Information criterion CLACLA
 cat("\nBest solutions using CLACLA\n")
 outsol <- tclust::tclustICsol(out, whichIC="CLACLA", nsol=2)
 print(outsol$CLACLAbs)
 
 }

 #--- EXAMPLE 3 ------------------------------------------
 \donttest{
 data(swissbank)
 (out <- tclustIC(swissbank, whichIC="ALL"))
 
 outsol <- tclust::tclustICsol(out, whichIC="ALL", nsol=2)
 print(outsol$CLACLAbs)
 
 }

}
\references{
Cerioli, A., Garcia-Escudero, L.A., Mayo-Iscar, A. and Riani M. (2017).
     Finding the Number of Groups in Model-Based Clustering via Constrained Likelihoods,
     \emph{Journal of Computational and Graphical Statistics}, pp. 404--416,
     https://doi.org/10.1080/10618600.2017.1390469.

 Hubert L. and Arabie P. (1985), Comparing Partitions, \emph{Journal of Classification}, 
     Vol. 2, pp. 193--218.
}
\seealso{
\code{\link{tclust}}, \code{\link{tclustIC}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ctlcurves.R
\name{ctlcurves}
\alias{ctlcurves}
\alias{print.ctlcurves}
\title{Classification Trimmed Likelihood Curves}
\usage{
ctlcurves(
  x,
  k = 1:4,
  alpha = seq(0, 0.2, len = 6),
  restr.fact = 50,
  parallel = FALSE,
  trace = 1,
  ...
)
}
\arguments{
\item{x}{A matrix or data frame of dimension n x p, containing the observations (row-wise).}

\item{k}{A vector of cluster numbers to be checked. By default cluster numbers from 1 to 5 are examined.}

\item{alpha}{A vector containing the alpha levels to be checked. By default \code{alpha} 
levels from 0 to 0.2 (continuously increased by 0.01), are checked.}

\item{restr.fact}{The restriction factor passed to \code{\link{tclust}}.}

\item{parallel}{A logical value, to be passed further to \code{tclust()}.}

\item{trace}{Defines the tracing level, which is set to \code{1} by default. 
Tracing level \code{2} gives additional information on the current iteration.
Tracing level \code{} suppresses all trace messages.}

\item{\ldots}{Further arguments (as e.g. \code{restr}), passed to \code{\link{tclust}}}
}
\value{
The function returns an S3 object of type \code{ctlcurves} containing the following components:
\itemize{
 \item \code{par} A list containing all the parameters passed to this function 
\item \code{obj}  An array containing the objective functions values of each computed cluster-solution 
\item \code{min.weights} An array containing the minimum cluster weight of each computed cluster-solution 
 }
}
\description{
The function applies \code{\link{tclust}} several times on a given dataset while parameters 
 \code{alpha} and \code{k} are altered. The resulting object gives an idea of the optimal 
 trimming level and number of clusters considering a particular dataset.
}
\details{
These curves show the values of the trimmed classification (log-)likelihoods 
 when altering the trimming proportion \code{alpha} and the number of clusters \code{k}. 
 The careful examination of these curves provides valuable information for choosing 
 these parameters in a clustering problem. For instance, an appropriate \code{k} 
 to be chosen is one that we do not observe a clear increase in the trimmed classification 
 likelihood curve for k with respect to the k+1 curve for almost all the range 
 of alpha values. Moreover, an appropriate choice of parameter alpha may be derived 
 by determining where an initial fast increase of the trimmed classification 
 likelihood curve stops for the final chosen k. A more detailed explanation can 
 be found in García-Escudero et al. (2011).
}
\examples{

\donttest{

 #--- EXAMPLE 1 ------------------------------------------

 sig <- diag (2)
 cen <- rep (1, 2)
 x <- rbind(MASS::mvrnorm(108, cen * 0,   sig),
 	       MASS::mvrnorm(162, cen * 5,   sig * 6 - 2),
 	       MASS::mvrnorm(30, cen * 2.5, sig * 50))

 ctl <- ctlcurves(x, k = 1:4)
 ctl

   ##  ctl-curves 
 plot(ctl)  ##  --> selecting k = 2, alpha = 0.08

   ##  the selected model 
 plot(tclust(x, k = 2, alpha = 0.08, restr.fact = 7))

 #--- EXAMPLE 2 ------------------------------------------

 data(geyser2)
 ctl <- ctlcurves(geyser2, k = 1:5)
 ctl
 
   ##  ctl-curves 
 plot(ctl)  ##  --> selecting k = 3, alpha = 0.08

   ##  the selected model
 plot(tclust(geyser2, k = 3, alpha = 0.08, restr.fact = 5))


 #--- EXAMPLE 3 ------------------------------------------
 
 data(swissbank)
 ctl <- ctlcurves(swissbank, k = 1:5, alpha = seq (0, 0.3, by = 0.025))
 ctl
 
   ##  ctl-curves 
 plot(ctl)  ##  --> selecting k = 2, alpha = 0.1
 
   ##  the selected model
 plot(tclust(swissbank, k = 2, alpha = 0.1, restr.fact = 50))
 
}

}
\references{
\enc{García}{Garcia}-Escudero, L.A.; Gordaliza, A.; \enc{Matrán}{Matran}, C. and Mayo-Iscar, A. (2011), 
   "Exploring the number of groups in robust model-based clustering." \emph{Statistics and Computing}, \bold{21} 
   pp. 585-599, <doi:10.1007/s11222-010-9194-z>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{estepRR}
\alias{estepRR}
\title{Function to perform the E-step for a Gaussian mixture distribution}
\arguments{
\item{ll}{Rcpp::NumericMatrix, n-by-k where \code{n} is the number of 
observations and \code{k} is the number of clusters.}
}
\value{
The function returns a list with the following elements:
\itemize{
   \item obj The value of the objective function (total log-likelihood)
   \item postprob an \code{n-by-k} matrix with the posterior probablilities
   \item logpdf a vector of length \code{n} containing the log PDF for 
        each observation
}
}
\description{
Compute the log PDF for each observation, the posterior probabilities
    and the objective function (total log-likelihood) for a Gaussian 
    mixture distribution
}
\details{
Formally a mixture model corresponds to the mixture distribution that
  represents the probability distribution of observations in the overall
  population. Mixture models are used
  to make statistical inferences about the properties of the
  sub-populations given only observations on the pooled population, without
  sub-population-identity information.
  Mixture modeling approaches assume that data at hand $\eqn{y_1, ..., y_n} in
  \eqn{R^p} come from a probability distribution with density given by the sum of k components
  \deqn{\sum_{j=1}^k  \pi_j \phi( \cdot, \theta_j)}
  with \eqn{\phi( \cdot, \theta_j)} being the
  \eqn{p}-variate  (generally multivariate normal) densities with parameters
  \eqn{\theta_j}, \eqn{j=1, \ldots, k}. Generally \eqn{\theta_j= (\mu_j, \Sigma_j)}
  where \eqn{\mu_j} is the population mean  and   \eqn{\Sigma_j} is the covariance
  matrix for component \eqn{j}.
  \eqn{\pi_j} is the (prior) probability of component \eqn{j}.
  The objective function is obj is equal to
   \deqn{ obj = \log   \left( \prod_{i=1}^n  \sum_{j=1}^k \pi_j \phi (y_i; \; \theta_j)    \right) 
    }

   or
 
    \deqn{ obj =  \sum_{i=1}^n  \log   \left( \sum_{j=1}^k \pi_j \phi (y_i; \; \theta_j)    \right)
    }
    where \eqn{k} is the number of components of the mixture and \eqn{\pi_j} are the 
    component probabilitites and \eqn{\theta_j} are the  parameters of the \eqn{j}-th 
    mixture component.
}
\examples{
##      Generate two Gaussian normal distributions
##      and do not produce plots

       mu1 = c(1,2)
       sigma1 = matrix(c(2, 0, 0, .5), nrow=2, byrow=TRUE)    #[2 0; 0 .5];
       mu2 = c(-3, -5)
       sigma2 = matrix(c(1, 0, 0, 1), nrow=2, byrow=TRUE)
       n1 = 100
       n2 = 200
       Y = rbind(MASS::mvrnorm(n1, mu1, sigma1), 
                 MASS::mvrnorm(n2, mu2, sigma2))
       k = 2
       pi = c(1/3, 2/3)
       mu = rbind(mu1, mu2)
       sigma = array(0, dim=c(2,2,2))
       sigma[,,1] = sigma1
       sigma[,,2] = sigma2
       
       ll = matrix(0, nrow=n1+n2, ncol=2)
       for(j in 1:k)
           ll[,j] = log(pi[j]) +  tclust:::dmvnrm(Y, mu[j,], sigma[,,j])

       dd = tclust:::estepRR(ll)
       dd$obj
       dd$logpdf
       dd$postprob
}
\references{
McLachlan, G.J.; Peel, D. (2000). Finite Mixture Models. Wiley. ISBN 0-471-00626-2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tclustIC.R
\name{plot.tclustIC}
\alias{plot.tclustIC}
\title{The \code{plot} method for objects of class \code{tclustIC}}
\usage{
\method{plot}{tclustIC}(x, whichIC, cc, main, xlab, ylab, xlim, ylim, col, lty, ...)
}
\arguments{
\item{x}{The \code{tclustIC} object to be shown}

\item{whichIC}{A string indicating which information criterion will be used. See the details section for more information.}

\item{cc}{choose which curves to plot (for which restriction factors \code{c}).
If missing, by default all curves will be printed.}

\item{main}{A character-string containing the title of the plot.}

\item{xlab, ylab, xlim, ylim}{Arguments passed to plot().}

\item{col}{A single value or vector of line colors passed to \code{\link[graphics]{lines}}.}

\item{lty}{A single value or vector of line types passed to \code{\link[graphics]{lines}}.}

\item{\ldots}{Arguments to be passed to or from other methods.}
}
\description{
The \code{plot} method for class \code{tclustIC}: This function implements 
 a series of plots, which display characteristic values 
 of each model, computed with different values for \code{k} and \code{c} for a fixed \code{alpha}.
}
\examples{

 \donttest{
 sig <- diag (2)
 cen <- rep (1, 2)
 x <- rbind(MASS::mvrnorm(108, cen * 0,   sig),
 	       MASS::mvrnorm(162, cen * 5,   sig * 6 - 2),
 	       MASS::mvrnorm(30, cen * 2.5, sig * 50))

 (out <- tclustIC(x, whichIC="ALL"))

 plot(out)
 }

}
\references{
Cerioli, A., Garcia-Escudero, L.A., Mayo-Iscar, A. and Riani M. (2017).
     Finding the Number of Groups in Model-Based Clustering via Constrained Likelihoods,
     \emph{Journal of Computational and Graphical Statistics}, pp. 404-416,
     https://doi.org/10.1080/10618600.2017.1390469.
}
